#ifndef N_CAMERACONTROLLER_H
#define N_CAMERACONTROLLER_H
//------------------------------------------------------------------------------
/**
    @class nCameraController

    @brief attachs to the camera, intercepts key presses and routes them as physics effects on the camera

    (C)	2005	Chris Miles
*/
#ifndef N_FSM_H
#include "enkiduo/nfsm.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nCameraController
#include "kernel/ndefdllclass.h"

class nPhysics;
class nInputServer;
class nStabilizer;
class nInterfaceServer;
class nTriggerServer;
#include "enkiduo/ngameserver.h"
//------------------------------------------------------------------------------
class nCameraController : public nFSM {
    enum STATES {
        Free,                           //descent style
        RTSCamera,                      ///Overhead camera
        GrabbingGround,                 //grabbing some ground
        Pivot,                          //Pivot around a point in the world
        FirstPerson,                    //On the bridge
        ThirdPerson,                    //descent style bound to a boat
        Homeworld,                      //homeworld style watching 
        TetheredWatch,
        Pan,
    };
public:
    /// constructor
    nCameraController();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nCameraController();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    //virtual void Tick( float time, float dtime );
    bool States( StateMachineEvent event, uint state, float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    virtual void PanToPoint( vector3 point );
    virtual void PanToEnts( EntArray ents );
    virtual void PanToEnts( const char* entPaths );
protected:
    var pos;
    var rot;
    var poffset;
    var roffset;
    var ptightness;
    var rtightness;
    var vel;
    var rvel;

    var attachedTo;
    var attachedPos;
    var attachedRot;

    var mass;
    var inertia;
    var relativeMovementMassCoef;
    var relativeMovementInertiaCoef;

    var watching;
    var watchingPos;

    var realignDelay;
    float lastMovementF;
    float lastMovementT;

    nDynAutoRef<nPhysics>               physics;
    nAutoRef<nInputServer>              ref_input;
    nAutoRef<nGameServer>               ref_game;
    nDynAutoRef<nStabilizer>            ref_stabilizer;
    nAutoRef<nInterfaceServer>          ref_interface;

    var force;
    var torque;
    var panTime;
    var panTimeRemaining;
    var panDestination;

    var windowSettings;
    var aspectRatio;
    var rtsCameraDeclination;
    var rtsCameraDeclinationFixRate;
    nAutoRef<nTriggerServer>            ref_trigger;

    float actualTime0;

    vector2 pivotMousePos;
    vector3 pivotPoint;
    vector3 groundGrabbedPosition;

    STATES mainMovementMode;
    var mouseMovementDelay;
    float mouseMovementStart;
};
//------------------------------------------------------------------------------
#endif
