#ifndef N_WEAPON_H
#define N_WEAPON_H
//------------------------------------------------------------------------------
/**
    @class nWeapon

    @brief responsible for firing (Creating) weapon ents at targets / points

    nweapon should look at what type of weapon it is firing and fill in information appropriately
    it should also be responsible for dealing the damage that weapon does, unless it is a delayed area effect weapon - in which case it will be done by the entity itself upon impact
    first of all what are the different weapon types:
        projectile / dumb fire
        beam
        missle
        any of these can do damage or not.
        how do we find this out? look at /ent/weapon type
    lets take all the fancy calculations out of the parent classes

    what kind of firing do we want
    direct always hit                   #boring laser beam
    fire + random angle (hit/miss)      #for machine gun type sprays
    fire + random position (hit/miss)   #for shell firing

    alternatives? 
    (C)	2004	Chris Miles
*/
#ifndef N_OFFSETASPECT_H
#include "physics/noffsetaspect.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nWeapon
#include "kernel/ndefdllclass.h"

class nCollisionNode;
class nGameServer;
//------------------------------------------------------------------------------
class nWeapon : public nOffsetAspect
{
public:
    /// constructor
    nWeapon();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nWeapon();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    virtual bool CanFire();
    //virtual nEnt* FireAlwaysHit( const char* target );
    virtual nEnt* Fire( const char* target = 0, float range = -1.0f );
    virtual nEnt* Fire( const char* target, quaternion rot, float range = -1.0f, float damage = -1.0f, float impulse = -1.0f, float effectradius = -1.0f, int level = 0 );
protected:
    myvar fires;
    myvar cooldown;
    float lastFired;
    float time;
    var vel;
    var side;
    myvar firingForce;
    nCollisionNode* collision;
    nAutoRef<nGameServer> game;

    var firingEnabled;
    //nAutoRef<nEnt> firingEntRef;
};
//------------------------------------------------------------------------------
#endif

