#ifndef EVENT_H
#define EVENT_H    
//-----------------------------------------------------------------------------//
const int numCategories = 2;
//-----------------------------------------------------------------------------//
enum EventCategory {
    EVENT_CATEGORY_LOCAL,
    EVENT_CATEGORY_BROADCAST,
};
//-----------------------------------------------------------------------------//
inline EventCategory parseEventCategoryString ( const char* string ) {
    static const char* EventCategoryStrings[] = {
        "local",
        "broadcast",
    };
    for( int i = 0; i < numCategories; i ++ ) {
        if ( strcmp(string, EventCategoryStrings[i]) == 0 )
            return EventCategory(i);
    }
    printf("warning eventCategoryString %s is not recognized\n", string);
    return EventCategory(0);
}
//-----------------------------------------------------------------------------//
/**-----------------------------------------------------------------------------//
 * a pullable event, can link to either python events or c++ functions
--------------------------------------------------------------------------------*/
class nRoot;
struct cFuncInfo {
    nRoot* r;
    ulong cmdId;
};
class Event : public nHashNode {
public:
    Event( EventCategory category ) {
        this->category = category;
        //printf("new event category:%i\n", category );
    }
    void addPythonFunction( PyObject* function ) { 
        this->pythonFunctions.Append( function ); 
    }
    void addCFunction( nRoot* r, ulong id ) {
        //printf("Event.addCFunction(%p,%i)\n", r, id );
        cFuncInfo& info                 = cFunctions.Push();
        info.r                          = r;
        info.cmdId                      = id;
    }
    void pull(const char* tag, const char* msg) const {
        //n_printf("Event(%s).pull(%s,%s)->[%i,%i]\n", this->GetName(), tag, msg, cFunctions.Size(), pythonFunctions.Size());
        for( int i = 0; i < cFunctions.Size(); i ++ ) {
            //printf("pulling cfunction(%p->%i)\n", cFunctions[i].r, cFunctions[i].cmdId );
            nCmd* cmd = cFunctions[i].r->GetClass()->FindCmdById( cFunctions[i].cmdId )->NewCmd();
            //printf("cmd:%p\n", cmd );
            cmd->In()->SetS( msg );
            cFunctions[i].r->Dispatch( cmd );
        }
        for( int i = 0; i < pythonFunctions.Size(); i++ ) {
            PyObject* arglist;
            if (msg && msg[0]!='\0')    arglist = Py_BuildValue("(s)", msg );
            else                        arglist = Py_BuildValue("(O)", Py_None );
            PyObject* result = PyEval_CallObject(pythonFunctions[i], arglist);
            Py_XDECREF(arglist);
            Py_XDECREF(result);
        }
    }
    EventCategory getCategory() { return this->category; }
protected:
    nArray<cFuncInfo> cFunctions;
    nArray<PyObject*> pythonFunctions;
    EventCategory category;
};
//-----------------------------------------------------------------------------//
#endif /* EVENT_H */
