#ifndef NASTARNODE_H
#define NASTARNODE_H    
class nAStar;
enum ASTARNODE_STATE {
    ASTARNODE_UNUSED,
    ASTARNODE_OPEN,
    ASTARNODE_CLOSED,
};
enum ASTARNODE_TYPE {
    ASTARNODE_CORNER,
    ASTARNODE_LINK,
};

//-----------------------------------------------------------------------------//
class AStarNode {
public:
    AStarNode(nAStar* server ) { 
        this->server                    = server; 
        static int ids                  = 0; 
        this->id                        = ids++; 
        expanded                        = false; 
        parent                          = 0;
        this->state                     = ASTARNODE_UNUSED;
    }
    virtual ~AStarNode()                                                        { /*printf("destroying AStarNode(%i)\n", this->id); */}
    virtual float CalculateCostTo( AStarNode* )                                 { return 0.0f; } ///Calculate actual cost / fitness for the next node (must be neighbor)
    virtual float CalculateLongCostTo( AStarNode * )                            { return 0.0f; } ///doesn't have to be neighbor
    virtual float EstimateCostTo( AStarNode* )                                  { return 0.0f; } ///guess at how much it will cost (not neccessarily neighbor)
    virtual void Display( nGfxServer* )                                         { return; }
    virtual nArray<AStarNode*> GetNeighbors()                                   { return nArray<AStarNode*>();          }
    void SetParent( AStarNode* parent )                                         { this->parent = parent; }
    void SetExpanded( bool b )                                                  { this->expanded = b; }
public:
    AStarNode* GetParent()                                                      { return this->parent; }
    bool HasCost()                                                              { return this->hasCost; }
    float GetCost()                                                             { n_assert( this->hasCost ); return this->cost; }
    void SetCost( float f )                                                     { this->hasCost = true; this->cost = f; }
    void SetEstimatedCost( float f )                                            { this->estimatedCost = f; }
    float GetEstimatedCost( )                                                   { return this->estimatedCost; }
    void Reset(){
        this->state = ASTARNODE_UNUSED;
    }
    virtual AStarNode* operator=( const AStarNode& rhs ) {
        this->parent   = rhs.parent;
        this->hasCost  = rhs.hasCost;
        this->expanded = rhs.expanded;
        this->cost     = rhs.cost;
        this->server   = rhs.server;
        this->state    = rhs.state;
        return this;
    }
    virtual const char* toString() const{
        //printf("AStarNode.toString\n");
        char* c = strGet();
        snprintf(c, STRMAX, "(%s)\n", str(pos));
        return c;
        #ifdef __WIN32__
        _snprintf( c, STRMAX, "%i.%i:%i,%i,%f,%s", this->id, this->state, this->parent ?this->parent->id:-1, this->hasCost, this->cost, str(pos));
        #else
        snprintf( c, STRMAX, "%i.%i:%i,%i,%f,%s", this->id, this->state, this->parent ?this->parent->id:-1, this->hasCost, this->cost, str(pos));
        #endif 
        return c;
    }
    virtual bool operator == ( const AStarNode& rhs )                                   { return false; }                       //are these two nodes equal -> needed to check for goal state
    vector3 pos;                                                                //fixme shitty drawing system
    ASTARNODE_STATE state;
    ASTARNODE_TYPE type;
    nAStar* server;
protected:
    int id;
    AStarNode* parent;
    bool hasCost;
    bool expanded;
    float cost;
    float estimatedCost;
    friend const char* str(const AStarNode* n);
};
//-----------------------------------------------------------------------------//
inline const char* str( const AStarNode* n ) {
    return n->toString();
}

#endif /* NASTARNODE_H */
