#ifndef NASTARPATH_H
#define NASTARPATH_H    
#include "enkiduo/nastarnode.h"
#include "enkiduo/colours.h"
//-----------------------------------------------------------------------------//
class AStarPath {
public:
    //-----------------------------------------------------------------------------//
    AStarPath(){
        costCalculated = false;
        cost = 0.0f;
        static int pathsConstructed = 0;
        n_printf("pathsConstructed:%i\n", ++pathsConstructed);
    }
    AStarPath(const AStarPath& rhs){
        *this = rhs;
        printf("AStarPath::AStarPath()\n");
    }
    //-----------------------------------------------------------------------------//
    virtual ~AStarPath(){
        for( int i = 0; i < nodes; i ++ )
            delete nodes[i];
        static int pathsDeconstructed = 0;
        n_printf("pathsDeconstructed:%i\n", ++pathsDeconstructed);
    }
    //-----------------------------------------------------------------------------//
    virtual float GetCost(){
        if ( nodes.Size() == 0 )
            return -1.0f;
        float cost = 0.0f;
        for( int i = 1; i < nodes; i ++ ) {
            float edgeCost = nodes[i-1]->CalculateLongCostTo( nodes[i] );
            cost += edgeCost;
            //n_printf("%s->%s edgeCost:%f cost:%f\n", str(nodes[i-1]), str(nodes[i]), edgeCost, cost );
        }
        return cost;
        /*
           if ( nodes[-1]->HasCost() == false )
           return -1.0f;
           return nodes[-1]->GetCost();
           if ( this->costCalculated == false ) {
           this->costCalculated = true;
           for( int i = 0; i < nodes; i ++ )
           cost += nodes[i]->GetCost();
           }
           return cost;
           */
    }
    //-----------------------------------------------------------------------------//
    virtual void Reverse() {             //reverse my direction -> reorder all my nodes
        nArray<AStarNode*> newNodes;
        while( nodes.Size() > 0 )
            newNodes.Append( nodes.Pop() );
        nodes = newNodes;
        //nArray<vector3> newPoints;
        //while( points.Size() > 0 )
            //newPoints.Append( points.Pop() );
        //points = newPoints;
    }
    //-----------------------------------------------------------------------------//
    void Append( AStarNode* node )                                              { 
        this->nodes.Append( node );
        //this->points.Append( vector3(node->pos) );
    }
    //-----------------------------------------------------------------------------//
    virtual void Display( nGfxServer* gfx )                                     { 
        for( int i = 0; i < nodes; i ++ ) 
            nodes[i]->Display(gfx); 
        gfx->Begin( N_PTYPE_LINE_LIST );
        for( int i = 0; i < nodes; i ++ ) {
            if( i == 0 ) {
                gfx->Rgba( 0,1,0,1 );
                gfx->Coord( nodes[i]->pos + vector3 (.5,10,.5));
                gfx->Coord( nodes[i]->pos + vector3 (.5,10,.5));
            } else {
                gfx->Rgba( .3,.3,1,1 );
                gfx->Coord( nodes[i-1]->pos + vector3 (0,1, 0) );
                gfx->Coord( nodes[i]  ->pos + vector3 (0,1, 0) );

                if( i < nodes.Size() - 1 ) {
                    gfx->Rgba( 0,0,.75,1 );
                    gfx->Coord( nodes[i]->pos ); gfx->Coord( nodes[i]->pos + vector3 (0,20,0));

                    gfx->Rgba( 1,0,0,1 );
                    gfx->Coord( nodes[i]->pos + vector3 (.3, nodes[i]->GetCost(), .5));
                    gfx->Coord( nodes[i]->pos + vector3 (.3, nodes[i]->GetCost(), .5));

                    gfx->Rgba( .5,1,.5,1 );
                    gfx->Coord( nodes[i]->pos + vector3 (.4, nodes[i]->GetEstimatedCost(), .5));
                    gfx->Coord( nodes[i]->pos + vector3 (.4, nodes[i]->GetEstimatedCost(), .5));
                } else {
                    for( int j = 0; j < 10; j ++ ) {
                        if( j % 2 == 0 ) 
                            gfx->Rgba( 1,1,1,1 );
                        else
                            gfx->Rgba( 0,0,0,1 );
                        gfx->Coord( nodes[i]->pos + vector3 (.5,10,.5) * (j));
                        gfx->Coord( nodes[i]->pos + vector3 (.5,10,.5) * (j+1));
                    }
                }
            }
        }
        gfx->End();
    }
    //-----------------------------------------------------------------------------//
    virtual void Print(){
        printf("nodes:%i cost:%f\n", nodes.Size(), this->GetCost() );
        for( int i = 0; i < nodes.Size(); i ++ ) {
            printf("    %s\n", str(nodes[i]));
        }
    }
    /* -----------------------------------------------------------------------------//
     * determine which points are corner points
     * ie points where the direction of travel changes
    --------------------------------------------------------------------------------*/
    virtual bool IdentifyCorners(){
        //n_printf("Path.IdentifyCorners()\n");
        bool foundLinks = false;
        for( int i = 0; i < nodes; i ++ ) {
            nodes[i]->type = ASTARNODE_CORNER;
        }
        for( int i = 2; i < nodes; i ++ ) {
            vector3 dx0 = nodes[i-1]->pos - nodes[i-2]->pos;
            vector3 dx1 = nodes[i]->pos - nodes[i-2]->pos;
            bool redundant = false;
            dx0.norm();
            dx1.norm();
            const float equalityThreshold = .01f;
            if ((dx1-dx0).len() < equalityThreshold ){
                nodes[i-1]->type = ASTARNODE_LINK;
                foundLinks = true;
            }
        }
        nodes[-1]->type = ASTARNODE_CORNER;
        return foundLinks;
    }
    /* -----------------------------------------------------------------------------//
     * identify the corner points in the path and get rid of the rest
    --------------------------------------------------------------------------------*/
    virtual bool Reduce(){ 
        //n_printf("Path.Reduce()\n");
        this->IdentifyCorners();
        bool reduced = false;
        for( int i = 0; i < nodes.Size()-1; i++ ) {
            if ( nodes[i]->type == ASTARNODE_LINK || (nodes[i]->pos - nodes[i+1]->pos).len() < .1f ) {
                //nodes[i+1]->SetParent( nodes[i-1] );
                nodes.Erase(i);
                //points.Erase(i);
                i--;
                reduced = true;
            }
        }
        return reduced;
    }
    /* -----------------------------------------------------------------------------//
     * subdivides it into equal length segments less then some threshold length
    --------------------------------------------------------------------------------*/
    virtual bool Segment( float maxSegmentLength ) {
        //n_printf("AStarPath.Segment()\n");
        bool segmented = false;
        for( int i = 1; i < nodes; i ++ ) {
            //printf("looking between nodes[%i] and nodes[%i]\n", i-1, i);
            n_assert( nodes[i]->type == ASTARNODE_CORNER && nodes[i-1]->type == ASTARNODE_CORNER );
            vector3 dx                  = nodes[i]->pos - nodes[i-1]->pos;
            int numSegments             = (int) ceil( dx.len() / maxSegmentLength );
            //printf("dx:%s numSegments:%i\n", str(dx), numSegments );
            dx                          = dx / maxSegmentLength;
            for( int j = 1; j < numSegments; j ++ ) {
                vector3 newPoint        = nodes[i-1]->pos + dx;
                //printf("inserting %s into spot(%i)\n", str(newPoint), i);
                AStarNode* newNode      = new AStarNode( nodes[i]->server );
                newNode->type           = ASTARNODE_LINK;
                newNode->pos            = newPoint;
                nodes.Insert            ( i, newNode );
                //points.Insert           ( i, newNode->pos );
                segmented               = true;
                i++;
            }
            //for( int i = 0; i < nodes; i ++ ) {
                //printf("    %s %i\n" , str( nodes[i]->pos), nodes[i]->type);
            //}
        }
        return segmented;
    }
    /* -----------------------------------------------------------------------------//
     * smooths a path into something less chess boardee
     * 1) reduce it to segmented corner paths
     * 2) try each corners friends
    --------------------------------------------------------------------------------*/
    virtual bool Smooth(){
        return false;
        //n_printf("AStarPath.Smooth()\n");
        //this->IdentifyCorners();
        //this->Reduce();
        //return true;
        //this->Segment( 100.0f );
        n_assert( nodes[0]->type == ASTARNODE_CORNER );
        n_assert( nodes[-1]->type == ASTARNODE_CORNER );
        int cornerA, cornerB, cornerC;
        cornerA = -1;
        cornerB = -1;
        cornerC = 0; //this will prop up with each corner, so once we have found 2 more we have our first triangle
        for( int i = 1; i < nodes; i ++ ) {
            if ( nodes[i]->type == ASTARNODE_CORNER ){ //if we find antoher corner
                cornerA = cornerB;
                cornerB = cornerC;
                cornerC = i;
                if( cornerA >= 0 && cornerB >= 0 && cornerC >= 0 ) { //found three corners, try to smooth them out nicely
                    nArray<AStarNode*> preLinkNodes;
                    nArray<AStarNode*> pstLinkNodes;
                    for( int j = cornerA + 1; j < cornerB; j ++ ) {
                        if ( nodes[j]->type == ASTARNODE_LINK ) {
                            preLinkNodes.Append( nodes[j] );
                        }
                    }
                    for( int j = cornerB + 1; j < cornerC; j ++ ) {
                        if ( nodes[j]->type == ASTARNODE_LINK ) {
                            pstLinkNodes.Append( nodes[j] );
                        }
                    }
                    //n_printf("trying to smooth corner, A:%i, B:%i C:%i\n", cornerA, cornerB, cornerC);
                    //n_printf("trying to smooth corner, A:%s, B:%s C:%s\n", str(nodes[cornerA]), str(nodes[cornerB]), str(nodes[cornerC]));
                    //n_printf("preLinkNodes:%i\n", preLinkNodes.Size());
                    //n_printf("pstLinkNodes:%i\n", pstLinkNodes.Size());
                    //to actually smooth the nodes we check to see if we can remove the corner in the middle
                    float costAB = nodes[cornerA]->CalculateLongCostTo( nodes[cornerB] );
                    float costBC = nodes[cornerB]->CalculateLongCostTo( nodes[cornerC] );
                    float costAC = nodes[cornerA]->CalculateLongCostTo( nodes[cornerC] );
                    //n_printf("costAB:%f costBC:%f costAC:%f\n", costAB, costBC, costAC);
                    if( costAC < .9f * costAB + costBC ) { //remove hte center corner
                        //n_printf("removing middle node\n");
                        nodes.Erase(cornerB);
                        //points.Erase(cornerB);
                        i--;
                        cornerC--;
                        cornerB = cornerA;
                        cornerA = -1;
                    }
                }
            }
        }
        return true;
    }
    /**-----------------------------------------------------------------------------//
     * operator =
    --------------------------------------------------------------------------------*/
    virtual const AStarPath& operator =(const AStarPath& rhs) {
        printf("AStarPath::operator =\n");
        for( int i = 0; i < rhs.nodes; i++ ) {
            AStarNode* node = new AStarNode(*rhs.nodes[i]);
            //*node           = rhs.nodes[i];
            this->nodes.Append( node );
        }
        this->costCalculated = rhs.costCalculated;
        this->cost           = rhs.cost;
        return *this;
    }
public:
    nArray<AStarNode*> nodes;
    bool costCalculated;
    float cost;
    friend const char* str(const AStarPath& n);
    //nArray<vector3> points;
};
//-----------------------------------------------------------------------------//
inline const char* str( const AStarPath& n ) {
    char* c = strGet();
    //c[0] = '\0';
    snprintf(c, STRMAX, "  %i->%f\n", n.nodes.Size(), n.cost );
    for( int i = 0; i < n.nodes.Size(); i ++ ) {
        strncat( c, "    ", STRMAX - strlen(c));
        strncat( c, str(n.nodes[i]), STRMAX - strlen(c));
        strncat( c, "\n", STRMAX - strlen(c));
    }
    return c;
}
//-----------------------------------------------------------------------------/
#endif /* NASTARPATH_H */
