#ifndef N_DISPLAYER_H
#define N_DISPLAYER_H
//------------------------------------------------------------------------------
/**
    @class nDisplayer

    @brief a brief description of the class

    a detailed description of the class

    (C)	2006	Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nDisplayer
#include "kernel/ndefdllclass.h"

//------------------------------------------------------------------------------
class nDisplayer : public nThing {
public:
    /// constructor
    nDisplayer();
    virtual void Initialize();
    /// destructor
    virtual ~nDisplayer();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    int getContext() { 
        contexts.Push(); 
        return contexts.Size() - 1; 
    }
    void clearContext( int contextHandle ) {
        contexts[contextHandle].lines.Clear();
        this->updateDisplay();
    }
    virtual void drawLine( int contextHandle, vector3 from, vector3 to, uint colour ){
        LineInfo& line = contexts[contextHandle].lines.Push();
        line.a = from;
        line.b = to;
        line.colour = colour;
        this->updateDisplay();
    }
    virtual void drawAngle( int contextHandle, vector3 pos, float heading, uint colour, float length = 100.0f ){
        AngleInfo& angle = contexts[contextHandle].angles.Push();
        angle.pos = pos;
        angle.heading = heading;
        angle.length = length;
        angle.colour = colour;
        this->updateDisplay();
    }
    virtual void drawPoint( int contextHandle, vector3 p, vector3 size, uint colour ){
        PointInfo& point = contexts[contextHandle].points.Push();
        point.pos = p;
        point.size = size;
        point.colour = colour;
        this->updateDisplay();
    }
    virtual void drawCircle( int contextHandle, vector3 p, uint colour, float r = 5 ){
        CircleInfo& circle = contexts[contextHandle].circles.Push();
        circle.pos = p;
        circle.radius = r;
        circle.colour = colour;
        this->updateDisplay();
    }
    virtual void drawSphere( int contextHandle, vector3 p, uint colour, float r = 5 ){
        SphereInfo& sphere = contexts[contextHandle].spheres.Push();
        sphere.pos = p;
        sphere.radius = r;
        sphere.colour = colour;
        this->updateDisplay();
    }
    void updateDisplay();
    void Display( nGfxServer* gfx );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    struct LineInfo{
        vector3 a;
        vector3 b;
        uint colour;
    };
    struct AngleInfo {
        vector3 pos;
        float heading;
        float length;
        uint colour;
    };
    struct PointInfo {
        vector3 pos;
        vector3 size;
        uint colour;
    };
    struct CircleInfo {
        vector3 pos;
        float radius;
        uint colour;
    };
    struct SphereInfo {
        vector3 pos;
        float radius;
        uint colour;
    };
    struct Context{
        nArray<LineInfo> lines;
        nArray<AngleInfo> angles;
        nArray<PointInfo> points;
        nArray<CircleInfo> circles;
        nArray<SphereInfo> spheres;
    };
    nArray<Context> contexts;
};
//------------------------------------------------------------------------------
#endif

