#ifndef N_MPISERVER_H
#define N_MPISERVER_H
//------------------------------------------------------------------------------
/**
    @class nMPIServer

    @brief manages communications through MPI

    initializes/finalizes MPI
    allows simpler communication through MPI

    (C)	2004	Chris Miles
*/
#include <mpi.h>
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nMPIServer
#include "kernel/ndefdllclass.h"

enum MPI_MESSAGE_TAG {
    HEADER,
    DATA,
    DIE,
    EVALUATE,   EVALUATE_DATA,
    FITNESS,    FITNESS_DATA,
    PROFILE,    PROFILE_DATA,
    NEWPLAN,    NEWPLAN_DATA,
};
//------------------------------------------------------------------------------
const int MAX_CALLBACKS = 1024;
class nMPIServer : public nThing
{
    typedef void(*callbackFunc)(void*,int);
public:
    /// constructor
    nMPIServer();
    virtual void Initialize();
    /// destructor
    virtual ~nMPIServer();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    int Rank(){
        return rank;
    }
    int Numprocs(){
        return numprocs;
    }
    void AddCallback( int tag, void* clazz, callbackFunc func, int proc );
    //tick should check recieves, if any activated then activate the callback
    void Tick( float time, float dtime );
    void Send( int tag, int proc );
protected:
    void InitRecv( int index );
    int rank, numprocs;
    int numCallbacks;
    int             tags    [MAX_CALLBACKS];
    void*           clazzes [MAX_CALLBACKS];
    callbackFunc    funcs   [MAX_CALLBACKS];
    MPI_Request     requests[MAX_CALLBACKS];
    int        recievingFrom[MAX_CALLBACKS];
};
//------------------------------------------------------------------------------
#endif

