#ifndef N_PROFSERVER_H
#define N_PROFSERVER_H
//------------------------------------------------------------------------------
/**
    @class nProfServer

    @brief Does in house profiling
    outputs its data to the screne

    (C)	2004	Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nProfServer
#include "kernel/ndefdllclass.h"
#include "kernel/ntimeserver.h"

class nGfxServer;
struct ProfNode {
    nString name;
    float sampleStart;
    float sampleStop;
    float sampleTime;
    float frameTime;
    float totalSampleTimes;
    float totalPercentage;
    float subtreePercentage;
    float childPercentage;

    float maxFrameTime;
    float maxTotalPercentage;
    float maxSubtreePercentage;
    float maxChildPercentage;

    nArray<ProfNode*> children;
    ProfNode( nString name ) 	                                        { this->name = name; totalSampleTimes = 0.0f; this->frameTime = 0.0f; this->parent = 0; this->ClearStats();}
    ~ProfNode()		{ for( int i = 0; i < children; i ++ ) delete children[i]; }
    void Start( float time );
    void Stop( float time );		
    void CalculateStats( float parentFrameTime = 0.0f, float totalFrameTime = 0.0f );
    void ClearStats();
    void Display( nGfxServer* gfx, int indent );
    void BeginFrame();
    void EndFrame();
    nString FullNameString();
    ProfNode* parent;
};
//------------------------------------------------------------------------------
class nProfServer : public nThing
{
    public:
        /// constructor
        nProfServer();
        virtual void Initialize();
        /// destructor
        virtual ~nProfServer();
        /// persistency
        virtual bool SaveCmds(nPersistServer* persistServer);

        virtual void BeginFrame();
        virtual void EndFrame();
        void Begin	( nString name );
        void End	( nString name );

        virtual void Display( nGfxServer* gfx );

        /// pointer to nKernelServer
        static nKernelServer* kernelServer;
    protected:
        virtual void ClearStats();
        virtual void CalculateStats();
        nAutoRef<nTimeServer> timeServer;
        float frameStart;
        float frameStop;
        ProfNode* root;
        ProfNode* current;
};
//------------------------------------------------------------------------------
#endif

