#ifndef N_THING_H
#define N_THING_H
//------------------------------------------------------------------------------
/**
    @class nThing

    @brief Generic Super Class

    Adds Trigger/Tick functionality and message passing on top of nVisNode.

    (C)	2004	Chris Miles
*/
#ifndef N_VISNODE_H
#include "kernel/nroot.h"
#endif
#ifndef N_KERNELSERVER_H
#include "kernel/nkernelserver.h"
#endif
#ifndef N_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nThing
#ifndef N_TRIGGER_SERVER
#include "kernel/ndefdllclass.h"
#endif
#include "gfx/ngfxserver.h"
#include "mathlib/line.h"
//#include "enkiduo/colours.h"
#include "enkiduo/nmessage.h"

class n3DNode;
class nTriggerServer;
class nMessageServer;
class nGameServer;
class nOpenALServer;

enum TICKS_PER_FRAME{
    TICKS_ONCE = 0,
    TICKS_TWICE = 1,
    TICKS_FOUR = 2,
    TICKS_EIGHT = 3,
    TICKS_SIXTEEN = 4,
};

class nProfServer;
class nThing : public nRoot
{
protected:
public:
    /// constructor
    nThing();
    virtual void Initialize();
    /// destructor
    virtual ~nThing();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    virtual void Trigger (nTriggerServer* trigger);
    virtual void Tick( float time, float dtime );
    //virtual void Up();
    //virtual void Down();
    inline bool IsAlive(){ return alive; }
    virtual void Die(){ 
        //if( !alive )
            //n_printf("%s->dying twice\n", fn );
        this->alive = false; 
    }
    virtual void Send( nMessage* message );
    virtual void Message( nMessage* message );
    virtual nRoot* CreateChild( const char* childClass, const char* path )                                              { return kernelServer->New(childClass, this->ChildPath(path).Get()); }
    virtual void SetTicksPerFrame( int i );
    static nKernelServer* kernelServer;

    virtual void Display( nGfxServer* gfx );
    virtual void AddToTriggerList();
    virtual void AddToDisplayList();
    virtual void RemoveFromDisplayList();
    virtual void AddToReleaseList();
    virtual void DrawLine( vector3 a, vector3 b, uint colour, float elevation = 0)                                     { linesToDraw.Append( LineInfo( line3(a + vector3(0,elevation,0),b + vector3(0,elevation,0)) , colour )); }
    virtual void DrawAngle( vector3 pos, float t, uint colour, float length = 100.0f );
    virtual void DrawPoint( vector3 p, uint colour, float w, float h, float d );
    virtual void DrawPoint( vector3 p, uint colour, float w = 5 );
    virtual void DrawCircle( vector3 p, uint colour, float r = 5 );
    virtual void DrawSphere( vector3 p, uint colour, float r = 5 );
    virtual void NotifyChannelSetDirty()                                                                                { } 
    virtual nString CreateVis( const char* nodePath, const char* destinationPath );
    virtual void log( int level, const char* string, ... );
    virtual void log( const char* string, ... ); //dumps data to external log
    virtual void logTo( const char* logName, const char* string, ... ); //dumps data to external log
    virtual void SetLogging( bool b ) { this->logging = b; }
    virtual void AddLogTag( const char* cat )                                                                           { logName += nString(" ") + cat; }
    virtual nRoot* PushChild( const char* className );
    virtual nRoot* QueueChild( const char* className );
    virtual nRoot* ReplaceChildren( const char* className );

    virtual void Up();
    virtual void Down();
    virtual bool Simple();
    virtual void ToggleDisplay(){ 
        if ( this->displayInfo.onList )                                         this->RemoveFromDisplayList();
        else                                                                    this->AddToDisplayList();
    }
protected:
    virtual void UpdateLists();
    TICKS_PER_FRAME ticksPerFrame;
    bool alive;
    nMessageServer* messageServer;
    //nAutoRef<nMessageServer> messageServer;
    class LineInfo{
    public:
        line3 line;
        uint colour;
        LineInfo( const line3& l, uint c ){
            line = l;
            colour = c;
        }
        LineInfo(){}
    };
    ///should lines be cleared every frame - default = yes
    bool clearLines;
    nArray<LineInfo> linesToDraw;
    struct ListInfo {
        bool onList, initialized;
    };
    ListInfo triggerInfo, displayInfo, releaseInfo;

    nString logName;
    bool logging;
    int loggingLevel;
    nAutoRef<nGameServer> game;
    nAutoRef<nOpenALServer> soundServer;
    static nProfServer* prof;
    bool& displayed;
};
#endif
