#ifndef VAR_H
#define VAR_H
//------------------------------------------------------------------------------
/**
    @class var

    @brief Wrapper class for nVar, handles all data connections

    var is the prefered way to interact with nVars.
    it holds effecient links to them, as well as some convience stuff such as dirty checks.

    (C)    2004    Chris Miles
*/
#ifndef N_ROOT_H
#include "kernel/nroot.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nVar
#include "kernel/ndefdllclass.h"

#include "enkiduo/nent.h"

//------------------------------------------------------------------------------
class var
{
public:
    var():nvar(0),parentVar(0),lastCounter(0){}
    virtual ~var()                                                              { if( parentVar) parentVar->RemoveChildVar( this );                             }
    inline int GetType()                                                        { CheckValid(); return nvar->GetType();                                         }
    inline void SetI(const int i)                                               { CheckValid(); nvar->SetI(i);                                                  }
    inline void SetF(const float f)                                             { CheckValid(); nvar->SetF(f);                                                  }
    inline void SetV(const vector3& v)                                          { CheckValid(); nvar->SetV(v);                                                  }
    inline void SetV(const float x, const float y, const float z)               { CheckValid(); nvar->SetV(x,y,z);                                              }
    inline void SetQ(const quaternion& q)                                       { CheckValid(); nvar->SetQ(q);                                                  }
    inline void SetQ(const float x, const float y, const float z, const float w){ CheckValid(); nvar->SetQ(x,y,z,w);                                            }
    inline void SetS(const nString& s)                                          { CheckValid(); nvar->SetS(s);                                                  }
    inline void SetS(const char* s)                                             { CheckValid(); nvar->SetS(s);                                                  }
    inline void SetO(void* const o )                                            { CheckValid(); nvar->SetO(o);                                                  }
    inline void SetRef( const char* name )                                      { CheckValid(); nvar->SetRef( name );                                           }
    inline void SetRef( nEnt* e )                                               { CheckValid(); nvar->SetRef( e );                                              }
    inline int GetI()                                                           { CheckValid(); return nvar->GetI();                                            }
    inline float GetF()                                                         { CheckValid(); return nvar->GetF();                                            }
    inline vector3& GetV()                                                      { CheckValid(); return nvar->GetV();                                            }
    inline quaternion& GetQ()                                                   { CheckValid(); return nvar->GetQ();                                            }
    inline nString& GetS()                                                      { CheckValid(); return nvar->GetS();                                            }
    inline nEnt* GetRef()                                                       { CheckValid(); return nvar->GetRef();                                          }
    inline const char* GetRefPath()                                             { CheckValid(); return nvar->GetRefPath();                                      }
    inline void DefaultI( int i )                                               { CheckValid(); nvar->DefaultI( i );                                            }
    inline void DefaultF( float f )                                             { CheckValid(); nvar->DefaultF( f );                                            }
    inline void DefaultV( vector3 v )                                           { CheckValid(); nvar->DefaultV( v );                                            }
    inline void DefaultV( float x, float y, float z )                           { CheckValid(); nvar->DefaultV( x,y,z );                                        }
    inline void DefaultQ( quaternion q )                                        { CheckValid(); nvar->DefaultQ( q );                                            }
    inline void DefaultQ( float x, float y, float z, float w )                  { CheckValid(); nvar->DefaultQ( x,y,z,w );                                      }
    inline void DefaultS( const char* s )                                       { CheckValid(); nvar->DefaultS( s );                                            }
    inline void DefaultRef( const char* name )                                  { CheckValid(); nvar->DefaultRef( name );                                       }
    inline void DefaultRC( RC_DEFAULT_TYPES type, float min, float max )        { CheckValid(); nvar->DefaultRC( type, min, max );                              }
    inline void ToString( char* s )                                             { nvar->ToString(s);                                                            }
    inline float Response( float f )                                            { CheckValid(); return nvar->Response( f );                                     }
    inline void DefaultInputRange( float min, float max, int samples )          { CheckValid(); nvar->DefaultInputRange( min, max, samples );                   }
    inline void SetInputRange( float min, float max, int samples )              { CheckValid(); nvar->SetInputRange( min, max, samples );                       }
    inline void DefaultOutputRange( float min, float max )                      { CheckValid(); nvar->DefaultOutputRange( min, max );                           }
    inline void SetOutputRange( float newmin, float newmax )                    { CheckValid(); nvar->SetOutputRange( newmin, newmax );                         }
    inline void SetBucketEdge(   int index, float val )                         { CheckValid(); nvar->SetBucketEdge( index, val);                               }
    inline int GetBucketCount()                                                 { CheckValid(); return nvar->GetBucketCount();                                  }
    inline float GetBucketEdge( int i )                                         { CheckValid(); return nvar->GetBucketEdge( i );                                }
    virtual void LoadRC( const char* filename )                                 { nvar->LoadRC( filename );                                                     }
    virtual void SaveRC( const char* filename )                                 { nvar->SaveRC( filename );                                                     }
    virtual void Connect( nVar* v )                                             { n_assert( v ); this->nvar = v; this->varName = nvar->GetName();               }
    virtual void Connect( nEnt* e, const char* varName )                        {
                                                                                    this->lastCounter = 0;
                                                                                    n_assert( e );
                                                                                    nvar = e->CreateVar(varName);
                                                                                    this->varName = varName;
                                                                                    n_assert( nvar != 0 );
                                                                                }
    virtual void Connect( var& v, const char* varName )                         {
                                                                                    n_assert( parentVar == 0 );
                                                                                    if( v.nvar == 0 )  
                                                                                        n_error( "trying to connect to a var that has not been connected\n" );
                                                                                    v.nvar->AddChildVar( this, varName );
                                                                                    this->varName = varName;
                                                                                    if( v.isvalid() ){
                                                                                        nThing* t = v.GetRef();
                                                                                        if( t )  {
                                                                                            this->Connect( (nEnt*)t, varName );
                                                                                        }
                                                                                    }
                                                                                }
    virtual void Connect( var& v, nString varName )                             { this->Connect( v, varName.Get()); }
    virtual void Connect( nEnt* e, nString varName )                            { this->Connect( e, varName.Get()); }
    virtual void Disconnect( )                                                  {
                                                                                    this->nvar = 0;
                                                                                }
    bool isvalid()                                                              { if( nvar != 0 ) return nvar->isvalid(); else return false;    }
    bool isdirty()                                                              {
                                                                                    n_assert( isvalid() && "dirty check on an invalid element" );
                                                                                    int c = nvar->GetCounter();
                                                                                    if( lastCounter == c )
                                                                                        return false;
                                                                                    else {
                                                                                        lastCounter = c;
                                                                                        return true;
                                                                                    }
                                                                                }
    const char* GetVarName()                                                    { return varName.get(); }
    nVar* GetVar()                                                              { return nvar; }
    void Attach( nVar* v )                                                      { n_assert( parentVar == 0 ); this->parentVar = v;    }
    void Detach( nVar* v )                                                      { n_assert( parentVar == v ); this->parentVar = 0;    }
    void DontCount()                                                            { n_assert(isvalid()); nvar->DontCount(); }
protected:
    inline void CheckValid()                                                    {
                                                                                    if( nvar == 0 )
                                                                                        n_error( "var %s is invalid\n", varName.Get() );
                                                                                }
    nVar* nvar;
    nVar* parentVar;
    nString varName;
    int lastCounter;
};
//------------------------------------------------------------------------------
#endif
