#ifndef N_CASEINJECTOR_H
#define N_CASEINJECTOR_H
//------------------------------------------------------------------------------
/**
    @class nCaseInjector

    @brief injects cases into the GA as it runs

    watches the generation
    whenever the generation changes it will inject individuals from the casebase into the population

    (C)	2004	Chris Miles
*/
#ifndef N_ASPECT_H
#include "enkiduo/naspect.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nCaseInjector
#include "kernel/ndefdllclass.h"

//------------------------------------------------------------------------------
class nGenome;
class nCaseInjector : public nAspect
{
public:
    /// constructor
    nCaseInjector();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nCaseInjector();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Inject( const char* );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    var generation;
    var injectionFrequency;
    var injectionAmount;
    var generationLastInjected;
    var injectionStartGen;
    var injectionStopGen;
    var popsize;
    var numgens;
    var numInjected;
    struct Case {
        int similarity;	//hamming distance between me and best individual
        float selectionValue;	//my value for being selected to be injected
        float selectionValuePosition;	//my value for being selected to be injected
        nGenome* genome;
    };
};
//------------------------------------------------------------------------------
#endif
