#ifndef N_GABLUE_H
#define N_GABLUE_H
//------------------------------------------------------------------------------
/**
    @class nGABlue

    @brief a GA player for Blue in the strike planning game

    a detailed description of the class

    (C)	2004	Chris Miles
*/
#ifndef N_GAPROBLEM_H
#include "ga/ngaproblem.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nGABlue
#include "kernel/ndefdllclass.h"
#include "enkiduo/npathfinder.h"

//------------------------------------------------------------------------------
class nGABlue : public nGAProblem
{
public:
    /// constructor
    nGABlue();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nGABlue();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    virtual nGenome* BuildEncoding();   //constructs the encoding

    virtual void StartEvaluation( nGenome* genome );        //starts the evaluation of the specified individual
        virtual void ResetWorld( nGenome* genome );
        virtual void LoadMission();
        virtual void LoadGenomePlan( nString genomePlan );
    virtual void StopEvaluation();
    virtual nString GenomeToPlan( nGenome* genome );    	//converts a genome into an XML plan of attack

    virtual bool IsEvaluationDone();  		//checks to see if the evaluation is finished every tick
    virtual float ParseFitness();                          	//called when a game is over to parse out the fitness
    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:

//------------------------------------SCENARIO REFERENCES-----------------------
    nDynAutoRef<nRoot> platformRoot, assetRoot, targetRoot, threatRoot;
    nArray<nString> targetNames;
    nArray<nString> threatNames;
    nAutoRef<nPathfinder> 	pathfinder;   //manages routing
//------------------------------------GAME REFERENCES FOR PARSING FITNESS-------
    nAutoRef<nRoot> platformEntRoot,    //the root of the platform tree
                    targetEntRoot,      //root of the target tree
                    threatEntRoot;      //root of the threat tree
};
//------------------------------------------------------------------------------
#endif

