#ifndef N_GENOME_H
#define N_GENOME_H
//------------------------------------------------------------------------------
/**
    @class nGenome

    @brief a brief description of the class

    a detailed description of the class

    (C)	2005	Chris Miles
*/
#ifndef N_FSM_H
#include "enkiduo/nfsm.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nGenome
#include "kernel/ndefdllclass.h"
#include <tnl.h>

enum States {
    UNEVALUATED,
    EVALUATED,
    BEING_EVALUATED,
};
class nGene;
class nEventServer;
//------------------------------------------------------------------------------
class nGenome : public nFSM {
public:
    /// constructor
    nGenome();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nGenome();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    virtual void EvaluationFinished( float fitness ) { 
        this->fitness = fitness; 
        this->SetState(EVALUATED);
        evaluationsDone.SetI( evaluationsDone.GetI() + 1 );
        mostRecentFitness.SetF( fitness );
        //numEvaluations.SetI( numEvaluations.GetI() + 1 );
    }
    /// pointer to nKernelServerw
    static nKernelServer* kernelServer;
    nGene* GetGeneRoot() { return this->geneRoot; }
    int GetBits( nGene* gene, int numBits );
    void SetBitString( nArray<bool> bits )                                      { int i; for( i = 0; i < bits; i ++ ) this->bits[i] = bits[i]; n_assert( i == bitsBeingUsed ); }
    TNL::Vector<bool> GetTNLBitString();
    bool operator[]( int index )                                                { n_assert( index >= 0 && index < bitsBeingUsed ); return this->bits[index]; }
    bool GetBit( int index )                                                    { return (*this)[index]; }
    const char* GetBits();
    virtual void Randomize();
    float GetFitness()                                                          { return this->fitness; }
    void SetSimilarity ( float f )                                              { this->similarity = f; }
    float GetSimilarity()                                                       { return this->similarity; }
    void SetSharedFitness ( float f )                                           { this->sharedFitness = f; }
    float GetSharedFitness()                                                    { return this->sharedFitness; }
    void SetNormalizedFitness ( float f )                                       { this->normalizedFitness = f; }
    float GetNormalizedFitness()                                                { return this->normalizedFitness; }
    void SetFitnessPercentile( float f )                                        { this->fitnessPercentile = f; }
    float GetFitnessPercentile()                                                { return this->fitnessPercentile; }
    virtual bool operator < ( const nGenome& rhs ) const                        { return this->fitness < rhs.fitness; 	}
    virtual bool operator > ( const nGenome& rhs ) const                        { return this->fitness > rhs.fitness; 	}
    void SetRank( int i )                                                       { this->rank = i; }
    int GetRank()                                                               { return this->rank; }
    void CopyFrom( nGenome* genome );
    void SetGeneration( int i )                                                 { this->generation = i; }
    int GetGeneration()                                                         { return this->generation; }
    int GetBitLength()                                                          { return this->bitsBeingUsed; }

    void CrossOverWith( nGenome* genome );
    void Mutation();
protected:
    bool States( StateMachineEvent event, uint state, float time, float dtime );
    nGene* geneRoot;
    bool* bits;
    int bitsAllocated;
    int bitsBeingUsed;
    
    float fitness;
    float similarity;
    float sharedFitness;
    float normalizedFitness;
    float fitnessPercentile;
    int rank;
    int generation;

    var mutationProb;
    var crossoverProb;
    var evaluationsDone;
    var mostRecentFitness;
    nAutoRef<nEventServer> ref_event;
};
//------------------------------------------------------------------------------
#endif
