#ifndef N_INFLUENCEMAP_H
#define N_INFLUENCEMAP_H
//------------------------------------------------------------------------------
/**
    @class nInfluenceMap

    @brief contains an influence map

    keeps general information on the placement / effectiveness of various units
    converts this into a grid form -> allowing for easy placement of things
    this is the general superclass which uses tcl scripts to evaluate the expressions
    can be subclasses for far > effeciency


    (C)	2004	Chris Miles
*/
#ifndef N_THING_H
#include "enkiduo/nthing.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nInfluenceMap
#include "kernel/ndefdllclass.h"
#include "enkiduo/narrayops.h"

class nMap;
//------------------------------------------------------------------------------
class nInfluenceMap : public nThing
{
public:
    /// constructor
    nInfluenceMap();
    virtual void Initialize();
    /// destructor
    virtual ~nInfluenceMap();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;

    virtual void SetSize( int s );
    virtual void SetVal( int x, int z, float val );
    virtual void Normalize();
    virtual void CalcBest();
    virtual void Display( nGfxServer* gfx );
    virtual float GetVal( vector2 v ) const;		
    virtual float GetVal( int x, int z ) const;
    virtual float GetNormVal( vector2 v ) const;	
    virtual float GetNormVal( int x, int z ) const;
    virtual int GetSize() const			{ return size; } 
    virtual vector2 GetMinPoint() const		{ return minPoint * squareSize; } 
    virtual vector2 GetMaxPoint() const		{ return maxPoint * squareSize; } 
    virtual float GetMin() const		{ return minVal; } 
    virtual float GetMax() const		{ return maxVal; } 
    virtual const char* GetExp() const		{ return this->exp.Get(); }
    virtual void Set( const nInfluenceMap& rhs );
    virtual void Add( const nInfluenceMap& rhs );

    virtual void SetCoef( float f ) 		{ this->coef = f; }
    virtual float GetCoef() 			{ return this->coef; }

    virtual bool Calculate( ) 			{ return false; }
    virtual void Clear();

    virtual float GetA()			{ return A;}
    virtual float GetB()			{ return B;}
    virtual float GetC()			{ return C;}
    virtual void SetA ( float f ) 		{ this->A = f; }
    virtual void SetB ( float f ) 		{ this->B = f; }
    virtual void SetC ( float f ) 		{ this->C = f; }
protected:
    virtual void GetColor( float val, nGfxServer* gfx );
    float coef; 	//my setvals are modifed by this coef
    int size;
    float squareSize;
    nArray<nArray<float> > grid;
    nArray<nArray<float> > normGrid;
    nArray<nArray<vector3> > points;
    vector2 minPoint, maxPoint;
    float minVal, maxVal;
    nAutoRef<nMap> map;
    nString exp;
    float A, B, C;
};
//------------------------------------------------------------------------------
#endif

