#ifndef N_TURRETAI_H
#define N_TURRETAI_H
//------------------------------------------------------------------------------
/**
    @class nTurretAI

    @brief Controls a turret attached to an entity.

    1. The turret shall follow whatever the ents target is, firing at it as much as possible
    2. The turret shall calculate the chance of hitting the target,  based upon MySensors, targetEcm, weaponRange, targetRange
    3. the turret shall determine suitible theta / phi values to offset the beam in order to hit/miss the target
    4. the turret shall call the weapon to create the beam and do damage to the target

    (C)	2005	Chris Miles
*/
#ifndef N_AI_H
#include "ai/nai.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nTurretAI
#include "kernel/ndefdllclass.h"
#include "aspect/nweapon.h"
#include "map/nmap.h"

class nSoundEmitter;
class nHardPointAttach;
class nTurret;
//------------------------------------------------------------------------------
class nTurretAI : public nAI
{
public:
    /// constructor
    nTurretAI();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nTurretAI();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
protected:
    virtual float CalculateHitChance();
    virtual void CalculateAngles( bool connects, quaternion& q, float& range, vector3& v );
    virtual void DoDamage( float time, float range );
    virtual void FindTarget();
    var side;
    ///link to the ship on which I am attached
    var mother;
    ///what I should target
    var motherTarget;
    var target;
    var rotLocal;
    var targetPos;
    var trackingSpeed;
    ///what kinda weapon do i fire
    var fires;

    ///variables to determine hit %
    var mySensors;
    var targetECM;
    var weaponLevel;
    var weaponPower;
    var weaponRange;
    var weaponImpulse;
    var weaponEffectRadius;

    var targetRange;
    var sensorRC;
    var rangeRC;
    ///variables to deteremine damage

    nAutoRef<nMap> map;
    nDynAutoRef<nTurret> turret;
    nDynAutoRef<nWeapon> weapon;
    nDynAutoRef<nSoundEmitter> emitter;
    nDynAutoRef<nHardPointAttach> attach;

    bool firingMissles;

    var autoFire;

};
//------------------------------------------------------------------------------
#endif

