#ifndef N_TNL_H
#define N_TNL_H
#include "tnl.h" 
#include "tnlGhostConnection.h" 
#include "tnlNetInterface.h" 
#include "tnlRPC.h" 
#include "tnlSymmetricCipher.h"
#include "tnlAsymmetricKey.h"
#include "tnlCertificate.h"
#include "tnlBitStream.h"
//------------------------------------------------------------------------------
/**
    @class nTNL

    @brief Wraps the torque network library 

    maintains a number of classes outside the nebula heirarchy - torque stuff doesn't mesh well...

    nTNLConnection extends from GhostConnection and represents an individual master-client connection


    (C)	2005	Chris Miles
*/
#ifndef N_FSM_H
#include "enkiduo/nfsm.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nTNL
#include "kernel/ndefdllclass.h"

#include "tnl/ntnlplayer.h"
#include "tnl/ntnlent.h"
#include "tnl/ntnlconnection.h"

using namespace TNL;
//------------------------------------------------------------------------------
class nTNL : public nFSM
{
    enum States {
        Void,
        Server,
        Client,
    };
public:
    /// constructor
    nTNL();
    virtual void Initialize();
    /// destructor
    virtual ~nTNL();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);

    virtual void Tick( float time, float dtime );
    /// pointer to nKernelServer
    static nKernelServer* kernelServer;

    virtual void OpenServer( const char* ip );
    virtual void ConnectToServer( const char* ip);
    virtual void RegisterEnt( nEnt* e );
    virtual void RemoveEnt( nEnt* e );
    virtual void SetScopeAll( nTNLConnection* c);
    nTNLConnection* getConnection()                                             { return this->connection; }
    void playersChanged();
protected:
    virtual bool States( StateMachineEvent event, uint state, float time, float dtime );
protected:
    nTNLConnection* 	connection;
    nTNLConnection*                     serverConnection;
    nArray<nEnt*>                       registeredEnts;
    nArray<nTNLEnt*> 	tnlents;
};
//------------------------------------------------------------------------------
#endif

