#ifndef NTNLCONNECTION_H
#define NTNLCONNECTION_H    
#include "tnl.h" 
#include "tnlGhostConnection.h" 
#include "tnlNetInterface.h" 
#include "tnlRPC.h" 
#include "tnlSymmetricCipher.h"
#include "tnlAsymmetricKey.h"
#include "tnlCertificate.h"
#include "tnlBitStream.h"

#include "enkiduo/nincludes.h"
#include "enkiduo/nent.h"

class nKernelServer;
class nTNL;
class nEventServer;
//------------------------------------------------------------------------------
class nTNLConnection : public TNL::GhostConnection
{
    typedef TNL::GhostConnection Parent;
public:
    nTNLConnection();
    ~nTNLConnection();
    TNL_DECLARE_NETCONNECTION(nTNLConnection); 

    virtual void setKernelServer( nKernelServer* ks ) 	{ this->kernelServer = ks; }
    virtual void openServer( const char* ip );
    virtual void connectToServer( const char* ip);
    virtual void connectToLocalServer( const char* ip, nTNLConnection* serverConnection );
    virtual nTNL* getTNL()		{ return tnl; }

    virtual void packetDropped		( PacketNotify* notify );
    virtual void packetReceived		( PacketNotify* notify );
    virtual void writePacket		( TNL::BitStream* stream, PacketNotify* notify);
    virtual void readPacket		( TNL::BitStream* stream );
    virtual void processEvent		( TNL::NetEvent* event );
    virtual void writeConnectRequest	                                        ( TNL::BitStream* stream );
    virtual bool readConnectRequest	                                        ( TNL::BitStream* stream, const char** errorString );
    virtual void writeConnectAccept	                                        ( TNL::BitStream* stream );
    virtual bool readConnectAccept	                                        ( TNL::BitStream* stream, const char** errorString);

    virtual void onConnectionEstablished();

    virtual void Tick( float time, float dtime );

    /**NETWORKING RPCS*********************************************************************/
    TNL_DECLARE_RPC(rpcMessageClientToServer, (TNL::StringTableEntry s)); 
    TNL_DECLARE_RPC(rpcMessageServerToClient, (TNL::StringTableEntry s)); 

    /**GENERAL RPCS************************************************************************/
    TNL_DECLARE_RPC(rpcSetVarF, (TNL::StringTableEntry entname, TNL::StringTableEntry varname, TNL::F32 f)); 
    TNL_DECLARE_RPC(rpcSetVarI, (TNL::StringTableEntry entname, TNL::StringTableEntry varname, TNL::S32 i)); 
    TNL_DECLARE_RPC(rpcSetVarV, (TNL::StringTableEntry entname, TNL::StringTableEntry varname, TNL::F32 x, TNL::F32 y, TNL::F32 z)); 

    /**Event Server RPCS*******************************************************************/
    TNL_DECLARE_RPC(rpcBroadcastEvent, (TNL::StringTableEntry tag, TNL::StringTableEntry msg)); 
    TNL_DECLARE_RPC(rpcReceiveEvent,   (TNL::StringTableEntry tag, TNL::StringTableEntry msg)); 

    /**Side Managment RPCS*****************************************************************/
    //server sending it side info to the client
    TNL_DECLARE_RPC(rpcServerUpdatePlayers,  (TNL::Vector<TNL::U8>                playerIds, 
                                        TNL::Vector<TNL::StringTableEntry>  playerNames, 
                                        TNL::Vector<TNL::StringTableEntry>  playerSides,
                                        TNL::Vector<TNL::StringTableEntry>  playerFactions));
    TNL_DECLARE_RPC(rpcClientUpdatePlayers,  (TNL::Vector<TNL::U8>                playerIds, 
                                        TNL::Vector<TNL::StringTableEntry>  playerNames, 
                                        TNL::Vector<TNL::StringTableEntry>  playerSides,
                                        TNL::Vector<TNL::StringTableEntry>  playerFactions));
    void playersChanged();


    static nKernelServer* kernelServer;
protected:
    nTNL* tnl;
    TNL::NetInterface* myNetInterface; 
    nAutoRef<nEventServer> eventServer;
    int id;
};
#endif /* NTNLCONNECTION_H */
