#ifndef N_PLANNERUI_H
#define N_PLANNERUI_H
//------------------------------------------------------------------------------
/**
    @class nPlannerUI

    @brief master mouse interface class,  tracks when the mouse is pressed / released. If it has no planner children, or if none block / take the mouse command then it will move things as the default behavior

    a detailed description of the class

    (C)	2004	Chris Miles
*/
#ifndef N_FSM_H
#include "enkiduo/nfsm.h"
#endif
#ifndef E_INCLUDES_H
#include "enkiduo/nincludes.h"
#endif

#undef N_DEFINES
#define N_DEFINES nPlannerUI
#include "kernel/ndefdllclass.h"
#include "enkiduo/ngameserver.h"
#include "misc/noctree.h"
#include "enkiduo/ncategoryserver.h"

class nEventServer;
class nPlanner;
class nInterfaceServer;
class nUIElement;
//------------------------------------------------------------------------------
class nPlannerUI : public nFSM
{
    enum States {
        Disabled,
        Normal,
        LeftMouseButtonDown,
        MiddleMouseButtonDown,
        RightMouseButtonDown,
        EntPlacementPre,
        EntPlacement,
        MultipleEntPlacementPre,
        MultipleEntPlacement,
        EntRotation,
        BoxSelection,
        RMenu,
    };
public:
    /// constructor
    nPlannerUI();
    virtual void Initialize();
    virtual void InitVars();
    /// destructor
    virtual ~nPlannerUI();
    /// persistency
    virtual bool SaveCmds(nPersistServer* persistServer);
    virtual const EntArray* GetObjectsSelected()	{ return refCategoryServer->GetEntsInCategory( categorySelected ); }
    virtual const EntArray* GetObjectsOver()	{ return refCategoryServer->GetEntsInCategory( categoryOver); }
    virtual void AddSelectedEnt( nEnt* e );
    virtual void AddSelectedEnt( const char* );
    virtual void RemoveSelectedEnt( nEnt* e );
    virtual void MouseEvent( int button );
    virtual void AddPlanner( const char* path )	{ this->AddPlanner( kernelServer->Lookup(path)); }
    virtual void AddPlanner( nRoot* r ); 	
    virtual void Block()		{ blockedCounter++; }
    virtual void UnBlock()		{ blockedCounter--; n_assert( blockedCounter >= 0 ); }
    virtual void PlaceEnt( const char* path )                                   { this->PlaceEnt( (nEnt*) kernelServer->Lookup(path)); }
    virtual void PlaceEnt( nEnt* ent );
    virtual void PlaceMultipleEnts( const char* paths );
    virtual void ClearSelection();
    virtual void BoxSelect();
    //virtual bool Attach(nSceneGraph2* sceneGraph);
    virtual void ClearCatSelection();
    virtual void AddCatSelectedEnt(const char* s );
    //virtual void ClearTargettedEntities();
    //virtual void AddTarget(const char* s );
    //virtual void RemoveDeletedEntFromLists( const char* entPath );
    //virtual void UpdateOverLinks();

    /// pointer to nKernelServer
    static nKernelServer* kernelServer;
    nString getObjectsSelectedString();
    nString getObjectsOverString();
protected:
    virtual void UpdateSelection();
    virtual void HideSelectedMenus();
    virtual void ShowSelectedMenus();
    //virtual void ClearObjectsOver();
    virtual void UpdateObjectsOver();
    bool States( StateMachineEvent event, uint state, float time, float dtime );
    virtual void AttachRings(const EntArray& ents, nAutoRef<n3DNode>& node, nSceneGraph2* sceneGraph);
    void FindGroupMembers(const EntArray& ents, EntArray& groupMembers);
    var selected;
    var mousePos;
    var selectionRadius;
    var selectionThreshold;
    var rotateYSensitivity;
    nAutoRef<nGameServer> gameServer;
    nAutoRef<nOctree> octree;
    MouseInfo lastMouseInfo;

    int categoryOver, categorySelected, categorySelectedGroup, categoryOverGroup, categoryCatSelected, categoryTargetted;
    /*EntArray objectsSelected;
    EntArray objectsOver;
    EntArray selectedGroupEntities;
    EntArray overGroupEntities;
    EntArray catSelectedEntities;
    EntArray targettedEntities;
    */

    ///To open whatever...okay..we want too..ummm find the best candidate! we need camera pos/rotation
    var camera;
    var cameraPos;
    var cameraRot;
    nArray<nRef<nPlanner> > planners;

    int blockedCounter;
    var entbeingplaced;
    var entbeingplacedpos;

    struct entsBeingPlacedInfo {
        nEnt* ent;
        var pos;
        vector3 offset;
    };
    nArray<entsBeingPlacedInfo> entsBeingPlaced;

    void* selectionEvent;
    void* selectionBoxEvent;
    void* mouseOverEvent;
    void* rightClickEvent;
    nAutoRef<nEventServer> eventServer;
    var windowSettings;
    var aspectRatio;
    nAutoRef<nInterfaceServer> interface;

    float timedown[3];
    vector3 boxSelectionPoints[2];

    float timeRightMouseButtonDown;
    float timeMiddleMouseButtonDown;
    var rightClickTime;

    nEnt* rmenu;
    nArray<nRef<nEnt> > group0;

    //nAutoRef<n3DNode> overVis;
    //nAutoRef<n3DNode> selectedVis;
    //nAutoRef<n3DNode> overGroupVis;
    //nAutoRef<n3DNode> selectedGroupVis;
    //nAutoRef<n3DNode> catSelectedVis;
    //nAutoRef<n3DNode> targettedVis;

    nAutoRef<nCategoryServer> refCategoryServer;
    //bool catMenu;
    bool multiSelect;
    vector3 entRotationCenter;
    int ticksSinceLastBoxSelection;
};
//------------------------------------------------------------------------------
#endif


