import wx
import math
import nEventServer
import catBehavior
#import Numeric

from inverseCPA import *
from common.defs import *
from utils_navigation import *
from common.log import Log
from common import EventNames
from cat.sensors.relativepositionsensor import *
from common import OptionManager
from cat import panelInfo
from cat import behaviorInfo

log = Log(tags=['all','sensors'])
DEFAULTSIZE          = panelInfo.CONTROLWRAPPERPANEL

SAFE_BIGENTITYAVOIDFACTOR = 3
MIN_BIGENTITYAVOIDFACTOR = 1

SAFE_SMALLENTITYAVOIDFACTOR = 10
MIN_SMALLENTITYAVOIDFACTOR = 3
BIGENTITYSAFECPATIME = 600
MAXRANGE_OBJECTIVECONST = 148.4131591025766 # equivalent of exp(100/20)
MAX_OBJECTIVE_LIMIT = 0.3
OBJECTIVE_CONST = 20

class CatAvoidEntity(catBehavior.CatBehavior):
    def __init__(self, cPanelA, entity=None, bname = None, sensors = None, actuators = None, recorder = None, size=DEFAULTSIZE,  show=False):
        catBehavior.CatBehavior.__init__(self, cPanelA, entity=entity, bname=bname, size = size,  show=show)

        self.sensors = sensors
        
        self.relpos = RelativePositionSensor(entity = self.entity, behavior = self)
        self.sens = sensors
        pushsel(self.entityPath)

        # Set self dimensions and size
        self.width, self.height,self.length = gv('dimension')
        self.size = gi ('size')

        # Set self max speed
        self.maxSpeed = gf('maxSpeed')
        popsel()
        

    def makeBehaviorPanel(self, panel, size=None, entity = None, bname = None, behavior = None,  show=False):
        from cat.Interface import catStartPanel, catTargetPanel
	from common import behaviorNames
        from cat import panelInfo
        bsize = panelInfo.BEHAVIORPANEL
        aePanel = catStartPanel.CatStartPanel(panel, -1, color=None, size=bsize, behaviorType=bname , entity=entity, behavior = behavior)
	tsize = panelInfo.TARGETPANEL
	targetPanel = wx.Panel(panel, -1, size=tsize) 
        if(show):
            panel.addPanel(aePanel, targetPanel, bname)
        return [aePanel, targetPanel]
    

    def computeRelativeBearing(self, absHeading, thetat):
        ab1 = 2*math.pi - absHeading
        ab2 = ab1 - math.pi

        if ab2 <0:
            ab2 += 2*math.pi

        heading2 = thetat
        if heading2 < 0:
            heading2 += 2*math.pi
        heading2 = 2*math.pi - heading2

        rb2 = ab2 - heading2
        if rb2 < 0:
            rb2 += 2*math.pi

        return rb2

    def GetOwnEntityPosition(self):
        pushsel(self.entityPath)
        if call('hasvar', 'pos'):
            oy, oz, ox = gv('pos')
        else:
            oy, oz, ox = (0, 0, 0)
        popsel()
        return (oy, oz, ox)

    def GetOwnEntitySpeed(self):
        pushsel(self.entityPath)
        if call('hasvar', 'speed'):
            oSpeed = gf('speed')
        else:
            oSpeed = 0.0
        popsel()
        return oSpeed

    def GetOwnEntityHeading(self):
        pushsel(self.entityPath)
        if call('hasvar', 'heading'):
            oHeading = gf('heading')
        else:
            oHeading = 0.0
        popsel()
        return oHeading

    def GetAvoideePosition(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'pos'):
            ay, az, ax = gv('pos')
        else:
            ay, az, ax = (0, 0, 0)
        popsel()
        return (ay, az, ax)

    def GetAvoideeSpeed(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'speed'):
            aSpeed = gf('speed')
        else:
            aSpeed = 0.0
        popsel()
        return aSpeed

    def GetAvoideeHeading(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'heading'):
            aHeading = gf('heading')
        else:
            aHeading = 0.0
        popsel()
        return aHeading

    def GetAvoideeSize(self, avoidee):
        pushsel(avoidee)
        if call('hasvar', 'size'):
            aSize = gi('size')
        else:
            aSize = 0.0
        popsel()
        #print "Size = ", aSize
        return aSize

    def IsAngleInInterval(self, theta, start, end):
        if end < start:
            end = end + 2*math.pi
        if theta > end:
            theta = theta - 2*math.pi
        else:
            if theta < start:
                theta = theta + 2*math.pi

        #print "start = ", math.degrees(start), " theta = ", math.degrees(theta), " end = ", math.degrees(end)
        if start <= theta and theta <= end:
            if (theta - start) <= (end - theta):
                closerToStart = 1
            else:
                closerToStart = 0
            isAngleInInterval = True
        else:
            isAngleInInterval = False
            closerToStart = 0
        return (isAngleInInterval, closerToStart)
    
    def CheckRightOfWay (self, rb2, thetam, thetat):
        thetam = NormalizeHeadingRad(math.radians(convertToMoboard (math.degrees(thetam))))
        thetat = NormalizeHeadingRad(math.radians(convertToMoboard (math.degrees(thetat))))
        angleBehindShip = 0
        
        #print "thetam = ", math.degrees(thetam), " thetat = ", math.degrees(thetat), " rb2 = ", math.degrees(rb2)

        if rb2 >=0 and rb2 < math.pi/2:
            # own ship in first quadrant w.r.t. target
            #print "CASE 1"
            isAngleInInterval, closerToStart = self.IsAngleInInterval(thetam, thetat + math.pi + rb2, thetat)
            if isAngleInInterval:
                collisionPossible = True
                rightOfWay = True
            else:
                collisionPossible = False
                rightOfWay = True
            angleBehindShip = thetat + math.pi + rb2 - math.pi/4
##             if closerToStart:
##                 angleBehindShip = thetat + math.pi + rb2 - math.pi/2
##             else:
##                 angleBehindShip = thetat + math.pi/2
                
        if rb2 >= math.pi/2 and rb2 < math.pi:
            #print "CASE 2"
            isAngleInInterval, closerToStart = self.IsAngleInInterval(thetam, thetat + math.pi + rb2, thetat)
            if isAngleInInterval:
                collisionPossible = True
                rightOfWay = False
            else:
                collisionPossible = False
                rightOfWay = True
            angleBehindShip = thetat + math.pi + rb2 - math.pi/4
##             if closerToStart:
##                 angleBehindShip = thetat + math.pi + rb2 - math.pi/2
##             else:
##                 angleBehindShip = thetat + math.pi/2

        if rb2 >= math.pi and rb2 < (3.0/2.0)* math.pi:
            # target in third quadrant
            #print "CASE 3"
            isAngleInInterval, closerToStart = self.IsAngleInInterval(thetam, thetat, thetat + rb2 - math.pi)
            if isAngleInInterval:
                collisionPossible = True
                rightOfWay = False
            else:
                collisionPossible = False
                rightOfWay = True
            angleBehindShip = thetat + rb2 - math.pi + math.pi/4
##             if closerToStart:
##                 angleBehindShip = thetat -  math.pi/2
##             else:
##                 angleBehindShip = thetat + rb2 - math.pi + math.pi/2 

        if rb2 >= (3.0/2.0)* math.pi and rb2 < 360:
            # target in second quadrant
            #print "CASE 4"
            isAngleInInterval, closerToStart = self.IsAngleInInterval(thetam, thetat, thetat + rb2 - math.pi)
            if isAngleInInterval:
                collisionPossible = True
                isAngleInInterval1, closerToStart1 = self.IsAngleInInterval(thetam, thetat + rb2 - math.pi - math.pi/2, thetat + rb2 - math.pi)
                if isAngleInInterval1:
                    rightOfWay = False
                else:
                    rightOfWay = True
            else:
                collisionPossible = False
                rightOfWay = True
            angleBehindShip = thetat + rb2 - math.pi + math.pi/4
##             if closerToStart:
##                 angleBehindShip = thetat - math.pi/2
##             else:
##                 angleBehindShip = thetat + rb2 - math.pi + math.pi/2


        angleBehindShip =  NormalizeHeadingRad(angleBehindShip)
        angleBehindShip =  NormalizeHeadingRad(2*math.pi-angleBehindShip)
        #print "collisionPossible = ", collisionPossible, " rightOfWay = ", rightOfWay, "angleBehindShip = ", math.degrees(NormalizeHeadingRad(angleBehindShip))        
        return collisionPossible, rightOfWay, angleBehindShip

    def getEntityDimensions(self, entityPath):
        if exists(entityPath):
            pushsel(entityPath)
            if call('hasvar', 'dimension'):
                dim = gv('dimension')
            else:
                dim = (20, 20, 100)
                
            popsel()     
            return dim
        else:
            return None

    def computeDesiredCPADistance(self, sizet, avoideeLength):
        if sizet > self.size: # you are smaller
            safeCPAdist = avoideeLength * SAFE_BIGENTITYAVOIDFACTOR
            minCPAdist = avoideeLength * MIN_BIGENTITYAVOIDFACTOR
        if sizet < self.size: # you are bigger
            safeCPAdist = self.length * SAFE_BIGENTITYAVOIDFACTOR
            minCPAdist = self.length * MIN_BIGENTITYAVOIDFACTOR
        if sizet == self.size: # you have the same size as the avoidee
            if sizet == 0:
                safeCPAdist = self.length * SAFE_SMALLENTITYAVOIDFACTOR
                minCPAdist = self.length * MIN_SMALLENTITYAVOIDFACTOR
            else:
                safeCPAdist = self.length * SAFE_BIGENTITYAVOIDFACTOR
                minCPAdist = self.length * MIN_BIGENTITYAVOIDFACTOR
        return (safeCPAdist, minCPAdist)

    def checkNeedToAvoidBehind(self, avoidee, rb2):
        useAngleBehindShip = False
        isAvoideeMyTarget, desiredBearing = self.entity.isAvoideeMyTarget(avoidee)
        if isAvoideeMyTarget :
            #print "rb2 = ", math.degrees(rb2), "desiredbearing = ", desiredBearing
            rb2Deg = math.degrees(rb2)
            if (desiredBearing == 0 or desiredBearing == 360):# Do you want to use ==  or near?
                useAngleBehindShip = False
            else:
                if (0 < desiredBearing and desiredBearing <= 180 and rb2Deg > 180 and rb2Deg < 360) or (0 < rb2Deg and rb2Deg <= 180 and desiredBearing > 180 and desiredBearing < 360):
                    useAngleBehindShip = True
                else:
                    useAngleBehindShip = False
        #if useAngleBehindShip:
            #print "****************>>> useAngleBehindShip = ", useAngleBehindShip
        return useAngleBehindShip

    def ComputeDirectionAwayFromBoat(self, absHeading):
        nautAbsBearing = 2 * math.pi - absHeading
        nautHeading = self.GetOwnEntityHeading()
        if nautHeading < 0:
            nautHeading += 2 * math.pi
        nautHeading = 2 * math.pi - nautHeading
        rb1 = nautAbsBearing - nautHeading
        if rb1 < 0:
            rb1 += 2*math.pi
        if rb1 > math.pi: #if the avoidee is on my left
            b_new = self.GetOwnEntityHeading() - math.pi / 4.0
            #print "----------> LLLLLLLLLLLLLLL, b_new =", math.degrees(b_new)
        else:             #if the avoidee is on my right
            b_new = self.GetOwnEntityHeading() + math.pi / 4.0
            #print "----------> RRRRRRRRRRRRRRR , b_new =", math.degrees(b_new)
        return b_new

    def ComputeObjectiveFactor(self):
        objective = self.entity.getObjective()
        #objectiveFactor = MAX_OBJECTIVE_LIMIT * Numeric.exp(objective/OBJECTIVE_CONST)/MAXRANGE_OBJECTIVECONST
        objectiveFactor = MAX_OBJECTIVE_LIMIT * objective/100.0
        avoidFactor = 1.0-objectiveFactor
        #print "objective = ", objectiveFactor, "avoid = ", avoidFactor
        return (objectiveFactor, avoidFactor)

    def ComputeSafetyFactor(self):
        # Note: currently not used
        safety = self.entity.getSafety()
        safetyFactor = Numeric.exp(safety/20.0)/MAXRANGE_OBJECTIVECONST
        #print "safety = ", safetyFactor
        return safetyFactor

    
    def smallAvoidSingleEntity(self, x0m, y0m, vm, thetam, avoidee, entlist, fs, totalx, totaly):
    
        v_new = fs
        
        # Get information about the target
        thetat = self.GetAvoideeHeading(avoidee) 
        y0t, tz, x0t = self.GetAvoideePosition(avoidee) 
        vt = self.GetAvoideeSpeed(avoidee)
        sizet = self.GetAvoideeSize(avoidee)
        #print "TARG: x0t = ", x0t, "y0t = ", y0t, "vt = ", vt, "thetat = ", math.degrees(thetat)

        absHeading = entlist[getfullname()][1]
        rb2 = self.computeRelativeBearing(absHeading, thetat)
        # print "rb2 = ", math.degrees(rb2)

        # Check if current entity has right of Way
        collisionPossible, rightOfWay, angleBehindShip = self.CheckRightOfWay(rb2, thetam, thetat)

        # Get the dimensions of the avoidee ship
        avoideeDim = self.getEntityDimensions(avoidee)
        if avoideeDim:
            avoideeWidth, y, avoideeLength = avoideeDim
        else:
            #print "No entity to avoid"
            return totalx, totaly, v_new

        # Compute the safe CPA distance range based on the avoidee's lenght
        # safeCPAdist - for high safety
        # minCPAdist - for low safety
        safeCPAdist, minCPAdist = self.computeDesiredCPADistance(sizet, avoideeLength)

        # Compute the desired CPA distance based on the "safety" level of the entity
        desiredCPAdist = minCPAdist + self.entity.getSafety()*(safeCPAdist-minCPAdist)

        # Compute the safe CPA time
        if vm < 0.1:
            safeCPAtime = BIGENTITYSAFECPATIME
        else:
            safeCPAtime = 2*safeCPAdist/vm

        # Compute the current distance between the boats
        currentDistance = math.sqrt((x0m-x0t)**2 + (y0m-y0t)**2)

        # Compute the CPA time and distance
        # print "vm = ", vm, "fs = ", fs
        t_CPA, dist_CPA =  directCPA (x0t, y0t, vt, math.degrees(thetat), x0m, y0m, vm, math.degrees(thetam))
        #print "------>  t_CPA = ", t_CPA, " dist_CPA = ", dist_CPA
        #print "------>  dtime = ", safeCPAtime, "dDist = ", desiredCPAdist, "cDist = ", currentDistance, "safety =", self.entity.getSafety()

        ttc = self.entity.getTTC()
        #print "------> TTC =", ttc

        collisionImminent = (currentDistance < desiredCPAdist)

        # Check if the boat needs to avoid, based on size, right of way and potential for collision
	noNeedToAvoid = self.size > sizet or (self.size == sizet and rightOfWay and (not collisionImminent))
        if noNeedToAvoid:
            #print "No need to avoid"
            return totalx, totaly, v_new

        # Check if avoidee is own ship's target and
        # if the maneuver requires own ship to change the side on which it is maneuvering
        # If true, avoid the ship from behind
        useAngleBehindShip = self.checkNeedToAvoidBehind(avoidee, rb2)

        # Need to avoid if:
        # - positive CPA time, CPA distance less than desired CPA distance and CPA time less than desired CPA time OR
        # - positive CPA time and time to complete maneuver is greater than CPA time (Note: this might not be needed anymore)

        #if (( t_CPA >= 0 and dist_CPA < desiredCPAdist and t_CPA < safeCPAtime) or (currentDistance < desiredCPAdist and useAngleBehindShip) ):
        part1 = ( t_CPA >= 0 and dist_CPA < desiredCPAdist and t_CPA < safeCPAtime)
        part2 = (currentDistance < desiredCPAdist)
        isAvoideeMyTarget, value = self.entity.isAvoideeMyTarget(avoidee)
        part3 = (useAngleBehindShip or (not isAvoideeMyTarget))
        #print "part 1", part1, "part2 = ", part2, "part3 = ", part3, "isAvoideeMyTarget ", isAvoideeMyTarget
        if (part1 or part2) and part3:
        #if (( t_CPA >= 0 and dist_CPA < desiredCPAdist and t_CPA < safeCPAtime and not useAngleBehindShip) or (currentDistance < desiredCPAdist and useAngleBehindShip)): 
            pushsel(avoidee)
            #print "*********Avoiding: ", getname()
            popsel()

            part1 = (( collisionPossible and not rightOfWay) or (collisionPossible and self.size < sizet ))
            part2 = (useAngleBehindShip and currentDistance > avoideeLength)
            #print "part 1", part1, "part2= ", part2
            if part1 or part2: 
            #if ((( collisionPossible and not rightOfWay) or (collisionPossible and self.size < sizet )) and not self.entity.isAvoideeMyTarget(avoidee)) or (useAngleBehindShip and currentDistance > avoideeLength):
                b_new = angleBehindShip
                #print "====================> angleBehindShip = ", math.degrees(b_new)
            else:
                # Compute a vector that points away from the other boat
                # This vector is used whenever inverseCPA cannot find a solution
                b_new = self.ComputeDirectionAwayFromBoat(absHeading)

            if vt > 0.1:
                solExists, v_newcpa, b_newcpa = inverseCPA (x0t, y0t, vt, math.degrees(thetat), x0m, y0m, vm, math.degrees(thetam), desiredCPAdist, v_new)
                # If inverseCPA returns a solution, then use it
                # Only solutions that involve change of speed are returned and used
                if solExists:
                    v_new = v_newcpa

            # Compute the coordinates of the avoidance vector
            xcoord = math.cos(b_new)
            ycoord = math.sin(b_new)
            totalx += xcoord
            totaly += ycoord
            if v_new < 0:
                v_new = 0
        return totalx, totaly, v_new
        
    def loop(self,time,dtime):
        pushsel()
        if exists(self.entityPath):
            
            # Initialize avoid vector to zero
            totalx = totaly = 0

            # Get own ship desired speed and heading
            vm, thetam = self.entity.actuators.getCurrentVec()

            #sel(self.entityPath)

             # Get own ship position and speed
            y0m,oz,x0m = self.GetOwnEntityPosition() #gv('pos')
            oSpeed = self.GetOwnEntitySpeed()
            oHeading = self.GetOwnEntityHeading()

##             print "\n *****************************************************************\n"
##             print "Entity: ", self.entity.getName()
##             print "\n *****************************************************************\n"
            

            # Get list of other entities
            entlist = self.relpos.getRelPos()

            # Initialize the return values
            returntuple = (0,thetam,vm)
            #v_new = self.maxSpeed
            v_new = vm                                      # Sushil --------------
            
            # Initialize the list of boats to avoid
            avoidList = []
            
            # Iterate through the list of entities
            sel('/game/ents/')
            dir = ndir()
            for subdir in dir:
                sel('/game/ents/'+subdir)
                dir2 = ndir()
                for subdir2 in dir2:
                    xcoord = ycoord = 0
                    avoidee = '/game/ents/'+subdir+'/'+subdir2
                    sel(avoidee)
                    if ((not getselectable()) or (getfullname() == self.entityPath)) :
                        #print "Fullname of entity: ", getfullname(), self.entityPath
                        pass
                    else:
                        # Compute avoid information for avoidee
                        #print "\n===============> smallAvoidSingleEntity"
                        totalx, totaly, v = self.smallAvoidSingleEntity(x0m, y0m, oSpeed, oHeading, avoidee, entlist, vm, totalx, totaly)
                        if v <= v_new:
                            v_new = v

        # Compute the vector projection from the desired heading
        if vm == 0:
            fx = 0
            fy = 0
        else:
            fx = math.cos(thetam)
            fy = math.sin(thetam)
        #print "thetam = ", thetam, "fx =", fx, "fy=", fy

        # Compute objective factor from the value set by the user
        objectiveFactor, avoidFactor = self.ComputeObjectiveFactor()
        finalTotalx = totalx*avoidFactor+fx*objectiveFactor
        finalTotaly = totaly*avoidFactor+fy*objectiveFactor
        returntuple = (.5, math.atan2(finalTotaly, finalTotalx), v_new)
        #returntuple = (.5, math.atan2(totaly*avoidFactor+fy*objectiveFactor,totalx*avoidFactor+fx*objectiveFactor), v_new) #vm
        popsel()
        returntupleknots = (returntuple[0], returntuple[1], returntuple[2]*behaviorInfo.METERS_PER_SEC_TO_KNOTS)
        return returntupleknots

    def checkPostconditions(self):
        return False


def getBehaviorType():
    return behaviorType


