#
# @(#) catPrimitiveApproach.py 2005/10/17
# @author Monica Nicolescu and Sushil Louis
# Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

"""
Implementation of Approach
"""

import wx
from common.defs import *
import math
from common.log import Log
from common import OptionManager
log = Log(tags=['approach', 'maintain'])


MAINTAIN_STATION          = 1
RAM_SHIP                  = 2
MOVE_TO                   = 3
APPROACH                  = 1
DEFAULT                   = 3

SPEED_CONST_MANEUVER      = 1
TIME_CONST_MANEUVER       = 2
BEARING_CONST_MANEUVER    = 3

# In minutes
MIN_TIME_SLIDER           = 1
MAX_TIME_SLIDER           = 10

# In degrees
MIN_BEARING_SLIDER        = 0
MAX_BEARING_SLIDER        = 360
DEFAULT_BEARING_SLIDER    = 0

#In yards
MIN_RANGE_SLIDER          = 0
if OptionManager.getOption('map') == 'openwater':
    MAX_RANGE_SLIDER      = 20000
else:
    MAX_RANGE_SLIDER      = 5000
DEFAULT_RANGE_SLIDER      = 200

# In knots
MIN_SPEED_SLIDER          = 1
MAX_SPEED_SLIDER          = 20

# Default values for maneuver value slider
DEFAULT_SPEED_MANEUVER    = 10
DEFAULT_TIME_MANEUVER     = 1
DEFAULT_BEARING_MANEUVER  = 0

from cat import panelInfo
import catBehavior
from utils_navigation import *

DEFAULTSIZE          = panelInfo.CONTROLWRAPPERPANEL
RANGE_FACTOR         = 3
#YDS_PER_SEC_TO_KNOTS = 1.7774514
#KNOTS_TO_YDS_PER_SEC = 0.562603286
from cat import behaviorInfo

class CatPrimitiveApproach(catBehavior.CatBehavior):
    """
    General Navigation Behavior
    """
    
    def __init__(self, cPanelA, entity = None, bname = None, sensors = None, actuators = None, recorder = None, size=DEFAULTSIZE,  show=False):

        """
        Creates the approach behavior.
        navigation_type: whether this is a maintain station, ram or move-to behavior
        sens (Sensors): The sensors approach should use. By default, it should
        use those from the gui.
        """
        #catBehavior.CatBehavior.__init__(self, cPanelA, cPanelE, entity=entity, bname=bname, size=size, show=show)
#         self.wrapperA = cPanelA
#         self.wrapperE = cPanelE        
#         self.entity = entity
        self.type = bname
        self.sensors = sensors
        self.actuators = actuators
        self.recorder = recorder

        pushsel(entity.getEVAPath())
        x, z, length = gv('dimension')
        self.rangeLimit = length * RANGE_FACTOR
        popsel()


        from common import behaviorNames
        if (bname.startswith(behaviorNames.MOVETO)):
            self.navigation_type = MOVE_TO
        elif (bname.startswith(behaviorNames.MAINTAIN) or bname.startswith(behaviorNames.APPROACH)):
            self.navigation_type = MAINTAIN_STATION
        elif (bname.startswith(behaviorNames.RAM)):
            self.navigation_type = RAM_SHIP
        else:
            print "Error: unknown approach behavior", bname
            self.navigation_type = DEFAULT

##         if bname == behaviorNames.MOVETO:
##             self.navigation_type = MOVE_TO
##         elif (bname == behaviorNames.MAINTAIN or bname == behaviorNames.APPROACH):
##             self.navigation_type = MAINTAIN_STATION
##         elif bname == behaviorNames.RAM:
##             self.navigation_type = RAM_SHIP
##         else:
##             print "Error: unknown approach behavior", bname
##             self.navigation_type = DEFAULT

        self.desiredStBearingDef = DEFAULT_BEARING_SLIDER
        self.desiredStBearingMin = MIN_BEARING_SLIDER
        self.desiredStBearingMax = MAX_BEARING_SLIDER

        self.desiredStRangeDef = DEFAULT_RANGE_SLIDER
        self.desiredStRangeMin = MIN_RANGE_SLIDER
        self.desiredStRangeMax = MAX_RANGE_SLIDER

#        self.authPanel = self.makeApproachPanel(cPanelA, color="RED", size = self.wrapperSize, entity = entity, behavior = self)
#        self.execPanel = self.makeApproachPanel(cPanelE, color="RED", size = self.wrapperSize, entity = entity, behavior = self)

        self.reachedStation = False

        self.weight = 1
        self.bm_computed = 0
        self.vm_computed = 0
        self.t_computed = 0

        self.tSpeed = 0
        self.tHeading = 0
        self.timerRemaining = 0
        self.timerStart = 0

        self.tickTimerStart = 0
        self.justReset = True
	##Z
        catBehavior.CatBehavior.__init__(self, cPanelA, entity=entity, bname=bname, size=size, show=show)

        self.AddBool('maintainBearing', True)
        self.AddBool('maneuverChanged', True)
        self.AddBool('activated',       False)
        self.AddBool('absoluteBearing', False)
        self.AddBool('reachedStation', True)
        self.AddBool('absoluteBearingLast', False)
        self.AddInt ('maneuverType', SPEED_CONST_MANEUVER, BEARING_CONST_MANEUVER, SPEED_CONST_MANEUVER)
        
        pushsel(self.entityPath)
        self.maxSpeed = gf('maxSpeed')
        maxSpeed = self.maxSpeed * behaviorInfo.METERS_PER_SEC_TO_KNOTS
        popsel()
        
        self.AddFloat('entMaxSpeed',    int(maxSpeed),   MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddFloat('maneuverValue',  int(maxSpeed/2), MIN_SPEED_SLIDER, int(maxSpeed))

        self.AddFloat('desiredStBearing', DEFAULT_BEARING_SLIDER, MIN_BEARING_SLIDER, MAX_BEARING_SLIDER*math.pi/180)
        self.AddFloat('desiredStRange',   DEFAULT_RANGE_SLIDER, MIN_RANGE_SLIDER, MAX_RANGE_SLIDER)
        self.range = -1

        self.AddInt('desiredX', 22500, 0, 45000)
        self.AddInt('desiredY', 22500, 0, 45000)

        # Variables that represent the sliders min/max/default values; needed for the behavior panel
        self.AddInt ('maneuverValueDef', int(maxSpeed/2), MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddInt ('maneuverValueMin', MIN_SPEED_SLIDER, MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddInt ('maneuverValueMax', int(maxSpeed), MIN_SPEED_SLIDER, int(maxSpeed))
        self.EXCLUSION_LIST = ()

        


    def GetOwnEntityHeadingLagDeg(self):
        pushsel(self.entityPath)
        if call('hasvar', 'heading'):
            oHeadingLagRad = gf('heading')
        else:
            oHeadingLagRad = 0.0
        oHeadingLagDeg = math.degrees(oHeadingLagRad)
        oHeadingMobDeg = convertToMoboard (oHeadingLagDeg)
        popsel()
        return oHeadingMobDeg

    def GetOwnEntitySpeed(self):
        pushsel(self.entityPath)
        if call('hasvar', 'speed'):
            oSpeed = gf('speed')
        else:
            oSpeed = 0.0
        popsel()
        return oSpeed
      
    def GetOwnEntityPosition(self):
        pushsel(self.entityPath)
        if call('hasvar', 'pos'):
            ox, oz, oy = gv('pos')
        else:
            ox, oz, oy = (0, 0, 0)
        popsel()
        return (ox, oz, oy)

    def GetTargetHeadingLagDeg(self):
        if exists(self.getTarget()):
            pushsel(self.getTarget())
            if call('hasvar', 'heading'):
                tHeadingLagRad = gf('heading')
            else:
                tHeadingLagRad = 0.0
            tHeadingLagDeg = math.degrees(tHeadingLagRad)
            tHeadingMobDeg = convertToMoboard (tHeadingLagDeg)
            popsel()
        else:
            tHeadingMobDeg = 0.0
        return tHeadingMobDeg
        

    def GetTargetSpeed(self):
        if exists(self.getTarget()):
            pushsel(self.getTarget())
            if call('hasvar', 'speed'):
                tSpeed = gf('speed')
            else:
                tSpeed = 0.0
            popsel()
        else:
            tSpeed = 0.0
        return tSpeed
        
      
    def GetTargetPosition(self):
        if exists(self.getTarget()):
            pushsel(self.getTarget())
            if call('hasvar', 'pos'):
                ox, oz, oy = gv('pos')
            else:
                ox, oz, oy = (0, 0, 0)
            popsel()
        else:
            ox, oz, oy = (0, 0, 0)
        return (ox, oz, oy)

    def UpdateManeuverChangedStatus(self, tSpeed, tHeading):
        if tSpeed - self.tSpeed > 0.5 or tHeading - self.tHeading > 1:
            self.SetVar('maneuverChanged', True)
            self.tSpeed = tSpeed
            self.tHeading = tHeading
        
    def GetDesiredRange(self):
        return self.GetVar('desiredStRange')

    def GetDesiredBearing(self, tHeading):
        if self.GetVar('absoluteBearing') == True:
            desiredStBearing = self.GetVar('desiredStBearing')
        else:
            desiredStBearing = self.GetVar('desiredStBearing') + tHeading;
        desiredStBearing = NormalizeHeading(desiredStBearing)
        return desiredStBearing

    def GetRangeLimit(self, desiredStRange):
        return self.rangeLimit;

    def GetBearingLimit(self):
        return 15

    def UpdateReachedStation(self, rangeDiff, rangeLimit, bearingDiff, bearingLimit):
        if rangeDiff < rangeLimit:
            self.reachedStation = True
        else:
            self.reachedStation = False

    def GetSpeedAtStation(self):
        return 0
    
    def GetHeadingAtStation(self):
        #return 0
        pushsel(self.entityPath)
        if call('hasvar', 'heading'):
            oHeadingLagRad = gf('heading')
        else:
            oHeadingLagRad = 0.0
        popsel()
        return oHeadingLagRad



    def CheckReduceSpeedNeeded(self, rangeDiff, bearingDiff):
        return False


    def GetReducedSpeed (self, vm, rangeDiff, tSpeed):
        vm_yrds_per_min = vm*2000/60
        tSpeed_yrds_per_min = tSpeed*2000/60
        #print "==>> vm_yrds_per_min = ", vm_yrds_per_min, "tSpeed_yrds_per_min = ", tSpeed_yrds_per_min

        timeToStation = rangeDiff/vm_yrds_per_min
        
        #print "==>> rangeDiff =", rangeDiff, "timeToStation [min]= ", timeToStation
        
        # Compute proportional term
        speedDifference_yrds_per_min = tSpeed_yrds_per_min - vm_yrds_per_min
        # Constants that work well (0.6, 4) (0.4, 4) (0.5, 6)
        Kp = 0.45
        #Kp = 0.3 + 0.07*speed
        #Kp = 0.2 + 0.08*speed + 0.01*speed**2
        #Kp = 0.1+ 0.1*speed + 0.01*speed**2
        pTerm = Kp*speedDifference_yrds_per_min
        #print "==>> speed_difference [yrds_per_min] = ", speedDifference_yrds_per_min
        #print "==>> pTerm = ", pTerm
        
        # Compute derivative term
        deceleration = (tSpeed_yrds_per_min - vm_yrds_per_min)/(rangeDiff/vm_yrds_per_min)
        Kd = 4
        #Last Kd = 0.6*speed
        dTerm = Kd*deceleration/600                 
        #print "==>> dTerm = ", dTerm
        
        vm_yrds_per_min = vm_yrds_per_min + pTerm + dTerm
        vm = vm_yrds_per_min*60/2000.0
        if vm < 0:
            vm = 0
        #print "==>> new vm_yrds_per_min = ", vm_yrds_per_min, "vm[knots] = ", vm

        return vm
    
    def loop(self, time, dtime):
        """
        Processes the sensors to produce actuator output.
        Returns a tuple of the weight, heading, and speed.
        """

        if self.justReset == True:
            self.tickTimerStart = time
            self.justReset = False
        else:
            if (time - self.tickTimerStart) < 1:
                self.justReset = False
                pass
            else:
                self.justReset = True
        
        if not exists(self.entityPath):
            return (0, 0, 0)
        else:
            pushsel(self.entityPath)###-----------------

            # Get own ship information
            oHeading =  self.GetOwnEntityHeadingLagDeg()
            oSpeed = self.GetOwnEntitySpeed()
            ox, oz, oy = self.GetOwnEntityPosition()
            
            # Get target information
            tHeading = self.GetTargetHeadingLagDeg()
            tSpeed = self.GetTargetSpeed()
            tx, tz, ty = self.GetTargetPosition()
            
            # Compute the current distance to the target
            currentStDistance = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            currentStBearing = math.degrees(math.atan2(tx-ox, oy-ty))
            currentStBearing = NormalizeHeading(currentStBearing)

            # Compute the desired range, and distance to target
            desiredStRange = self.GetDesiredRange()
            desiredStBearing = self.GetDesiredBearing(tHeading)

            # Get range and bearing limit
            rangeLimit = self.GetRangeLimit(desiredStRange)
            bearingLimit = self.GetBearingLimit()
           
            # Compute rangeDiff and bearingDiff
            rangeDiff = abs(currentStDistance - desiredStRange)
            bearingDiff = abs(currentStBearing - desiredStBearing)
            if bearingDiff > 180:
                bearingDiff = 360 - bearingDiff
            #print "PA: rangeDiff = ", rangeDiff, " bearingDiff = ", bearingDiff
 
            self.UpdateReachedStation(rangeDiff, rangeLimit, bearingDiff, bearingLimit)

            if self.reachedStation == True:
                print "========================> Reached station"
                speedAtStation = self.GetSpeedAtStation()
                headingAtStation = self.GetHeadingAtStation()
                return (0.5, headingAtStation, speedAtStation)

            # If maintain course
            if self.GetVar ('maneuverType') == BEARING_CONST_MANEUVER: #maintain bm
                bm, vm, t = maintainBM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)
                
            # If maintain time
            if self.GetVar ('maneuverType') == TIME_CONST_MANEUVER: #maintain t
                if (self.GetVar('activated')):
                    self.timerStart = time
                    timeRemaining_min = self.GetVar('maneuverValue')
                else:
                    timeSpent = time - self.timerStart
                    timeRemaining_sec = 60*self.GetVar('maneuverValue') - timeSpent
                    timeRemaining_min = timeRemaining_sec/60.0
                    if timeRemaining_min < 0:
                        timeRemaining_min = 0

                bm, vm, t = maintainT(tSpeed, tHeading, timeRemaining_min, currentStDistance, currentStBearing, desiredStRange, desiredStBearing)

            # If maintain speed
            if self.GetVar ('maneuverType') == SPEED_CONST_MANEUVER: #maintain vm
                bm, vm, t = maintainVM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)

            bm = convertToMoboard (bm)
            bm = math.radians(bm)

            # PD control: reduce the speed as approaching the final station
            if self.CheckReduceSpeedNeeded(rangeDiff, bearingDiff) == True:
                vm = self.GetReducedSpeed(vm, rangeDiff, tSpeed)
                
            #print "Approach : course = ", bm, "speed = ", vm, "time =", t
            self.weight = 0.5
            self.bm_computed = bm
            self.vm_computed = vm
            self.t_computed = t
            self.entity.setTTC(t * 60) # convert to seconds
            self.SetVar ('maneuverChanged', False)
            self.SetVar ('activated', False)
            self.SetVar ('absoluteBearingLast', self.GetVar('absoluteBearing'))
            return (0.5, bm, vm)

        

                                         

    def checkPreconditions(self):
        return True

    def handleTargetSelection(self, tp, msg):
        self.setTarget(tp)
        self.SetVar ('maneuverChanged', True)
        print "CatPrimitiveApproach: handleTargetSelection: Target Changed to", str(tp)

    def checkPostconditions(self):
        #print self.reachedStation
        return self.reachedStation


