#
# @(#) catPrimitiveBehavior.py 2005/11/19
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
import wx
from xml.sax import saxutils

from common.log import Log
log = Log(tags=['behavior'])
from common.defs import *
from common import behaviorNames
from cat.ports import port
from cat import behaviorInfo
import nEventServer
from common import EventNames
from triggers import trigger
from cat.xml import behaviorSpec
from common import tickmanager


TIME_STEP              = 1     # How long between speed/heading updates
SIGNIFICANT_FRACTION   = 0.25  # What fraction of param range constitutes significant difference
PARAM_TIME_CHANGE_MIN  = 4
TRIGGER_SEPERATOR      = '_'
SENSOR_CLOSE_FRACTION  = 0.1
from cat import panelInfo
from cat import behaviorInfo
import random

class CatPrimitiveBehavior:
    """Ancestor of all behaviors"""
    def __init__(self, entity=None, bname=None):
        self.entity         = entity
        self.entityPath     = entity.getEVAPath()
        self.bname          = bname
        if (self.bname.startswith(behaviorInfo.AVOID_START)):
            self.amAvoid = True
        else:
            self.amAvoid = False
        self.saveNet        = []
        self.sid            = None
        self.params         = {}
        self.slots          = {}
        self.triggers       = {}
        self.oldParams      = {}
        self.deltaParams    = {}
        self.paramInfo      = {}
        self.sensors        = entity.getSensors()
        self.recorder       = entity.getRecorder()
        self.actuators      = entity.getActuators()
        #self.preTriggers    = []
        #self.postTriggers   = []
        
        self.xmlBehavior    = {}
        self.preConditions  = None
        self.postConditions = None
        self.links          = None
        self.runnable       = False
        self.isRecording    = False
        self.oldTime        = 0
        self.currentTime    = 0
        self.paramChangeTime = 0
        self.targetPath     = None
        self.moveLocation   = None
        self.prePorts       = []
        self.outPorts       = []
        #self.preSensorPorts = []
        #self.postSensorPorts= []
        self.trigID         = 0
        self.active         = False
        self.savedPreTriggers = []

        self.targetBindingName = None
        self.environment       = None
        self.checkDelta        = False
        self.saveBehavior      = False

        #print "In Behavior name:", bname

        self.funcname = "CatBehavior:"+str(bname)
        #nEventServer.register(EventNames.CAT_TRIGGER_CHANGE, self.handleTriggerChange)
        #tickmanager.register(self.funcname,  self.Tick )
        self.timeStep = TIME_STEP + random.random()



    def setEnvironment(self, env):
        self.environment = env

    def setTargetBindingName(self, name):
        self.targetBindingName = name

    def getTargetBindingName(self):
        return self.targetBindingName

    def isActive(self):
        return self.active

    # --------------------------
    # is this one of the avoids?
    # --------------------------
    def isAvoid(self):
        return self.amAvoid

    def getName(self):
        return self.bname

    def getMaxSpeed(self):
        return self.entity.getMaxSpeed()

    def handleTargetSelection(self, targetPath, msg):
        self.setTarget(targetPath)

    def getTarget(self):
        self.entity.setCurrentTarget(self.targetPath) # so the entity level target gets updated everytime a behavior runs
        return self.targetPath

    def setTarget(self, tp):
        #print "&&&&&&&&&&&&&&&&&&&&& Setting Target", self, tp
        self.entity.setCurrentTarget(tp)
        if(self.environment and self.targetPath):
            self.environment.changeBinding(self.targetPath, tp)
        self.entity.updateTriggers(self.targetPath, tp)
        self.targetPath = tp # most importantly set this at the end
            
    def setMoveLocation(self, pos):
        self.moveLocation = pos

    def getMoveLocation(self):
        return self.moveLocation

    def SetVar(self, name, val):
        if(self.params.has_key(name)):
            ov, type, min, max = self.params[name]
            self.params[name] = [val, type, min, max]
        else:
            print "Must add var before setting %s = %s\n" % (name, val)

    def GetVar(self, name): 
        if(self.params.has_key(name)):
            val, type, min, max = self.params[name]
            return val
        else:
            return None

    def AddVar(self, name, val, type, min, max):
        if(type):
            self.params[name] = [val, type, min, max]
        else:
            self.params[name] = [val, 'var', min, max]            
        
    def AddBool(self, name, val):
        self.params[name] = [val, 'bool', False, True]

    def AddInt(self, name, val, min, max):
        self.params[name] = [val, "int", min, max]
    
    def AddFloat(self, name, val, min, max):
        self.params[name] = [val, "float", min, max]
    
    def AddVector(self, name, val, min, max):
        self.params[name] = [val, "vector", min, max]

    def setupFusion(self):
        self.actuators.addBehavior(self.getName())

    def stopFusion(self):
        self.actuators.deleteBehavior(self.getName())

    def behaviorImpl(self, time, dtime):
        self.active = True
        w, dh, ds = self.loop(time, dtime)
        maxSpeed = self.getMaxSpeed()
        ds = ds * behaviorInfo.KNOTS_TO_METERS_PER_SEC
        if(ds > maxSpeed):
            ds = maxSpeed
        self.actuators.setDesiredHeading(self.bname, dh)
        self.actuators.setDesiredSpeed(self.bname, ds)
        self.actuators.setWeight(self.bname, w)
        self.actuators.update()

    def Tick(self, time, dtime):
        self.currentTime = time
        self.active = False
        if(self.runnable):
            if(time - self.oldTime > self.timeStep):
                self.oldTime = time

                if(self.entity.isAuthorMode()):
                    self.runRecordTick(time, dtime)
                else:
                    self.runExecTick(time, dtime)

    def runExecTick(self, time, dtime):
        #FIXED: No longer need checkPreSensorPorts
        #if (not (self.checkPrePorts() and self.checkPreSensorPorts())):
        if not self.checkPrePorts():
            # is this AND or OR?
            self.actuators.setWeight(self.bname, 0.0)
            return
        print "Running behavior:", self.bname
        self.behaviorImpl(time, dtime)
        self.setOutPorts(True)
        #self.setPostSensorPorts()

    def runRecordTick(self, time, dtime):
        self.paramChangeTime += dtime
        self.behaviorImpl(time, dtime)
        if (self.recorder.isRecording):
            self.recordImpl()

    def recordImpl(self):
        pass
    
    def checkTime(self):
        return self.paramChangeTime >= PARAM_TIME_CHANGE_MIN

    def resetTime(self):
        self.paramChangeTime = 0.0

    def start(self):
        self.currentTime = tickmanager.register(self.funcname,  self.Tick )
        print "CPB: Starting: ", self.getName()
        self.setupFusion()
        self.startStop = 'stop'
        self.updatePanels()
        self.entity.stopRelated(self.getName())
        if(self.runnable): # if already running, handle change
            self.stopRecord()
            # print "CatBehavior-Start: %s %s " % (self.getName() + behaviorInfo.TIME_SEP_CHARACTER + str(self.oldTime), self.getName())
        self.startRecord()
        self.resetPorts()
        self.runnable = True

    def startRecord(self):
        self.sid = self.getName() + behaviorInfo.TIME_SEP_CHARACTER + str(int(self.currentTime))
        self.behSpec = behaviorSpec.BehaviorSpec(self.getName(), self.sid, '0', start=self.currentTime)
        
    def stopRecord(self, triggerSpec = None):
        if(not self.sid):
            return
        if self.recorder.isRecording:
            self.behSpec.end = self.currentTime
            #Set targets
            if self.getTarget():
                self.behSpec.targets = [self.getTarget()]
            #Add Parameters
            for pname, pinfo in self.params.iteritems():
                value, type, min, max = pinfo
                param = behaviorSpec.Parameter(pname, type, value, min, max)
                self.behSpec.parameters.append(param)
            if triggerSpec:
                self.behSpec.triggers.append(triggerSpec)
                #triggerSpec.id = len(self.behSpec.triggers)
            self.recorder.addToNetSpec(self.behSpec)
        self.behSpec = None
        self.sid = None

    def stop(self, triggerSpec = None):
        print "CPB: Stopping: ", self.getName()
        self.stopFusion()
        
	self.runnable = False
        self.startStop = "start"

        #self.updatePanels()
        self.stopRecord(triggerSpec)

        if not (self.bname.startswith(behaviorNames.AVOIDENTITY) or self.bname.startswith(behaviorNames.AVOIDLAND)):
            self.actuators.setDesiredSpeed(self.bname, 0.0)
            self.actuators.setWeight(self.bname, 0.0)
            self.actuators.update()
        tmp = tickmanager.unregister(self.funcname, self.Tick)

    def triggerStop(self, triggerSpec):
        if self.runnable:
            self.stop(triggerSpec)

    def pause(self):
        # self.actuators.setWeight(self.bname, 0.0)
        print "Pausing: ", self.getName()
        self.stopFusion()
        self.runnable = False
        if not (self.bname.startswith(behaviorNames.AVOIDENTITY) or self.bname.startswith(behaviorNames.AVOIDLAND)):
            self.actuators.setDesiredSpeed(self.bname, 0.0)
            self.actuators.setWeight(self.bname, 0.0)
            self.actuators.update()
#             pass

    def loop(self, time, dtime): 
        print "Pretending to run behavior: ", self.bname
        return (0, 0, 0)

    def getParams(self):
        return self.params

    def addTrigger(self, trigger):
        self.triggers[trigger.id] = trigger

    def addRunningPort(self, port):
        #FIXED: combine port with trigger
        #FIXED: add trigger to sensors
        #port.trigger = trigger
        self.outPorts.append(port)
        #self.postTriggers.append(trigger)

    def addPreconditionPort(self, prt):
        pc = port.PortCondition(prt, False)
        self.prePorts.append(pc)

    def resetPorts(self):
        for p in self.outPorts:
            p.setIsDone(False)
            p.setIsRunning(False)
        for pc in self.prePorts:
            pc.setCondition(False)

    def setOutPorts(self, val):
        #FIXED: move postcondition check out so it is only done once
        checkPost = self.checkPostconditions()
        for port in self.outPorts:
            #FIXED: also check to see if trigger is met

            trigger = self.triggers.get(port.trigid)
            triggerFired = False
            if trigger:
                trigger.checkThreshold()
                triggerFired = trigger.hasFired()
            if not trigger and not port.trigid in ["None", None]:
                print "something went wrong looking up link trigger", port.trigid, self.triggers.keys()

            checkPost = checkPost or triggerFired
            
            if checkPost: ## done running
                port.setIsDone(True)
                self.Hide()
            else:
                #port.setIsDone(False)
                port.setIsRunning(val)
        if checkPost:
            print "Postconditions met, %s should be stopping" % self.bname
            if (self.entity.getMode() == panelInfo.EXECUTION_MODE or (not self.recorder.isRecording)):
                print "Postconditions met, %s is actually stopping" % self.bname                
                self.stop()
                self.Hide()
            
    def checkPrePorts(self):
        #print "Checking Preports", self.bname,
        for pc in self.prePorts:
            port = pc.getPort()
            linkType = port.name
            if(linkType.startswith(behaviorInfo.ORDERING)):
                if (port.isDone()):
                    pc.setCondition(True)
            elif(linkType.startswith(behaviorInfo.ENABLING)):
                if (port.isRunning()):
                    pc.setCondition(True)
            elif(linkType.startswith(behaviorInfo.PERMANENT)):
                pc.setCondition(port.isRunning() and (not port.isDone()))
            #print pc.getCondition()
            if(not pc.getCondition()):
                return False
        if(self.prePorts):
            self.Show()
        return True

            
    def forceOutPorts(self):
        for port in self.outPorts:
            port.setIsDone(True)
            port.setIsRunning(False)


    def Show(self):
        pass

    def Hide(self):
        pass


    def updatePanels(self):
        return # to be overriden by catBehavior.py/panel

    def checkPostconditions(self):
        return False # Should be overridden by behavior
