#
# @(#) plan.py 2006/03/12
# @author Sushil Louis
# Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

"""
Uses Chris' A* implementation
"""

import wx
from common.defs import *
import math
from common.log import Log
from common import OptionManager
log = Log(tags=['catA*'])


from cat import panelInfo
import catBehavior
from utils_navigation import *

DEFAULTSIZE          = panelInfo.CONTROLWRAPPERPANEL
RANGE_FACTOR         = 3
from cat import behaviorInfo

SPEED_CONST_MANEUVER   = 0
BEARING_CONST_MANEUVER = 1

MAXXCOORD = 45000
MAXYCOORD = 45000

class Plan(catBehavior.CatBehavior):
    """
    General Navigation Behavior
    """
    
    def __init__(self, cPanelA, entity = None, bname = None, sensors = None, actuators = None, recorder = None, size=DEFAULTSIZE,  show=False):

        """
        Constructor
        """
        self.type = bname
        self.sensors = sensors
        self.actuators = actuators
        self.recorder = recorder
        self.entityPath = entity.getEVAPath()

        pushsel(entity.getEVAPath())
        x, z, length = gv('dimension')
        self.rangeLimit = length * RANGE_FACTOR
        maxSpeed = gf('maxSpeed')
        self.maxSpeed = maxSpeed * behaviorInfo.METERS_PER_SEC_TO_KNOTS
        popsel()

        catBehavior.CatBehavior.__init__(self, cPanelA, entity=entity, bname=bname, size=size, show=show)

        self.AddBool('maintainBearing', True)
        self.AddBool('absoluteBearing', False)
        self.AddBool('reachedStation', True)
        self.AddInt ('maneuverType', SPEED_CONST_MANEUVER, BEARING_CONST_MANEUVER, SPEED_CONST_MANEUVER)
        
        self.AddFloat('entMaxSpeed',    int(maxSpeed),   MIN_SPEED_SLIDER, int(maxSpeed))
        self.AddFloat('maneuverValue',  int(maxSpeed/2), MIN_SPEED_SLIDER, int(maxSpeed))

        self.AddFloat('desiredStBearing', DEFAULT_BEARING_SLIDER, MIN_BEARING_SLIDER, MAX_BEARING_SLIDER*math.pi/180)
        self.AddFloat('desiredStRange',   DEFAULT_RANGE_SLIDER, MIN_RANGE_SLIDER, MAX_RANGE_SLIDER)
        self.range = -1

        self.AddInt('desiredX', 22500, 0, 45000)
        self.AddInt('desiredY', 22500, 0, 45000)


    def makeMoveNode(self, from, to):
        """
        from and to are 3vecs that specify from where to where to move. Returns a BehaviorSpec
        """
        targets = []
        params = makeMoveParams(from, to)
        bid = behaviorNames.MOVETO + "*" + str(to)
        return behaviorSpec.BehaviorSpec(name = behaviorNames.MOVETO, id = bid, parameters = params) 

    def makeMoveParams(self, from, to):
        """
        Move at constant speed to to
        """
        params = []
        params.add(behaviorSpec.Parameter(name = "maneuverType", type = "int", val = SPEED_CONST_MANEUVER, min = SPEED_CONST_MANEUVER, max = BEARING_CONST_MANEUVER))
        params.add(behaviorSpec.Parameter(name = "maneuverValue", type = "int", val = self.maxSpeed/2.0, min = 0, max = self.maxSpeed))
        x, z, y  = to
        params.add(behaviorSpec.Parameter(name = 'desiredX', type = "int", val = x, min = 0, max = MAXX))
        params.add(behaviorSpec.Parameter(name = 'desiredX', type = "int", val = y, min = 0, max = MAXY))
        return params

    def getAstarPath(self, from, to):
        print call('/game/sides/blue/allocatorastar.findpath', from[0], from[1], from[2], to[0], to[1], to[2])
        pathlength = call('/game/sides/blue/allocatorastar.getpathlength')
        path = []
        for i in range(pathlength):
            path.add( call('/game/sides/blue/allocatorastar.getpathnode', i))
        return path
         
    def makeAvoidEntityNode(self):
        return None

    def makeAvoidLandNode(self):
        return None

    def loop(self, time, dtime):
        """
        Processes the sensors to produce actuator output.
        Returns a tuple of the weight, heading, and speed.
        """

        if self.justReset == True:
            self.tickTimerStart = time
            self.justReset = False
        else:
            if (time - self.tickTimerStart) < 1:
                self.justReset = False
                pass
            else:
                self.justReset = True
        
        if not exists(self.entityPath):
            return (0, 0, 0)
        else:
            pushsel(self.entityPath)###-----------------

            # Get own ship information
            oHeading =  self.GetOwnEntityHeadingLagDeg()
            oSpeed = self.GetOwnEntitySpeed()
            ox, oz, oy = self.GetOwnEntityPosition()
            
            # Get target information
            tHeading = self.GetTargetHeadingLagDeg()
            tSpeed = self.GetTargetSpeed()
            tx, tz, ty = self.GetTargetPosition()
            
            # Compute the current distance to the target
            currentStDistance = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            currentStBearing = math.degrees(math.atan2(tx-ox, oy-ty))
            currentStBearing = NormalizeHeading(currentStBearing)

            # Compute the desired range, and distance to target
            desiredStRange = self.GetDesiredRange()
            desiredStBearing = self.GetDesiredBearing(tHeading)

            # Get range and bearing limit
            rangeLimit = self.GetRangeLimit(desiredStRange)
            bearingLimit = self.GetBearingLimit()
           
            # Compute rangeDiff and bearingDiff
            rangeDiff = abs(currentStDistance - desiredStRange)
            bearingDiff = abs(currentStBearing - desiredStBearing)
            if bearingDiff > 180:
                bearingDiff = 360 - bearingDiff
            #print "PA: rangeDiff = ", rangeDiff, " bearingDiff = ", bearingDiff
 
            self.UpdateReachedStation(rangeDiff, rangeLimit, bearingDiff, bearingLimit)

            if self.reachedStation == True:
                print "========================> Reached station"
                speedAtStation = self.GetSpeedAtStation()
                headingAtStation = self.GetHeadingAtStation()
                return (0.5, headingAtStation, speedAtStation)

            # If maintain course
            if self.GetVar ('maneuverType') == BEARING_CONST_MANEUVER: #maintain bm
                bm, vm, t = maintainBM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)
                
            # If maintain time
            if self.GetVar ('maneuverType') == TIME_CONST_MANEUVER: #maintain t
                if (self.GetVar('activated')):
                    self.timerStart = time
                    timeRemaining_min = self.GetVar('maneuverValue')
                else:
                    timeSpent = time - self.timerStart
                    timeRemaining_sec = 60*self.GetVar('maneuverValue') - timeSpent
                    timeRemaining_min = timeRemaining_sec/60.0
                    if timeRemaining_min < 0:
                        timeRemaining_min = 0

                bm, vm, t = maintainT(tSpeed, tHeading, timeRemaining_min, currentStDistance, currentStBearing, desiredStRange, desiredStBearing)

            # If maintain speed
            if self.GetVar ('maneuverType') == SPEED_CONST_MANEUVER: #maintain vm
                bm, vm, t = maintainVM(tSpeed, tHeading, self.GetVar('maneuverValue'), currentStDistance, currentStBearing, desiredStRange, desiredStBearing)

            bm = convertToMoboard (bm)
            bm = math.radians(bm)

            # PD control: reduce the speed as approaching the final station
            if self.CheckReduceSpeedNeeded(rangeDiff, bearingDiff) == True:
                vm = self.GetReducedSpeed(vm, rangeDiff, tSpeed)
                
            #print "Approach : course = ", bm, "speed = ", vm, "time =", t
            self.weight = 0.5
            self.bm_computed = bm
            self.vm_computed = vm
            self.t_computed = t
            self.entity.setTTC(t * 60) # convert to seconds
            self.SetVar ('maneuverChanged', False)
            self.SetVar ('activated', False)
            self.SetVar ('absoluteBearingLast', self.GetVar('absoluteBearing'))
            return (0.5, bm, vm)

        

                                         

    def checkPreconditions(self):
        return True

    def handleTargetSelection(self, tp, msg):
        self.setTarget(tp)
        self.SetVar ('maneuverChanged', True)
        print "CatPrimitiveApproach: handleTargetSelection: Target Changed to", str(tp)

    def checkPostconditions(self):
        #print self.reachedStation
        return self.reachedStation


