import math
from cat import behaviorInfo

def NormalizeHeading (heading):
    if heading > 360:
        heading = heading - 360
    elif heading <0:
        heading = heading + 360
    return heading

def convertToMoboard (headingDeg):
    heading = 360 - headingDeg
    normHeading = NormalizeHeading (heading)
    return normHeading

def convertToLagoon (headingDeg):
    heading = 360 - headingDeg
    normHeading = NormalizeHeading (heading)
    return normHeading        

twoPI = 2 * math.pi

def NormalizeHeadingRad (heading):
    if heading >= twoPI:
        heading = heading - twoPI
    elif heading <0:
        heading = heading + twoPI
    return heading

    

def maintainBM(vr, br, bm, r0, b0, r1, b1):
    # This function provides the time and speed for a maneuvering ship, given
    # a precise course for the maneuver
    # - the time value is informational
    
    #  Input:
    #  vr - reference ship speed (in knots)
    #  br - reference ship true bearing (in degrees)
    #  bm - maneuvering ship required course to perform the maneuver (in degrees)
    #  r0 - maneuvering ship initial range to the reference ship (in yards)
    #  b0 - maneuvering ship initial true bearing from reference ship (in degrees)
    #  r1 - maneuvering ship desired range to the reference ship (in yards)
    #  b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
    
    #  Output:
    #  t - the amount of time in which the maneuver has to be completed (in minutes)
    #  vm - maneuvering ship required speed to perform the maneuver (in knots)
    
    
    #print "maintainBM:", vr, br, bm, r0, b0, r1, b1
    
    
    # Convert to radians
    br_rad = math.radians(br)
    bm_rad = math.radians(bm)
    b0_rad = math.radians(b0)
    b1_rad = math.radians(b1)
    
    dx = r0*math.cos(b0_rad) - r1*math.cos(b1_rad)
    dy = r0*math.sin(b0_rad) - r1*math.sin(b1_rad)
    
    bd_rad = math.atan2(dy, dx)
    # Compute -bd
    bd_rad = bd_rad + math.pi
    bd = bd_rad*180/math.pi
    bd_rad = math.radians(bd)
    if bd < 0:
        bd = br + 360
    if br >= 360:
        br = br - 360
        
    if bd - br <= 180:
        if bm < br and bm > bd:
            print "===>> error input range: [", br, bd, "]"
            bm_rad = bd_rad
        else:
            print "====== input range: [", br, bd, "]"
    else:
        if bm < bd and bm > br:
            print "===>> error input range: [", bd, br, "]"
            bm_rad = bd_rad
        else:
            print "====== input range: [", bd, br, "]"

        
    # Convert speed to yards/minute
    vr = vr*2000.0/60.0

    if vr < 0.01:
        dx = -r0*math.cos(b0_rad) + r1*math.cos(b1_rad)
        dy = -r0*math.sin(b0_rad) + r1*math.sin(b1_rad)
        bm_rad = math.atan2(dy,dx)
        vm = 10
        t = -(60/2000)*dx/(math.cos(bm_rad)*vm)
        bm = bm_rad*180/math.pi
    else:
        vrx = vr*math.cos(br_rad)
        vry = vr*math.sin(br_rad)
        dx = r0*math.cos(b0_rad) - r1*math.cos(b1_rad)
        dy = r0*math.sin(b0_rad) - r1*math.sin(b1_rad)
        
        t = (-dy + dx*math.tan(bm_rad))/(vrx*math.tan(bm_rad) - vry)
        vm = (60.0*(dx*vry - dy*vrx))/(2000.0*(dx*math.sin(bm_rad) - dy*math.cos(bm_rad)))
        # Put in different error checking mechanism for speed
        #if vm > 22:
        #    vm = 22
        if vm < 0:
            vm = 0
            
        bm = bm_rad*180/math.pi
    #print "maintainBM: bm = ", bm, "vm = ", vm, "t = ", t
    return (bm, vm, t)


def maintainT (vr, br, t, r0, b0, r1, b1):

    # This function provides the course and speed for a maneuvering ship, given
    # a precise time for the completion of the maneuver
    
    # Input:
    # vr - reference ship speed (in knots)
    # br - reference ship true bearing (in degrees)
    # t - the amount of time in which the maneuver has to be completed (in minutes)
    # r0 - maneuvering ship initial range to the reference ship (in yards)
    # b0 - maneuvering ship initial true bearing from reference ship (in degrees)
    # r1 - maneuvering ship desired range to the reference ship (in yards)
    # b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
    
    # Output:
    # bm - maneuvering ship required course to perform the maneuver (in degrees)
    # vm - maneuvering ship required speed to perform the maneuver (in knots)
    
    # print "maintainT:", vr, br, t, r0, b0, r1, b1
    
    if r0 == r1 and b0 == b1:
        #            print "MaintainT: nothing to do - already at destination"
        return (0, 0, 0)

    if t == 0:
        #            print "MaintainT: zero time selected: going with max speed"
        vm = 20
        bm, vm, t = maintainVM (vr, br, vm, r0, b0, r1, b1)
        return (bm, vm, t)
        
    # Convert to radians
    br = math.radians(br) 
    b0 = math.radians(b0) 
    b1 = math.radians(b1)
        
    # Convert speed to yards/minute
    vr = vr*2000/60
    dx = r0*math.cos(b0) - r1*math.cos(b1)
    dy = r0*math.sin(b0) - r1*math.sin(b1)

    if vr == 0:
        bm = math.atan2(-dy,-dx)
        vm = -(dx+dy)/((math.cos(bm)+math.sin(bm))*t)
    else:
        vrx = vr*math.cos(br)
        vry = vr*math.sin(br)
        vmx = (vrx*t-dx)/t
        vmy = (vry*t-dy)/t
        bm = math.atan2(vmy, vmx)
        vm = math.sqrt(vmx**2 + vmy**2)

    vm = (vm*60.0)/2000.0
    bm = bm*180/math.pi
    # print "maintainT: bm = ", bm, "vm = ", vm, "t = ", t
    return (bm, vm, t)

def maintainVM (vr, br, vm, r0, b0, r1, b1):
    
    # This function provides the course and time required for a maneuvering ship, given
    # a precise speed for performing the maneuver
    # - the time value is informational
    
    # Input:
    # vr - reference ship speed (in knots)
    # br - reference ship true bearing (in degrees)
    # vm - maneuvering ship required speed to perform the maneuver (in knots)
    # r0 - maneuvering ship initial range to the reference ship (in yards)
    # b0 - maneuvering ship initial true bearing from reference ship (in degrees)
    # r1 - maneuvering ship desired range to the reference ship (in yards)
    # b1 - maneuvering ship desired true bearing from reference ship at the end of the maneuver (in degrees)
 
    # Output:
    # bm - maneuvering ship required course to perform the maneuver (in degrees)
    # t - the amount of time in which the maneuver has to be completed (in minutes)

    # print "maintainVM:", vr, br, vm, r0, b0, r1, b1
    
    if r0 == r1 and b0 == b1:
        print "MaintainVM: nothing to do - already at destination"
        return (0, 0, 0)
        
    # Convert to radians
    br = math.radians(br)
    b0 = math.radians(b0)
    b1 = math.radians(b1)

    # Convert speed to yards/minute
    vm_knots = vm
    vr = vr*2000/60
    vm = vm*2000/60
    
    vrx = vr*math.cos(br)
    vry = vr*math.sin(br)
    
    dx = r0*math.cos(b0) - r1*math.cos(b1)
    dy = r0*math.sin(b0) - r1*math.sin(b1)
    
    if vr == 0:
        if vm == 0:
            print "MaintainVM - error: must supply maneuver ship speed greater than zero"
            return (0, 0, 0)
        
        bm = math.atan2(-dy,-dx)
        #t = -(60.0/2000)*dx/(math.cos(bm)*vm)
        t = -(dx+dy)/(vm*(math.cos(bm)+math.sin(bm)))
    else:
        a = vrx**2 + vry**2 - vm**2
        b = -2*(vrx*dx + vry*dy)
        c = dx**2 + dy**2
        b2m4ac = b**2 - 4*a*c
        if (b2m4ac < 0):
            print "Error: maintainVM - no solution to the problem exists"
            return (0, 0, 0)                           
                
        delta = math.sqrt(b2m4ac)
        if delta < 0 or a == 0 :
            print "Error: maintainVM - no solution to the problem exists"
            return (0, 0, 0)                           
        t1 = (-b + delta)/(2*a)
        t2 = (-b - delta)/(2*a)
        
        if (t1 > 0):
            t = t1
        else:
            t = t2
        vmx = (vrx*t - dx)/t
        vmy = (vry*t - dy)/t
        bm = math.atan2(vmy, vmx)
            
    # Convert bm to degrees
    bm = bm*180/math.pi
    vm = vm_knots
    #print "maintainVM: bm = ", bm, "vm = ", vm, "t = ", t
    return (bm, vm, t)

