#
# @(#) controller.py 2005/9/29 changed on 2006/2/22
# @author Sushil Louis, Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
import wx

from common.log import Log
log = Log(tags=['entity'])
from common.defs import *
import nEventServer
from common import EventNames

PANELSIZE = (225, 800)

from cat.Controllers import primitiveBehaviorController
from cat import panelInfo
from cat import behaviorInfo
from cat.ports import port
from cat.xml import environment
from cat.sensors import entityDistanceSensor
from cat.Interface import flagsensor
from cat import catInfo
from cat.triggers.trigger import Trigger

MOVE_VARS   = ('desiredX', 'desiredY')
TARGET_VARS = ('target')
IGNORE_VARS = ('entMaxSpeed', 'maneuverValueMax', 'maneuverValueMin')
AVOID_BASE  = 'AVOID'

class Controller:
    """A controller and controller manager for an entity"""
    def __init__(self, panelA, entity = None, filename = None, params=None, spec = None):
        self.authorPanel      = panelA 
        self.netSpec             = spec
        self.entity           = entity
        self.filename         = filename   # filename from which to load controller
        self.params           = params     # unit interaction parameter dictionary
        self.environment      = None

        self.behaviorSet      = {}
        self.behaviorNet      = {}
        self.pbcList          = []
        self.name             = spec.name

        self.makeNetwork(self.netSpec)

    def getNetName(self):
        return self.name

    def getName(self):
        return self.name

    def makeNetwork(self, netSpec):
        blist = netSpec.behaviors
        for bSpec in blist:
            pbc = self.makeBehavior(bSpec)
            self.pbcList.append(pbc)
            behavior = pbc.getBehavior()
            bname = bSpec.name
            bid   = bSpec.id
            self.setBehaviorParams(behavior, bSpec)
            self.addBehaviorToController(bname = bname, id = bid, behavior = behavior)

        self.hookUpBehaviors(netSpec)

    #-------------------------------------------------------------
    # Trigger handling code
    #-------------------------------------------------------------

    def createTriggers(self):
        blist = self.netSpec.behaviors
        behaviorTriggerMap = {}
        triggerMap = {}
        for bSpec in blist:
            triggerList = []
            for trigger in bSpec.triggers:
                t = self.initTrigger(trigger, triggerMap)
                triggerList.append(t)
            behaviorTriggerMap[bSpec.id] = triggerList
        return behaviorTriggerMap

    #-------------------------------------------------------------

    def hookupTriggers(self, tmap):
        for bid, triggerList in tmap.iteritems():
            behavior = self.getBehavior(bid)
            if not behavior:
                print "NO Behavior to hook up triggers to:", bid
                return
            for triggerMap in triggerList:
                for id, trigger in triggerMap.iteritems():
                    trigger.id = id
                    behavior.addTrigger(trigger)

    #-------------------------------------------------------------

    def initTrigger(self, tspec, triggerMap):
        id = tspec.id
        if triggerMap.has_key(tspec.id):
            return triggerMap
        toPath = None
        if tspec.toPath:
            toPath = self.environment.lookup(tspec.toPath)
            if self.netSpec.ownship.startswith(toPath):
                self.environment.changeBinding(toPath, self.entity.getEVAPath())
                toPath = self.environment.lookup(tspec.toPath)
        fromPath = None
        if tspec.fromPath:
            fromPath = self.environment.lookup(tspec.fromPath)
            if self.netSpec.ownship.startswith(fromPath):
                self.environment.changeBinding(fromPath, self.entity.getEVAPath())
                fromPath = self.environment.lookup(tspec.fromPath)

        trig = Trigger(self.entity, tspec.name, tspec.type, subtype=tspec.subtype,
                       threshold=tspec.threshold, toPath = toPath, fromPath = fromPath)
        triggerMap[tspec.id] = trig
        return triggerMap

    #-------------------------------------------------------------
    
    def bind(self):
        self.environment = environment.Environment(self.netSpec.name)
        blist = self.netSpec.behaviors
        for bSpec in blist:
            bindings = bSpec.bindings
            for binding in bindings:
                name = binding.name
                val  = binding.val
                hr   = binding.humanReadable
                self.environment.addBinding(name = name, val = val, humanReadable=hr)
            bid      = bSpec.id
            targets  = bSpec.targets
            for target in targets:
                behavior = self.getBehavior(bid)
                if (behavior): ### is behavior created?
                    targetPath = behavior.getTarget()
                    behavior.setTargetBindingName(target)
                #self.environment.addBinding(name = target, val = targetPath, humanReadable=bid+'Target')
            
    def makeBehavior(self, bSpec):
        from cat.Controllers import primitiveBehaviorController
        from cat import panelInfo
        pbc = primitiveBehaviorController.PrimitiveBehaviorController(self.authorPanel, entity=self.entity, bname = bSpec.id, size = panelInfo.CONTROLWRAPPERPANEL, show = True)
        return pbc

    def setBehaviorParams(self, behavior, spec):
        paramList = spec.parameters
        for param in paramList:
            type = param.type
            name = param.name
            val  = param.val
            min  = param.min
            max  = param.max
            behavior.SetVar(name, val)

    def setSPArgs(self, args):
        for name, val in args.iteritems():
            varName = name.split("_")[-1]
            for id, behavior in self.behaviorNet.iteritems():
                print "Setting args: ", behavior.getName, varName
                if (not (behavior.getName().startswith(AVOID_BASE))):
                    if (varName in TARGET_VARS):
                        behavior.setTarget(val)
                    elif (not (varName in IGNORE_VARS)):
                        behavior.SetVar(varName, val)
        return
            
    #port is only added if behaviors found
    def hookUpBehaviors(self, netSpec):
        blist = netSpec.behaviors
        for bSpec in blist:
            links = bSpec.links
            behavior = self.getBehavior(bSpec.id)
            if behavior:
                for link in links:
                    if (link.dir.startswith("to")):
                        prt = port.Port(link.type, link.triggerid)
                        destBehavior = self.getBehavior(link.dest)
                        if destBehavior:
                            behavior.addRunningPort(prt)
                            destBehavior.addPreconditionPort(prt)

    def getParams(self):
        return self.params

    def setParams(self, paramsDict):
        self.params = paramsDict

    def getControllerPanels(self):
        return [self.authorPanel]

    def setControllerPanels(self, cPanelA):
        self.authorPanel = cPanelA

    def getControllerFile(self):
        return self.filename

    def setControllerFile(self, filename):
        self.filename = filename

    def getBehaviorNet(self):
        return self.behaviorNet

    def setBehaviorNet(self, net):
        self.behaviorNet = net

    #None is returned when id isn't in behaviorNet.keys
    def getBehavior(self, id):
        if(self.behaviorNet and id in self.behaviorNet.keys()):
            return self.behaviorNet[id]
        else:
            return None

    def setBehavior(self, bid, behavior):
        self.behaviorNet[bid] = behavior

    def addBehaviorToController(self, bname=None, id = None, behavior=None):
        if(not bname):
            print "No behavior name to add"
            return
            
        if(not self.behaviorSet.has_key(bname)):  # if set already has the name do not add bname:behavior
            if(behavior):
                self.behaviorSet[bname] = behavior
            else:
                print "I've already got a behavior called: ", bname
        # if we have behavior name only, then create behavior then add to network
        if(not behavior):
            from cat.Behaviors import catBehavior
            b = []
        else:
            b = behavior

        # Finally, if id exists, add to behavior net
        if(id):
            self.behaviorNet[id] = b

    def getBehaviorSet(self):
        return self.behaviorSet

    def setBehaviorSet(self, bset):
        self.behaviorSet = bset

#--------------------------------------------------------
# if you want to change a controller's behavior's target use this method
#--------------------------------------------------------
    def handleTargetSelection(self, targetPath, msg):
        if(self.behaviorNet):
            for id, behavior in self.behaviorNet.iteritems():
                if(behavior.getTarget()):
                    self.environment.changeBinding(behavior.getTarget(), targetPath)
                else:
                    targetBindingName = behavior.getTargetBindingName()
                    self.environment.addBinding(targetBindingName, targetPath)
                self.handleBindingChange(behavior.getTarget(), targetPath)
                behavior.setTarget(targetPath)                
        else:
            log.write("No behavior net to set Target on")

    def handleBindingChange(self, oldPath, newPath):
        for triggerList in self.triggerMap.values():
            for triggerDict in triggerList: ## added this to fix DC
                for id, trigger in triggerDict.iteritems(): ## added this to fix DC
                    if(trigger):  ## added this to fix DC
                        trigger.sensor.handleBindingChange(oldPath, newPath)


    def hasAvoidActive(self):
        for id, behavior in self.behaviorNet.iteritems():
            if(id.startswith("AVOID")):
                if (behavior.runnable ):
                    return True
        return False
    
    def start(self):
        if self.behaviorNet:
            for id, behavior in self.behaviorNet.iteritems():
                behavior.start()
                print "Controller:Controller starting: ", behavior.getName()                
        else:
            log.write('No behavior net to start')

    def init(self):
        print "Starting controller initialization"
        if(self.behaviorNet):
            self.bind()
            self.entity.clearTriggers() # Shouldn't we clear old triggers before creating them?            
            self.triggerMap = self.createTriggers()
            self.hookupTriggers(self.triggerMap)
            for id, behavior in self.behaviorNet.iteritems():
                behavior.setEnvironment(self.environment)
                bindingName = behavior.getTargetBindingName()
                target = self.environment.lookup(bindingName)
                if(target):
                    behavior.setTarget(target)
                if(behavior.checkPrePorts()):
                    behavior.Show()
                else:
                    behavior.Hide()
                #self.entity.clearTriggers()
                for triggerList in self.triggerMap.values():
                    for triggerDict in triggerList:
                        for id, trigger in triggerDict.iteritems():
                            trigger.id = id
                            self.entity.addTrigger(trigger)
                print "C: Ready to start"
        else:
            log.write("No behavior net to start")

    def stop(self):
        if(self.behaviorNet):
            for id, behavior in self.behaviorNet.iteritems():
                #print "stopping behavior net behavior: ", behavior.getName()
                behavior.stop()
                behavior.Hide()
        else:
            log.write("No behavior net to stop")


    def pause(self):
        if self.behaviorNet:
            for id, behavior in self.behaviorNet.iteritems():
                behavior.pause()
        else:
            log.write('No behavior net to pause')

    def record(self):
        if(self.behaviorNet):
            self.behaviorNet.record()
        else:
            log.write("No behavior net to record")
        
    def update(self):
        print "Updating Controller"

    def updatePanels(self):
        print " "

    def show(self):
        if(self.behaviorNet):
            for id, behavior in self.behaviorNet.iteritems():
                behavior.Show()
        else:
            log.write("No behavior net to Show")

    def hide(self):
        if(self.behaviorNet):
            for id, behavior in self.behaviorNet.iteritems():
                behavior.Hide()
        else:
            log.write("No behavior net to Show")


    def handlePageChange(self, mode):
        if(self.behaviorNet):
            for id, behavior in self.behaviorNet.iteritems():
                behavior.handlePageChange(mode)
