#
# @(#) primitiveBehaviorController.py 2005/19/29
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

import wx
import wx.lib.fancytext as fancytext

from common.log import Log
log = Log(tags=['entity'])
from common.defs import *

import controller
from common import behaviorNames

from cat.Behaviors import catBehavior	

DEFAULTSIZE = (220, 400)

class PrimitiveBehaviorController:
    """A controller for an entity"""
    def __init__(self, panelA, entity = None, filename = None, bname=None, params=None, size=DEFAULTSIZE, show = True, bid = None):
        self.behaviorName = bname
        self.behaviorId   = bid
        self.entity = entity
        self.authorPanel = panelA
        self.wrapper = None
        self.size = size # wrapper size
        self.show = show
        self.behavior = self.initBehavior(entity=entity, bname=bname, size = size, show = show)

    def initBehavior(self, entity=None, bname="APPROACH", size = None, show = False):
        """Creates the behavior class instance"""
	contPanelA = entity.getControllerPanels()
        self.wrapper = contPanelA.getWrapper()

        if(bname.startswith(behaviorNames.FIRE)):
            from cat.Behaviors import catFire
            # create Fire Behavior, create panels, add panels to wrapper
	    b = catFire.CatFire(self.wrapper, entity = entity, bname = bname, size=size, show=show)            
        elif (bname.startswith(behaviorNames.EMPTY)):
            # create EMPTY Behavior, create panels, add panels to wrapper            
            b = catBehavior.CatBehavior(self.wrapper, entity=entity, bname = bname, size=size, show=show)            
        elif (bname.startswith(behaviorNames.APPROACH)):
            # create Approach Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catApproach
            b = catApproach.CatApproach(self.wrapper, entity = entity, bname = bname, size = size, show=show)            
        elif (bname.startswith(behaviorNames.MAINTAIN)):
            # create MAINTAIN Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catMaintain
            b = catMaintain.CatMaintain(self.wrapper, entity = entity, bname = bname, size = size, show=show)          
        elif (bname.startswith(behaviorNames.RAM)):
            # create RAM Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catRam
            b = catRam.CatRam(self.wrapper,  entity = entity, bname = bname, size = size, show=show)            
        elif (bname.startswith(behaviorNames.MOVETO)):
            # create MOVETO Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catMoveTo
            b = catMoveTo.CatMoveTo(self.wrapper, entity = entity, bname = bname, size = size, show=show)            
        elif (bname.startswith(behaviorNames.AVOIDENTITY)):
            # create AVOIDENTITY Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catAvoidEntity
            b = catAvoidEntity.CatAvoidEntity(self.wrapper, entity = entity, bname = bname, size = size, show=show)            
        elif (bname.startswith(behaviorNames.AVOIDLAND)):
            # create AVOIDLAND Behavior, create panels, add panels to wrapper            
            from cat.Behaviors import catAvoidLand
            b = catAvoidLand.CatAvoidLand(self.wrapper, entity = entity, bname = bname, size = size, show=show)            
        else:
            # default behavior and behavior panel
            b = catBehavior.CatBehavior(self.wrapper,entity=entity, bname = bname, size = size, show=show)
        return b

    def getBehavior(self):
        return self.behavior

    def getBehaviorName(self):
        return self.behaviorName

    def setBehavior(self, bname, behavior):
        self.behaviorName = bname
        self.behavior = behavior

    def Show(self):
        self.behavior.Show()

    def Hide(self):
        self.behavior.Hide()

    def start(self):
        if(self.behavior):
            self.behavior.start()
        else:
            print("No behavior to start")

    def stop(self):
        if(self.behavior):
            self.behavior.stop()
            self.behavior.Hide()
        else:
            print("No behavior to stop")

    def triggerStop(self, triggerSpec):
        if self.behavior:
            self.behavior.triggerStop(triggerSpec)

