#
# @(#) catApproachesPanel 2005/12/7
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

"""
Cat Approaches panel

Authoring and Execution UI for approach

Sergiu: check Jan 28 2006
 
"""
import wx
import catSelectorPanel
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'cat'])

import nEventServer
from common import EventNames
from cat.Behaviors import catPrimitiveApproach as mapp

class CatApproachesPanel(catSelectorPanel.CatSelectorPanel):
    """
    Sets up the GUI for changing the parameters of approach.
    """
    def __init__(self, parent, panelID, color="CYAN", size=None, behaviorType = None, entity = None, behavior = None):
        """
        Creates the new panel.
        parentPanel (wx.Panel): Approach, Retreat, and Maintain are mutually
        exclusive. These behaviors are put under one tab and they need this
        parent tab to change its title.
        """
        catSelectorPanel.CatSelectorPanel.__init__(self, parent, panelID, color, size, behaviorType, entity, behavior)
	
        # self.bearingSlider = None
        # self.rangeSlider   = None
        # self.rb            = None

	from common import behaviorNames
       
        # Initialize range slider
        
	self.sizer.Add(wx.StaticText(self, -1, "New station range"))
        self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.rangeSlider = wx.Slider(self, -1, mapp.DEFAULT_RANGE_SLIDER, mapp.MIN_RANGE_SLIDER, mapp.MAX_RANGE_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        self.labelSizer.Add(self.rangeSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        self.labelSizer.Add(wx.StaticText(self, -1, 'yards'))
        self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND)
        wx.EVT_SLIDER(self.rangeSlider, -1, self.OnRangeChange)

        # Initialize bearing slider
	
        self.sizer.Add(wx.StaticText(self, -1, "New station bearing"))
        self.labelSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.bearingSlider = wx.Slider(self, -1, mapp.DEFAULT_BEARING_SLIDER, mapp.MIN_BEARING_SLIDER, mapp.MAX_BEARING_SLIDER, style=wx.SL_LABELS|wx.SL_HORIZONTAL)
        self.labelSizer.Add(self.bearingSlider, 1, border=5, flag=wx.RIGHT|wx.EXPAND)
        self.labelSizer.Add(wx.StaticText(self, -1, 'degs '))
        self.sizer.Add(self.labelSizer, 1, flag=wx.EXPAND)
        wx.EVT_SLIDER(self.bearingSlider, -1, self.OnBearingChange)

        # Initialize bearing options
	
	bearingSizer = wx.BoxSizer(wx.HORIZONTAL) 
	bearingOptions = ['Abs', 'Rel']

        self.rb = wx.RadioBox(
            self, -1, "", wx.DefaultPosition, wx.DefaultSize,
            bearingOptions, 1, wx.RA_SPECIFY_ROWS
            )
            
        self.Bind(wx.EVT_RADIOBOX, self.OnEvtRadioBox, self.rb)
        bearingSizer.Add(self.rb, 0)#, wx.ALL, 20)
	
	# Add Start/Stop button
	self.startStopButton = wx.Button(self, -1, "Start", style=wx.BU_EXACTFIT)
	self.startStopButtonState = "start"
	wx.EVT_BUTTON(self.startStopButton, -1, self.OnClick)
	wx.EVT_SET_FOCUS(self, self.handleEvent)
	
        # Space between radio buttons and start button
	spacerY =wx.StaticText(self, -1, "", size=(65, 15), style=wx.ALIGN_BOTTOM)

        bearingSizer.Add(spacerY)
        bearingSizer.Add(self.startStopButton, 0, wx.ALIGN_RIGHT|wx.ALIGN_BOTTOM)

        self.sizer.Add(bearingSizer)
	
	spacer_1 =wx.StaticText(self, -1, "", size=(180,2), style=wx.ALIGN_LEFT)
	self.sizer.Add(spacer_1)


    def WriteNotifMsg(self, msg):
        #self.notifMsg.SetLabel (msg)
        print str(msg)
	
    def OnEvtRadioBox(self, event):
	behavior = self.behavior
        if(behavior):
	    selection = self.rb.GetSelection()
	    if selection == 0:
	        behavior.SetVar('absoluteBearing', True)
	    else:
		behavior.SetVar('absoluteBearing', False)
	
    def stop(self):
        print "Stopping panel"
	self.WriteNotifMsg ("Stopping panel")
	    
    def OnRangeChange(self, event):
         """
         Called on station range change
         """
         # print "On Range Change: ", event
         behavior = self.behavior
         if(behavior):
             behavior.SetVar('maneuverChanged', True)
             behavior.SetVar('reachedStation', False)
             behavior.SetVar('desiredStRange', self.rangeSlider.GetValue())            
             # print "Range value: ", str(self.rangeSlider.GetValue())
         else:
             print "No attached behavior to range change"
             self.WriteNotifMsg ("no att b to range chg")
             
    def OnBearingChange(self, event):
         """
         Called on station bearing change
         """
#         print "On station bearing Change: ", event
         behavior = self.behavior
         if(behavior):
             behavior.SetVar('maneuverChanged', True)
             behavior.SetVar('reachedStation', False)
             behavior.SetVar('desiredStBearing', self.bearingSlider.GetValue())
         else:
             print "No attached behavior to bearing change"
	     self.WriteNotifMsg ("no att b to bearing chg")
             
    def OnClick(self, event):
        """
        Called when the activate button is pressed.
        Turns off other behaviors and changes the parent tab's name.
        """
        if(self.behavior):
            self.behavior.handleStateSwitch()
            self.startStopButton.SetLabel(self.behavior.startStop)
        else:
            print "ApproachType: I am on a time out due to bad behavior"

    def SetStartStop(self):
        self.startStopButton.SetLabel(self.behavior.startStop)
#         if(self.maneuverValueSlider):
#             val = self.behavior.GetVar('maneuverValue')
#             print "Maneuver Val: " + str(val)
#             self.maneuverValueSlider.SetValue(val)
       
    def updatePanels(self):
 #       print "Handling Page change in " + self.behavior.getName()
        self.SetStartStop()
        if(self.maneuverTypeList):
            mtype = self.behavior.GetVar('maneuverType')
            if(mtype == mapp.SPEED_CONST_MANEUVER):
                self.maneuverTypeList.SetValue('Maintain speed')
            elif(mtype == mapp.TIME_CONST_MANEUVER):
                self.maneuverTypeList.SetValue('Maintain time')

        if(self.maneuverValueSlider):
            val = self.behavior.GetVar('maneuverValue')
#            print "Page Changed: Maneuver Val: " + str(val)
            self.maneuverValueSlider.SetValue(val)
        if(self.bearingSlider):
#            print "Changeing bearing slider"
            self.bearingSlider.SetValue(self.behavior.GetVar('desiredStBearing'))
        if(self.rangeSlider):
#            print "Changeing range slider"            
            self.rangeSlider.SetValue(self.behavior.GetVar('desiredStRange'))
        if(self.rb):
            if(self.behavior.GetVar('absoluteBearing')):
                val = 0
            else:
                val = 1
            self.rb.SetSelection(val)
        self.Layout()
        self.Validate()
        self.Show()

    def handleEvent(self, event):
#        print "Handling window focus Event"
        self.updatePanels()

    def handlePageChange(self, msg):
        self.updatePanels()
