#
# @(#) controlPanel.py 2005/09/27
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

'''
Defines the Control Panel in GUI CAT.

'''
import wx
import wx.lib.fancytext as fancytext
import common.WindowManager as WindowManager
from common.defs import *
import xml.sax
from xml.sax.handler import ContentHandler
import math

from common import EventNames
import nEventServer

from cat.Behaviors import catPrimitiveApproach as mapp
from cat import behaviorInfo

DESIREDHEADING = " DesiredHeading"
DESIREDSPEED   = " DesiredSpeed"

HEADINGSPEEDCONTROLSIZE   = (50, 20)
BUTTON_PAD 	          = 8
CONTROLPANELSIZE          = (220, 500)


class ControlPanel(wx.Panel):
    
    '''Control panel class in CAT GUI'''
    
    def __init__(self, parent, size=None, color = "CYAN", entity = None, wsize=None):
        
        wx.Panel.__init__(self, parent, -1, size=size, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
        self.entity = entity
        from cat import panelInfo
        parentColor = color
        self.bgcolor = color
        self.SetBackgroundColour(self.bgcolor)
	
	labelControl  = wx.StaticText(self, -1, "CONTROL", size=(200,-1))
	
	spacer1 = wx.StaticText(self, -1, "", size= (10, 10), style = wx.ALIGN_RIGHT)
	spacer2 = wx.StaticText(self, -1, "", size= (195, 5), style = wx.ALIGN_RIGHT)
	spacer3 = wx.StaticText(self, -1, "", size= (215, 1), style = wx.ALIGN_RIGHT)
	spacer4 = wx.StaticText(self, -1, "", size= (215, 1), style = wx.ALIGN_RIGHT)
	spacer5 = wx.StaticText(self, -1, "", size= (195, 5), style = wx.ALIGN_RIGHT)

        wrapperSize = wsize
#        print "ControlPanel size: %s, Control Wrapper size: %s"% (str(self.GetSize()), str(wrapperSize))
	import catControlWrapper
	self.controlWrapper = catControlWrapper.CatControlWrapper(self, -1, color, size=wrapperSize)
	
	import os	
        prefix = os.path.dirname(__file__)+"/"
	
	####################### create buttons to select behaviors ################## 
	from common import behaviorNames
	bmpApproach 	= wx.Bitmap(prefix+"approach.gif", wx.BITMAP_TYPE_GIF)
        b0 		= wx.BitmapButton(self, 30, bmpApproach, (20, 20),
                       (bmpApproach.GetWidth()+BUTTON_PAD, bmpApproach.GetHeight()+BUTTON_PAD), name=behaviorNames.APPROACH)
        bname=behaviorNames.APPROACH
        b0.SetToolTipString("Approach")
	wx.EVT_BUTTON(self, 30, self.entity.showBehavior)
	
        
	bmpMaintain 	= wx.Bitmap(prefix+"maintain.gif", wx.BITMAP_TYPE_GIF)
	#mask1 		= wx.Mask(bmpMaintain, "WHITE")
        #bmpMaintain.SetMask(mask1)

	b1 		= wx.BitmapButton(self, 30, bmpMaintain, (20, 20),
                       (bmpMaintain.GetWidth()+BUTTON_PAD, bmpMaintain.GetHeight()+BUTTON_PAD), name=behaviorNames.MAINTAIN)
        bname=behaviorNames.MAINTAIN
        b1.SetToolTipString("Maintain Station")
	wx.EVT_BUTTON(self, 30, self.entity.showBehavior)
	
	bmpRam 		= wx.Bitmap(prefix+"ram.gif", wx.BITMAP_TYPE_GIF)
	#mask2 		= wx.Mask(bmpRam, "GREEN")
        #bmpRam.SetMask(mask2)
	b2 		= wx.BitmapButton(self, 30, bmpRam, (20, 20),
                       (bmpRam.GetWidth()+BUTTON_PAD, bmpRam.GetHeight()+BUTTON_PAD), name=behaviorNames.RAM)
        bname=behaviorNames.RAM
        b2.SetToolTipString("Ram")
     	wx.EVT_BUTTON(b2, 30, self.entity.showBehavior)
	
	bmpMoveTo 	= wx.Bitmap(prefix+"moveto.gif", wx.BITMAP_TYPE_GIF)
	#mask3 		= wx.Mask(bmpMoveTo, "GREEN")
        #bmpMoveTo.SetMask(mask3)
        b3 		= wx.BitmapButton(self, 30, bmpMoveTo, (20, 20),
                       (bmpMoveTo.GetWidth()+BUTTON_PAD, bmpMoveTo.GetHeight()+BUTTON_PAD), name=behaviorNames.MOVETO)
        bname=behaviorNames.MOVETO
        b3.SetToolTipString("Move To")
     	wx.EVT_BUTTON(b3, 30, self.entity.showBehavior)
	
	bmpAvoidLand 	= wx.Bitmap(prefix+"gray.gif", wx.BITMAP_TYPE_GIF)
	#mask4 = wx.Mask(bmpAvoidLand, "CADET BLUE")
        #bmpMoveTo.SetMask(mask4)
        b4 = wx.BitmapButton(self, 30, bmpAvoidLand, (20, 20),
                       (bmpAvoidLand.GetWidth()+BUTTON_PAD, bmpAvoidLand.GetHeight()+BUTTON_PAD), name=behaviorNames.AVOIDLAND)
        bname=behaviorNames.AVOIDLAND
        b4.SetToolTipString("Avoid Land")
     	wx.EVT_BUTTON(b4, 30, self.entity.showBehavior)

        bmpAvoidEntity  = wx.Bitmap(prefix+"gray.gif", wx.BITMAP_TYPE_GIF)
        #mask6           = wx.Mask(bmpAvoidEntity,"MAGENTA")
        #bmpAvoidEntity.SetMask(mask6)
        b6              = wx.BitmapButton(self,30, bmpAvoidEntity, (20,20),
                       (bmpAvoidEntity.GetWidth()+BUTTON_PAD, bmpAvoidEntity.GetHeight()+BUTTON_PAD),name=behaviorNames.AVOIDENTITY)
        bname=behaviorNames.AVOIDENTITY
        b6.SetToolTipString("Avoid Entities")
        wx.EVT_BUTTON(b6, 30, self.entity.showBehavior)

        bmpFire         = wx.Bitmap(prefix+"fire.gif", wx.wx.BITMAP_TYPE_GIF)
        #mask7           = wx.Mask(bmpFire, "MAGENTA")
        #bmpAvoidEntity.SetMask(mask7)
        b7              = wx.BitmapButton(self,30, bmpFire, (20,20),
                       (bmpFire.GetWidth()+BUTTON_PAD, bmpFire.GetHeight()+BUTTON_PAD),name=behaviorNames.FIRE)
        bname=behaviorNames.FIRE
        b7.SetToolTipString("Fire")
        wx.EVT_BUTTON(b7, 30, self.entity.showBehavior)
	
	bmpStop 	= wx.Bitmap(prefix+"stopall.gif", wx.wx.BITMAP_TYPE_GIF)
	#mask5 		= wx.Mask(bmpStop, "RED")
        #bmpStop.SetMask(mask5)
	b5 		= wx.BitmapButton(self, 30, bmpStop, (20, 20),
                       (bmpStop.GetWidth()+BUTTON_PAD-3, bmpStop.GetHeight()+BUTTON_PAD-3), name=behaviorNames.STOPALL)
        bname=behaviorNames.STOPALL
        b5.SetToolTipString("Stop all behaviors")
     	wx.EVT_BUTTON(b5, 30, self.entity.stopAll)
		
	############### place behavior control buttons on a bar #######################
		
	box 		= wx.StaticBox(self, -1, "", style=wx.SUNKEN_BORDER)
        bsizer 		= wx.StaticBoxSizer(box, wx.HORIZONTAL)
        bsizer.Add	(b0, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b2, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b3, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b4, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add      (b6, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add      (b7, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
	bsizer.Add	(spacer1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b5, 0, wx.CENTER|wx.ALIGN_RIGHT, 1)
	
	border 		= wx.BoxSizer()
        border.Add	(bsizer, 1, wx.EXPAND|wx.ALL, 2)
 
    	############## add control wrapper panel to show selected behavior) ###########
	
	#########  putting all together ######################################################

	sizerControl = wx.BoxSizer(wx.VERTICAL)
	sizerControl.Add(labelControl, 0, wx.ALIGN_LEFT)

	sizerControl.Add(spacer5, 0)        
	sizerControl.Add(border,  0) # behavior buttons        
        sizerControl.Add(self.controlWrapper, 1, flag=wx.FIXED_MINSIZE)
	self.SetSizer(sizerControl)

    def getWrapper(self):
        return self.controlWrapper

    def getPanel(self):
        return self
