# @(#) catGUI.py 2005/07/06
# @author Sushil J Louis Copyright (c) 2005 The
#University of Nevada, Reno. All rights reserved.  This software is
#provided "AS IS," without a warranty of any kind. ALL EXPRESS OR
#IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES, INCLUDING ANY
#IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE
#OR NON-INFRINGEMENT, ARE HEREBY EXCLUDED. THE UNIVERSITY OF NEVADA,
#RENO AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES SUFFERED
#BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING THE
#SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL THE UNIVERSITY OF
#NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT
#OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL
#OR PUNITIVE DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF
#LIABILITY, ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE,
#EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN ADVISED OF THE
#POSSIBILITY OF SUCH DAMAGES.  TODO: Fix Header TODO: Look into
#horizontal layout TODO: Set up recorder TODO: Tighten interface '''
#Defines the GUI and its operations for cat.
'''
This is where all the UI elements for CAT are defined as well as
the functions for manipulating that GUI. Ideally, this file should just
be UI elements, but because the behaviors have GUI elements, the code for
behavior fusion and output is also handled here. 
'''
import wx
import wx.lib.fancytext as fancytext
import common.WindowManager as WindowManager
from common.defs import *
import knob
import math

import nEventServer
from common import EventNames
from cat.xml import xmlReader



class CATPanel(wx.Panel):
    '''
    The wxFrame that defines the GUI
    '''
    ID_PAUSE = 102
    ID_SAVE = 103
    ID_RECORD = 104
    ID_STOP = 105
    ID_PLAY = 106
    ID_START = 107
    ID_QUERY = 108

    def __init__(self, parent, ID=-1, pos = wx.DefaultPosition, size = wx.DefaultSize):
        '''
        Creates and layouts the CAT GUI
        '''	
        wx.Panel.__init__(self, parent, ID, size = size)

        #nEventServer.register(EventNames.RMENU_AUTHORING_MODE,  self.onSetAuthoring)
        #nEventServer.register(EventNames.RMENU_EXECUTION_MODE,  self.onSetExecution)
        nEventServer.register(EventNames.CAT_SELECT_ENT,        self.onSelectEnt)
	
        self.entitySet = {}
        self.bgcolor = "CYAN"
    
        color1 = wx.Colour (185, 239, 232)
        color2 = wx.Colour (123, 219, 235)
        color3 = wx.Colour (123, 198, 235)
        color4 = wx.Colour (123, 163, 235)

        color5 = wx.Colour (236, 231, 190)
        color6 = wx.Colour (221, 211, 124)
        color7 = wx.Colour (236, 223,  79)
        color8 = wx.Colour (236, 184,  79)
        xSize, ySize = size
        #print "Panel Size = ", str(size)
        from cat import panelInfo
#        notebookSize = (xSize, ySize - 200)
        notebookSize = size
        recordSize   = panelInfo.RECORDPANEL
        self.notebook = wx.Notebook(self, -1, size=notebookSize)

        wx.EVT_NOTEBOOK_PAGE_CHANGED(self.notebook, -1, self.pageChanged)
    
        self.panelAuthoring = wx.Panel(self.notebook, -1, size=notebookSize, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
        self.panelAuthoring.SetBackgroundColour(panelInfo.ENTITY_AUTHOR_COLOR)
        self.notebook.AddPage(self.panelAuthoring, "Command")

        self.netVis = wx.Panel(self.notebook, -1, size=notebookSize, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
        self.netVis.SetBackgroundColour(panelInfo.ENTITY_EXEC_COLOR)
        self.notebook.AddPage(self.netVis, "Visuals")

        self.everythingSizer = wx.BoxSizer(wx.VERTICAL)
        self.everythingSizer.Add(self.notebook)

        self.SetSizer(self.everythingSizer)
        self.SetAutoLayout(1)
        self.Show(1)

        # Let's create all entities
        from cat.entity import entitySet
	##Z
        self.entitySet = entitySet.EntitySet(self.panelAuthoring, self.netVis, notebookSize)        
        entMap = self.entitySet.makeCatEntities()
        keys = entMap.keys()
        if(len(keys) == 0):
            print "No entities in Lagoon"
        else:
            entMap[keys[0]].Show()

    def getEntitySet(self):
        return self.entitySet
    
    def tick(self, time, dtime):
        pass
    
    def onSetAuthoring(self, msg):
        self.notebook.SetSelection(0)
    
    def onSetExecution(self, msg):
	    ##Z
        self.notebook.SetSelection(0)

    def pageChanged(self, event):
        print "Page changed to: " + str(event.GetEventObject().GetSelection())
        self.entitySet.handlePageChange(event)
        #nEventServer.pull(EventNames.CAT_PAGE_CHANGED, str(event.GetEventObject().GetSelection())) 
        
    def onSelectEnt(self, msg):
        args = parseRMenuMsg(msg)
        if(args.objectsSelected):
            entityPath = args.objectsSelected[0]
            entMap = self.entitySet.getEntities()
            if(entMap.has_key(entityPath)):
                self.entitySet.setSelectedEntity(entityPath) # SETs current entity and call Show()
                #entMap[entityPath].Show()
            else:
                print "No such entity: " + entityPath
            #endif
        else:
            entityPath = ""
        #endif
        print "Entity selected: " + str(entityPath)

    def setCoveClient(self, cc):
        self.coveClient = cc

entSet = {}

def getGUI(parent, size):
    pnl = CATPanel(parent, size = size)
    global entSet    
    entSet = pnl.getEntitySet()
    netSpecList = LoadEVATree(True)
    entSet.setNetSpecList(netSpecList)
    return pnl

def AttachController(entPath, task, args):
    print "+++++++++++++++CatGUI: Attaching controller: ", str(entPath), str(task), str(args)
    entSet.AttachController(entPath, task, args)

def LoadEVATree(loadable):
    import glob
    cpath = 'cat/networks/'
    files = glob.glob(cpath+'*.bn')
    netSpecList = []
    #print "Controllers:",  files
    pushsel()
    sn('nthing', '/comm/tasks')
    for pathname in files:
        netSpec = LoadNet(pathname)
        netSpecList.append((netSpec, pathname))
    popsel()
    return netSpecList

def createTypedEvaVar(type, name, val):
    if(type.startswith('bool')):
        ci(str(name), val)
    elif(type.startswith('int')):
        ci(str(name), val)
    elif(type.startswith('float')):
        cf(str(name), val)
    elif(type.startswith('string')):
        cs(str(name), val)
    elif(type.startswith('vector')):
        x, y, z = val
        cv(name, x, y, z)
    else:
        print "Cannot handle unknown type", type, name, val

def LoadNet(pathname):
    netSpec = xmlReader.slurp(pathname)
    netName = netSpec.name
    sn('nent', str(netName))
    cs('colour', 'black')
    blist = netSpec.behaviors
    for bSpec in blist:
        bid = bSpec.id
        gid = bSpec.group
        plist = bSpec.parameters
        for pSpec in plist:
            type = pSpec.type
            min  = pSpec.min #xmlReader.convert(pSpec.min, type)
            max  = pSpec.max #xmlReader.convert(pSpec.max, type)
            val  = pSpec.val #xmlReader.convert(pSpec.val, type)
            if(type.startswith('bool')):
                val = int(val)
            name = str(pSpec.name)
            sn('nent', str(gid + "_" + name))
            createTypedEvaVar(type, 'min', min)
            createTypedEvaVar(type, 'max', max)
            createTypedEvaVar(type, 'val', val)
            createTypedEvaVar('string', 'type', type)
            sel('..')
        # end for pSpec
    # end for bSpec
    sel('..')
    return netSpec   
        
    
    
