#
# @(#) catRecordPanel.py 2005/11/05
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#
'''
Defines the Network Loading, and Playback GUI and its operations for cat.

'''
import wx
from common import EventNames
import nEventServer

from common import tickmanager
from cat import panelInfo

rootlabel = "RECORDER: "
timeconstant = 0.75

from common.defs import *
class CatPlaybackPanel(wx.Panel):
    '''Behavior Network Management panel class - Execution only'''

    ID_PAUSE = 23102
    ID_SAVE = 23103
    ID_RECORD = 23104
    ID_STOP = 23105
    ID_PLAY = 23106
    ID_START = 23107
    ID_QUERY = 23108
    ID_DELETE = 23109

    def __init__(self, parent, size = None, color = "CYAN",recType = "Recorder: ", entity=None, recorder=None):
        
        wx.Panel.__init__(self, parent, -1, size, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)

        self.dtime = 0.0
        self.dots = 1
        self.entity = entity
        self.recorder = recorder
        self.defaultFilename = "test.bn"
        self.defaultNetname  = "testNetwork"
        self.isRecording     = False
        nEventServer.register(EventNames.CAT_DELETE_NETWORK, self.onDelete)

        self.SetBackgroundColour(panelInfo.ERECORDPANELCOLOR)

        ###############################################################
        # Main recorder panel, holds the toolbar and the dropdown menu
        self.recPanel = wx.Panel(self, -1, size)
        self.recPanel.SetBackgroundColour(panelInfo.ERECORDPANELCOLOR)

	self.labelM = wx.StaticText(self.recPanel, -1, recType)
	
	self.spacer1 = wx.StaticText(self.recPanel, -1, "", size= (215, 5), style = wx.ALIGN_RIGHT)
	self.spacer2 = wx.StaticText(self.recPanel, -1, "", size= (215, 90), style = wx.ALIGN_RIGHT)
	self.spacer3 = wx.StaticText(self.recPanel, -1, "", size= (215, 4), style = wx.ALIGN_RIGHT)
	self.spacerS = wx.StaticText(self.recPanel, -1, "", size= (188, 4), style = wx.ALIGN_CENTER)

	self.tb = wx.ToolBar(self.recPanel)
        self.tb.SetBackgroundColour(panelInfo.ERECORDPANELCOLOR)
	 
	import os
        prefix = os.path.dirname(__file__)+"/"
        
	del os
        # Load button bitmaps
        self.stop_bmp = wx.Bitmap(prefix+"stop.gif", wx.BITMAP_TYPE_GIF)
        self.pause_bmp = wx.Bitmap(prefix+"pause.gif", wx.BITMAP_TYPE_GIF)
        self.play_bmp = wx.Bitmap(prefix+"play.gif", wx.BITMAP_TYPE_GIF)
        self.delete_bmp = wx.Bitmap(prefix+"delete.gif", wx.BITMAP_TYPE_GIF)
        self.tb.AddSeparator()

        self.loadNetworks()
        
        # Set toolbar with button bitmaps
        self.tb.AddSimpleTool(self.ID_STOP,   self.stop_bmp,   "Stop",    "Stop Recording Behavior Network")
        self.tb.AddSimpleTool(self.ID_PAUSE,  self.pause_bmp,  "Pause",   "Pause Recording Behavior Network")
        self.tb.AddSimpleTool(self.ID_PLAY,  self.play_bmp,    "Run",     "Execute Behavior Network")
        self.tb.AddSimpleTool(self.ID_DELETE, self.delete_bmp, "Delete",  "Delete Behavior Network")


        self.tb.EnableTool(self.ID_STOP,   False)
        self.tb.EnableTool(self.ID_PAUSE,  False)
        self.tb.EnableTool(self.ID_PLAY,   False)        
        self.tb.EnableTool(self.ID_DELETE, True)

        self.tb.Realize()

        # Toolbar event handlers
        wx.EVT_TOOL(self.tb, self.ID_STOP,   self.stop)
        wx.EVT_TOOL(self.tb, self.ID_PAUSE,  self.pause)
        wx.EVT_TOOL(self.tb, self.ID_PLAY,   self.play)
        wx.EVT_TOOL(self.tb, self.ID_DELETE, self.delete)

        # Delete behavior confirmation panel ########################################################
        self.deletePanel = wx.Panel(self, -1, size)
        self.deletePanel.SetBackgroundColour("ORANGE")

        tmp = wx.BoxSizer(wx.VERTICAL)
        tmp.Add(wx.StaticText(self.deletePanel, -1, "DELETING NETWORK FILE! \n Are you sure you want to delete?", 
                    size=(220,50), style=wx.ALIGN_CENTER), 1, flag=wx.EXPAND)

        # OK and Cancel buttons
        self.confirmDelete = wx.Button(self.deletePanel, wx.ID_OK, '')
        self.cancelDelete = wx.Button(self.deletePanel, wx.ID_CANCEL, '')
        # Event handlers
        wx.EVT_BUTTON(self.cancelDelete, -1, self.onCancelDelete)
        wx.EVT_BUTTON(self.confirmDelete, -1, self.onConfirmDelete)

        # Flexible grid sizer for OK and Cancel buttons
        tmp2 = wx.FlexGridSizer(cols = 2, hgap=4, vgap=2)
        tmp2.AddMany([self.cancelDelete,self.confirmDelete])

        tmp.Add(tmp2, flag=wx.ALIGN_CENTER)
        tmp.Add(wx.StaticText(self.deletePanel, -1, ''), 1)

        # Set sizer for delete panel
        self.deletePanel.SetSizer(tmp)

	########################################################################

        self.assembleSizers()


    def assembleSizers(self):
        bsizer = wx.BoxSizer(wx.VERTICAL)
	bsizer.Add(self.tb, 0, wx.TOP|wx.ALIGN_CENTER, 2)
	bsizer.Add(self.spacerS, 0, wx.CENTER|wx.ALIGN_CENTER, 2)
	bsizer.Add(self.tb2, 0, wx.BOTTOM|wx.ALIGN_CENTER, 2)
    
	sizerE = wx.BoxSizer(wx.VERTICAL)
	sizerE.Add(self.labelM)
	sizerE.Add(self.spacer1)	
	sizerE.Add(bsizer)
	sizerE.Add(self.spacer2)
	self.recPanel.SetSizer(sizerE)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sizer.Add(self.recPanel)
        sizer.Add(self.deletePanel)
        self.deletePanel.Hide()
        self.SetSizer(sizer)
        
    def play(self, event):
        # print "catRecordPanel: playing network"+ str(event)
        netName = self.filename.GetStringSelection()
        self.entity.handlePlayRequest(netName, {})
        self.enablePlay()

    def enablePlay(self):
        self.tb.EnableTool(self.ID_STOP,   True)
        self.tb.EnableTool(self.ID_PAUSE,  True)
        self.tb.EnableTool(self.ID_PLAY,   False)
        self.tb.EnableTool(self.ID_DELETE, True)


    def stop(self, event):
        # print "stop recording/playback network"+ str(event)
        #if(self.isRecording):
        #    self.recorder.handleStopRecording()
        #    self.isRecording = False
        #    self.tb.EnableTool(self.ID_SAVE,   True)
        #    self.tb.EnableTool(self.ID_RECORD, True)                
        #else:
        netName = self.filename.GetStringSelection()
        self.entity.handleStopRequest(netName)
        self.tb.EnableTool(self.ID_STOP,   False)
        self.tb.EnableTool(self.ID_PAUSE,  False)
        self.tb.EnableTool(self.ID_PLAY,   True)
        self.tb.EnableTool(self.ID_DELETE, True)
        


    def pause(self, event):
        #print "pause recording/playback network"+ str(event)
        #if(self.isRecording):
        #    self.recorder.handlePauseRecording()
        #    self.tb.EnableTool(self.ID_PLAY,   False)
        #    self.tb.EnableTool(self.ID_RECORD, True)                
        #    self.tb.EnableTool(self.ID_SAVE,   True)
        #else:
        netName = self.filename.GetStringSelection()
        self.entity.handlePauseRequest(netName)
        self.tb.EnableTool(self.ID_PLAY,   True)
        self.tb.EnableTool(self.ID_STOP,   True)
        self.tb.EnableTool(self.ID_PAUSE,  False)
        self.tb.EnableTool(self.ID_DELETE, True)

    def onCancelDelete(self, event):
        '''
        Cancel delete button event handler.
        Hides the delete panel and shows the recorder panel.
        '''
        self.deletePanel.Hide()
        self.recPanel.Show()
        self.Layout()

    def onConfirmDelete(self, event):
        '''
        Confirm delete button event handler.
        Hides the delete panel, shows the recorder panel,
        and records the selected recorder.
        '''
        self.deletePanel.Hide()
        self.recPanel.Show()
        self.Layout()
        f = self.filename.GetStringSelection()
        nEventServer.pull(EventNames.CAT_DELETE_NETWORK, str(f))

    def onDelete(self, msg):
        '''
        CAT_DELETE_NETWORK event handler.
        '''
        f = unicode(msg)
        filenames = self.entity.getFilenames()
        print filenames
        filename = filenames[f]
        import os
        try:
            os.remove(filename)
            filenames.pop(filename)
            nlist = self.entity.getNetSpecDict()
            nlist.pop(f)
            self.updateNets()
        except:
            printExcept("On deleting")

    def delete(self, event):
        '''
        Delete from toolbar event handler.
        Hides the recorder panel and shows the confirm
        delete panel.
        '''
        self.deletePanel.Show()
        self.recPanel.Hide()
        self.Layout()

    def getPanel(self):
        return self

    def show(self, data = {}):
        self.Show()
        if data.get('state') == 'ok':
            myfilename = data['filename']
            netname = data['netname']
            if not myfilename.endswith('.bn'):
                myfilename = myfilename + '.bn'
            #self.recorder.setNetInfo(myfilename, netname)
            self.recorder.saveNetwork(myfilename, netname)
            self.recorder.updateNetDict(myfilename, netname)
        else:
            #print "Cannot handle state: %s, with data: %s" % (data.get('state'), str(data))                            #chris
            pass

    def updateNets(self):
        self.loadNetworks()
        self.assembleSizers()
        self.Layout()

    def loadNetworks(self, msg=None):
        self.tb2 = wx.ToolBar(self.recPanel)
        self.filename = wx.Choice(self.tb2, -1, size=(180, -1))
        wx.EVT_CHOICE(self.filename, -1, self.OnSelect)
        self.tb2.AddControl(self.filename)
        self.tb2.Realize()

        nets = ["< Select Network >"]
        nlist = self.entity.getNetSpecDict()
        nets.extend(nlist.keys())
        self.filename.Clear()
        self.filename.AppendItems(nets)
        self.filename.SetSelection(0)

    def OnSelect(self, event):
        f = self.filename.GetStringSelection()
        #if(f.find('new behavior') >= 0):
        #    self.stop(event)
        #    self.tb.EnableTool(self.ID_PLAY,   False)
        #    self.tb.EnableTool(self.ID_DELETE, False)
        #    self.tb.EnableTool(self.ID_SAVE,   False)            
        #else:
        print "CatRecordPanel: Selected:", f
        self.tb.EnableTool(self.ID_STOP,   True)
        self.tb.EnableTool(self.ID_PAUSE,  False)
        self.tb.EnableTool(self.ID_PLAY,   True)        
        self.tb.EnableTool(self.ID_DELETE, True)
        #self.recorder.loadNetwork('/cat/networks/'+f)
        #self.play(event)
        netName = self.filename.GetStringSelection()
        self.entity.handleInitRequest(netName, {})

    def updatePanel(self, cname):
        nlist = self.entity.getNetSpecDict()
        #print "Nets: ", str(nets)
        names = nlist.keys()
        if(cname in names):
            n = names.index(cname)
            self.filename.SetSelection(n + 1)
            self.enablePlay()

    def updatePanels(self, cname):
        self.updatePanel(cname)
