#
# @(#) catTargetPanel.py 2005/10/20
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#TODO: Fix Header
#TODO: Look into horizontal layout
#TODO: Set up recorder
#TODO: Tighten interface
'''
Defines the GUI and its operations for cat.

'''
import wx
#import wx.lib.fancytext as fancytext
#import common.WindowManager as WindowManager
from common.defs import *
from common.widgets.drawline import *
import math
import nEventServer

from common import EventNames

PANELSIZE = (215,300)
TIME_STEP = 1

# Line status
NO_LINE = 0
LINE_ACTIVE = 1
LINE_HIDDEN = 2

class CatTargetPanel(wx.Panel):
    
    '''Entity panel class'''
    
    def __init__(self, parent, size=None, color = "CYAN", entity=None, behavior=None):
	
        wx.Panel.__init__(self, parent, -1, size=size, style = wx.RAISED_BORDER | wx.DOUBLE_BORDER)
	self.entity = entity
        self.behavior = behavior
	self.bgcolor 	= parent.GetBackgroundColour()
	
	spacer1 = wx.StaticText(self, -1, "", size= (215, 2), style = wx.ALIGN_RIGHT)
	spacer2 = wx.StaticText(self, -1, "", size= (215, 2), style = wx.ALIGN_RIGHT)
	
	l1 		= wx.StaticText (self, -1,"TARGET:   ")
	self.t1 	= wx.TextCtrl   (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l2 		= wx.StaticText (self, -1," Speed:  ")
	self.t2 	= wx.TextCtrl   (self, -1, "0.0",  size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l3 		= wx.StaticText (self, -1," Heading:   ")
	self.t3 	= wx.TextCtrl   (self, -1, "0",    size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l6 		= wx.StaticText (self, -1," Bearing:   ")
	self.t6 	= wx.TextCtrl   (self, -1, "0",    size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l4 		= wx.StaticText (self, -1," Range:   ")
	self.t4 	= wx.TextCtrl   (self, -1, "0",    size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	l5 		= wx.StaticText (self, -1," Position (X,Y):   ")
	self.t5 	= wx.TextCtrl   (self, -1, "00000, 00000", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)

	space = 2
	sizer = wx.FlexGridSizer (cols = 2, hgap = space, vgap = space)
	sizer.AddMany ([l1, self.t1, l2, self.t2, l3, self.t3, l4, self.t4, l6, self.t6, l5, self.t5])
	
	sizerTarget = wx.BoxSizer(wx.VERTICAL)

	sizerTarget.Add(spacer1)
	sizerTarget.Add (sizer)
        sizerTarget.Add(spacer2)
	self.SetSizer(sizerTarget)

        self.t1.SetValue('none')
        self.setTarget(None)
        self.t2.SetValue('0.0')
        self.t3.SetValue('0.0')
        self.t6.SetValue('0')
        self.t4.SetValue('0')
        self.t5.SetValue('00000, 00000')
        self.oldTime = 0

        from common import tickmanager
        tickmanager.register( 'TargetPanel'+str(self.bgcolor)+str(self.behavior.getName()), self.Tick )

        self.prevTargetPath=None
        self.triggerCount=0
        self.lineStatus=NO_LINE
        self.prevLineName=None
        self.lineName=None

	
    def getPanel(self):
        return self

    def setTarget(self, targetPath):
        self.behavior.setTarget(targetPath)

    def getTarget(self):
        return self.behavior.getTarget()

    
    def Tick(self, time, dtime):
        if(time - self.oldTime < TIME_STEP):
            return
        self.oldtime = time

        targetPath = self.getTarget()
        if(not targetPath):
            return 
        target = targetPath.split('/')[-1]
        if(not exists(targetPath)):
            return

        # Line drawing stuff ###################################
        if targetPath != self.prevTargetPath:
                # Draw new line if target has changed
                self.lineName = drwTargetLine('trig'+`self.triggerCount`+ ' ' + self.entity.getEVAPath()+' '+targetPath, self.behavior.getName())
                # If drawing target line failed
                if not self.lineName:
                    return

                self.triggerCount += 1
                self.lineStatus=LINE_ACTIVE

                # Delete old line
                if self.prevLineName:
                    deleteTargetLine(self.prevLineName)

                self.prevTargetPath = targetPath
                self.prevLineName = self.lineName

        # Hide line vis node if target panel is not active
        if not self.IsShown():
             # if line node has been hidden once, then return
            if self.lineStatus == LINE_HIDDEN:
                return

            visnode = call('%s.getvis' % self.lineName)
            call('%s.setactive' % visnode, False)
            self.lineStatus = LINE_HIDDEN
        else:
            if self.lineStatus == LINE_HIDDEN:
                visnode = call('%s.getvis' % self.lineName)
                call('%s.setactive' % visnode, True)
                self.lineStatus = LINE_ACTIVE
        #######################################################

        pushsel(targetPath)

        # Set target name text field
        self.t1.SetValue(target)
            
        # Set speed text field
        val = gf("speed")
        s = '%5.1f' % val
        self.t2.SetValue(s)
	    
        # Set position text field
        ox, oz, oy = gv('pos')
        self.t5.SetValue(`int(ox)`+', '+`int(oy)`)
            
        # Set heading text field
        hd = gf("heading")
        val = math.degrees(hd)
        if val < 0:
            val += math.pi * 2
        s = '%5.1f' % val
        self.t3.SetValue(s)

        popsel()


        pushsel(self.entity.getEVAPath())
        tx, tz, ty = gv('pos')
        hd = gf('heading')
        popsel()
        
        range = math.sqrt((ox-tx)**2 + (oy-ty)**2)
        self.t4.SetValue('%.1f'%range)

        dy = abs(oy-ty)
        dx = abs(ox-tx)

        bear1 = math.atan2(oy-ty, ox-tx) - math.pi/2
        
        if bear1 < -math.pi:
            bear1 += math.pi*2
        bear2 = bear1 + hd
        if bear2 < -math.pi:
            bear2 += math.pi*2
        if bear2 < 0:
            sign="L"
        else:
            sign="R"
        valb = abs(math.degrees(bear2))
        s = '%5.1f ' % valb
        self.t6.SetValue(s+sign)
