#
# @(#) triggerInfoPanel.py 2006/3/8
# @author Copyright (c) 2006 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

'''
Defines GUI for creating triggers
'''

import wx
from common.defs import *
from cat import catInfo

import triggerPanelBase
CREATEID = 100
CANCELID = 100

class TriggerInfoPanel(triggerPanelBase.TriggerPanelBase):
    '''Panel for creating Triggers'''

    def __init__(self, parent, size=None, color='CYAN', entity=None):
        triggerPanelBase.TriggerPanelBase.__init__(self, parent, size, color, entity)

        self.trigtypes = ['Alert', 'Distance', 'Damage']
        self.subtypes = [catInfo.LESS_THAN, catInfo.GREATER_THAN, catInfo.APPROX]
        
	sizer = wx.BoxSizer(wx.VERTICAL)
        
        title = wx.StaticText(self, -1, "TRIGGER INFO")
        
	sizer.Add(title)

	l1 	= wx.StaticText (self, -1, "   Name:     ")
	self.t1 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	
	l2 	= wx.StaticText (self, -1,"   Type:     ")
	self.t2 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
		
	l3 	= wx.StaticText (self, -1, "   Subtype:   ")
	self.l3 = l3
	self.t3 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	
        # Need a reference to the From and To labels in order to hide/show them
	l4 = wx.StaticText (self, -1,"   To:   ")
	self.l4 = l4
	self.t4 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	
	l5 = wx.StaticText (self, -1, "   From:   ")
	self.l5 = l5
	self.t5 = wx.TextCtrl (self, -1, "none", size = (100, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
    
	#l6	= wx.StaticText (self, -1,"   Value: ")
	#self.t6 = wx.TextCtrl (self, -1, "none", size = (130, 20), style = wx.TE_READONLY | wx.TE_RIGHT)
	
	space 	= 2

	sizerA 	= wx.FlexGridSizer (cols = 2, hgap = 30, vgap = space)
	sizerA.AddMany ([l1, self.t1, l2, self.t2, l3, self.t3, l4, self.t4, l5, self.t5])
	
	self.okButton = wx.Button(self, -1, "  OK  ", style=wx.BU_EXACTFIT)
	wx.EVT_BUTTON(self.okButton, -1, self.OnOKClick)

# 	sizerB 	= wx.FlexGridSizer (cols = 1, hgap = space, vgap = space)
# 	sizerB.AddMany ([self.okButton])
	
	
	spacer1 = wx.StaticText(self, -1, "", size= (215, 2), style = wx.ALIGN_RIGHT)
	spacer2 = wx.StaticText(self, -1, "", size= (215, 2), style = wx.ALIGN_RIGHT)

	sizerInfo = wx.BoxSizer(wx.VERTICAL)
	sizerInfo.Add(spacer1)	
	sizerInfo.Add(sizer)
        sizerInfo.Add(sizerA)
	sizerInfo.Add(spacer2)
	sizerInfo.Add(self.okButton, 0, wx.ALIGN_CENTER|wx.ALIGN_BOTTOM)
	
	self.SetSizer(sizerInfo)
        sizerInfo.Layout()
        
    def OnOKClick(self, event):
	self.GetParent().show("main")

    def show(self, name=''):
        trigger = self.entity.getTrigger(name)
        if trigger:
            self.t1.SetValue(trigger.getName())
            self.t2.SetValue(trigger.getType())

            # alert sensor
            if trigger.getType() == 'AlertSensor':
                self.l3.Hide()
                self.t3.Hide()
                self.l4.Hide()
                self.t4.Hide()
                self.l5.Hide()
                self.t5.Hide()
                self.GetSizer().Layout()
            else:
                self.t3.SetValue(trigger.getSubtype())
                getTo = trigger.getTo()
                # Set entity name only
                self.t4.SetValue(getTo.split('/')[-1])

                self.l3.Show()
                self.t3.Show()
                self.l4.Show()
                self.t4.Show()

                # damage sensor
                if trigger.getType() == 'DamageSensor':
                    self.l5.Hide()
                    self.t5.Hide()
                # distance sensor
                else:
                    getFrom = trigger.getFrom()
                    # Set entity name only
                    self.t5.SetValue(getFrom.split('/')[-1])

                    self.l5.Show()
                    self.t5.Show()

                self.GetSizer().Layout()

        else:
            self.t1.SetValue('none')
            self.t2.SetValue('none')
            self.t3.SetValue('none')
            self.t4.SetValue('none')
            self.t5.SetValue('none')
        self.Show()
