#
# @(#) triggerAuthorPanel.py 2006/3/7
# @author Copyright (c) 2006 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
'''
Defines Gui for trigger authoring in cat
'''
import wx
#import wx.lib.fancytext as fancytext
#import common.WindowManager as WindowManager
from common.defs import *
import triggerPanelBase

from cat.triggers.trigger import Trigger
from cat import catInfo

CREATEID = wx.NewId()
DELETEID = wx.NewId()
INFOID = wx.NewId()
FIREID = wx.NewId()
LISTBOXID = wx.NewId()

CHECKLISTSIZE = (210,100)
TIME_STEP = 0.5


class TriggerMainPanel(triggerPanelBase.TriggerPanelBase):
    '''
    Trigger Panel for authoring (creating/deleting) and firing triggers
    '''
    def __init__(self, parent, size=None, color = "CYAN", entity=None):
        triggerPanelBase.TriggerPanelBase.__init__(self, parent, size=size, color=color, entity=entity)


        sbox = wx.StaticBox(self, -1, "Triggers", size= (size[0]-10, 20))
        self.sizer = wx.StaticBoxSizer(sbox, wx.VERTICAL)

        self.oldTriggerList = self.triggerList = []

        # Set trigger check list box
        self.triggerListBox = wx.CheckListBox(self, LISTBOXID, size=CHECKLISTSIZE, choices=self.triggerList)
        self.sizer.Add(self.triggerListBox)

	self.createButton = wx.Button(self, CREATEID, "  New  ", style = wx.BU_EXACTFIT)
        self.deleteButton = wx.Button(self, DELETEID, " Delete ", style = wx.BU_EXACTFIT)
        self.createButton.SetBackgroundColour(color)
        self.deleteButton.SetBackgroundColour(color)        

        self.mode = self.entity.getMode()
        # Disable New and Delete buttons if in exec mode
        if not self.entity.isAuthorMode():
            self.createButton.Enable(False)
            self.deleteButton.Enable(False)

        self.infoButton = wx.Button(self, INFOID, "   Info  ", style = wx.BU_EXACTFIT)
        self.fireButton = wx.Button(self, FIREID, "  Fire  ", style = wx.BU_EXACTFIT)

        # Set event handlers for the buttons
        wx.EVT_BUTTON(self.createButton, CREATEID, self.createTrigger)
        wx.EVT_BUTTON(self.infoButton, INFOID, self.infoTrigger)
        wx.EVT_BUTTON(self.deleteButton, DELETEID, self.deleteTrigger)
        wx.EVT_BUTTON(self.fireButton, FIREID, self.fireTrigger)

        # Set layout of buttons
        self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        self.buttonSizer.Add(self.createButton)
        self.buttonSizer.Add(self.infoButton)
        self.buttonSizer.Add(self.deleteButton, flag=wx.ALIGN_RIGHT)
        self.buttonSizer.Add(self.fireButton, flag=wx.ALIGN_RIGHT)

        self.sizer.Add(self.buttonSizer, 0)
        self.buttonSizer.Layout()
        self.sizer.Layout()
        self.SetSizer(self.sizer)

        self.oldtime = 0
        # Register tick function
        from common import tickmanager
        tickmanager.register( 'TriggerPanel'+str(self.entity.getName()), self.Tick )
	

    def Tick(self, time, dtime):
        '''
        Updates list of triggers for the entity.
        '''
        if (time - self.oldtime) < TIME_STEP:
            return

        self.oldtime = time

        self.triggerList = self.entity.getTriggers()

        size = len(self.triggerList)
        # Check if trigger list has changed
        #if self.triggerList != self.oldTriggerList:
        #    self.oldTriggerList = self.triggerList
            # if triggers have changed, then clear the list and repopulate
        selection = self.triggerListBox.GetSelection()
        self.triggerListBox.Clear()
        for i in range(size):
            self.triggerList[i].tick(time, dtime)
            if(self.triggerList[i].getValue()):
                self.triggerListBox.Insert('%s - %s' % (self.triggerList[i].getName(),str(self.triggerList[i].getValue())),i)

            # Add checkmarks to triggers that have been fired
        for i in range(size):
            if  self.triggerList[i].hasFired():
                self.triggerListBox.Check(i, True)
        self.triggerListBox.SetSelection(selection)


        # Check entity mode auth/exec to disable/enable buttons
        if self.mode != self.entity.getMode():
            # If auth mode enable all buttons
            if self.entity.isAuthorMode():
                self.createButton.Enable(True)
                self.deleteButton.Enable(True)
            else:
             # If exec mode disable new and delete
                self.createButton.Enable(False)
                self.deleteButton.Enable(False)
            self.mode=self.entity.getMode()


    def createTrigger(self, event):
        '''
        Calls create trigger dialog.
        '''
        self.GetParent().show('new', 'trigger name')

    def infoTrigger(self, event):
        '''
        Calls trigger info dialog.
        '''
        index = self.triggerListBox.GetSelection()
        if  index != wx.NOT_FOUND:
                self.GetParent().show('info', self.triggerList[index].getName())


    def deleteTrigger(self, event):
        '''
        Calls delete trigger dialog.
        '''
        index = self.triggerListBox.GetSelection()
        if  index != wx.NOT_FOUND:
                self.GetParent().show('delete', self.triggerList[index].getName())

    def fireTrigger(self, event):
        '''
        Fires a trigger.  The trigger on the list is added a checkmark.
        '''
        index = self.triggerListBox.GetSelection()
        if  index != wx.NOT_FOUND:
            if not self.triggerList[index].hasFired():
                self.triggerList[index].fire()
                self.triggerListBox.Check(index,True)

    def show(self, passthru=''):
        self.Show()
