#
# @(#) entity.py 2005/09/29
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
import wx
import wx.lib.fancytext as fancytext
from common.log import Log
log = Log(tags=['entity'])
from common.defs import *
from common import behaviorNames
from cat.entity import actuator
from cat.sensors import sensor
from cat import panelInfo

from cat.Interface import catButtonPanel
from cat.Interface import catSwitchRecordWrapper
from cat.Interface import catTriggerWrapper

from cat.xml import xmlReader

from cat.Controllers import controller
from cat.Interface import catEntityPanel
from common.displayer import Displayer
import waypoint

class Entity:
    """An entity or unit in the system"""
    def __init__(self, entPath, panelA, panelV, size=None, netSpecList = None):
        self.authorPanel  = panelA 	# main panel (will change name to command")
        self.netVisPanel  = panelV

        self.netSpecList = netSpecList
        self.entPath     = entPath
        self.EVAPath     = entPath
        self.name        = entPath.split('/')[-1]
        self.actuators   = actuator.Actuators(self)
        self.sensors     = sensor.Sensors(self)
        self.track       = []
        self.displayer   = Displayer()
        
        self.currentControllerName = None
        self.playback              = False
        self.controllerDict        = {}
        self.netSpecDict           = {}
        self.filenames             = {}
        self.timeToCompletion      = 3601 # one hour = 3600 secs
        self.safety                = 100.0
        self.objective             = 50.0
        self.debug                 = False
        self.moveLoc               = None
        self.currentTarget         = None

        from cat import panelInfo
        x, y = size
        eSize = panelInfo.ENTITYPANEL
        cSize = panelInfo.BEHAVIORCONTROLPANEL
        wSize = panelInfo.CONTROLWRAPPERPANEL
        rSize = panelInfo.RECORDPANEL

        ## panels
        self.entityPanelA = self.initEntityPanel(self.authorPanel, size=eSize)

        # target panel is with primitive behavior
        self.sbcDict = {}

        self.triggerDict = {}

        from cat.Interface import catControlPanel
        self.contAuthPanel = catControlPanel.ControlPanel(self.authorPanel, size=cSize, color=panelInfo.CONTROL_AUTHOR_COLOR, entity=self, wsize=wSize)
	
        # Create recorder wrapper to switch between auth/exec
        self.switchRecWrapper = catSwitchRecordWrapper.CatSwitchRecordWrapper(self.authorPanel, size=size, color=panelInfo.CONTROL_AUTHOR_COLOR, entity=self)
        import recorder
        self.recorder = recorder.Recorder(self.switchRecWrapper, size=rSize, entity=self)
        self.authRecordPanel, self.execRecordPanel = self.recorder.getPanels()
        
        # Add record panels to wrapper
        self.switchRecWrapper.add(self.authRecordPanel, "Auth")
        self.switchRecWrapper.add(self.execRecordPanel, "Exec")

        ## controllers
        self.initControllers()
        self.recorder.updateNets()

	self.sbcDict     = self.initPrimitiveBehaviorControllers(self.authorPanel, size=wSize)
        self.currentPrimitiveBehaviorName = behaviorNames.EMPTY
        self.mode = panelInfo.EXECUTION_MODE
        
        self.aSizer = wx.BoxSizer(wx.VERTICAL)

        # Create button panel for exec/auth switching and add it to command panel
        self.buttonPanel = catButtonPanel.CatButtonPanel(self.authorPanel, color="CYAN", size=eSize, entity=self)
        self.aSizer.Add(self.buttonPanel, 0)

        self.aSizer.Add(self.entityPanelA, 0)
        self.aSizer.Add(self.contAuthPanel, 0, flag=wx.FIXED_MINSIZE)

        # Triggers panel
        self.triggerPanel = catTriggerWrapper.CatTriggerWrapper(self.authorPanel, color=panelInfo.TRIGGERPANELCOLOR, size=panelInfo.TRIGGERPANEL, entity=self)

        self.aSizer.Add(self.triggerPanel)
        
        # Make exec recorder active
        self.switchRecWrapper.show("Exec")
        # Keep a reference of recorder wrapper in the button panel class
        self.buttonPanel.SetRecorderPanels(self.switchRecWrapper)
	self.aSizer.Add(self.switchRecWrapper,0)
	
        self.authorPanel.SetSizer(self.aSizer)
        self.authorPanel.Layout()

        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
	
        if(bc):
            bc.Show()
        self.authorPanel.Show()

        from cat import netvis
        self.visualizer = netvis.initialize(self.netVisPanel,self)


        #print "Finished Making Entity", self.getName()

        call('/sys/servers/audio.play', 'entityCreated')

    #-------------------------------------------------------------------------
    # Waypoint functionality
    #-------------------------------------------------------------------------
    def addWaypoint(self, wp, index):
        self.track.insert(index, wp)

    def appendWaypoint(self, wp):
        print "Appending waypoint: ", str(wp)
        self.track.append(wp)

    def getWaypoint(self, index):
        return self.track[index]

    def getTrack(self):
        return self.track

    #-------------------------------------------------------------------
    # This is called from cat/Interface/catEntityPanel every tick.
    # Before calling this method, we always call clearWaypoints (below)
    #-------------------------------------------------------------------
    def displayWaypoints(self):
        self.displayer.clear()
        pushsel(self.entPath)
        pos = gv('pos')
        popsel()
        wp0 = waypoint.Waypoint(lPos = pos)
        for i in range(len(self.track)):
            wp1 = self.track[i]
            self.displayer.addLine(wp0.lagoonPos, wp1.lagoonPos)
            wp0 = wp1
            #displayWaypoint(wp0)            

    def displayWaypoint(self, wp):
        print "Displaying waypoint: ", str(wp)
        self.displayer.addLine(wp.lagoonPos, (wp.lagoonPos[0], wp.lagoonPos[1] + 100, wp.lagoonPos[2]))

    def clearWaypoints(self):
        self.displayer.clear()
        
    #-------------------------------------------------------------------------
    # Turn debug on/off
    #-------------------------------------------------------------------------

    def setDebug(self):
        if(self.debug):
            self.debug = False
        else:
            self.debug = True

    def isDebug(self):
        return self.debug

    #-------------------------------------------------------------------------
    # Current target
    #-------------------------------------------------------------------------

    def setCurrentTarget(self, target):
        self.currentTarget  = target

    def getCurrentTarget(self):
        return self.currentTarget

    #-------------------------------------------------------------------------

    def getMaxSpeed(self):
        pushsel(self.getEVAPath())
        if (varexists(self.getEVAPath(), 'maxSpeed', 'float')):
            mspeed = float(gf('maxSpeed'))
        else:
            print "Cannot calculate maxSpeed at this point in time"
            mspeed = 5.0
        popsel()
        return mspeed

    def setTTC(self, val):
        self.timeToCompletion = val

    def getTTC(self):
        return self.timeToCompletion

    def getMode(self):
        return self.mode

    def getSafety(self):
        return self.safety/100.0

    def getObjective(self):
        return self.objective

    def setSafety(self, safety):
        self.safety = safety

    def setObjective(self, objective):
        self.objective = objective

    def isAuthorMode(self):
        return (self.mode == panelInfo.AUTHORING_MODE)

    def setMode(self, mode):
        self.mode = mode

    def isAvoidActive(self):
        if (self.playback):
            cntlr = self.getController(self.currentControllerName)
            if (cntlr):
                if(cntlr.hasAvoidActive()):
                    return True
            return False
        else:
            avoidLand = self.getPrimitiveBehaviorController(behaviorNames.AVOIDLAND)
            avoidEntity = self.getPrimitiveBehaviorController(behaviorNames.AVOIDENTITY)
            return (avoidEntity.getBehavior().runnable or avoidLand.getBehavior().runnable )

    #----------------------------------------------------------------------------
    # Not used?
    #----------------------------------------------------------------------------
    def handleAvoidTarget(self, tp):
        if(self.currentControllerName):
            cntlr = self.getController(self.currentControllerName)
            if (cntlr):
                for id, behavior in cntlr.getBehaviorNet().iteritems():
                    if(id.startswith("AVOID")):
                        behavior.setTarget(tp)

    def initControllers(self):
        if(self.netSpecList):
            for netSpec, filepath in self.netSpecList:
                self.makeNetSpecDict(netSpec, filepath)                
        else:
            import glob
            cpath = 'cat/networks/'
            files = glob.glob(cpath+'*.bn')
            for pathname in files:
                self.loadNetSpecDict(pathname)                
        return

    def makeNetSpecDict(self, netSpec, filename):
        self.netSpecDict[netSpec.name] = netSpec
        self.filenames[netSpec.name] = filename

    def loadNetSpecDict(self, pathname):
        try:
            fname = pathname.split('/')[-1]
            netSpec = xmlReader.slurp(pathname)
            self.makeNetSpecDict(netSpec, pathname)
        except:
            printExcept("LoadNetSpecDict: Cannot load file: %s" % pathname)

    def getNetSpecDict(self):
        return self.netSpecDict

    def makeController(self, netSpec = None, pathname = None):
        try:
            controlNet = controller.Controller(self.authorPanel, entity=self, filename = pathname, spec = netSpec)
            self.controllerDict[controlNet.getNetName()] = controlNet
        except:
            printExcept("makeController: Cannot make Controller: %s" % netSpec.name)
        print "Done making network : ", netSpec.name

    def loadController(self, pathname):
        try:
            fname = pathname.split('/')[-1]
            netSpec    = xmlReader.slurp(pathname)
            self.makeController(netSpec = netSpec, filename = pathname)
        except:
            printExcept("Cannot load file: %s" % pathname)
        

    def getNetList(self):
        return [self.controllerDict, {}]

    def getFilenames(self):
        return self.filenames

    def getControllerPanels(self):
        return self.contAuthPanel

    def getName(self):
        return self.name

    def getActuators(self):
        return self.actuators

    def getSensors(self):
        return self.sensors

    def setBehaviorSizer(self, sizer):
        self.behaviorSizer = sizer

    #------------------------------------------------------------
    # trigger methods
    #------------------------------------------------------------

    def handleTrigger(self, triggerSpec):
        for key, val in self.sbcDict.iteritems():
            val.triggerStop(triggerSpec)

    #------------------------------------------------------------

    def addTrigger(self, trigger):
        self.triggerDict[trigger.getName()] = trigger

    #------------------------------------------------------------

    def addTriggers(self, triggerlist):
        for trigger in triggerlist:
            self.addTrigger(trigger)
    #------------------------------------------------------------

    def deleteTrigger(self, triggerName):
        if self.triggerDict.has_key(triggerName):
            trigger = self.triggerDict.pop(triggerName)
            trigger.delete()

    #------------------------------------------------------------
            
    def clearTriggers(self):
        for key, trig in self.triggerDict.iteritems():
            trig.delete()
        self.triggerDict.clear()

    #------------------------------------------------------------        

    def getTriggers(self):
        return self.triggerDict.values()

    #------------------------------------------------------------

    def getTrigger(self, triggerName):
        return self.triggerDict.get(triggerName)

    #------------------------------------------------------------
    # Side effects self.triggerDict
    #------------------------------------------------------------
    def updateTriggers(self, oldTargetPath, newTargetPath):
        from cat import catInfo
        for key, trig in self.triggerDict.iteritems():
            if (trig.getType() == catInfo.DISTANCE_SENSOR):
                self.updateDistanceTrigger(trig, oldTargetPath, newTargetPath)
            elif (trig.getType() == catInfo.DAMAGE_SENSOR):
                self.updateDamageTrigger(trig, oldTargetPath, newTargetPath)

    #------------------------------------------------------------        
    # In place update/change of trigger
    # -----------------------------------------------------------
    def updateDistanceTrigger(self, trig, oldPath, newPath):
        if(self.playback):
            trig.handleBindingChange(oldPath, newPath)
        
    #------------------------------------------------------------        
    # In place update/change of trigger
    # -----------------------------------------------------------

    def updateDamageTrigger(self, trig, oldPath, newPath):
        if(self.playback):
            pass
        
    #------------------------------------------------------------        

    def delete(self):
        self.entityPanelA.Destroy()
        self.contAuthPanel.Destroy()

    def Show(self):
        """Show all panels associated with Entity"""
        self.authorPanel.Show()
        self.entityPanelA.Show()
        self.contAuthPanel.Show()
        self.visualizer.Show()
        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
        if(bc):
            bc.Show()
        self.displayController(self.currentControllerName)

        self.switchRecWrapper.Show()
        self.buttonPanel.Show()
        self.triggerPanel.Show()

        #if(self.isDebug()):
        #self.displayWaypoints()


    def Hide(self):
        """Hide all information associated with Entity"""
        self.authorPanel.Hide()
        self.entityPanelA.Hide()
        self.contAuthPanel.Hide()
        self.visualizer.Hide()
        self.hideAllPBCs()
        self.hideControllers()

        self.buttonPanel.Hide()
        self.switchRecWrapper.Hide()
        self.triggerPanel.Hide()

        self.displayer.clear()

        
    # Stop all primitive behaviors
    def stopAll(self, event):
        for key, val in self.sbcDict.iteritems():
            val.stop()
            #val.Hide()

        for key, val in self.controllerDict.iteritems():
            val.stop()


    def stopRelated(self, bname):
        # fix for networks
        if (not behaviorNames.RELATED.has_key(bname)):
            return
        related = behaviorNames.RELATED[bname]
        for name in related:
            bc = self.getPrimitiveBehaviorController(name)
            if(bc):
                bc.stop()


    # creates primitive behavior controllers for this entity.
    # Allows controlling an entity from CAT's primitive behavior panels
    def initPrimitiveBehaviorControllers(self, panelA, size = None):
        from cat.Controllers import primitiveBehaviorController
        #from common import behaviorNames
        sbcDict = {}
        for sbc in behaviorNames.BEHAVIOR_NAMES:
            sbcDict[sbc] = primitiveBehaviorController.PrimitiveBehaviorController(panelA, entity=self, bname=sbc, size=size, show = True)
        return sbcDict

    def getPrimitiveBehaviorController(self, bname):
        if (self.sbcDict.has_key(bname)):
            return self.sbcDict[bname]
        else:
            return None
        
    # Show the panel associated with the behavior with name bname,
    # event handler for controlPanel behavior switching buttons
    def showBehavior(self, event):
        bname = event.GetEventObject().GetName()
        self.displayBehavior(bname)

    def displayBehavior(self, bname):
        self.currentPrimitiveBehaviorName = bname
        bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)

        self.contAuthPanel.Hide()
        self.contAuthPanel.Show()

        if (bc):
            bc.Show()
            bc.getBehavior().updatePanels()

    def isAnyBehaviorActive(self):
        for key, val in self.sbcDict.iteritems():
            if(val.behavior.runnable):
                print val.getBehaviorName(), " is active"
                return True
        return False

    # Checks if avoidee is a target of any of the entity's running behaviors
    # If yes, the desired bearing of entity w.r.t. target(avoidee) is returned
    # Used by catAvoidEntity
    # Modified by SJL to take controllers into account
    def isAvoideeMyTarget (self, avoidee):
        if (self.playback):
            cntlr = self.getController(self.currentControllerName)
            if (cntlr):
                behaviors = cntlr.getBehaviorNet().values()
        else:
            sbcs = self.sbcDict.values()
            behaviors = []
            for b in sbcs:
                behaviors.append(b.behavior)
        # now we've got a list of behaviors
        for val in behaviors:
            if val.isAvoid():
                continue
            if(val.runnable):
                #print val.getBehaviorName(), " is active"
                targetName = val.getTarget()
                #print "targetName = ", targetName, "avoideename = ", avoideeFullName
                if targetName == avoidee:
                    #print "Target and avoidee are the same"
                    return (True, val.GetDesiredBearing(val.GetTargetHeadingLagDeg()))
                else:
                    return (False, 0)

        return (False, 0)


    def hideAllPBCs(self):
        #from common import behaviorNames
        for key, val in self.sbcDict.iteritems():
            val.Hide()

    def hideControllers(self):
        cntl = self.getController(self.currentControllerName)
        if(cntl):
            cntl.hide()

    def initEntityPanel(self, panel, size = None):
        """entityPanel designed by Sergiu"""
	
        panel1 = catEntityPanel.EntityPanel(panel, size=size, entity=self)
	
        return panel1

    def getRecorder(self):
        return self.recorder

    def getEntityPanel(self):
        return self.entityPanel

    def setEntityPanel(self, ePanel):
        self.entityPanel = ePanel

    def getEVAPath(self):
        return self.EVAPath

    def setEVAPath(self, newPath):
        self.EVAPath = newPath
        
 
    def setTarget(self, targetPath):
        if(self.currentPrimitiveBehaviorName):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.setTarget(targetPath)
        else:
            print "No current behavior selected for target: ", targetPath
    
    def getTarget(self):
        if(self.currentPrimitiveBehaviorName):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            return bc.getTarget()
        else:
            return None
        
    def handleTargetSelection(self, targetPath, msg):
        if(self.playback):
            self.handleControllerTargetSelection(targetPath, msg)
            return
        if(self.currentPrimitiveBehaviorName):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.getBehavior().handleTargetSelection(targetPath, msg)
        else:
            print "handleTargetSelection: No current behavior selected for target: ", targetPath

    def getMoveLoc(self):
        return self.moveLoc

    def handleSetMovePos(self, msg):
        args     = parseRMenuMsg(msg)
        moveLoc  = args.mousePos
        self.moveLoc = moveLoc
        # Set destination vector for moveto point
        pushsel(self.getEVAPath())
        sv('destination', moveLoc)
        popsel()

        if(self.playback):
            self.handleControllerMovePositioning(moveLoc, msg)
            return
        if (self.currentPrimitiveBehaviorName):
            #from common import behaviorNames
            bc = self.getPrimitiveBehaviorController(behaviorNames.MOVETO)
            bc.getBehavior().setMoveLocation(moveLoc)
            return
        print "entity.py: handleSetMove: No MOVETO available! for move position: ", moveLoc

    def handleBehaviorChange(self, msg):
        if (self.playback):
            print "%%%%%%%%%%%%%%%%%%%%%CANNOT play behavior while controller is running", msg
            return
        args     = parseRMenuMsg(msg)
        moveLoc  = args.mousePos
        menuItem = args.menuItem
        text     = args.text
        bname    = str(menuItem)
        bc = self.getPrimitiveBehaviorController(bname)
        behavior = bc.getBehavior()
        #from common import behaviorNames
        if(bname.startswith(behaviorNames.MOVETO)):
            self.moveLoc = moveLoc
            behavior.setMoveLocation(moveLoc)
            # Set destination vector for moveto point    
            pushsel()
            sel(self.getEVAPath())
            sv('destination', moveLoc)
            popsel()
        elif (bname in behaviorNames.FUSABLE_NAMES):
            targetPath = args.objectsOver[0]
            behavior.handleTargetSelection(targetPath, msg)
        behavior.start()
        self.displayBehavior(bname)
        self.sayBehavior(bname)

    def sayBehavior(self, bname):
        if(bname == behaviorNames.MOVETO):
            call('/sys/servers/audio.play', 'moveTo')
        elif (bname == behaviorNames.MAINTAIN):
            call('/sys/servers/audio.play', 'maintain')            
        elif (bname == behaviorNames.APPROACH):
            call('/sys/servers/audio.play', 'approach')            
        elif (bname == behaviorNames.RAM):
            call('/sys/servers/audio.play', 'ram')
        elif (bname == behaviorNames.FIRE):
            call('/sys/servers/audio.play', 'fire')
            
    def updatePanels(self, mode=None):
        return
        # not needed with only command panels
        #from common import behaviorNames        
        if(not self.currentPrimitiveBehaviorName.startswith(behaviorNames.EMPTY)):
            bc = self.getPrimitiveBehaviorController(self.currentPrimitiveBehaviorName)
            bc.getBehavior().handlePageChange("Tryng hard to GOOOO")
        elif (self.playback):
            cntrllr = self.getController(self.currentControllerName)
            if (cntrllr):
                cntrllr.handlePageChange(mode)
        else:
            print "Update Panels: handlePageChange: No current behavior: "

    def stopRecording(self):
        for key, val in self.sbcDict.iteritems():
            if(val.behavior.runnable):            
                val.behavior.stopRecord()

    ##################Controller related methods######################                

    def displayController(self, cname):
        self.currentControllerName = cname
        c = self.getController(cname)
        if (c):
            c.show()
            self.execRecordPanel.updatePanels(cname)            
        else:
            print "No controller exists"

    def getController(self, cname):
        if(self.controllerDict.has_key(cname)):
            cntrllr = self.controllerDict[cname]
            return cntrllr
        elif(self.netSpecDict.has_key(cname)):
            self.makeController(netSpec = self.netSpecDict[cname])
            cntrllr = self.controllerDict[cname]
            return cntrllr
        else:
            return None
        
    def handleInitRequest(self, netName, args):
        cntl = self.getController(netName)
        print "Entity: Initializing controller: ", netName
        if (cntl):
            if (args):
                cntl.setSPArgs(args)
            self.currentControllerName = netName
            self.playback = True
            cntl.init()

    def handlePlayRequest(self, netName, args):
        self.stopAll(None)
        cntl = self.getController(netName)
        print "Entity: Playing controller:", netName
        if (cntl):
            if (args):
                cntl.setSPArgs(args)
            self.currentControllerName = netName
            cntl.init()
            self.playback = True
            self.buttonPanel.SwitchToExecution(None)
            self.execRecordPanel.updatePanel(netName)
            cntl.start()

    def handleStopRequest(self, netName):
        cntl = self.getController(netName)
        if (cntl):
            self.currentControllerName = None
            cntl.stop()
            self.playback = False

    def handlePauseRequest(self, netName):
        cntl = self.getController(netName)
        if cntl:
            cntl.pause()
            self.playback = True
            
    def handleControllerTargetSelection(self, targetPath, msg): 
        cntl = self.getController(self.currentControllerName)
        if (cntl):
            cntl.handleTargetSelection(targetPath, msg)
        else:
            print "Cannot select target, no current controller selected"

    def handleControllerMovePositioning(self, moveLoc, msg): 
        cntl = self.getController(self.currentControllerName)
        bn = cntl.getBehaviorNet()
        if(bn):
            for id, behavior in bn.iteritems():
                if(behavior.getName().startswith(behaviorNames.MOVETO)):
                    behavior.setMoveLocation(moveLoc)

