#
# @(#) entitySet.py 2005/09/29
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
from common.log import Log
log = Log(tags=['newcat'])
from common.defs import *
import nEventServer
from common import EventNames
from common import behaviorNames
from cat.xml.behaviorSpec import *
from cat.xml.xmlWriter import *
from cat import Controllers

import waypoint


class EntitySet:
    """A set of unique entities in the system"""
    def __init__(self, panelA, panelV, size = None):
        self.authorPanel      = panelA
        self.netVisPanel      = panelV
        self.size = size
        self.netSpecList = []
        self.entityList  = []
        self.entityDict  = {}
        pushsel("/game/ents/")
        for entType in ndir():
            sel(entType)
            for ent in ndir():
                if hasvar(ent, 'selectable') and call('%s.gi' % ent, 'selectable'):
                    entPath = "/game/ents/" + entType + "/" + ent
                    self.entityList.append(entPath)
            sel("..")
        print("EntityList: " + str(self.entityList))
        self.currentEntPath = None
        popsel()

        nEventServer.register(EventNames.SELECTION_CHANGED,       self.leftSelectEnt)
        #nEventServer.register(EventNames.RMENU_DELETE_ENT,        self.handleDeletion) #chris
        nEventServer.register(EventNames.ENTITY_CREATED,          self.handleEntCreation)
        nEventServer.register(EventNames.ENTITY_DESTROYED,        self.handleEntDestruction) #chris
        nEventServer.register(EventNames.SP_CREATE_ENT,           self.handleSPEntCreation)
        nEventServer.register(EventNames.RMENU_CAT_SELECT_TARGET, self.handleTargetSelection)
        nEventServer.register(EventNames.RMENU_BEHAVIOR_CHANGE,   self.handleBehaviorChange)
        nEventServer.register(EventNames.RMENU_ASSIGN_OBJECTIVE,  self.handleObjectiveChange)
        nEventServer.register(EventNames.RMENU_SET_MOVE_POS,      self.handleSetMovePos)           
        #nEventServer.register(EventNames.RMENU_ROUTE,             self.handleRoute)           
        nEventServer.register(EventNames.RMENU_ADD_WAYPOINT,      self.handleWaypoint)           
        nEventServer.register(EventNames.RMENU_UNIT_DEBUG,        self.handleDebug)           
        nEventServer.register(EventNames.CAT_DELETE_NETWORK,      self.onDelete)
        nEventServer.register(EventNames.RMENU_PLOT_COURSE_NET,   self.handleCreateMoveToNetwork)
        self.pulledStrings = []

    def makeBehaviorSpec(self, bname, nspec):
        #make a bid
        bid = nspec.makeNewBid(bname)
        entity = self.entityDict[nspec.ownship]

        #make a pbc to get default params out of
        panelA = entity.authorPanel
        pbc = Controllers.primitiveBehaviorController.PrimitiveBehaviorController(None,entity,bname=bname,bid=bid)
        behavior = pbc.behavior
        #bid = behavior.bname

        #init some BehaviorSpec attributes
        sensors = BehaviorSensorSpec([],[])
        params = []

        #copy pbc params
        for paramName in behavior.params.keys():
            val, typ, min, max = behavior.params[paramName]
            params.append(Parameter(paramName,typ,val,min,max))

        #make BehaviorSpec
        bspec = BehaviorSpec(bname,bid,"1",[],sensors,params,[],[])

        return bspec

        
    def handleCreateMoveToNetwork(self,msg):
        pushsel()
        info = parseRMenuMsg(msg)
        entity = self.entityDict[info.objectsSelected[0]]
        #first, select the entity for plotting
        sel(info.objectsSelected[0])

        #then, get it's position
        x0,y0,z0 = gv('pos')
        #then, get the mouse's position
        x1,y1,z1 = info.mousePos
        #then,select the A*system
        sel('/game/sides/blue/allocatorastar')
        #tell A* to create the path
        call('.findpath',x0,y0,z0,x1,y1,z1)
        #determine the # of nodes in the path
        pathlength = call('getpathlength')

        nSpec = NetworkSpec("astarnetwork",info.objectsSelected[0],[],[],[])
        nodes = []
        print 'pathlength:',pathlength
        #next get each node in the path
        for i in range(pathlength):
            node = call('.getpathnode',i)
            #create the behaviorspec and add it to the netspec
            nSpec.behaviors.append(self.makeBehaviorSpec(behaviorNames.MOVETO,nSpec))
            #set the desired x and y for the bspec (and maneuverchanged, but I don't know why)
            nSpec.behaviors[i].setParam('desiredX',node[0])
            nSpec.behaviors[i].setParam('desiredY',node[2])
            nSpec.behaviors[i].setParam('maneuverChanged',False)
            print 'created behavior',nSpec.behaviors[i].id

        #create the links between the bspecs
        for i in range(len(nSpec.behaviors)):
            for j in range(i+1,len(nSpec.behaviors)):
                nSpec.behaviors[i].links.append(Link('to','ordering',nSpec.behaviors[j].id))
                print 'created link:',i,'->',j

        #add avoidland and avoidentity
        avoidEntityIndex = len(nSpec.behaviors)
        avoidLandIndex = avoidEntityIndex + 1
        nSpec.behaviors.append(self.makeBehaviorSpec(behaviorNames.AVOIDENTITY,nSpec))
        nSpec.behaviors.append(self.makeBehaviorSpec(behaviorNames.AVOIDLAND,nSpec))
        for i in range(pathlength):
            nSpec.behaviors[avoidEntityIndex].links.append(Link('to','permanent',nSpec.behaviors[i].id))
            nSpec.behaviors[avoidLandIndex].links.append(Link('to','permanent',nSpec.behaviors[i].id))
        #create a controller based on our netspec
        cntrllr = Controllers.controller.Controller(entity.authorPanel,entity,'',[],nSpec)
        generateNetSpecXML(nSpec,'cat/networks/astarnetwork.bn')
        entity.stopAll(None)
        entity.currentControllerName = cntrllr.name
        entity.controllerDict[cntrllr.name] = cntrllr
        entity.playback = True
        call('/game/sides/blue/allocatorastar.display')
        cntrllr.start()
        

        popsel()
    
    def onDelete(self, msg):
        for ns, fn in self.netSpecList:
            if ns.name == unicode(msg):
                self.netSpecList.remove((ns, fn))
                break
        
    def setNetSpecList(self, nsl):
        self.netSpecList = nsl

    def handlePageChange(self, event):
        mode = int(event.GetEventObject().GetSelection())
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].setMode(mode)
        else:
            print "No own ship selected"

    def handleTargetSelection(self, msg):
        args    = parseRMenuMsg(msg)
        targetPath  = args.objectsOver[0]
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].handleTargetSelection(targetPath, msg)
            call('/sys/servers/audio.play', 'targetSelected')
        else:
            print "No ownship for target: ", target

    def handleDebug(self, msg):
        args    = parseRMenuMsg(msg)
        me      = args.objectsOver[0]
        
        if exists(me):
            print "Debugging: ", me
            self.entityDict[me].setDebug()
        else:
            print "No one to debug: ", me
            
    def handleWaypoint(self, msg):
        args    = parseRMenuMsg(msg)
        pos     = args.mousePos
        if (self.currentEntPath):
            wp = waypoint.Waypoint(lPos = pos)
            self.entityDict[self.currentEntPath].appendWaypoint(wp)
        else:
            print "No current entity, we are clicking at: ", str(msg)
            
        
    def handleObjectiveChange(self, msg):
        print "Handling Objective Change: ", str(msg)

    def handleBehaviorChange(self, msg):
        '''
        Behavior handler.
        '''
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].handleBehaviorChange(msg)
        else:
            print "No Entity for handling behavior change: ", str(msg)
        
    def handleSetMovePos(self, msg):
        '''
        Change the move location for the entity, so current move behavior knows where to go
        '''
        if(self.currentEntPath):
            self.entityDict[self.currentEntPath].handleSetMovePos(msg)
        else:
            print "No Entity for handling behavior change: ", str(msg)
        
    def handleDeletion(self, msg):
        args = parseRMenuMsg(msg)
        who = args.objectsSelected[0]
        if(self.currentEntPath.startswith(who)):
            self.currentEntPath = None
        delete (who)

    def delete(self, entPath):
        if(self.entDict.has_key(entPath)):
            ent = self.entDict[entPath]
            ent.Hide()
            ent.delete()
        
    def handleEntCreation(self, path):
         self.makeEntity(path, self.authorPanel, self.netVisPanel)
         # self.entityDict[entPath].Show()

    def handleEntDestruction( self, path):                                      #chris
        print 'DO SOMETHING HERE TO DEALLOC WHATEVER WAS MADE IN MAKE ENTITY'   #chris

    def handleSPEntCreation(self, msg):
        #for entPath in self.pulledStrings:
        entPath = str(msg)
        if(not self.entityDict.has_key(entPath)):
		##Z
            self.makeEntity(entPath, self.authorPanel, self.netVisPanel)
        #self.pulledStrings = []
        
    def leftSelectEnt(self, msg):
        """link into the standard selection mechanism"""
        ent = call('/ui/planner.getobjectsselected')
        if ent:
            from sp.playermanager import playerManager                          #chris
            if playerManager.canControlEnt(ent):                                #chris -> cant cat select entities on another team
                ents = ent.split(' ')
                if(len(ents) >= 1):
                    entPath = ents[0].strip() 
                else:
                    entPath = ent.strip()
                call('/ui/planner.clearcatselection')
                call('/ui/planner.addcatselectedent', entPath)
                if (self.currentEntPath):
                    self.entityDict[self.currentEntPath].Hide()
                self.currentEntPath = entPath
                            
                if(self.entityDict.has_key(entPath)):
                    self.entityDict[entPath].Hide()
                    self.entityDict[entPath].Show()
                else:
                    self.makeEntity(entPath, self.authorPanel, self.netVisPanel)
                    self.entityDict[entPath].Show()
                    from cat import panelInfo
                    self.entityDict[entPath].setMode(self.entityDict[entPath].mode)
            else:
                print "Sides: Cannot control ent: ", str(ent)
        else:
            print "Entity does not exist: " + str(ent)

    def makeEntity(self, entPath, authorPanel, netVisPanel):
        import entity
        log.write("Making entity: " + str(entPath))
        #print "Making entity: " + str(entPath)

        ent = entity.Entity(entPath, authorPanel, netVisPanel, size = self.size, netSpecList = self.netSpecList)
        self.entityDict[entPath] = ent

    def makeCatEntities(self):
        import entity
        for entPath in self.entityList:
            self.makeEntity(entPath, self.authorPanel, self.netVisPanel)
        return self.entityDict

    def getEntities(self):
        return self.entityDict

    def setEntities(self, eDict):
        self.entityDict = eDict

    def getEntity(self, entityPath):
        if(self.entityDict.has_key(entityPath)):
            return self.entityDict[entityPath]
        else:
            log.write("No entity of that name: " + entityPath)
            print("No entity of that name: " + entityPath)
            return None

    def setSelectedEntity(self, entityPath):
        if (self.entityDict.has_key(entityPath)):
            self.currentEntPath = entityPath
            self.entityDict[entityPath].Show()
        
    def AttachController(self, entPath, task, args):
        if(not self.entityDict.has_key(entPath)):
		##Z
            self.makeEntity(entPath, self.authorPanel, self.netVisPanel)
            ## creates side effect by adding entry to self.entityDict
        ent = self.getEntity(entPath)
        if(ent):
            print "EntitySet Handling network play request:", task, entPath, args
            # should we not change entity's mode to execution?
            from cat import panelInfo
            ent.setMode(panelInfo.EXECUTION_MODE)
            ent.handleInitRequest(task, args)            
            ent.handlePlayRequest(task, args)
