#
# @(#) recorder.py 2005/11/12
# @author SJL Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
from common.log import Log
log = Log(tags=['recorder'])
from common.defs import *
import nEventServer

from common import behaviorNames
from cat.xml import xmlWriter
from cat.xml import xmlReader
from cat.xml import behaviorSpec
from sets import Set
from cat.xml.bindings import Bindings

class Recorder:
    """Recording/Authoring Manager"""
    def __init__(self, panelA, size=None, entity=None):
        self.entity  = entity
        self.entPath = entity.getEVAPath()
        self.authPanel = panelA
        self.size      = size
        self.netspec = None
        self.pauseRecording = False
        self.isRecording = False
        self.filename    = 'test.bn'
        self.netname     = 'testNet'
        self.trignum     = 0

        from cat.Interface import catRecordManager
        from cat import panelInfo
        self.authRecordPanel = catRecordManager.CatRecordManager(self.authPanel, color = panelInfo.ENTITY_AUTHOR_COLOR, size = size, recType  = "Author:        ", entity = entity, recorder=self)
        from cat.Interface import catPlaybackPanel
        # Set parent of exec record panel to be the authoring panel
        self.execRecordPanel = catPlaybackPanel.CatPlaybackPanel(self.authPanel, color=panelInfo.ENTITY_EXEC_COLOR, size = size, recType  =   "Load/Execute:  ", entity = entity, recorder=self)

    def updateNets(self):
        self.execRecordPanel.updateNets()

    def getPanels(self):
        return [self.authRecordPanel, self.execRecordPanel]
        
    def getNetwork(self):
        return self.netspec

    def getNetSpecDict(self):
        return self.entity.getNetSpecDict()

#-------------------------------------------------------------------------------
# Adding a behavior to a netSpec. Do not directly add to netspec data structure
#-------------------------------------------------------------------------------

    def addToNetSpec(self, behspec):
        if(self.pauseRecording):
            return

        slots = Set()
        bindings = Bindings(behspec.id)
        newTargets = []
        for target in behspec.targets:
            targetbind = bindings.addBinding('target', target)
            slots.add(targetbind)
            newTargets.append(targetbind)
        behspec.targets = newTargets

        for trigger in behspec.triggers:
            trigger.id = self.trignum
            self.trignum += 1
            if trigger.toPath:
                tobind = bindings.addBinding('to', trigger.toPath)
                slots.add(tobind)
                trigger.toPath = tobind
            if trigger.fromPath:
                frombind = bindings.addBinding('from', trigger.fromPath)
                slots.add(frombind)
                trigger.fromPath = frombind
        
        for binding in slots:
            path = bindings.lookupPath(binding)
            bindSpec = behaviorSpec.Binding(binding, path, behspec.id)
            behspec.bindings.append(bindSpec)

        for beh in self.netspec.behaviors:
            if beh.start < behspec.start: #add links to beh
                if beh.triggers:
                    trigid = beh.triggers[0].id
                else:
                    trigid = None
                if beh.end >= behspec.end:
                    beh.links.append(behaviorSpec.Link('to', 'permanent', behspec.id, triggerid=trigid))
                if beh.end < behspec.end and beh.end > behspec.start:
                    beh.links.append(behaviorSpec.Link('to', 'enabling', behspec.id, triggerid=trigid))
                if beh.end <= behspec.start:
                    beh.links.append(behaviorSpec.Link('to', 'ordering', behspec.id, triggerid=trigid))
            elif behspec.start < beh.start: #add links to behspec
                if behspec.triggers: trigid = behspec.triggers[0].id
                else: trigid = None
                if behspec.end >= beh.end:
                    behspec.links.append(behaviorSpec.Link('to', 'permanent', beh.id, triggerid=trigid))
                if behspec.end < beh.end  and behspec.end > beh.start:
                    behspec.links.append(behaviorSpec.Link('to', 'enabling', beh.id, triggerid=trigid))
                if behspec.end <= beh.start:
                    behspec.links.append(behaviorSpec.Link('to', 'ordering', beh.id, triggerid=trigid))
            else:
                #don't know what to do if they started at the same time.
                print "Behaviors started at same time"
        self.netspec.behaviors.append(behspec)
    
    def getTargetPath(self, name):
        pbc = self.entity.getPrimitiveBehaviorController(name)
        if(pbc):
            tPath = pbc.getBehavior().getTarget()
        else:
            tPath = None
        return tPath

    def handleRecordRequest(self):
        self.netspec = behaviorSpec.NetworkSpec(ownship=self.entPath)
        self.pauseRecording = False
        self.isRecording = True
        self.trignum = 0

    def handlePlayRequest(self):
        print "Executing Controller"
        
    def handleStopRecording(self):
        self.netspec = None
        self.isRecording = False

    def handlePauseRecording(self):
        self.pauseRecording = True
        self.isRecording = False

    def saveNetwork(self, filepath = "test.bn", netname = "testNetwork", saveToFile = True):
        self.entity.stopRecording()
        path = filepath.strip()
        netname = netname.strip()
        self.netspec.name = netname
        xmlWriter.generateNetSpecXML(self.netspec, "cat/networks/"+path)
    
    def updateNetDict(self, filename, netname):
        path = filename.strip()
        netname = netname.strip()
        infileName = "cat/networks/"+path
        netSpec = xmlReader.slurp(infileName)
        self.entity.makeNetSpecDict(netSpec = netSpec, filename = path)
        self.updateNets()
