import wx
import wx.lib.ogl as ogl

from cat import *
from common import *

class BehaviorMenu(wx.Panel):
    def __init__(self, parent, size, behaviorShape, entity, controller, reloadShapes):
        self.behaviorShape = behaviorShape
        self.behavior = self.behaviorShape.behavior
        self.entity = entity
        self.controller = controller
        self.reloadShapes = reloadShapes

        wx.Panel.__init__(self,parent,size=size)

        self.vsizer = wx.BoxSizer(wx.VERTICAL)

        self.SetBackgroundColour(wx.Colour(100,150,255))

        self.makeParamBox(size)
        self.makeTargetPanel()

        self.SetPosition((0,0))
        self.SetSize(size)

        w,h = size
        self.paramPanel.SetPosition((0,0))
        self.paramPanel.SetSize((w,h/2))
        self.targetPanel.SetPosition((0,h/2))
        self.targetPanel.SetSize((w,h/2))

        #self.SetSizer(self.vsizer)
        #self.SetAutoLayout(1)
        self.Show(1)
        #self.vsizer.Fit(self)

        #tickmanager.register("behaviormenutick",self.tick)

    def cleanup(self):
        self.targetPanel.Hide()
        self.targetPanel.SetBackgroundColour(self.oldTargetPanelColour)
        self.targetPanel.Reparent(self.oldTargetPanelParent)
        self.paramPanel.Hide()
        self.paramPanel.SetBackgroundColour(self.oldParamPanelColour)
        self.paramPanel.Reparent(self.oldParamPanelParent)
        #tickmanager.unregister("behaviormenutick",self.tick)
        self.behavior.Hide()
        return None

    def tick(self, time, dtime):
        self.paramPanel.Show()
        self.targetPanel.Show()
        self.Refresh()

        #print "\n\n\nbehaviorMenu.tick:"
        #self.printWindowAttribs("parent",self.GetParent())
        #self.printWindowAttribs("self",self)
        #self.printWindowAttribs("param",self.paramPanel)
        #self.printWindowAttribs("target",self.targetPanel)

    def printWindowAttribs(self, name, window):
        x,y = window.GetPosition()
        w,h = window.GetSize()
        print "%s: pos = (%d, %d), size = (%d, %d)" % (name,x,y,w,h)

    def makeTargetPanel(self):
        self.targetPanel = self.behavior.tPanelA
        self.oldTargetPanelParent = self.targetPanel.GetParent()
        self.targetPanel.Reparent(self)
        self.oldTargetPanelColour = self.targetPanel.GetBackgroundColour()
        self.targetPanel.SetBackgroundColour(self.GetBackgroundColour())
        self.targetPanel.Show()
        self.vsizer.Add(self.targetPanel,1,wx.EXPAND)

    def makeParamBox(self, totalSize):
        self.paramPanel = self.behavior.authBehaviorPanel
        self.paramPanel.updatePanels()
        self.paramPanel.Refresh()
        self.oldParamPanelParent = self.paramPanel.GetParent()
        self.paramPanel.Reparent(self)
        self.oldParamPanelColour = self.paramPanel.GetBackgroundColour()
        self.paramPanel.SetBackgroundColour(self.GetBackgroundColour())
        self.paramPanel.Show()
        self.vsizer.Add(self.paramPanel,1,wx.EXPAND)

    def onOK(self, e):
        self.targetPanel.Reparent(self.oldTargetPanelParent)
        self.targetPanel.SetBackgroundColour(self.oldTargetPanelColour)
        self.paramPanel.Reparent(self.oldParamPanelParent)
        self.paramPanel.SetBackgroundColour(self.oldParamPanelColour)
        #tickmanager.unregister("behaviormenutick",self.tick)
        self.behaviorShape.behavior.bname = self.behaviorNameBox.GetValue()
        self.popBehavior()
        self.popWindow()

    def onDelete(self, e):
        netSpec = self.controller.netSpec
        for bspec in netSpec.behaviors:
            if bspec.id == self.behaviorShape.behavior.bname:
                netSpec.behaviors.remove(bspec)
        self.controller.makeNetwork(netSpec)
        self.controller.start()

        self.targetPanel.Reparent(self.oldTargetPanelParent)
        self.targetPanel.SetBackgroundColour(self.oldTargetPanelColour)
        self.paramPanel.Reparent(self.oldParamPanelParent)
        self.paramPanel.SetBackgroundColour(self.oldParamPanelColour)
        #tickmanager.unregister("behaviormenutick",self.tick)
        self.popBehavior()
        self.popWindow()
