import string
import os
import wx
import wx.lib.ogl as ogl

from cat import *
from common import *
import nEventServer

import behaviorLink
import behaviorShape
import behaviorMenu
import linkMenu
import mainMenu
import lineLayout
import loadMenu
import saveMenu
import triggerShape
import triggerMenu

class NetForm(wx.Panel):
    def __init__(self, parent, id, entity):
        #init cat interface
        self.entity = entity
        self.controller = self.entity.getController(self.entity.currentControllerName)
        self.netSpec = None
        self.lastControllerName = "none"

        #init shape dicts
        self.behaviorShapes = {}
        self.linkShapes = {}
        self.triggerShapes = {}
        self.triggerLinks = {}
        self.triggerIgnoreList = []

        #init activity pointers
        self.activeShape = None
        self.selectedShape = None
        self.selectedLink = None
        self.targetPath = None
        self.running = False
        self.addingLinkState = "done"
        self.addingLinkFromShape = None
        self.behaviorTarget = {}
        self.triggerMenuShown = False

        #init this window
        wx.Panel.__init__(self,parent)
        self.vsizer = wx.BoxSizer(wx.VERTICAL)

        #init the sizes
        size = (220,940)
        w,h = size
        self.SetSize(size)
        self.topSize = (w,h/3)
        self.bottomSize = (w,h*2/3)
        topw, toph = self.topSize
        bottomw, bottomh = self.bottomSize

        #init the ogl canvas
        self.canvas = ogl.ShapeCanvas(self)
        self.diagram = ogl.Diagram()
        self.canvas.SetDiagram(self.diagram)
        self.diagram.SetCanvas(self.canvas)
        self.canvas.SetSize(self.topSize)

        #make the top sizer and put the canvas and buttons in it
        self.topSizer = wx.BoxSizer(wx.VERTICAL)
        self.topSizer.Add(self.canvas,9,wx.EXPAND)
        self.topSizer.Add(self.makeBehaviorBar(),1,wx.EXPAND)

        addAvoidID = 119
        addLinkID = 120
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        self.addLinkButton = wx.Button(self,addLinkID,"Add Link")
        self.addLinkButton.SetToolTipString("Switch to link adding mode")
        hsizer.Add(self.addLinkButton,1,wx.EXPAND)
        wx.EVT_BUTTON(self,addLinkID,self.onAddLink)
        addAvoidButton = wx.Button(self,addAvoidID,"Add Avoid")
        addAvoidButton.SetToolTipString("Add an avoidEntity behavior and link it in")
        hsizer.Add(addAvoidButton,1,wx.EXPAND)
        wx.EVT_BUTTON(self,addAvoidID,self.addAvoidEntity)
        self.topSizer.Add(hsizer,1,wx.EXPAND)

        startID = 121
        stopID = 122
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        startButton = wx.Button(self,startID,"Start")
        startButton.SetToolTipString("Run This Network")
        wx.EVT_BUTTON(self,startID,self.onStartButton)
        hsizer.Add(startButton,1,wx.EXPAND)
        stopButton = wx.Button(self,stopID,"Stop")
        stopButton.SetToolTipString("Stop Any Running Network")
        wx.EVT_BUTTON(self,stopID,self.onStopButton)
        hsizer.Add(stopButton,1,wx.EXPAND)
        self.topSizer.Add(hsizer,1,wx.EXPAND)

        saveID = 123
        loadID = 124
        hsizer = wx.BoxSizer(wx.HORIZONTAL)
        saveButton = wx.Button(self,saveID,"Save")
        saveButton.SetToolTipString("Save This Network")
        wx.EVT_BUTTON(self,saveID,self.onSaveButton)
        hsizer.Add(saveButton,1,wx.EXPAND)
        loadButton = wx.Button(self,loadID,"Load")
        loadButton.SetToolTipString("Load a Network")
        wx.EVT_BUTTON(self,loadID,self.onLoadButton)
        hsizer.Add(loadButton,1,wx.EXPAND)
        self.topSizer.Add(hsizer,1,wx.EXPAND)
        
        triggerID = 125
        self.triggerButton = wx.Button(self,triggerID,"Edit Triggers")
        self.triggerButton.SetToolTipString("Edit the triggers for this network")
        wx.EVT_BUTTON(self,triggerID,self.onTriggerLeftClick)
        self.topSizer.Add(self.triggerButton,1,wx.EXPAND)

        self.vsizer.Add(self.topSizer,1,wx.EXPAND)

        #init the bottom window
        self.bottomWindow = wx.Panel(self,size=self.bottomSize)
        self.bottomWindow.SetBackgroundColour(wx.Colour(100,150,255))
        self.bottomWindowChild = None
        self.vsizer.Add(self.bottomWindow,2,wx.EXPAND)
        
        #reload the default controller
        #self.reloadNetSpec(self.netSpec)

        #finish window setup
        self.diagram.ShowAll(1)
        self.SetSizer(self.vsizer)
        self.SetAutoLayout(1)
        self.vsizer.Fit(self)
        self.Show(1)

        #register the target callback
        nEventServer.register(EventNames.RMENU_CAT_SELECT_TARGET,self.handleTargetSelection)

    #makes the bar of bitmap buttons used to make behaviors
    #called by __init__
    def makeBehaviorBar(self):
        prefix = os.path.dirname(__file__) + "/gifs/"
        buttonPad = 8

        approachID = 110
        maintainID = 111
        ramID = 112
        moveToID = 113
        avoidLandID = 114
        avoidEntityID = 115
        fireID = 116

        bmp = wx.Bitmap(prefix+"approach.gif",wx.BITMAP_TYPE_GIF)
        b0 = wx.BitmapButton(self,approachID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.APPROACH)
        b0.SetToolTipString("Approach")
        wx.EVT_BUTTON(self,approachID,self.makeApproach)

        bmp = wx.Bitmap(prefix+"maintain.gif",wx.BITMAP_TYPE_GIF)
        b1 = wx.BitmapButton(self,maintainID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.MAINTAIN)
        b1.SetToolTipString("Maintain")
        wx.EVT_BUTTON(self,maintainID,self.makeMaintain)

        bmp = wx.Bitmap(prefix+"ram.gif",wx.BITMAP_TYPE_GIF)
        b2 = wx.BitmapButton(self,ramID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.RAM)
        b2.SetToolTipString("Ram")
        wx.EVT_BUTTON(self,ramID,self.makeRam)

        bmp = wx.Bitmap(prefix+"moveto.gif",wx.BITMAP_TYPE_GIF)
        b3 = wx.BitmapButton(self,moveToID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.MOVETO)
        b3.SetToolTipString("Move To")
        wx.EVT_BUTTON(self,moveToID,self.makeMoveTo)

        bmp = wx.Bitmap(prefix+"gray.gif",wx.BITMAP_TYPE_GIF)
        b4 = wx.BitmapButton(self,avoidLandID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.AVOIDLAND)
        b4.SetToolTipString("Avoid Land")
        wx.EVT_BUTTON(self,avoidLandID,self.makeAvoidLand)

        bmp = wx.Bitmap(prefix+"gray.gif",wx.BITMAP_TYPE_GIF)
        b5 = wx.BitmapButton(self,avoidEntityID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.AVOIDENTITY)
        b5.SetToolTipString("Avoid Entities")
        wx.EVT_BUTTON(self,avoidEntityID,self.makeAvoidEntity)

        bmp = wx.Bitmap(prefix+"fire.gif",wx.BITMAP_TYPE_GIF)
        b6 = wx.BitmapButton(self,fireID,bmp,(20,20),(bmp.GetWidth()+buttonPad,bmp.GetHeight()+buttonPad),name=behaviorNames.FIRE)
        b6.SetToolTipString("Fire")
        wx.EVT_BUTTON(self,fireID,self.makeFire)

    	box = wx.StaticBox(self, -1, "", style=wx.SUNKEN_BORDER)
        bsizer 		= wx.StaticBoxSizer(box, wx.HORIZONTAL)
        bsizer.Add	(b0, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b1, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add	(b2, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b3, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
      	bsizer.Add	(b4, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add  (b5, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        bsizer.Add  (b6, 0, wx.CENTER|wx.ALIGN_CENTER, 1)
        return bsizer

    #loads netSpec into the controller and reloads the display
    def reloadNetSpec(self, netSpec):
        if self.selectedShape:
            selectedShapeBehavior = self.selectedShape.behavior

        #make a netSpec if needed
        if not netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])
        else:
            self.netSpec = netSpec

        #hide the authoring and exec panels to keep things straight
        self.entity.stopAll(None)
        for behavior, shape in self.behaviorShapes.items():
            behavior.Hide()

        #make a controller if needed and load it into entity
        if not self.controller:
            panela = self.entity.authorPanel
            self.controller = Controllers.controller.Controller(panela,self.entity,"",[],self.netSpec)
            self.controller.init()
            self.entity.playback = False
            self.entity.controllerDict[self.controller.getNetName()] = self.controller
            #self.entity.makeController(self.netSpec)
        #else load the netSpec into the existing controller
        else:
            #copy params out
            paramDict = {}
            for bid, behavior in self.controller.behaviorNet.items():
            #for behavior, shape in self.behaviorShapes.items():
            #    bid = behavior.bname
                for paramName in behavior.params.keys():
                    paramDict[(bid,paramName)] = behavior.GetVar(paramName)

            self.controller.behaviorNet = {}
            self.controller.makeNetwork(self.netSpec)
            self.stopController()

            #copy params back in
            for (bid, paramName), val in paramDict.items():
                if bid in self.controller.behaviorNet.keys():
                    self.controller.behaviorNet[bid].SetVar(paramName,val)

        self.controller.hide()

        #reset the activity pointers
        self.activeShape = None
        #self.selectedShape = None
        #self.selectedLink = None

        #clear out any open editing window
        #self.clearBottomWindow()

        #reload the ogl display
        self.reloadShapes()

        if self.selectedShape and selectedShapeBehavior in self.behaviorShapes.keys():
            self.selectedShape = self.behaviorShapes[selectedShapeBehavior]
            self.selectedShape.makeBox()
            self.diagram.ShowAll(1)
            self.Refresh()

        #reset the target of all behaviors to targetPath
        #TODO: change this to set individually set targets back right
        if self.targetPath:
            for behavior, shape in self.behaviorShapes.items():
                targetPath = self.targetPath
                if behavior.bname in self.behaviorTarget.keys():
                    targetPath = self.behaviorTarget[behavior.bname]
                self.setBehaviorTarget(behavior,targetPath)

    #reset the display with a controller
    def reloadController(self, controller):
        #copy the controller and get its netSpec
        self.controller = controller
        if self.controller:
            self.netSpec = self.controller.netSpec
        else:
            self.netSpec = None

        #reset the activity pointers
        self.activeShape = None
        self.selectedShape = None
        self.selectedLink = None

        #clear out any open editing window
        self.clearBottomWindow()

        #reload the ogl display
        self.reloadShapes()

    #returns the behavior that matches bspec
    #called by reloadShapes() and reloadLinks() and onAddLink()
    def getBehaviorByBspec(self, bspec):
        ret = None
        if self.controller:
            for bid, behavior in self.controller.behaviorNet.items():
                if bid == bspec.id:
                    ret = behavior
        return ret
    
    #sets the background of the canvas to blue
    #called by reloadShapes()
    def initCanvasBackground(self):
        bgRect = ogl.RectangleShape(1000,1000)
        bgRect.SetX(0)
        bgRect.SetY(0)
        bgRect.SetPen(wx.Pen(wx.Colour(0,0,0),0,wx.TRANSPARENT))
        bgRect.SetBrush(wx.Brush(wx.Colour(0,0,100),wx.SOLID))
        bgRect.SetDraggable(False)
        self.canvas.AddShape(bgRect)

        nameRect = ogl.RectangleShape(220,50)
        nameRect.SetX(110)
        nameRect.SetY(25)
        nameRect.SetPen(wx.Pen(wx.Colour(0,0,0),0,wx.TRANSPARENT))
        nameRect.SetBrush(wx.Brush(wx.Colour(0,0,0),wx.TRANSPARENT))
        nameRect.SetDraggable(False)
        nameRect.SetTextColour('WHITE',0)
        self.canvas.AddShape(nameRect)

    #makes a behaviorShape
    #called by reloadShapes()
    def makeBehaviorShape(self, bspec, x, y):
        behavior = self.getBehaviorByBspec(bspec)
        shape = behaviorShape.BehaviorShape(x,y,behavior,self.canvas,self.onBehaviorLeftClick,self.onBehaviorRightClick)
        self.behaviorShapes[behavior] = shape
        return shape

    #reloads the ogl display
    def reloadShapes(self):
        #don't try to reload a null or empty netSpec
        if not self.netSpec or len(self.netSpec.behaviors) == 0:
            self.initCanvasBackground()
            self.reloadTriggers()
            self.diagram.ShowAll(1)
            self.Refresh()
            return

        #make a list of behaviors with shapes and behaviors without shapes
        oldBspecs = []
        newBspecs = []
        for bspec in self.netSpec.behaviors:
            found = False
            for behavior, shape in self.behaviorShapes.items():
                if behavior.bname == bspec.id:
                    found = True
            if found:
                x = -1
                y = -1
                for behavior, shape in self.behaviorShapes.items():
                    if behavior.bname == bspec.id:
                        x = shape.GetX()
                        y = shape.GetY()
                if x != -1 and y != -1:
                    oldBspecs.append((bspec,x,y))
                else:
                    newBspecs.append(bspec)
            else:
                newBspecs.append(bspec)

        #clear out the canvas
        self.behaviorShapes = {}
        self.diagram.DeleteAllShapes()
        self.initCanvasBackground()

        #put old shapes back where they were
        for bspec, x, y in oldBspecs:
            self.makeBehaviorShape(bspec,x,y)

        #search for places to put new shapes
        xStep = 50
        yStep = 50
        xGutter = 50
        yGutter = 50
        w,h = self.canvas.GetSize()
        w = w - xGutter
        h = h - yGutter
        for bspec in newBspecs:
            x = xGutter
            y = yGutter
            placed = False
            while y < h and not placed:
                while x < w and not placed:
                    collides = False
                    for behavior, shape in self.behaviorShapes.items():
                        if shape.collides(x,y):
                            collides = True
                    if not collides:
                        self.makeBehaviorShape(bspec,x,y)
                        placed = True
                    x = x + xStep
                x = xGutter
                y = y + yStep
            if y >= h:
                print "unable to find a good place to put behavior %s" % bspec.id
                self.makeBehaviorShape(bspec,10,10)

        #reload links and triggers
        self.reloadTriggers()

    #returns a trigger from a tspec
    #called by makeTriggerShape
    def getTriggerByTspec(self, tspec):
        ret = None
        for name, trigger in self.entity.triggerDict.items():
            if name == tspec.name:
                ret = trigger
        return ret

    #makes a trigger shape from a tspec and adds it to the canvas
    def makeTriggerShape(self, tspec, bspec, x, y):
        trigger = self.getTriggerByTspec(tspec)
        behavior = self.getBehaviorByBspec(bspec)
        bshape = self.behaviorShapes[behavior]
        shape = triggerShape.TriggerShape(x,y,bshape,trigger,self.canvas,self.onTriggerLeftClick,self.onTriggerRightClick)
        self.triggerShapes[trigger] = shape
        return shape

    #reload the triggers in the ogl display
    def reloadTriggers(self):
        #don't try to reload from a null netspec
        if not self.netSpec:
            return
            
        #clear off the old shapes
        for trigger, tshape in self.triggerShapes.items():
            self.canvas.RemoveShape(tshape)
        
        #split the tspecs into new and old lists
        oldTspecs = []
        newTspecs = []
        for bspec in self.netSpec.behaviors:
            for tspec in bspec.triggers:
                found = False
                for trigger, triggerShape in self.triggerShapes.items():
                    if trigger.name == tspec.name and not found:
                        found = True
                        oldTspecs.append((tspec,bspec,triggerShape.GetX(),triggerShape.GetY()))
                if not found:
                    newTspecs.append((tspec,bspec))

        #add the old tspecs in the same places                    
        for tspec, bspec, x, y in oldTspecs:
            self.makeTriggerShape(tspec,bspec,x,y)
            
        #use the same method from reloadShapes to layout new shapes
        xStep = 50
        yStep = 50
        xGutter = 50
        yGutter = 50
        w,h = self.canvas.GetSize()
        w = w - xGutter
        h = h - yGutter
        for tspec, bspec in newTspecs:
            x = xGutter
            y = yGutter
            placed = False
            while y < h and not placed:
                while x < w and not placed:
                    collides = False
                    for behavior, shape in self.behaviorShapes.items():
                        if shape.collides(x,y):
                            collides = True
                    for trigger, shape in self.triggerShapes.items():
                        if shape.collides(x,y):
                            collides = True
                    if not collides:
                        self.makeTriggerShape(tspec,bspec,x,y)
                        placed = True
                    x = x + xStep
                x = xGutter
                y = y + yStep
            if y >= h:
                print "unable to find a good place to put trigger %s" % tspec.name
                self.makeTriggerShape(tspec,10,10)
                
        self.reloadLinks()

    #reload the arrows in the ogl display
    def reloadLinks(self):
        #don't try to reload from a null netspec
        if not self.netSpec:
            return
            
        linex = []
        liney = []
        lineStep = 5

        #clear old lines off the canvas
        for key, linkShape in self.linkShapes.items():
            self.canvas.RemoveShape(linkShape)
        for key, linkShape in self.triggerLinks.items():
            self.canvas.RemoveShape(linkShape)

        #reset the top and bottom x for shapes
        for behavior, shape in self.behaviorShapes.items():
            shape.topx = 0
            shape.bottomx = 0
        for trigger, shape in self.triggerShapes.items():
            shape.topx = 0
            shape.bottomx = 0
    
        #init vars
        self.linkShapes = {}
        bspecDict = {}
        for bspec in self.netSpec.behaviors:
            bspecDict[bspec.id] = bspec
        lc = self.onLinkLeftClick
        rc = self.onLinkRightClick

        #use netspec to load lines
        for bspec in self.netSpec.behaviors:
            fromBehavior = self.getBehaviorByBspec(bspec)
            if fromBehavior in self.behaviorShapes.keys():
                fromShape = self.behaviorShapes[fromBehavior]
                for link in bspec.links:
                    if link.dest in bspecDict:
                        tobspec = bspecDict[link.dest]
                        toBehavior = self.getBehaviorByBspec(tobspec)
                        if toBehavior in self.behaviorShapes.keys():
                            toShape = self.behaviorShapes[toBehavior]
                            linkShape = behaviorLink.BehaviorLink(fromShape,toShape,link.type, self.canvas,linex,liney,lineStep,lc,rc)
                            self.linkShapes[(fromBehavior,toBehavior)] = linkShape
                    else:
                        bspec.links.remove(link)
                        
        #load trigger lines too
        for tspec, tshape in self.triggerShapes.items():
            link = behaviorLink.BehaviorLink(tshape,tshape.behaviorShape,"trigger",self.canvas,linex,liney,lineStep,lc,rc)
            self.triggerLinks[(tshape.trigger,tshape.behaviorShape.behavior)] = link

        #refresh the display
        self.diagram.ShowAll(1)
        self.Refresh()

    #adds a bspec matching behavior to self.netSpec and reloads it
    def addBehavior(self, behavior):
        if self.netSpec:
        
            sensors = xml.behaviorSpec.BehaviorSensorSpec([],[])
            i = string.find(behavior.bname,'_')
            name = behavior.bname[0:i]
            bspec = xml.behaviorSpec.BehaviorSpec(name,behavior.bname,"0",[],sensors,[],[],[])
            
            self.netSpec.behaviors.append(bspec)
            #self.netSpec.addBehavior(bspec,False)
            #self.reloadNetSpec(self.netSpec)

    #removes any bspec matching behavior from self.netSpec and reloads it
    def removeBehavior(self, behavior):
        if self.netSpec:
            if self.selectedShape and self.selectedShape.behavior == behavior:
                self.selectedShape = None
            self.clearBottomWindow()
            
            for bspec in self.netSpec.behaviors:
                if bspec.id == behavior.bname:
                    self.netSpec.behaviors.remove(bspec)
            self.reloadNetSpec(self.netSpec)
            #if self.netSpec.removeBehavior(behavior.bname):
            #    self.reloadNetSpec(self.netSpec)

    def addLinkNoReload(self,fromBehavior,toBehavior,type='ordering'):
        if self.netSpec:
            for bspec in self.netSpec.behaviors:
                if bspec.id == fromBehavior.bname:
                    link = xml.behaviorSpec.Link("to",type,toBehavior.bname)
                    bspec.links.append(link)
            #self.reloadNetSpec(self.netSpec)
            #if self.netSpec.addLink(fromBehavior.bname,toBehavior.bname,type):
            #    self.reloadNetSpec(self.netSpec)
        

    #adds a link to netSpec and reloads it
    def addLink(self, fromBehavior, toBehavior, type = "ordering"):
        if self.netSpec:
            for bspec in self.netSpec.behaviors:
                if bspec.id == fromBehavior.bname:
                    link = xml.behaviorSpec.Link("to",type,toBehavior.bname)
                    bspec.links.append(link)
            self.reloadNetSpec(self.netSpec)
            #if self.netSpec.addLink(fromBehavior.bname,toBehavior.bname,type):
            #    self.reloadNetSpec(self.netSpec)

    #removes a link from netSpec and reloads it
    def removeLink(self, fromBehavior, toBehavior):
        if self.netSpec:
            for frombspec in self.netSpec.behaviors:
                if frombspec.id == fromBehavior.bname:
                    for link in frombspec.links:
                        if link.dest == toBehavior.bname:
                            frombspec.links.remove(link)
            self.reloadNetSpec(self.netSpec)
            #if self.netspec.removeLink(fromBehavior.bname,toBehavior.bname):
            #    self.reloadNetSpec(self.netSpec)

    #add trigger to behavior's triggers and reload
    def addTrigger(self, trigger, behavior = None):
        #default to behavior[0] TODO: fix this
        if not behavior:
            if len(self.behaviorShapes.keys()) == 0:
                self.triggerIgnoreList.append(trigger)
                return
            behavior = self.behaviorShapes[self.behaviorShapes.keys()[0]].behavior    
    
        tspec = None

        #first check if a tspec already exists
        for bspec in self.netSpec.behaviors:
            for testTspec in bspec.triggers:
                if testTspec.name == trigger.name:
                    tspec = testTspec
                    
        #if not found make one
        if not tspec:
            bspec = None
            for testBspec in self.netSpec.behaviors:
                if testBspec.id == behavior.bname:
                    bspec = testBspec
            if bspec:
                tspec = xml.behaviorSpec.TriggerSpec(trigger.name,trigger.type,trigger.subtype,trigger.getThreshold(),trigger.getTo(),trigger.getFrom())
                bspec.triggers.append(tspec)
                
        #and reload the netspec
        self.reloadNetSpec(self.netSpec)
            

    #remove trigger and reload        
    def removeTrigger(self, trigger):
        #search for a tspec and remove it, then reload
        found = False
        for bspec in self.netSpec.behaviors:
            for tspec in bspec.triggers:
                if tspec.name == trigger.name:
                    found = True
                    bspec.triggers.remove(tspec)

        #if any tspecs were deleted reload the netspec                    
        if found:
            print self.triggerLinks.items()
            for (testTrigger, behavior), link in self.triggerLinks.items():
                if testTrigger == trigger:
                    del self.triggerLinks[(testTrigger, behavior)]

            self.reloadNetSpec(self.netSpec)

    #called when a linkShape is left clicked
    def onLinkLeftClick(self, linkShape):
        fromName = linkShape.fromShape.getName()
        toName = linkShape.toShape.getName()
        shapeList = []
        for behavior, shape in self.behaviorShapes.items():
            shapeList.append(shape)
        self.selectedLink = linkShape
        parent = self.bottomWindow
        size = self.bottomSize
        fromBehavior = linkShape.fromShape.behavior
        toBehavior = linkShape.toShape.behavior
        type = linkShape.type
        if self.controller:
            behaviorNet = self.controller.behaviorNet
        else:
            behaviorNet = None
        window = linkMenu.LinkMenu(parent,size,fromBehavior,toBehavior,type,self.behaviorShapes,behaviorNet,self.netSpec,self.reloadNetSpec)
        self.addBottomWindow(window)

    #called when a linkShape is right clicked
    def onLinkRightClick(self, linkShape):
        fromName = linkShape.fromShape.getName()
        toName = linkShape.toShape.getName()
        self.removeLink(linkShape.fromShape.behavior,linkShape.toShape.behavior)

    #called when a behaviorShape is left clicked
    def onBehaviorLeftClick(self, behaviorShape):
        if self.addingLinkState == "done":
            if self.selectedShape == behaviorShape:
                self.selectedShape.removeBox()
                self.Refresh()
                self.selectedShape = None
                self.clearBottomWindow()
            else:
                if self.selectedShape:
                    self.selectedShape.removeBox()
                self.selectedShape = behaviorShape
                self.selectedShape.makeBox()
                self.diagram.ShowAll(1)
                self.Refresh()
                window = behaviorMenu.BehaviorMenu(self,self.bottomSize,behaviorShape,self.entity,self.controller,self.reloadShapes)
                self.addBottomWindow(window)
        
        elif self.addingLinkState == "settingFrom":
            print "setting fromShape"
            self.addingLinkFromShape = behaviorShape
            self.addingLinkFromShape.SetPen(wx.Pen(wx.Colour(255,0,0),2,wx.SOLID))
            self.addingLinkState = "settingTo"
            self.Refresh()
            
        elif self.addingLinkState == "settingTo":
            print "setting toShape"
            
            if (self.addingLinkFromShape.behavior, behaviorShape.behavior) in self.behaviorShapes.keys():
                print "that link already exists"
            else:
                self.addLink(self.addingLinkFromShape.behavior,behaviorShape.behavior)
                self.addingLinkFromShape.SetPen(wx.Pen(wx.Colour(0,0,0),0,wx.TRANSPARENT))
                self.addingLinkFromShape = None
                self.addingLinkState = "done"
                self.addLinkButton.Enable(True)
                self.Refresh()
        
        else:
            print "ERROR: unknown addingLinkState %s" % self.addingLinkState

    #called when a behaviorShape is right clicked
    def onBehaviorRightClick(self, behaviorShape):
        self.removeBehavior(behaviorShape.behavior)
        
    #called when a triggerShape is left clicked
    #or when the trigger button is clicked
    def onTriggerLeftClick(self, tShape):
        if self.triggerMenuShown:
            self.clearBottomWindow()
            self.triggerMenuShown = False
            self.triggerButton.SetLabel("Edit Triggers")
        else:
            if type(tShape) != triggerShape.TriggerShape:
                tShape = None
            window = triggerMenu.TriggerMenu(self.bottomWindow,self.bottomSize,tShape,self.entity,self.reloadTriggers())
            self.addBottomWindow(window)
            self.triggerMenuShown = True
            self.triggerButton.SetLabel("Hide Triggers")

    #called when a triggerShape is right clicked
    def onTriggerRightClick(self, triggerShape):
        self.removeTrigger(triggerShape.trigger)

    #called each time interval
    def tick(self, time, dtime):
        if self.IsShown():
            self.checkControllerChanged()
            self.checkShapesMoved()
            self.updateActiveShape()
            self.checkTriggersChanged()

    #check if new triggers were created and display them
    #called by tick
    def checkTriggersChanged(self):
        deleted = False
    
        for name, trigger in self.entity.triggerDict.items():
            if not self.triggerShapes.has_key(trigger) and trigger not in self.triggerIgnoreList:
                print "found new trigger %s in entity" % name
                self.addTrigger(trigger)
                
        for trigger, triggerShape in self.triggerShapes.items():
            if not self.entity.triggerDict.has_key(trigger):
                delete = True

        if self.controller:
            for bid, behavior in self.controller.behaviorNet.items():
                for name, trigger in behavior.triggers.items():
                    if not self.triggerShapes.has_key(trigger) and trigger not in self.triggerIgnoreList:
                        print "found new trigger %s in behavior %s" % (name,bid)
                        self.addTrigger(trigger,behavior)
                    
            for trigger, triggerShape in self.triggerShapes.items():
                for bid, behavior in self.controller.behaviorNet.items():
                    if not self.triggerShapes.has_key(trigger):
                        deleted = True
                        
        #if any triggers were deleted reload shapes
        if deleted:
            self.reloadShapes()

    #if the controller changed reload it
    #called by tick
    def checkControllerChanged(self):
        if self.entity.currentControllerName != self.lastControllerName:
            self.lastControllerName = self.entity.currentControllerName
            self.reloadController(self.entity.getController(self.entity.currentControllerName))

    #if any shapes moved reload links
    #called by tick
    def checkShapesMoved(self):
        moved = False
        for behaivor, shape in self.behaviorShapes.items():
            if shape.GetX() != shape.lastx or shape.GetY() != shape.lasty:
                moved = True
                shape.lastx = shape.GetX()
                shape.lasty = shape.GetY()
                if shape == self.selectedShape:
                    shape.updateBox()
        for trigger, shape in self.triggerShapes.items():
            if shape.GetX() != shape.lastx or shape.GetY() != shape.lasty:
                moved = True
                shape.lastx = shape.GetX()
                shape.lasty = shape.GetY()
                if shape == self.selectedShape:
                    shape.updateBox()
        if moved:
            self.reloadLinks()

    #activeShape is rendered in red
    #called by tick
    def updateActiveShape(self):
        if not self.running:
            return
        self.lastActiveShape = self.activeShape
        for behavior, shape in self.behaviorShapes.items():
            if shape.behavior.checkPrePorts() and shape.behavior != self.lastActiveShape:
                self.activeShape = shape
        if self.activeShape != self.lastActiveShape:
            for behavior, shape in self.behaviorShapes.items():
                shape.SetBrush(wx.Brush(wx.Colour(100,100,255),wx.SOLID))
                shape.active = False
            self.activeShape.SetBrush(wx.Brush(wx.Colour(255,0,0),wx.SOLID))
            self.activeShape.active = True
            self.Refresh()

    #callback for add link button, switches to link adding mode
    def onAddLink(self, e):
        fullyConnected = True
        for bspec in self.netSpec.behaviors:
            if len(bspec.links) < len(self.netSpec.behaviors):
                fullyConnected = False
        if len(self.netSpec.behaviors) > 1 and not fullyConnected:
            print "switching to link adding mode"
            self.addingLinkState = "settingFrom"
            self.addLinkButton.Disable()
            self.Refresh()
        else:
            print "not enough behaviors to link or fully connected"

    #these functions are callbacks for the behavior buttons
    #they just call makeBehavior with the right bname out of behaviorNames
    def makeApproach(self, e):
        self.makeBehavior(behaviorNames.APPROACH)

    def makeMaintain(self, e):
        self.makeBehavior(behaviorNames.MAINTAIN)

    def makeRam(self, e):
        self.makeBehavior(behaviorNames.RAM)

    def makeMoveTo(self, e):
        self.makeBehavior(behaviorNames.MOVETO)

    def makeAvoidLand(self, e):
        self.makeBehavior(behaviorNames.AVOIDLAND)

    def makeAvoidEntity(self, e):
        self.makeBehavior(behaviorNames.AVOIDENTITY)

    def makeFire(self, e):
        self.makeBehavior(behaviorNames.FIRE)

    #makes a behavior id based on a behavior name
    #called by makeBehavior
    def makeBid(self, bname):
        i = 1
        id = bname + "_1"
        ok = False
        while not ok and self.controller:
            id = bname + "_" + str(i)
            ok = True
            for bid, behavior in self.controller.behaviorNet.items():
                if id == bid:
                    ok = False
            i = i + 1
        return id

    def makeBehaviorNoPanels(self,bname):
        #make a bid
        bid = self.makeBid(bname)

        #make a pbc to get default params out of
        panelA = self.entity.authorPanel
        pbc = Controllers.primitiveBehaviorController.PrimitiveBehaviorController(None,self.entity,bname=bname,bid=bid)
        behavior = pbc.behavior
        #bid = behavior.bname

        #init some BehaviorSpec attributes
        sensors = xml.behaviorSpec.BehaviorSensorSpec([],[])
        params = []

        #copy pbc params
        for paramName in behavior.params.keys():
            val, typ, min, max = behavior.params[paramName]
            params.append(xml.behaviorSpec.Parameter(paramName,typ,val,min,max))

        #make BehaviorSpec
        bspec = xml.behaviorSpec.BehaviorSpec(bname,bid,"1",[],sensors,params,[],[])

        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])

        #add the BehaviorSpec to the netSpec and reload it
        self.netSpec.behaviors.append(bspec)
        self.reloadNetSpec(self.netSpec)
        
        #find the shape we added using its bid and simulate a click on it to open the edit window
        #also set its target if one has been specified
        shape = None
        for testBehavior, testShape in self.behaviorShapes.items():
            if testBehavior.bname == bid:
                shape = testShape
        if shape:
            pass
            #self.onBehaviorLeftClick(shape)

        return shape.behavior
        
    #makes a behavior, adds it to the controller, and clicks it
    #called by the make(some bname) functions
    def makeBehavior(self, bname):
        #make a bid
        bid = self.makeBid(bname)

        #make a pbc to get default params out of
        panelA = self.entity.authorPanel
        pbc = Controllers.primitiveBehaviorController.PrimitiveBehaviorController(panelA,self.entity,bname=bname,bid=bid)
        behavior = pbc.behavior
        #bid = behavior.bname

        #init some BehaviorSpec attributes
        sensors = xml.behaviorSpec.BehaviorSensorSpec([],[])
        params = []

        #copy pbc params
        for paramName in behavior.params.keys():
            val, typ, min, max = behavior.params[paramName]
            params.append(xml.behaviorSpec.Parameter(paramName,typ,val,min,max))

        #make BehaviorSpec
        bspec = xml.behaviorSpec.BehaviorSpec(bname,bid,"1",[],sensors,params,[],[])

        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])

        #add the BehaviorSpec to the netSpec and reload it
        self.netSpec.behaviors.append(bspec)
        self.reloadNetSpec(self.netSpec)

        #find the shape we added using its bid and simulate a click on it to open the edit window
        #also set its target if one has been specified
        shape = None
        for testBehavior, testShape in self.behaviorShapes.items():
            if testBehavior.bname == bid:
                shape = testShape
        if shape:
            pass
            #self.onBehaviorLeftClick(shape)

        return shape.behavior
        
    #starts controller
    def startController(self):
        if self.controller:
            self.running = True
            for behavior, shape in self.behaviorShapes.items():
                behavior.stop()
                behavior.Hide()
            self.entity.stopAll(None)
            self.entity.currentControllerName = self.controller.name
            self.entity.playback = True
            self.controller.start()
            #self.entity.handlePlayRequest(self.controller.name,None)

    #stops controller
    def stopController(self):
        if self.controller:
            self.running = False
            for behavior, shape in self.behaviorShapes.items():
                behavior.stop()
                behavior.Hide()
            self.entity.stopAll(None)
            self.entity.hideAllPBCs()
            #self.controller.stop()
            self.entity.handlePauseRequest(self.controller.name)
            self.entity.playback = False
            pbc = self.entity.getPrimitiveBehaviorController(self.entity.currentPrimitiveBehaviorName)
            if pbc:
                pbc.stop()

            #reset the active shape
            self.activeShape = None
            for behavior, shape in self.behaviorShapes.items():
                shape.SetBrush(wx.Brush(wx.Colour(100,100,255),wx.SOLID))
                shape.active = False
            self.Refresh()

    #restarts controller
    def restartController(self):
        self.stopController()
        self.startController()

    #button callback for start button
    def onStartButton(self, e):
        #switch the command panel to execution mode because we are executing now
        self.entity.buttonPanel.SwitchToExecution(None)
        self.entity.authorPanel.Refresh()
        self.startController()

    #button callback for stop button
    def onStopButton(self, e):
        #switch the command panel to authoring mode because we are authoring now
        self.entity.buttonPanel.SwitchToAuthoring(None)
        self.entity.authorPanel.Refresh()
        self.stopController()

    #handles target selection
    #if a behavior menu is open, it assigns the target to that behavior
    #else it just assigns it to the whole network
    def handleTargetSelection(self, msg):
        args = defs.parseRMenuMsg(msg)
        self.targetPath = args.objectsOver[0]
        if self.IsShown():
            if self.selectedShape:
                self.setBehaviorTarget(self.selectedShape.behavior,self.targetPath)
            else:
                for behavior, behaviorShape in self.behaviorShapes.items():
                    self.setBehaviorTarget(behaviorShape.behavior,self.targetPath)

    #sets behavior's target to targetPath
    def setBehaviorTarget(self, behavior, targetPath):
        if behavior:
            behavior.setTarget(targetPath)
            self.behaviorTarget[behavior.bname] = targetPath

    #callback for save button
    def onSaveButton(self, e):
        #make a netSpec if needed, this should've already happened in the constructor
        if not self.netSpec:
            self.netSpec = xml.behaviorSpec.NetworkSpec("VPT","no ownship",[],[],[])
        self.addBottomWindow(saveMenu.SaveMenu(self.bottomWindow,self.bottomSize,self.netSpec.name,self.onSave,self.clearBottomWindow))

    #saves netspec to filename
    def onSave(self, filename):
        if self.netSpec:
            xml.xmlWriter.generateNetSpecXML(self.netSpec,filename)

    #callback for load button
    def onLoadButton(self, e):
        netSpecName = "VPT"
        if self.netSpec:
            netSpecName = self.netSpec.name
        self.addBottomWindow(loadMenu.LoadMenu(self.bottomWindow,self.bottomSize,netSpecName,self.onLoad,self.clearBottomWindow))

    #tries to load netspec from filename
    def onLoad(self, filename):
        netSpec = None
        try:
            netSpec = xml.xmlReader.slurp(filename)
        except:
            print "unable to load netSpec from %s" % filename
        if netSpec:
            self.reloadNetSpec(netSpec)

    #callback for convenience button to make an avoidentity 
    #behavior if needed and link it to all non-avoid behaviors
    def addAvoidEntity(self, e):
        #make the behavior if needed
        avoidBehaviorShape = None
        for behavior, behaviorShape in self.behaviorShapes.items():
            if behavior.bname.startswith(behaviorNames.AVOIDENTITY):
                avoidBehaviorShape = behaviorShape
        if not avoidBehaviorShape:
            self.makeBehavior(behaviorNames.AVOIDENTITY)
            self.addAvoidEntity(None)
        
        #link it to non-avoid behaviors
        for behavior, shape in self.behaviorShapes.items():
            if not behavior.bname.startswith(behaviorNames.AVOIDENTITY) and not behavior.bname.startswith(behaviorNames.AVOIDLAND):
                self.addLink(avoidBehaviorShape.behavior,behavior,"permanent")

    #kill any open editing window
    def clearBottomWindow(self):
        if self.bottomWindowChild:
            self.bottomWindowRet = self.bottomWindowChild.cleanup()
            #self.bottomWindow.RemoveChild(self.bottomWindowChild) #lol segfault
            self.bottomWindowChild.Hide()
            #del self.bottomWindowChild
            self.bottomWindowChild = None
            self.Refresh()
            self.bottomWindow.Refresh()

    #show window in the bottom section of the overall form
    def addBottomWindow(self, window):
        if self.bottomWindowChild:
            self.clearBottomWindow()
        self.bottomWindowChild = window
        window.Reparent(self.bottomWindow)
        #window.SetSize(self.bottomSize)
        window.SetPosition((0,0))
        window.Show(1)
        window.Refresh()






