#
# @(#) entityDistanceSensor.py 2005/11/25
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

from common.defs import *
import math
import nEventServer
from common import EventNames

class EntityDistanceSensor:

    def __init__(self,own = None, other = None, range = 100, calculateDistance = True, id = None):
        """
        Creates a new EntityDistance object
        """
        self.toEntity = other
        self.fromEntity = own
        self.range = range
        self.parameters = []
        self.calculateDistance = calculateDistance
        self.id = id
        if self.calculateDistance:
            self.parameters.append({'name':'From','type':'ent'})
            self.parameters.append({'name':'To','type':'ent'})
        else:
            self.parameters.append({'name':'From','type':'ent'})
            self.parameters.append({'name':'Range','type':'int'})
        self.entsInRange = []
        if self.id:
            nEventServer.pull(EventNames.DISTANCE_SENSOR_CREATED, self.id+' '+self.fromEntity+' '+self.toEntity)
        
    def __del__(self):
        nEventServer.pull(EventNames.DISTANCE_SENSOR_DELETED, str(self.id))

    def handleBindingChange(self, oldPath, newPath):
        print "Changing entityDistanceSensor", oldPath, newPath, self.toEntity, self.fromEntity
        if self.toEntity == oldPath:
            self.toEntity = newPath
        if self.fromEntity == oldPath:
            self.fromEntity = newPath
        if self.id:
            nEventServer.pull(EventNames.DISTANCE_SENSOR_DELETED, str(self.id))
            nEventServer.pull(EventNames.DISTANCE_SENSOR_CREATED, self.id+' '+self.fromEntity+' '+self.toEntity)

    def setParameters(self,valueList):
        for i in range(len(self.parameters)):
            if self.parameters[i]['name'] == 'From':
                self.fromEntity = str(valueList[i])
            elif self.parameters[i]['name'] == 'To':
                self.toEntity = str(valueList[i])
            elif self.parameters[i]['name'] == 'Range':
                self.range = int(valueList[i])

    def getTriggerState(self):
        outgoingParams = self.parameters
        
        for param in outgoingParams:
            if param['name']=='From':
                param['value']=self.fromEntity
            elif param['name']=='To':
                param['value']=self.toEntity
            elif param['name']=='Range':
                param['value']=int(self.range)

        sensorInfo = {}
        sensorInfo['value'] = self.getValue()
        if self.calculateDistance:
            sensorInfo['type'] = 'float'
            sensorInfo['sensorType'] = 'Distance Between Entities'
        else:
            sensorInfo['type'] = 'entlist'
            sensorInfo['sensorType'] = 'Entities in Range'

        return (sensorInfo,outgoingParams)

            
        
    def Tick(self,time,dtime):
        if self.calculateDistance:
            return self.getDistance()
        else:
            shortlist = []
            list = self.getEntsInRange()
            for ent in list:
                pushsel(ent)
                shortlist.append(getname())
                popsel()
            return shortlist

    def getValue(self):
        if self.calculateDistance:
            return self.getDistance()
        else:
            return self.getEntsInRange()
        
    def getDistance(self,toEntity=None,fromEntity=None):
        try:
            if toEntity == None:
                toEntity = self.toEntity
            if fromEntity == None:
                fromEntity = self.fromEntity
            if not exists(toEntity) or not exists(fromEntity):
                return 0
            pushsel()
            sel(toEntity)
            #sel('/game/ents/cigarette/cigarette0')
            ox,oz,oy = gv('pos')
            #sel('/game/ents/ddg51/ddg510')
            sel(fromEntity)
            tx,tz,ty = gv('pos')

            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            self.range = dist
            popsel()
            return dist
        except:
            printExcept('EntityDistanceSensor')
            return 0

    def getEntsInRange(self,range=None,fromEntity=None):
        try:
            self.entsInRange = []
            if range == None:
                range = self.range
            if fromEntity == None:
                fromEntity = self.fromEntity
            if not exists(fromEntity):
                return []
            sel(fromEntity)
            ox,oz,oy = gv('pos')
            pushsel()
            sel('/game/ents')
            dir = ndir()
            for subdir in dir:
                sel('/game/ents/'+subdir)
                dir2 = ndir()
                for subdir2 in dir2:
                    sel('/game/ents/'+subdir+'/'+subdir2)
                    if getfullname() == fromEntity or not getselectable():
                        pass
                    else:
                        tx,tz,ty = gv('pos')
                        dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
                        if dist <= range:
                            self.entsInRange.append(getfullname())
            popsel()
            return self.entsInRange
        except:
            printExcept('EntityDistanceSensor')
            return []
        
