from sensor import Sensor
from cat import catInfo
from common.defs import *
import math
import nEventServer
from common import EventNames

class DistanceSensor(Sensor):
    def __init__(self, name, toPath, fromPath, subtype, threshold):
        Sensor.__init__(self)
        self.name = name
        self.toPath = toPath
        self.fromPath = fromPath
        self.threshold = DistanceThreshold(subtype, threshold)
        self.lineName = name + ' '+toPath+' '+fromPath
        nEventServer.pull(EventNames.DISTANCE_SENSOR_CREATED, self.lineName)

    def __del__(self):
        nEventServer.pull(EventNames.DISTANCE_SENSOR_DELETED, self.name)

    def tick(self, time, dtime):
        Sensor.tick(self, time, dtime)
        self.sensorVal = self.getDistance()

    def getDistance(self):
        try:
            if not exists(self.toPath) or not exists(self.fromPath):
                return -1
            pushsel()
            sel(self.toPath)
            ox, oz, oy = gv('pos')
            sel(self.fromPath)
            tx, tz, ty = gv('pos')
            popsel()

            dist = math.sqrt((ox-tx)**2 + (oy-ty)**2)
            return dist
        except:
            printExcept('DistanceSensor')
            return 0

    # -----------------------------------------------------------------------------------------------------
    # delegated call from triger. Delete old distanceline, create new one
    # -----------------------------------------------------------------------------------------------------
    def handleBindingChange(self, oldPath, newPath):
        if self.toPath == oldPath:
            self.toPath = newPath
            self.delete()
            self.lineName = self.name + ' '+self.toPath+' '+self.fromPath
            nEventServer.pull(EventNames.DISTANCE_SENSOR_CREATED, self.lineName)
        if self.fromPath == oldPath:
            self.fromPath = newPath
            self.delete()
            self.lineName = self.name + ' '+self.toPath+' '+self.fromPath
            nEventServer.pull(EventNames.DISTANCE_SENSOR_CREATED, self.lineName)


    def delete(self):
        self.__del__()


class DistanceThreshold:
    def __init__(self, type, value=None):
        self.type = type
        self.value = value
        if value == None:
            self.value = 0
            self.set = False
        else:
            self.value = value
            self.set = True

    def isSet(self):
        return self.set

    def fire(self, sensor=None):
        if sensor:
            self.value = sensor.getValue()
            self.set = True

    def check(self, sensor):
        val = sensor.getValue()
        if type == catInfo.LESS_THAN:
            return val < self.value
        elif type == catInfo.GREATER_THAN:
            return val > self.value
        elif type == catInfo.APPROX:
            return val < value*1.05 and val > value * 0.95

    def getValue(self):
        return self.value
