from cat import catInfo
from distancesensor import *
from alertsensor import *
from damagesensor import *
from cat.xml.behaviorSpec import TriggerSpec
from common import tickmanager

class Trigger:
    def __init__(self, entity, name, type, subtype=None, threshold=None, toPath=None, fromPath=None):
        self.entity        = entity
        self.name          = name
        self.type          = type
        self.subtype       = subtype
        self.fired         = False
        self.threshold     = threshold
        self.toPath        = toPath
        self.fromPath      = fromPath

        if self.entity.mode == 0:
            tickmanager.register(self.name, self.tick)
        self.sensor = self.createSensor(type)


    def createSensor(self, type):
        sensor = None
        if type == catInfo.DISTANCE_SENSOR:
            sensor = DistanceSensor(self.name, self.toPath, self.fromPath, self.subtype, self.threshold)
        elif type == catInfo.ALERT_SENSOR:
            sensor = AlertSensor()
        if type == catInfo.DAMAGE_SENSOR:
            sensor = DamageSensor(self.toPath, self.subtype, self.threshold)
        return sensor

    def delete(self):
        tickmanager.unregister(self.name, self.tick)
        self.sensor.delete()

    def tick(self, time, dtime):
        self.sensor.tick(time, dtime)
        #print "Ticking trigger: ", self.name, " value: ", self.getValue()
        if self.entity.mode == 0 and self.hasThreshold():
            self.checkThreshold()

    def getName(self):
        return self.name

    def getType(self):
        return self.type

    def getSubtype(self):
        return self.subtype

    def fire(self):
        if self.fired:
            return
        self.fired = True
        if self.entity.mode == 0: #authoring mode
            self.sensor.threshold.fire(self.sensor)
            triggerSpec = TriggerSpec(self.name, self.type, self.subtype,
                                      self.getThreshold(), self.getTo(), self.getFrom())
            self.entity.handleTrigger(triggerSpec)
        elif self.entity.mode == 1: #execution mode
            self.sensor.threshold.fire(self.sensor)
    
    def hasFired(self):
        return self.fired

    def hasThreshold(self):
        return self.sensor.threshold.isSet()

    def getThreshold(self):
        return self.sensor.threshold.getValue()

    def checkThreshold(self):
        if self.sensor.threshold.isSet():
            self.fired = self.fired or self.sensor.threshold.check(self.sensor)
        
    def setTo(self, path):
        self.sensor.setTo(path)

    def setFrom(self, path):
        self.sensor.setFrom(path)

    def getTo(self):
        return self.sensor.getTo()

    def getFrom(self):
        return self.sensor.getFrom()


    def getValue(self):
        return self.sensor.getValue()

    #-----------------------------------------------------------------------
    # Changes underlying sensor targets if needed. For example
    # if the target changes while execing, then we need to make
    # changes to the underlying sensor. The correct way to do this
    # is for triggers to track variable names and the environment
    # such that if the underlying value of a trigger entityVARIABLE changes
    # we modify the trigger and sensor to account for this change.
    #-----------------------------------------------------------------------
    def handleBindingChange(self, oldPath, newPath):
        self.sensor.handleBindingChange(oldPath, newPath)
