#
# @(#) xmlWriter.py 2005/11/11
# @author SJL Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

from xml.sax import saxutils
from common import behaviorNames
from xml.sax.saxutils import XMLGenerator
import StringIO
import xmlLogger
from sets import Set

from cat import behaviorInfo
from bindings import Bindings


def makeSlotsXML():
    return "\t<slots> </slots>\n"

#writes netspec to filePath if provided and returns string
def generateNetSpecXML(netSpec, filePath = None):
    outfile = StringIO.StringIO()
    logger = xmlLogger.xml_logger(outfile)

    logger.OpenTag('network',{'name':netSpec.name,'ownship':netSpec.ownship},True)

    for bspec in netSpec.behaviors:
        logger.OpenTag('behavior',{'name':bspec.name,'id':bspec.id,'group':bspec.group},True)

        if bspec.targets:
            logger.WriteEntry('target', {}, bspec.targets[0])

        logger.OpenTag('parameters',{},True)
        for param in bspec.parameters:
            logger.WriteEntry(str(param.type),{'name':param.name,'min':str(param.min),'max':str(param.max)},str(param.val))
        logger.CloseTag('parameters')

        logger.OpenTag('triggers', {}, True)
        for t in bspec.triggers:
            logger.OpenTag('trigger', {'name':t.name, 'type':t.type, 'id':str(t.id)}, True)
            if t.subtype:
                logger.WriteEntry('subtype', {}, t.subtype)
            if t.toPath:
                logger.WriteEntry('to', {}, t.toPath)
            if t.fromPath:
                logger.WriteEntry('from', {}, t.fromPath)
            if t.threshold:
                logger.WriteEntry('threshold', {}, str(t.threshold))
            logger.CloseTag('trigger')
        logger.CloseTag('triggers')

        #logger.OpenTag('sensors',{},True)
        #for sensor in bspec.sensors.pre:
        #    logger.OpenTag('pre',{'name':sensor.name,'id':sensor.id,'type':sensor.type},True)
        #    for param in sensor.params:
        #        logger.WriteEntry('parameter',{'name':param.name,'type':param.type,'value':param.value},'')
        #    logger.CloseTag('pre')
        #for sensor in bspec.sensors.post:
        #    logger.OpenTag('post',{'name':sensor.name,'id':sensor.id,'type':sensor.type},True)
        #    for param in sensor.params:
        #        logger.WriteEntry('parameter',{'name':param.name,'type':param.type,'value':param.value},'')
        #    logger.CloseTag('post')
        #logger.CloseTag('sensors')

        logger.OpenTag('bindings',{},True)
        for binding in bspec.bindings:
            logger.WriteEntry('binding',{'name':binding.name,'path':binding.val},'')
        logger.CloseTag('bindings')

        logger.OpenTag('links',{},True)
        for link in bspec.links:
            logger.WriteEntry('to',{'type':link.type, 'trigid':str(link.triggerid)}, link.dest)
        logger.CloseTag('links')

        logger.CloseTag('behavior')

    logger.CloseTag('network')

    strval = outfile.getvalue()
    outfile.close()

    if filePath:
        fp = open(filePath,'w')
        fp.write(strval)

    return strval

def generateXML(net, filepath, netname, ownship):
    outfile = StringIO.StringIO()
    logger  = xmlLogger.xml_logger(outfile)
    logger.OpenTag('network', {'name':netname, 'ownship':ownship}, True)
    slots = Set()
    for key, netInfo in sorted(net.iteritems()):
        bname, params, start, end, path, bid, preTriggers, postTriggers = netInfo
        bindings = Bindings(bid)
        slots.clear()
        #val = [name, p, s, e]
        #        print "Val: ", str(val)
        #bname, params, start, end = val
        logger.OpenTag('behavior', {'name': bname, 'id': key, 'group':'0'}, True)
        
        if path:
            targetbind = bindings.addBinding('target', path)
            logger.WriteEntry('target', {}, targetbind)
            slots.add(targetbind)
        logger.OpenTag('parameters', {}, True)
        for pname, pinfo in params.iteritems():
            value, type, min, max = pinfo
            #            print "XMLGenerator: ", pname, str(pinfo)
            logger.WriteEntry(type, {'name':pname, 'min':str(min), 'max':str(max)}, str(value))
        logger.CloseTag('parameters')
        logger.OpenTag('sensors', {}, True)
        for t in preTriggers:
            logger.OpenTag('pre', {'name':t.name, 'id':t.id, 'type':t.type}, True)
            for p in t.parameters:
                val = p.val
                if p.type == 'ent':
                    val = bindings.addBinding(p.name, p.val)
                    slots.add(val)
                logger.WriteEntry('parameter', {'name':p.name, 'type':p.type, 'value':str(val)}, '')
            logger.CloseTag('pre')
        for t in postTriggers:
            logger.OpenTag('post', {'name':t.name, 'id':t.id, 'type':t.type}, True)
            for p in t.parameters:
                val = p.val
                if p.type == 'ent':
                    val = bindings.addBinding(p.name, p.val)
                    slots.add(val)
                logger.WriteEntry('parameter', {'name':p.name, 'type':p.type, 'value':str(val)}, '')
            logger.CloseTag('post')
        logger.CloseTag('sensors')

        logger.OpenTag('bindings', {}, True)
        i = 0
        #for path in paths:
        #if path:
        #    logger.WriteEntry('binding', {'id' : str(bid) + "_" + str(i),  'path': path}, '')
        #    i = i + 1

        for binding in slots:
            path = bindings.lookupPath(binding)
            logger.WriteEntry('binding', {'name':binding, 'path':path}, '')
                                       
        logger.CloseTag('bindings')

        logger.OpenTag('links', {}, True)
        s1 = float(start)
        e1 = float(end)
        for k2, v2 in sorted(net.iteritems()):
            if before(key, k2):
                n, p, s, e, path2, bid2, pre, post = v2
                s2 = float(s)
                e2 = float(e)
                if e1 >= e2:
                    logger.WriteEntry('to', {'type':'permanent'}, str(k2))
                elif e1 < e2 and e1 > s2:
                    logger.WriteEntry('to', {'type':'enabling'}, str(k2))
                elif e1 <= s2:
                    logger.WriteEntry('to', {'type':'ordering'}, str(k2))

        # link stuff here
        logger.CloseTag('links')
        logger.CloseTag('behavior')
    logger.CloseTag('network')
    strval = outfile.getvalue()
    outfile.close()
    return strval

def before(k1, k2):
    name1, time1 = k1.split(behaviorInfo.TIME_SEP_CHARACTER)
    name2, time2 = k2.split(behaviorInfo.TIME_SEP_CHARACTER)
    #    print "Times: ", time1, time2
    return (float(time1) < float(time2))



################ old code#######################################

def makeBehaviorXML(id, bname, params):
    bs = '\t<behavior name=%s id=%s>\n'% (saxutils.quoteattr(bname), saxutils.quoteattr(id))
    triggerXml = makeTriggerXML()
    paramXml   = makeParamDictXML(params)
    bs = bs + triggerXml + paramXml
    bs = bs + "\t</behavior>\n"
#    print bs
    return bs

def makeTriggerXML():
    return '\t<triggers> </triggers>\n'

def makeParamDictXML(params):
    pds = ''
    pds = pds + '\t\t<parameters>\n'
    for key, val in params.iteritems(): 
        pds = pds + makeParamXML(key, val)
        #        print pds
    pds = pds + '\t\t</parameters>\n'
    return pds
    
def makeParamXML(key, val):
    ps = '\t\t\t<%s name=%s  min=%s max=%s> %s </%s>\n' % (str(val[1]), saxutils.quoteattr(str(key)),  saxutils.quoteattr(str(val[2])), saxutils.quoteattr(str(val[3])), str(val[0]), str(val[1]) )
    #       print ps
    return ps
