#
# @(#) GUIManager.py 2005/07/14
# @author Copyright (c) 2005 The University of Nevada, Reno. All
# rights reserved.
#
# This software is provided "AS IS," without a warranty of any
# kind. ALL EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND
# WARRANTIES, INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE OR NON-INFRINGEMENT, ARE HEREBY
# EXCLUDED. THE UNIVERSITY OF NEVADA, RENO AND ITS LICENSORS SHALL NOT BE
# LIABLE FOR ANY DAMAGES SUFFERED BY LICENSEE AS A RESULT OF USING,
# MODIFYING OR DISTRIBUTING THE SOFTWARE OR ITS DERIVATIVES. IN NO
# EVENT WILL THE UNIVERSITY OF NEVADA, RENO OR ITS LICENSORS BE LIABLE FOR
# ANY LOST REVENUE, PROFIT OR DATA, OR FOR DIRECT, INDIRECT, SPECIAL,
# CONSEQUENTIAL, INCIDENTAL OR PUNITIVE DAMAGES, HOWEVER CAUSED AND
# REGARDLESS OF THE THEORY OF LIABILITY, ARISING OUT OF THE USE OF OR
# INABILITY TO USE SOFTWARE, EVEN IF THE UNIVERSITY OF NEVADA, RENO HAS BEEN
# ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#

from npython import *
from common.defs import *
from common.log import Log
import wx
import sys
import nEventServer
from common import EventNames
from gamestate import gameState

try:
    import cat
except:
    printExcept('importing cat')
from wxPython.wx import *
log = Log(tags=['all', 'GUIManager'])


ID_EXEC = 2323101

class MainFrame(wx.Frame):
    """
    The Main frame, to let everyone know that we are here.
    """

    def __init__(self, parent, id):
        """
        Constuctor - open the window, and manage child panels.
        """

        from common import WindowManager 
        windowName                      = 'GUIManager'
        WindowManager.register          (windowName, self)
        position,size                   = WindowManager.get(windowName)


        frame = wx.Frame.__init__(self, parent, id, windowName, pos=position, size=size, style=wx.DEFAULT_FRAME_STYLE & wx.FRAME_TOOL_WINDOW)
#        self.CreateStatusBar()
#        self.SetStatusText("Ready to execute")
        self.infoSizer = wx.BoxSizer(wx.VERTICAL)

        menubar = wx.MenuBar()
        self.SetMenuBar(menubar)

        #setup file menu items
        fileMenu = wx.Menu()
        fileMenu.Append(5473, '&Save', 'Save the game state')
        fileMenu.Append(5474, '&Load', 'Load the game state')
        fileMenu.Append(5475, '&Exit', 'Exit')

        wx.EVT_MENU(self, 5473, self.OnSave)
        wx.EVT_MENU(self, 5474, self.OnLoad)
        wx.EVT_MENU(self, 5475, quit)
        menubar.Append(fileMenu, '&File')
	
        # Create cove comms for Lagoon
        #try:
            #import comms
            #coveClient = comms.getGUI(self, (0, 0))
        #except:
            #printExcept('No cove communications')

        import executive
        execReady = executive.getGUI(self, (0,0))

#       Create panels for Bulba

        WindowManager.register ("SP", self)
        p, s = WindowManager.get("SP")
        try:
            import sp
            spPanel = sp.getGUI(self, s)
        except:
            printExcept('loading sp panel')
            spPanel = wx.Panel(self, -1, name="SP", size=s)
        spPanel.SetBackgroundColour("RED")                        

	#######################

        WindowManager.register ("CAT1", self)
        p, s = WindowManager.get("CAT1")
        try:
            import cat
            #from cat.Interface import manager
            catPanel1 = cat.getGUI(self, s)
            flagmenu1 = wx.Menu()
            flagmenu1.Append(catPanel1.ID_START, '&Start Scenario', 'Start Scenario', wx.ITEM_CHECK)
            flagmenu1.Append(catPanel1.ID_QUERY, '&Target Query', 'Target Query', wx.ITEM_CHECK)
            #wx.EVT_MENU(self, catPanel1.ID_START, manager.SetStartScenario)
            #wx.EVT_MENU(self, catPanel1.ID_QUERY, manager.SetTargetQuery)
            menubar.Append(flagmenu1, "&Flags")
        except:
            printExcept('loading cat panel 1')
            catPanel1 = wx.Panel(self, -1, name="CAT1", size=s)
        catPanel1.SetBackgroundColour(wx.Colour(128, 128, 248))
	

        # Create cove comms for Lagoon, pass in catPanel1 so coveClient can get 
        try:
            import comms
            coveClient = comms.getGUI(self, (0, 0), catPanel1)
        except:
            printExcept('No cove communications')


	
    	print('OK so far 1')
        
        WindowManager.register ("3d", self)
        p, s = WindowManager.get("3d")

        graphicsPanel = wx.Panel(self, -1, name="Graphics", style=wx.TRANSPARENT_WINDOW, size=s)
        graphicsPanel.SetBackgroundColour("PURPLE")

        #CHRIS - added the new toolmenu, which allows you to open the various tool widgets
        toolmenu                        = wx.Menu()
        id = 1534
        toolmenu.Append                 ( id, 'Option Explorer', 'Show / Edit Command Line Options')
        self.Bind                       (wx.EVT_MENU, self.openOptionExplorer, id=id)
        id += 1
        toolmenu.Append                 ( id, 'Project Explorer', 'Shows all the project files loaded')
        self.Bind                       (wx.EVT_MENU, self.openProjectExplorer, id=id)
        id += 1
        toolmenu.Append                 ( id, 'EVA Explorer', 'Navigate the EVA Tree')
        self.Bind                       (wx.EVT_MENU, self.openEvaExplorer, id=id)
        id += 1
        toolmenu.Append                 ( id, 'Sound Explorer', 'Play available sounds')
        self.Bind                       (wx.EVT_MENU, self.openSoundExplorer, id=id)
        id += 1
        toolmenu.Append                 ( id, 'IM Explorer', 'Play available sounds')
        self.Bind                       (wx.EVT_MENU, self.openIMExplorer, id=id)
        id += 1

        menubar.Append                  ( toolmenu, 'Tools')

        titleMenu = wx.Menu()
        menubar.Append(titleMenu, "                                                                                         Plan/Controller Authoring Tool (PCAT)")

        useGrid = False
        if not useGrid:
            box1 = wx.BoxSizer(wx.VERTICAL)
            box1.Add(graphicsPanel, 1)
            box1.Add(spPanel, 2)

            box2 = wx.BoxSizer(wx.HORIZONTAL)
            box2.Add(box1, 0)

            box3 = wx.BoxSizer(wx.VERTICAL)
            box3.Add(catPanel1, 0)

            line = wx.StaticLine(self, -1, size=(20,-1), style=wx.LI_HORIZONTAL)
            box3.Add(line, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.RIGHT|wx.TOP, 5)
            
#            box3.Add(triggerPanel,0)
	    
	    #box4 = wxBoxSizer(wx.VERTICAL)
	    #box4.Add(catPanel1, 0)
	    
	    box2.Add(box3, 0)
	    #box2.Add(box4, 0)

#             import minimap.minimap
#             minimap = minimap.minimap.MiniMap(self, -1)
#             box2.Add(minimap, 0)

            self.SetAutoLayout(True)
            self.SetSizer(box2)
            self.Show(True)
        else:
            import minimap.minimap
            minimap = minimap.minimap.MiniMap(self, -1)
            gbs = wx.GridBagSizer(0,0)
            gbs.Add                         (graphicsPanel,(0,0), flag=wx.EXPAND)
            gbs.Add                         (catPanel,     (0,1), flag=wx.EXPAND)
            gbs.Add			    (catPanel1,    (0,1), flad=wx.EXPAND)
	    gbs.Add                         (spPanel,      (1,0), flag=wx.EXPAND)
            gbs.Add                         (minimap,      (1,1), flag=wx.EXPAND)
            gbs.AddGrowableRow(0)
            gbs.AddGrowableCol(0)

            self.SetSizer(gbs)
            self.Show(True)
    #/-----------------------------------------------------------------------------//
    def openOptionExplorer(self, event):
        print 'openOptionExplorer'
        from common import optionexplorer
        optionexplorer.open()
    #/-----------------------------------------------------------------------------//
    def openProjectExplorer(self, event):
        print 'openOptionExplorer'
        from common import projectexplorer
        projectexplorer.open()
    #/-----------------------------------------------------------------------------//
    def openEvaExplorer(self, event):
        print 'openOptionExplorer'
        import evaexplorer 
        evaexplorer.open()
    #/-----------------------------------------------------------------------------/
    def openSoundExplorer(self, event):
        print 'openSoundExplorer'
        from common import soundexplorer
        soundexplorer.open()
    #/-----------------------------------------------------------------------------/
    def openIMExplorer(self, event):
        import im
        im.im.frame.Show()
    #/-----------------------------------------------------------------------------//
    def OnSave(self, event):
        '''
        Save game state dialog.
        '''
        # Save File dialog
        parentDialog = wx.Dialog(self, size=(0,0), pos=(-1,-1), style=wx.NO_BORDER|wx.TRANSPARENT|wx.STAY_ON_TOP)
        fileDialog = wx.FileDialog(parentDialog,'Choose save file name.', style=wx.SAVE|wx.OVERWRITE_PROMPT)
        parentDialog.Show()

        # If user selects a file
        if fileDialog.ShowModal() == wx.ID_OK:
            # Pass file name to gameState
            gameState.save(fileDialog.GetPath())

        # Destroy dialogs
        fileDialog.Destroy()
        parentDialog.Destroy()

    #/-----------------------------------------------------------------------------//
    def OnLoad(self, event):
        '''
        Load game state dialog.
        '''
        # Open File dialog
        parentDialog = wx.Dialog(self, size=(0,0), pos=(-1,-1), style=wx.NO_BORDER|wx.TRANSPARENT|wx.STAY_ON_TOP)
        fileDialog = wx.FileDialog(parentDialog, 'Choose file name to load.', style=wx.OPEN)
        parentDialog.Show()

        # If user selects a file
        if fileDialog.ShowModal() == wx.ID_OK:
            # Pass event to gameState
            gameState.load(fileDialog.GetPath())

        # Destroy dialogs
        fileDialog.Destroy()
        parentDialog.Destroy()

    #/-----------------------------------------------------------------------------//
