from npython import *
from common.defs import *
from common.log import Log
log = Log(tags=['all', 'optionExplorer'])
import wx
import options
import OptionManager
import wx.lib.foldpanelbar as fpb
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
class OptionExplorer(wx.Panel):
    """The option explorer shows all the command line options in a nice gui, and presents easy way for users to modify them
    The main purpose of this is to allow people who are not chris miles to know all of the options, and use them appropriately
    """
    idToOptionMap = {}
    #/-----------------------------------------------------------------------------//
    def __init__(self, parent, id):
        wx.Panel.__init__(self, parent, id)
        #sizer = wx.GridBagSizer(2,2)
        #sizer.AddGrowableCol(0)
        #sizer.AddGrowableCol(1)
        #row = 0
        category                        = None
        self.pnl                        = fpb.FoldPanelBar(self, -1, wx.DefaultPosition, wx.DefaultSize, fpb.FPB_DEFAULT_STYLE, fpb.FPB_VERTICAL)
        style                           = fpb.CaptionBarStyle()
        style.SetCaptionStyle           (fpb.CAPTIONBAR_GRADIENT_V)

        for optionName in OptionManager.orderedOptions:
        #for option in OptionManager.options.values():
            option                      = OptionManager.options[optionName]
            if option.category != category:
                categoryPnl             = self.pnl.AddFoldPanel(option.category, collapsed=True)
                category                = option.category
                #pnl.AddFoldPanelSeperator( categoryPnl )

            #first a text control describing the option
            optionPanel                 = wx.Panel( categoryPnl, -1 )
            sizer                       = wx.GridBagSizer(2,2)
            sizer.AddGrowableCol(0)
            #sizer.AddGrowableCol(1)
            sizer.AddGrowableRow(0)
            sizer.AddGrowableRow(1)
            row                         = 0

            #chop the description up into different lines
            #subdesc = option.description
            #linedDesc = ''
            #maxCharsPerLine = 40
            #while subdesc:
                #linedDesc += subdesc[:maxCharsPerLine] + '\n'
                #subdesc = subdesc[maxCharsPerLine:]
            #optionDescr                 = wx.StaticText(optionPanel, -1, label=linedDesc)
            optionDescr                 = wx.TextCtrl( optionPanel, -1, value=option.description, style=wx.TE_MULTILINE, size=(10,40))
            optionDescr.SetEditable     ( False )
            sizer.Add                   ( optionDescr, (row, 0), (1,2), flag=wx.EXPAND)
            row                         += 1

            #add a simple text ctrl for the name
            optionName                  = wx.TextCtrl(optionPanel, -1, value=option.name, size=(150,20))
            optionName.SetEditable      ( False )
            sizer.Add                   ( optionName, (row,0), flag=wx.EXPAND)
            #if its a boolean option add a checkbox
            if option.possibilities == [0,1]:
                optionCheck             = wx.CheckBox(optionPanel, -1)
                if option.value == None:optionCheck.SetValue( 0 )
                else:                   optionCheck.SetValue( option.value )
                sizer.Add               ( optionCheck, (row,1), flag=wx.EXPAND)
                self.Bind               (wx.EVT_CHECKBOX, self.onCheckOptionChanged, optionCheck)
                self.idToOptionMap[optionCheck.GetId()] = option.name
            #if its an option with defined choices add a choice
            elif option.possibilities == []:
                optionText              = wx.TextCtrl(optionPanel, -1, size=(150,20))
                if option.value == None:  optionText.SetValue('')
                else:                     optionText.SetValue( str(option.value) )
                sizer.Add               ( optionText, (row,1), flag=wx.EXPAND)
                self.Bind               (wx.EVT_TEXT, self.onStringOptionChanged, optionText)
                self.idToOptionMap[optionText.GetId()] = option.name
            #otherwise allow a generic text ctrl so they can enter whatever
            else:
                optionChoice            = wx.Choice(optionPanel, -1, choices=option.possibilities)
                sizer.Add               ( optionChoice, (row, 1), flag=wx.EXPAND)
                self.Bind               (wx.EVT_CHOICE, self.onStringOptionChanged, optionChoice)
                self.idToOptionMap[optionChoice.GetId()] = option.name
            optionPanel.SetSizerAndFit( sizer )
            self.pnl.AddFoldPanelWindow  ( categoryPnl, optionPanel )
            self.pnl.AddFoldPanelSeparator(categoryPnl)

        size = self.GetClientSize()
        self.pnl.SetDimensions(0, 0, 400, 1006)
    #/-----------------------------------------------------------------------------//
    def onCheckOptionChanged(self, event):
        optionName = self.idToOptionMap[event.GetId()]
        OptionManager.setOption( optionName, int(event.GetInt()), 3)
    #/-----------------------------------------------------------------------------//
    def onStringOptionChanged(self, event):
        optionName = self.idToOptionMap[event.GetId()]
        optionValue = event.GetString()
        try:
            optionValue = int(optionValue)
        except:
            pass
        if not optionValue:
            optionValue = None
        OptionManager.setOption( optionName, optionValue, 3)
    #/-----------------------------------------------------------------------------//
    def OnSize(self, event):
        print 'OnSize'
        #wx.LayoutAlgorithm().LayoutWindow(self, None)
        size = self.GetClientSize()
        print 'size', size
        self.pnl.SetDimensions(0, 0, size.GetWidth(), size.GetHeight())
    #/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
def open():
    """Open an option explorer window"""
    global optionExplorer
    frame                               = wx.Frame(None,-1,title='Option Explorer')
    optionExplorer                      = OptionExplorer(frame, -1)
    frame.Show()
#/-----------------------------------------------------------------------------//
