from npython import *
from common.defs import *
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
class IM:
    """wraps a corresponding nim class"""
    autoAddStack = []
    #/-----------------------------------------------------------------------------//
    def __init__(self, name, path=None, cClass='nim', behavior='none'):
        self.children = {}
        self.name = name
        self.cClass = cClass
        self.behavior = behavior
        #print 'path', path, psel()
        if path:
            if path[0] != '/':
                path = psel() + '/' + path
            self.path = path
            self.createNIM(self.path)
        else:
            self.path = None
        #print 'self.path', self.path

        if len(self.autoAddStack) > 0:
            self.autoAddStack[-1].append(self)
    #/-----------------------------------------------------------------------------//
    def append(self, child):
        """add a child IM"""
#        print 'im.append', self, child
        self.children[child.name] = child
        assert self.path
        child.createNIM('%s/%s' % (self.path, child.name))
    #/-----------------------------------------------------------------------------//
    def createNIM(self, path):
        """instantiate my c++ nIM partner"""
        self.path = path
#        print 'new', self.cClass, path
        new(self.cClass, path)
        pushsel(self.path)
        self.update()
        popsel()
    #/-----------------------------------------------------------------------------//
    def add(self):
        self.autoAddStack.append(self)
        return self
    #/-----------------------------------------------------------------------------//
    def done(self):
        item = self.autoAddStack.pop()
        assert item == self, 'problems with nim add/done'
    #/-----------------------------------------------------------------------------//
    def update(self):
        call('.setbehavior', self.behavior)
    #/-----------------------------------------------------------------------------//
    def __str__(self):
        return '%s->%s' % (self.name, self.path)
    #/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
class IMRoot(IM):
    def __init__(self, name, path):
        IM.__init__(self, name, cClass='nent')
        self.createNIM(path)
        pushsel(self.path)
        call('.inheritfrom', '..')
        popsel()
    #/-----------------------------------------------------------------------------//
    def assignTo(self, group):
        print 'nIMRoot(%s).assignTo(%s)' % (self, group)
        call('%s.cref' % self.path, 'myGroup', '/comm/groups/%s' % group)
    #/-----------------------------------------------------------------------------//
    def update(self):
        pass
#/-----------------------------------------------------------------------------//
class IMObjective(IM):
    def __init__(self, name, path=None, powerCoef=0.0, valueCoef=0.0, speedCoef=0.0, maxUnits=0):
        self.powerCoef = powerCoef
        self.valueCoef = valueCoef
        self.speedCoef = speedCoef
        self.maxUnits = maxUnits
        IM.__init__(self, name, cClass='nimobjective', path=path)
    def update(self):
        IM.update(self)
        call('.setpowercoef', self.powerCoef)
        call('.setvaluecoef', self.valueCoef)
        call('.setspeedcoef', self.speedCoef)
        call('.setmaxunits', self.maxUnits)
#/-----------------------------------------------------------------------------//
SUM = 'SUM'
SUB = 'SUM'
MUL = 'MUL'
DIV = 'DIV'
MAX = 'MAX'
MIN = 'MIN'
#/-----------------------------------------------------------------------------/
NORMAL = 'NORMAL'
BINARIZE = 'BINARIZE'
class IMOperator(IM):
    def __init__(self, name, operator, mode='NORMAL', behavior='none'):
        self.operator = operator
        self.mode = mode
        IM.__init__(self,name, cClass='nimoperator', behavior=behavior)
    def update(self):
        IM.update(self)
        call('.setoperator', self.operator)
        call('.setmode', self.mode)
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
VALUE_OVER_THREAT                       = 'VALUE_OVER_THREAT'
POWER                                   = 'POWER'
LINEAR                                  = 'LINEAR'
INV_LINEAR                              = 'INV_LINEAR'
FLAT                                    = 'FLAT'
AREA                                    = 'AREA'
WEAPON                                  = 'WEAPON'
COLLISION_AVOIDANCE_RADIUS              = 'COLLISION_AVOIDANCE_RADIUS'
#MY_WEAPON                               = 'MY_WEAPON'
ENEMIES                                 = 'ENEMIES'
NEUTRALS                                = 'NEUTRALS'
FRIENDS                                 = 'FRIENDS'

MINIMIZE_ONLY                           = 'MINIMIZE_ONLY'
MAXIMIZE_ONLY                           = 'MAXIMIZE_ONLY'
#ME                                      = 'ME'
#/-----------------------------------------------------------------------------//
class IMEnt(IM):
    def __init__(self, name, func, ipol, radius, who, coef=1.0, radiusCoef=1.0, behavior='none', cClass='niment'):
#        print 'IMEnt(%s,%s,%s,%s,%s,%f)' % (name, func, ipol, radius, who, coef)
        self.func                       = func
        self.ipol                       = ipol
        self.radius                     = radius
        self.who                        = who
        self.coef                       = coef
        self.radiusCoef                 = radiusCoef
        IM.__init__                     (self, name, cClass=cClass, behavior=behavior)
    def update(self):
        IM.update(self)
        call('.setfunc', self.func)
        call('.setipol', self.ipol)
        call('.setradius', self.radius)
        call('.setwho', self.who)
        call('.setcoef', self.coef)
        call('.setradiuscoef', self.radiusCoef)
#/-----------------------------------------------------------------------------//
class IMRadial(IMEnt):
    def __init__(self, name, func, ipol, radius, frm, to, coef=1.0, radiusCoef=1.0, thetaOffset=0, thetaWidth=30, behavior='none', cClass='nimradial'):
        self.to                         = to
        self.thetaOffset                = thetaOffset
        self.thetaWidth                 = thetaWidth
        IMEnt.__init__(self, name=name, func=func, ipol=ipol, radius=radius, who=frm, coef=coef, radiusCoef=radiusCoef, behavior=behavior, cClass = cClass)
    #/-----------------------------------------------------------------------------//
    def update(self):
        IMEnt.update(self)
        call('.setto', self.to)
        call('.setthetaoffset', self.thetaOffset)
        call('.setthetawidth', self.thetaWidth)
#/-----------------------------------------------------------------------------//
class IMVisibility(IMEnt):
    def __init__(self, name, who, ipol=LINEAR, coef=1.0, radiusCoef=1.0, behavior='none', cClass='nimvisibility'):
        IMEnt.__init__(self, name=name, func=POWER, ipol=LINEAR, radius=AREA, who=who, coef=coef, radiusCoef=radiusCoef, behavior=behavior, cClass = cClass)
    #/-----------------------------------------------------------------------------//
    def update(self):
        IMEnt.update(self)
#/-----------------------------------------------------------------------------//
class IMTerrain(IM):
    def __init__(self, name):
        IM.__init__                     (self, name=name, cClass='nimterrain', behavior='none')
    #/-----------------------------------------------------------------------------//
    def update(self):
        IM.update(self)
#/-----------------------------------------------------------------------------//
class IMGaussianBlur(IM):
    def __init__(self, name, radius = 3.0, mode=NORMAL):
        self.radius                     = radius
        self.mode                       = mode
        IM.__init__                     (self, name=name, cClass='nimgaussianblur', behavior='none')
    #/-----------------------------------------------------------------------------//
    def update(self):
        IM.update(self)
        call('.setmode',                self.mode)
        call('.setradius',              self.radius)
#/-----------------------------------------------------------------------------//
