from npython import *
from common.defs import *
import wx
global miniMap
miniMap = None
backgroundImage = None
#/-----------------------------------------------------------------------------//
class MiniMap(wx.Panel):
    """Displays a nice minimap of the world
    If I was very clever I would get it to take a screen shot of the map from above and bind that in, but for now we pull it manually
    """
    def __init__(self, parent, id, size=(250,250)):
        global miniMap
        miniMap = self
        wx.Panel.__init__(self, parent, id, size=size)
        self.w = size[0]
        self.h = size[1]
        self.SetBackgroundColour(wx.BLACK)
        self.Bind(wx.EVT_PAINT, self.OnPaint)

        from common import tickmanager
        tickmanager.register( 'MiniMap', self.tick )
        self.worldMin = (0,0,0)
        self.worldMax = call('/map.gf', 'worldsize')
        self.worldMax = (self.worldMax, 1, self.worldMax)
        print 'self.worldMin', self.worldMin, 'self.worldMax', self.worldMax
        self.r = self.worldMax[0] - self.worldMin[0], self.worldMax[1] - self.worldMin[1] + 1, self.worldMax[2] - self.worldMin[2]
        self.mapBackground = None
        global backgroundImage
        if backgroundImage:
            self.setBackgroundImage(backgroundImage)
        self.Bind(wx.EVT_LEFT_DOWN, self.onMouseEvent)
        self.Bind(wx.EVT_MOTION, self.onMouseEvent)
    #/-----------------------------------------------------------------------------//
    def OnPaint(self, event):
        """draw our data to the screen"""
        dc = wx.PaintDC(self)
        #dc.Clear()
        dc.BeginDrawing()
        if self.mapBackground:
            dc.DrawBitmap(self.mapBackground,0,0)
        rectangles = []
        entityTypes = ndirin('/game/ents')
        for type in entityTypes:
            ents = ndirin('/game/ents/%s' % type)
            for ent in ents:
                path = '/game/ents/%s/%s' % (type,ent)
                if call('%s.getselectable' % path):
                    side = call('%s.getname' % call('%s.gref' % path, 'side'))
                    if side == 'blue':
                        dc.SetPen(wx.Pen('BLUE', 2))
                    if side == 'red':
                        dc.SetPen(wx.Pen('RED', 2))
                    if side == 'green':
                        dc.SetPen(wx.Pen('GREEN', 2))
                    if side == 'yellow':
                        dc.SetPen(wx.Pen('YELLOW', 2))
                    if side == 'neutral':
                        dc.SetPen(wx.Pen('GREY', 2))
                        
                    pos = call('%s.gv' % path, 'pos')
                    dp = pos[0]-self.worldMin[0],                               pos[1]-self.worldMin[1],                pos[2]-self.worldMin[2]
                    dp = dp[0] / self.r[0],                                     dp[1] / self.r[1],                      dp[2] / self.r[2]
                    dp = map(lambda(x):1.0-x, dp)
                    dc.DrawRectangle    (dp[0]*self.w, dp[2]*self.h, 3, 3)



        dc.SetPen(wx.Pen('WHITE', 1))
        pos = call('%s.gv' % '/game/camera', 'pos')
        cp = pos[0]-self.worldMin[0], pos[1]-self.worldMin[1], pos[2]-self.worldMin[2]
        cp = cp[0] / self.r[0], cp[1] / self.r[1], cp[2] / self.r[2]
        cp = map(lambda(x):1.0-x, cp)
        dc.DrawRectangle (cp[0]*self.w, cp[2]*self.h, 1, 1)
        points = []

        import math
        tp = [0,0,0]
        heading = call('/game/camera.gf', 'heading')
        heading += 3.1415
        fov = call('/sys/share/windowsettings.gf', 'fieldOfView')
        aspectRatio = call('/sys/share/windowsettings.gf', 'aspectRatio')
        fov = ( math.asin(fov) * aspectRatio ) 

        tp[0] = cp[0] + math.sin(heading + fov)
        tp[2] = cp[2] + math.cos(heading + fov)
        dc.DrawLine (cp[0]*self.w, cp[2]*self.h, tp[0]*self.w, tp[2]*self.h)
        tp[0] = cp[0] + math.sin(heading - fov)
        tp[2] = cp[2] + math.cos(heading - fov)
        dc.DrawLine (cp[0]*self.w, cp[2]*self.h, tp[0]*self.w, tp[2]*self.h)
        #dc.DrawLine (cp[0]*self.w, cp[2]*self.h, cp[0]*self.w, cp[2]*self.h)
        dc.EndDrawing()
    #/-----------------------------------------------------------------------------//
    redrawFreq                          = 0.2                                   #call .assign this often
    timeLastDrawn                       = -1.0
    #/-----------------------------------------------------------------------------//
    def tick(self, time, dtime):
        """our tick function (registered in __init__) - called every slice"""
        if time - self.timeLastDrawn > self.redrawFreq:
            self.timeLastDrawn = time
            self.Refresh()
    #/-----------------------------------------------------------------------------//
    def setBackgroundImage(self, filename):
        print 'setBackgroundImage', filename
        from common import FileManager
        file = FileManager.find( 'tex', filename)
        if not file:
            print 'failed to find minimap background', filename
            return
        else:
            print 'found mapbackgground', filename, 'at', file
            
        self.mapBackground = wx.Image(file, wx.BITMAP_TYPE_BMP)
        print 'ok', self.mapBackground.Ok()
        size = self.GetSize()
        self.mapBackground.Rescale(size[0], size[1])
        self.mapBackground = self.mapBackground.ConvertToBitmap()
    #/-----------------------------------------------------------------------------//
    def onMouseEvent( self, event ):
        if event.LeftDown() or event.Dragging():
            x, y = event.GetX(), event.GetY()                                   #get mouse pos
            px, py = self.w, self.h                                             #size of minimap window
            rx,ry = x / float(px), y / float(py)                                #relative % mouse position in window
            rx, ry = 1 - rx, 1 - ry                                             #0,0,0 is in lower right, not upper left
            mx, my, mz = call('/map.gf', 'worldsize'), 0, call('/map.gf', 'worldsize' ) #get world size
            wx, wy, wz = mx * rx, call('/game/camera.gv', 'pos')[1], mz * ry   #3d point in world
            #print 'onMouseEvent.event.LeftDown()', 'x, y', x, y, 'px, py', px, py, 'rx, ry', rx, ry, 'mx, my', mx, my, 'px, py', wx, wy, wz
            panToPoint( wx, wy, wz )
            event.Skip()
    #/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
def setBackgroundImage(filename):
    print 'setBackgroundImage', filename
    if not miniMap:
        global backgroundImage
        backgroundImage = filename
    else:
        miniMap.setBackgroundImage(filename)
#/-----------------------------------------------------------------------------//
