from npython import *
from common.defs import *
from common.log import Log
import nEventServer
from common import EventNames
log = Log(tags=['all', 'sp'])
from recorder import recorder
class GroupManager:
    """manages the creation / destruction and maintenace of groups"""
    #/-----------------------------------------------------------------------------//
    def __init__(self):
        self.groupSelected = None
        global groupManager
        groupManager = self
        nEventServer.register( EventNames.GROUPS_CHANGED,                       self.checkExists )
        nEventServer.register( EventNames.ENTITY_DESTROYED,                     self.entDied )
        nEventServer.register( EventNames.ENTITY_DESTROYED,                     self.checkExists )
        nEventServer.register( EventNames.UNIT_WITHOUT_GROUP,                   self.unitWithoutGroup )
        nEventServer.register( EventNames.ENTITY_CREATED,                        self.unitWithoutGroup ) #when an ent is created try to map it to a group
    #/-----------------------------------------------------------------------------//
    def entDied(self, msg):
        """an entity has been killed -> scan through my groups and remove any links to it"""
        print 'entDied', msg
        for group in ndirin('/comm/groups'):
            for ent in ndirin('/comm/groups/%s' % group):
                path = call('/comm/groups/%s/%s.gs' % (group, ent), 'path')
                if path == msg:
                    print 'removing unit %s from group %s' % (ent, group)
                    ndel('/comm/groups/%s/%s' % (group,ent))
    #/-----------------------------------------------------------------------------//
    def checkExists(self, msg=None):
        """check if our group entities still exist - erase any adhoc groups without units"""
        #print 'checkExists'
        log.open('checkExists')
        for group in ndirin('/comm/groups'):
            #print 'group', group, 'number of units', len(ndirin('/comm/groups/%s' % group) ) 
            if len(ndirin('/comm/groups/%s' % group) ) == 0 and call('/comm/groups/%s.gs' % group, 'grouptype') == 'adhoc':
                #print 'erasing empty adhoc group', group
                ndel('/comm/groups/%s' % group)
                nEventServer.pullLater(EventNames.GROUPS_CHANGED, 'ent died')
        log.close()
    #/-----------------------------------------------------------------------------//
    def findGroupFromIndex( self, index):
        #print 'findGroupFromIndex', index
        for group in ndirin('/comm/groups'):
            if hasvar('/comm/groups/%s' % group, 'groupIndex'):
                i = call('/comm/groups/%s.gi' % group, 'groupIndex')
                if i == index:
                    #print 'found group %s with index: %i' % (group, index)
                    return group
        return None
    #/-----------------------------------------------------------------------------//
    def findGroupsWithEnts(self, ents):
        """find the set of groups containing the ents"""
        #print 'findGroupsWithEnts', ents
        log.open('groupManager.findGroupsWithEnts', ents)
        groups = []
        pushsel()
        for ent in ents:
            #search through the groups to see if they contain this ent
            sel('/comm/groups')
            for group in ndirin():
                sel(group)
                log.write('checking if group', group, 'has ent', ent)
                for groupMember in ndirin():
                    log.write('groupMember', groupMember)
                    sel(groupMember)
                    path = gs('path')
                    log.write('has memeber', path)
                    if path == ent:
                        log.write('found matching ent', groupMember)
                        if group not in groups:
                            groups.append(group)
                    sel('..')
                sel('..')
        popsel()
        log.write('found groups', groups)
        log.close()
        return groups
    #/-----------------------------------------------------------------------------//
    def assignEnt(self, path, groupName = None, index = None):
        #print 'assignEnt', path
        recorder.assignEntToGroup( path=path, groupName=groupName, index=index)
        assert exists(path), 'trying to assign a non existant entity'
        if not call('%s.hasvar' % path, 'groupable') or call('%s.gi' % path, 'groupable') == 0:
            #print 'not assigning ungroupable ent', path, 'to a group'
            return


        #print 'groupManager.assignEnt', path, groupName
        entname = getname(path)
        if index:
            groupName = self.findGroupFromIndex( index )
            if groupName == None:
                groupName = 'Group%i' % index

        for group in ndirin('/comm/groups'):
            for ent in ndirin('/comm/groups/%s' % group):
                #print 'group:%s ent:%s' % (group, ent)
                if call('/comm/groups/%s/%s.gs' % (group,ent), 'path') == path:
                    #print 'removing %s from existing group' % path, groupName
                    ndel('/comm/groups/%s/%s' % (group, ent))

        type = 'standard'
        if not groupName:
            #print 'creating adhoc group for ent', path
            groupName = entname
            type = 'adhoc'

        if not exists('/comm/groups/%s' % groupName):
            new('nent', '/comm/groups/%s' % groupName)
            call('/comm/groups/%s.cs' % groupName, 'grouptype', type)
            call('/comm/groups/%s.cs' % groupName, 'label', groupName)
            if type != 'adhoc':
                if index == None:
                    for i in range(1,10)+[0]:
                        if self.findGroupFromIndex(index=i) == None:
                            index = i
                            break
            if index:
                call('/comm/groups/%s.ci' % groupName, 'groupIndex', index)
        
        #print 'adding ent %s to group %s' % (path, groupName)
        new('nent', '/comm/groups/%s/%s' % (groupName,entname))
        call('/comm/groups/%s/%s.cs' % (groupName, entname), 'path', path)
        call('%s.cref' % path, 'group', '/comm/groups/%s' % groupName)
        nEventServer.pullLater(EventNames.GROUPS_CHANGED, 'assignEnt')
    #/-----------------------------------------------------------------------------//
    def selectGroup(self, groupName=None, index=None):
        """select a group"""
        if index:
            groupName = self.findGroupFromIndex(index)
        from playermanager import playerManager
        if not playerManager.canControlGroup( groupName ):
            print 'warning cannot control group', groupName
            return False
        #print 'groupManager.selectGroup(%s)' % (groupName), index
        #idea -> check if we already ave something selected from the group? if not select the first ent in it
        alreadySelected = False
        entsSelected = call('/ui/planner.getobjectsselected')
        #print 'entsSelected'
        entsInGroup = []
        pushsel('/comm/groups/%s' % groupName)
        for ent in ndirin():
            entsInGroup.append( call('%s.gs' % ent, 'path'))

        #entsInGroup = ndirin('/comm/groups/%s' % groupName)
        if entsSelected:
            entsSelected = entsSelected.split(' ')
            for ent in entsSelected:
                if ent in entsInGroup:
                    alreadySelected = True
                    break
        if not alreadySelected:
            if groupName:                                                           #if the group exists
                call('/ui/planner.clearselection')                                  #how do we select it? we can just call select on it right - i believe so
                for ent in entsInGroup:
                    #print 'groupManager.selecting %s.select' % ent
                    call('%s.select' % ent)
                    break
            #print 'groupServer updating selection'
            nEventServer.pullLater(EventNames.SELECTION_CHANGED,'')
    #/-----------------------------------------------------------------------------//
    def panToGroup(self, groupName=None, index=None):
        if index:
            groupName = self.findGroupFromIndex(index)
        if groupName and exists('/comm/groups/%s' % groupName):
            paths = []
            for ent in ndirin('/comm/groups/%s' % groupName):
                path = call('/comm/groups/%s/%s.gs' % (groupName, ent), 'path')
                paths.append(path)
            panToEnts(paths)
        else:
            print 'failed to pan to non existant group %s' % groupName
    #/-----------------------------------------------------------------------------//
    def destroyGroup(self, groupName=None, index=None):
        if index:
            groupName = self.findGroupFromIndex(index)
        if groupName and exists('/comm/groups/%s' % groupName):
            for ent in ndirin('/comm/groups/%s' % groupName):
                path = call('/comm/groups/%s/%s.gs' % (groupName, ent), 'path')
                self.assignEnt( path=path, groupName=None )
            nEventServer.pullLater(EventNames.GROUPS_CHANGED, 'destroyGroup')
        else:
            print 'failed to destroy non existant group %s' % groupName
    #/-----------------------------------------------------------------------------//
    def getSelectedGroup(self):
        selectedEnt = call('/ui/planner.getobjectselected')
        if selectedEnt and selectedEnt != 'none':
            #do a cheesy search to figure out which group the selected unit belongs too
            groupCommanding = None
            for group in self.groups.itervalues():
                if selectedEnt in group.paths.values():
                    groupCommanding = group
            #assign it to its point
            if groupCommanding != None:
                return groupCommanding
            else:
                print 'entity', selectedEnt, 'is not assigned to a group'
        return None
    #/-----------------------------------------------------------------------------//
    def unitWithoutGroup(self, ent):
        #print 'groupManager.unitWithoutGroup', ent
        self.assignEnt(ent)
    #/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
groupManager = GroupManager()
