import os
from npython import *
from common.defs import *
from common.enum import *
import random
import nEventServer
from common                             import EventNames
from common import OptionManager
from sidemanager                        import sideManager
MAX_PLAYERS = 64
#/-----------------------------------------------------------------------------//
class PlayerManager:
    """manages the players and their info
    okay so in /game/players we keep a basicent for each player connected to the game
    ---- name
    ---- side
    ---- faction
    ---- human or ai
    ---- local or not

    also we mirror some of this information into /game/sides for each access
    /game/sides/$playerside/playerside = 1 
    othersides/playerside              = 0
    
    /game/players/localplayer  -> name of local player
    /game/players/localside    -> name of local side
    /game/players/localfaction -> faction of local player
    """
    #/-----------------------------------------------------------------------------//
    def __init__(self):
        new                   ( 'nbasicent', '/game/players')
        nEventServer.register ( EventNames.UPDATE_PLAYER,   self.updatePlayer )
        nEventServer.register ( EventNames.PLAYERS_CHANGED, self.playersChanged )
    #/-----------------------------------------------------------------------------//
    def createPlayerPath(self):
        """find a new / random name for a player"""
        for i in range(10):
            path = '/game/players/player%i' % random.randrange(0,10000)
            if not exists(path):
                return path
        assert i < 10, 'somehow failed to create a player name -> could happend randomly but extremely unlikely'
        return None
    #/-----------------------------------------------------------------------------//
    def createPlayer(self):
        """create a new player with a random name"""
        playerPath = self.createPlayerPath()
        print 'creating new player', playerPath
        new('nbasicent', playerPath )
        return playerPath
    #/-----------------------------------------------------------------------------//
    def createLocalPlayer(self):
        """create a new player and set it locally -> assign it sides / etc from options"""
        playerPath    = self.createPlayer()
        call          ('/game/players.cs', 'localPlayerPath', playerPath )
        playerName    = OptionManager.getOption('name')
        playerSide    = OptionManager.getOption('side')
        playerFaction = OptionManager.getOption('faction')

        if not playerName:
            playerName = os.getlogin()
        if not playerSide:
            playerSide = 'neutral'
        if not playerFaction:
            playerFaction = 'Space Marines'
            
        self.setLocalPlayerName   ( playerName )
        self.setLocalPlayerSide   ( playerSide )
        self.setLocalPlayerFaction( playerFaction )
        nEventServer.pullLater    ( EventNames.PLAYERS_CHANGED, 'createLocalPlayer' )
    #/-----------------------------------------------------------------------------//
    def setLocalPlayerName(self, playerName):
        """set a new name for this player
        also save it to the option file so it is loaded next time
        """
        playerPath                   = self.getLocalPlayerPath()
        print 'player %s.name:    %s' % ( playerPath, playerName)
        call                         ( '%s.cs' % playerPath, 'playerName', playerName)
        call                         ( '/game/players.cs', 'localPlayerName', playerName )
        OptionManager.setOption      ( 'name', playerName)
        nEventServer.pullLater       ( EventNames.PLAYERS_CHANGED, 'setLocalPlayerName')
    #/-----------------------------------------------------------------------------//
    def setLocalPlayerSide(self, sideName):
        """set a new side for this player
        also create the ui for playing that side -> display resources / etc
        """
        sel('/')
        print 'setLocalPlayerSide', sideName
        sidePath                     = '/game/sides/%s' % sideName
        playerPath                   = self.getLocalPlayerPath()
        print 'player %s.side:    %s' % ( playerPath, sidePath)
        call                         ( '%s.cs' % playerPath, 'playerSide', sidePath )
        call                         ( '/game/players.cs', 'localPlayerSide', sideName )
        call                         ( '/game/players.cs', 'localPlayerSidePath', sidePath )
        call                         ( '/game/players.cs', 'localPlayerSideType', call('%s.gs' % sidePath, 'sideType'))

        #delete any ui stuf for the old player
        if hasvar('/game/players', 'localPlayerSide'):
            oldPlayerSide = call('/game/players.gs', 'localPlayerSide')
            print         ( 'deleting GUI for side', oldPlayerSide)
            ndel          ( '/game/vis/ui/%s' % oldPlayerSide)

        #create a new gui for this player
        sideManager.createSideUI (sideName)

        nEventServer.pullLater               ( EventNames.SIDES_CHANGED, 'local:%s' % sideName)
        nEventServer.pullLater               ( EventNames.PLAYERS_CHANGED, 'setLocalPlayerSide')
    #/-----------------------------------------------------------------------------//
    def setLocalPlayerFaction(self, factionName):
        """assign a faction to the local player -> Just cosmetic at the moment
        """
        print 'setLocalPlayerFaction', factionName
        playerPath                   = self.getLocalPlayerPath( )
        print 'player %s.faction: %s' % ( playerPath, factionName)
        call                         ( '%s.cs' % playerPath, 'playerFaction', factionName)
        call                         ( '/game/players.cs', 'localPlayerFaction', factionName )
        nEventServer.pullLater       ( EventNames.PLAYERS_CHANGED, 'setLocalPlayerFaction')
    #/-----------------------------------------------------------------------------//
    def getLocalPlayerPath(self):
        return call('/game/players.gs', 'localPlayerPath')
    #/-----------------------------------------------------------------------------//
    def getLocalPlayerName(self):
        return call('/game/players.gs', 'localPlayerName')
    #/-----------------------------------------------------------------------------//
    def getLocalSide(self):
        return call('/game/players.gs', 'localPlayerSide')
    #/-----------------------------------------------------------------------------//
    def getLocalSidePath(self):
        return call('/game/players.gs', 'localPlayerSidePath')
    #/-----------------------------------------------------------------------------//
    def getLocalFaction(self):
        return call('/game/players.gs', 'localPlayerFaction')
    #/-----------------------------------------------------------------------------//
    def getLocalSideType(self):
        return call('/game/players.gs', 'localPlayerSideType')
    #/-----------------------------------------------------------------------------//
    def canControlGroup(self, groupName):
        """check to see if i can legally control this group"""
        #print 'canControlGroup', groupName, 
        playerPath = self.getLocalPlayerPath()
        sideType   = self.getLocalSideType()
        #check if i can legally control this unit
        if sideType == 'bm':            #bm can control everything
            return True
        elif sideType == 'observer':    #observer cant control anything
            return False
        elif sideType == 'combat' or sideType == 'neutral':                           #other types (neutral/combat) can control their own units
            canControl = True
            for entName in ndirin('/comm/groups/%s' % groupName):
                entPath = call('/comm/groups/%s/%s.gs' % (groupName, entName), 'path')
                canControl = (canControl and self.canControlEnt(entPath))
            return canControl
        else:
            print 'unknown sideType %s' % sideType
            return False
    #/-----------------------------------------------------------------------------//
    def canControlEnt(self, entPath):
        """check if i can control an individual entitiy"""
        entPath = entPath.split(' ')[0]
        if not exists(entPath):
            print 'calling canControlEnt on a non existent ent:', entPath
            return False

        entSidePath  = call('%s.gref' % entPath, 'side')
        relationship = sideManager.getRelationship( self.getLocalSidePath(), entSidePath )
        print 'relationship', relationship
        return relationship == RELATIONSHIPS.PLAYER or relationship == RELATIONSHIPS.LORD
    #/-----------------------------------------------------------------------------//
    def updatePlayer(self, msg):
        return
        print 'updatePlayer', msg
        try:
            id, playerName, playerSide, playerFaction = msg.split(':')
        except:
            print 'failed to parse updatePlayer message', msg
            return
        id                              = 'player%i' % int(id)
        playerName                      = str(playerName)
        playerSide                      = str(playerSide)
        playerFaction                   = str(playerFaction)

        changed = False
        pushsel('/game/players')
        #player doesn't exist locally - create
        if ( exists(id) == False):
            print 'creating new remote player', id
            sn('nbasicent',             id)
            ci('local',                 0)
            cs('playername',            playerName)
            cs('side',                  playerSide)
            cs('faction',               playerFaction)
            changed = True
        #check to see if any of the data has changed
        else:
            sel(id)
            if gs('playername') != playerName:
                print 'updating player %s name from %s to %s' % ( id, gs('playername'), playerName)
                ss('playername', playerName)
                changed = True
            if gs('playerside') != playerSide:
                print 'updating player %s side from %s to %s' % ( id, gs('playerside'), playerSide)
                ss('playerside', playerSide)
                changed = True
            if gs('playerFaction') != playerFaction:
                print 'updating player %s faction from %s to %s' % ( id, gs('playerFaction'), playerFaction)
                ss('playerFaction', playerFaction)
                changed = True
        if changed:
            nEventServer.pullLater(EventNames.PLAYERS_CHANGED, 'updatePlayer')
        popsel()
    #/-----------------------------------------------------------------------------//
    def playersChanged(self, msg):
        print 'playersChanged', msg
        call('/net/tnl.playersChanged')
    #/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
playerManager = PlayerManager()
playerManager.createLocalPlayer()
#print 'done loading playermanager'
