#define N_IMPLEMENTS nAI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ai/nai.h"

nNebulaScriptClass(nAI, "nfsm");

//------------------------------------------------------------------------------
/**
*/
nAI::nAI():
    nFSM()
{
}

void nAI::Initialize(){
    nFSM::Initialize();
}

void nAI::InitVars(){
    nFSM::InitVars();
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    vel.Connect( ent, "vel" );
    rvel.Connect( ent, "rvel" );
    thrust.Connect( ent, "thrust" );
    thrust.DefaultF( 1.0 );
    physics = (nPhysics*) ent->Find("physics");
    
    rc_distAngleCoef.Connect( ent, "distanglecoef" );
    rc_distAngleCoef.DefaultInputRange( 0, 10, 11 );
    rc_distAngleCoef.DefaultOutputRange( 0.0f, 4.0f );
    rc_distAngleCoef.DefaultRC( RC_LINE, 1.0f, 1.0f );

    rc_angleToTorque.Connect( ent, "angletotorque" );
    rc_angleToTorque.DefaultInputRange( -N_PI, N_PI, 21 );
    rc_angleToTorque.DefaultOutputRange( -1.0f, 1.0f );
    rc_angleToTorque.DefaultRC( RC_LINE, 0.5f, -0.5f );
    
    rc_rvelToTorque.Connect ( ent, "rveltotorque" );
    rc_rvelToTorque.DefaultInputRange( -N_PI, N_PI, 13 );
    rc_rvelToTorque.DefaultOutputRange( -1.0f, 1.0f );
    rc_rvelToTorque.DefaultRC( RC_LINE, 0.0f, 0.0f );

    rc_angleToForce.Connect(  ent, "angletoforce" );
    rc_angleToForce.DefaultInputRange( -N_PI, N_PI, 13 );
    rc_angleToForce.DefaultOutputRange( -1.0f, 1.0f );
    rc_angleToForce.DefaultRC( RC_LINE, 1.0f, 1.0f );
}

nAI::~nAI(){
}

float nAI::MoveTowards( const vector3& tar, float dtime ){
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    float heading = q.ToYaw();
    float rh = rvel.GetQ().ToYaw();
    vector3 d = tar - p;
    float th = (float) atan2( d.x, d.z );
    dh = n_normalizeRot( th - heading );
    float coef = rc_distAngleCoef.Response( d.len() );
    dh *= coef;
    /*
        DrawLine( p, tar, white );
        DrawAngle( p, th, red );
        DrawAngle( p, rvel, green );
        DrawAngle( p, th + dh, yellow );
    */
    float f = thrust.GetF() * rc_angleToTorque.Response(dh) + rc_rvelToTorque.Response(rh);
    f *= dtime;
    physics->ApplyAlignedTorque( vector3(0,-f,0) );

    f = rc_angleToForce.Response(dh);
    f *= dtime;
    physics->ApplyAlignedForce(  vector3(0,d.y * .1f * dtime,-f) );
    d.y = 0.0f;
    return d.len();
}

float nAI::Move3D( const vector3& tar, float dtime ){
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    vector3 forward = q.rotate( vector3(0,0,-1) );
    vector3 right = q.rotate( vector3(1,0,0) );
    vector3 up = q.rotate( vector3(0,1,0) );
    //DrawLine( p, p + forward, white );
    //DrawLine( p, p + right, blue );
    //DrawLine( p, p + up, green );
    
    vector3 d = tar - p;
    float td = d.len();
    lf = (d % forward) / td;
    lr = (d % right) / td;
    lu = (d % up) / td;
    //DrawLine( p, p + forward * lf, yellow );
    //DrawLine( p, p + right * lr, yellow );
    //DrawLine( p, p + up * lu, yellow );
    physics->ApplyAlignedTorque( vector3( lu * dtime, -lr * dtime, 0.0f ) );
    physics->ApplyAlignedForce( vector3(0.0f,0.0f,-1.0f * dtime)  );
    /*float heading = q.ToYaw();
    float rh = rvel.GetQ().ToYaw();
    vector3 d = tar - p;
    float th = atan2( d.x, d.z );
    dh = n_normalizeRot( th - heading );
    float coef = rc_distAngleCoef.Response( d.len() );
    dh *= coef;
    float f = rc_angleToTorque.Response(dh) + rc_rvelToTorque.Response(rh);
    f *= dtime;
    physics->ApplyAlignedTorque( vector3(0,-f,0) );

    f = rc_angleToForce.Response(dh);
    f *= dtime;
    physics->ApplyAlignedForce(  vector3(0,d.y * .1f * dtime,-f) );
    d.y = 0.0f;
    */
    return d.len();
}
