#define N_IMPLEMENTS nMissleAI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ai/nmissleai.h"
#include "aspect/nsoundemitter.h"

nNebulaScriptClass(nMissleAI, "nai");

//------------------------------------------------------------------------------
/**
*/
nMissleAI::nMissleAI():
emitter( kernelServer, this )
{
    startTime = -1.0f;
}

void nMissleAI::Initialize(){
    nAI::Initialize();
    stateNames.Append("inactive");
    stateNames.Append("drop");
    stateNames.Append("dumb");
    stateNames.Append("seek");
    stateNames.Append("explode");
}

void nMissleAI::InitVars(){
    nAI::InitVars();
    target.	Connect( ent, "target" );
    target.	DefaultRef("");
    targetPos.	Connect( target, "pos" );
    dropTime.	Connect( ent, "droptime" );
    dropTime.	DefaultF(0.5f);
    flyTime.	Connect( ent, "flytime" );
    flyTime.	DefaultF(5.0f);
    thrust.	Connect( ent, "thrust");
    thrust.	DefaultF(0.0f);
    attachedTo. Connect( ent, "attachedto");
    attachedTo. DefaultRef("");
    detonationRange.Connect( ent, "detonationrange" );
    detonationRange.DefaultF( 1.0 );
    emitter = ent->ChildPath("emitter");
    damage.	Connect( ent, "damage" );
    damage.	DefaultF( -1.0f );
    impulse.	Connect( ent, "impulse" );
    impulse.	DefaultF( -1.0f );
    weaponlevel.Connect( ent, "weaponlevel" );
    weaponlevel.DefaultI( 0 );
    effectradius.Connect( ent, "effectradius");
    effectradius.DefaultF( 0.0f );
    lastDistance = -1.0f;
}

nMissleAI::~nMissleAI(){
}
//------------------------------------------------------------------------------
/**
*/

void nMissleAI::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
}

bool nMissleAI::States( StateMachineEvent event, uint state, float time, float dtime ){
    if( ent->Held() ) return true;
    BeginStateMachine
        State( Inactive )
            OnEnter
        State( Drop )
            OnEnter
                startTime = time;
                thrust.SetF( .2f );
                attachedTo.SetRef("");
                //n_printf("%s->missledrop!!!\n", this->fn);
                //ifvalid( emitter )
                    //emitter->Queue("missledrop");
            OnTick
                physics->ApplyAlignedForce( vector3(0,-dtime/5.0, 0 ) );
                if( time - startTime > dropTime.GetF() ){
                    if( target.isvalid() )
                        SetState( Seek );
                    else
                        SetState( Dumb );
                }
        State( Dumb )
            OnEnter
	thrust.SetF( 1.0f );
            OnTick
                physics->ApplyAlignedForce(vector3(0,0,-dtime));
                if( time - startTime > flyTime.GetF() )
                    SetState( Explode );
        State( Seek )
            OnEnter
	thrust.SetF( 1.0f );
            OnTick
                //this->MoveTowards( targetPos.GetV(), dtime );
	if( time - startTime > flyTime.GetF() )
	    SetState( Explode );
	if( target.isvalid() ) {
	    float d = this->Move3D( targetPos.GetV(), dtime );
	    if( lastDistance > 0.0f && d > lastDistance && lastDistance < detonationRange.GetF() ) {
	        n_printf("ld: %f d: %f dr: %f\n", lastDistance, d, detonationRange.GetF() );
	        SetState( Explode );
	    }
	    lastDistance = d;
	} else {
	    SetState( Explode );
	}
        State( Explode )
            OnEnter {
	ifvalid( emitter ) 
	    emitter->Queue("missleexplosion");
                ent->Die();
	nMessage* m = new nMessage();
	m->from = this->fn;
	m->to = "all";
	m->title = "areadamage";
	nString weaponType;
	weaponType.PrintF("%s%i", ent->GetParent()->GetName(), weaponlevel.GetI () );
	m->data = weaponType;
	m->f0 = damage.GetF();
	m->f1 = impulse.GetF();
	m->f2 = effectradius.GetF();
	printf("radius: %f\n", m->f2 );
	m->v = pos.GetV() - vector3(0,-10,0);
	Send( m );
            }
    EndStateMachine
}
