#define N_IMPLEMENTS nConstructor
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "aspect/nconstructor.h"
#include "enkiduo/ngameserver.h"

nNebulaScriptClass(nConstructor, "naspect");

//------------------------------------------------------------------------------
nConstructor::nConstructor() {
}
//------------------------------------------------------------------------------
void nConstructor::Initialize(){
    nAspect::Initialize();
}
//------------------------------------------------------------------------------
void nConstructor::InitVars(){
    pos.Connect                         ( ent, "pos" );
    pos.DefaultV                        ( 0,0,0 );
    selectionCircleRadius.Connect       ( ent, "selectionCircleRadius" );
    selectionCircleRadius.DefaultF      ( 100.0f );
    constrRatio.Connect                 ( ent, "constrRatio" );
    constrRatio.DefaultF                ( 0.0f );
    constructionTimeAccumulated.Connect ( ent, "constructionTimeAccumulated" );
    constructionTimeAccumulated.DefaultF( 0.0f );
    entBeingBuilt.Connect               ( ent ,"entBeingBuilt" );
    entBeingBuilt.DefaultRef            ( "" );
    entBeingBuiltConstructionTime.Connect( entBeingBuilt, "constructionTime" );
    nAspect::InitVars();
}
//------------------------------------------------------------------------------
nConstructor::~nConstructor(){
}
/**-----------------------------------------------------------------------------//
 * if i have commands to do execute them
 * in the top commad is a build command i should apply points towards accomplishing that command
--------------------------------------------------------------------------------*/
void nConstructor::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    if( commands.Size() > 0 ) {
        Command& command = commands[0];
        switch (command.commandType){
            case Command::COMMAND_BUILD:
                if ( entBeingBuilt.isvalid() == false ) {
                    nString entPath;
                    entPath.PrintF("/lib/ents/%s", command.entType.Get());
                    n_printf("%s trying to construct ent %s\n", str(this), entPath.Get() );
                    entBeingBuilt.SetRef( entPath.Get() );
                    n_assert( entBeingBuilt.isvalid() && "trying to build a non existent ent" );
                    constructionTimeAccumulated.SetF( 0.0f );                   //reset my construction timer
                }
                constructionTimeAccumulated.SetF( constructionTimeAccumulated.GetF() + dtime );
                constrRatio.SetF( constructionTimeAccumulated.GetF() / entBeingBuiltConstructionTime.GetF() );
                //n_printf("constrRatio:%f\n", constrRatio.GetF() );
                if( constrRatio.GetF() > 1.0f ) {                               //have finished constructing the entity, actually create it
                    n_printf             ( "%s finished constructing %s\n", str(this), command.entType.Get() );
                    nEnt* childEnt       = game->GenEnt( command.entType.Get() );
                    float direction      = randF( 0, PI * 2.0f );
                    vector3 offset       ( cos(direction), 0, sin(direction));
                    offset              *= selectionCircleRadius.GetF();
                    childEnt->SetVarV    ( "pos", pos.GetV() + offset );
                    childEnt->SetVarRef  ( "side", ent->GetVarRef("side") );
                    commands.Pop         ( );
                    entBeingBuilt.SetRef ( "");
                }
                break;
            default:
                assert(0 && "invalid commandType");
        }
    }
}
/**-----------------------------------------------------------------------------//
 * queue up a new entity to be construted
--------------------------------------------------------------------------------*/
void nConstructor::ConstructEntity( const char* entityName ) {
    n_printf("nConstructor(%s).ConstructEntity(%s)\n", str(this), entityName );
    Command& newCommand = commands.Push();
    newCommand.commandType = newCommand.COMMAND_BUILD;
    newCommand.entType = entityName;
}
