#define N_IMPLEMENTS nSide
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "misc/nquickwatch.h"
#include "aspect/nside.h"
#include "ui/ntextdisplayer.h"
#include "enkiduo/neventserver.h"

nNebulaScriptClass(nSide, "naspect");


//------------------------------------------------------------------------------
nSide::nSide():ref_eventServer( kernelServer, this ){
}
//-----------------------------------------------------------------------------//
void nSide::Initialize(){
    nAspect::Initialize();
    this->AddToDisplayList();

    ref_eventServer                     = "/sys/servers/event";
    n_assert( ref_eventServer.isvalid() && "nSide failed to find eventserver" );
    ref_eventServer->registerCFunction      ( "playerRelationshipsChanged", this, 'PRC' );
    this->Tick(0,0);
    relations.SetFixedSize              ( RELATIONSHIPS.Size() );
}
//-----------------------------------------------------------------------------//
void nSide::InitVars(){
    nAspect::InitVars();
    playerSide.Connect( ent, "playerside");
    playerSide.DefaultI( 0 );
    playerRelationship.Connect( ent, "playerRelationship" );
    playerRelationship.DefaultI( VOID );
    dtimesum = 0;
}
//-----------------------------------------------------------------------------//
nSide::~nSide(){
}
//------------------------------------------------------------------------------
void nSide::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    if( playerSide.isdirty() ) {
        static void* eventPtr = 0;
        ref_eventServer->pull( eventPtr, "playerRelationshipsChanged", str(this));
    }

    foreachp( Resource*, r, resources ) {
        //printf("side(%s) checking resource %s\n", str(this), r->name.Get() );
        if( r->amount0 > 0 && dtime > 0 ){ 
            r->damount.SetF( (r->amount.GetF() - r->amount0) / (dtime / 10.0f) );
        }
        r->amount0 = r->amount.GetF();
        this->time = time;
        this->dtimesum += dtime;
    }
}
//-----------------------------------------------------------------------------//
void nSide::AddUnit( nEnt* e ) {
    //n_printf("side:%s adding %s to its ent list\n", this->fn, e->fn );
    units.Push() = e;
}
//-----------------------------------------------------------------------------//
void nSide::RemoveUnit( nEnt* e ) {
    //n_printf("side:%s removing %s from its ent list\n", this->fn, e->fn );
    for( int i = 0; i < units; i ++ ) {
        if( units[i] == e ) {
            //n_printf("found in slot: %i\n", i );
            units[i] = 0;
            units.Erase( i );
            //n_printf("done erasing\n");
        }
    }
}
/**-----------------------------------------------------------------------------//
 * set a particular relationship between this side and another side
--------------------------------------------------------------------------------*/
void nSide::SetRelationship( const char* side, const char* relationship) {
    //n_printf("%s->SetRelationship(%s,%s)\n", this->fn, side, relationship );
    nString sidePath;
    sidePath.PrintF("/game/sides/%s/side", side );
    nSide* s = (nSide*) kernelServer->Lookup(sidePath.Get() );

    if( s == 0 ) {
        n_printf("trying to set relationship (%s,%s) to a non existant side\n", side, relationship );
        return;
    } else {
        ///remove it from any old relations
        for( int i = 0; i < relations; i ++ ) {
            if( i == RELATIONSHIPS.getI( relationship ) )
                continue;
            bool b = relations[i].sides.Remove( s );
            if ( b ) {
                printf("side(%s) is no longer %s with %s\n", str(this), RELATIONSHIPS.getS(i), str(s));
            }
        }
        ///add it to the new relations
        Relation& relation = relations[RELATIONSHIPS.getI( relationship )];
        relation.sides.Append( s );

        static void* eventPtr = 0;
        ref_eventServer->pullLater ( "playerRelationshipsChanged", str(this));
    }
}
//-----------------------------------------------------------------------------//
int nSide::GetRelationship( const char* sidePath ) {
    //nString path;
    //path.PrintF("/game/sides/%s/side", sideName );
    nSide* side = LOOKUP(nSide*, sidePath );
    printf("nSide(%s,%p)::GetRelationship(%s,%p)\n", str(this), this, str(side), side);
    if( side == this ) 
        return PLAYER;
    if( side == 0 ) {
        printf("nSide(%s)::GetRelationship(%s) -> unknown side\n", str(this), sidePath );
        return 0;
    }

    for( int i = 0; i < relations; i ++ ) {
        if( this->relations[i].sides.Find( side ) ) {
            return i;
        }
    }
}
//-----------------------------------------------------------------------------//
void nSide::Display( nGfxServer* gfx ){
    nAspect::Display( gfx );
    foreachp( Resource*, r, resources ) {
        nString string;
        //string.PrintF("%3.0f(%+.1f)", r->amount.GetF(), r->damount.GetF() );
        ifvalid( r->td ) {
            string.PrintF("%i", (int) r->amount.GetF() );
            r->td->SetString( string.Get() );
        }
        ifvalid( r->tdDelta ) {
            string.PrintF("%+i", (int) r->damount.GetF() );
            r->tdDelta->SetString( string.Get() );
        }
    }
}
//-----------------------------------------------------------------------------//
void nSide::AddResource( const char* name ) {
    //n_printf("nSide(%s).AddResource(%s)\n", str(this), name );
    Resource* r                         = &resources.Push();
    r->name                             = name;
    r->amount.Connect                   ( ent, name );
    r->amount.DefaultF                  ( 0.0f );
    r->amount0                          = -1.0f;
    r->damount.Connect                  ( ent, (nString(name) + "d").Get() );
    r->damount.DefaultF                 ( 0.0f );

    nString path;
    r->tdName.initialize                ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/%sname", this->GetParent()->GetName(), name );
    r->tdName                           = path.Get();

    r->td.initialize                    ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/%s", this->GetParent()->GetName(), name );
    r->td                               = path.Get();

    r->tdDelta.initialize               ( kernelServer, this );
    path.PrintF                         ("/game/vis/ui/%s/d%s", this->GetParent()->GetName(), name );
    r->tdDelta                          = path.Get();
}
/**-----------------------------------------------------------------------------//
 * return a list of units which share a particular relationship with this side
 --------------------------------------------------------------------------------*/
const EntArray* nSide::GetRelatedUnits( int relationship ){
    Relation& relation = relations[relationship];
    nSide* side;
    nEnt* unit;
    for( int i = 0; i < relation.sides; i ++ ) {
        side = relation.sides[i];
        const EntArray& units = *side->GetUnits();
        foreach( unit, units )
            relation.units.Append( unit );
    }
    return &relation.units;
}
/**-----------------------------------------------------------------------------//
 * search through my relations, find who's the player and set my player relationship
 --------------------------------------------------------------------------------*/
void nSide::PlayerRelationshipsChanged(const char* msg ){
    //n_printf("nSide(%s).PlayerRelationshipsChanged(%s)\n", str(this), msg);
    nRoot* sidesRoot = ent->GetParent();
    nEnt* playerEnt = 0;
    foreachchild( nEnt, sideEnt, sidesRoot ) {
        if( sideEnt->GetVarI("playerside") == 1 ) {  //its the player side
            playerEnt = sideEnt;
            break;
        }
    }
    if ( playerEnt == 0 ) {
        n_printf("failed to find the player side somehow\n");
    } else {
        nSide* playerSide = dynamic_cast<nSide*>( playerEnt->Find("side"));
        if( playerSide == this ) 
            playerRelationship.SetI( PLAYER );
        else {
            for ( int i = 0; i < relations; i ++ ) {
                if( relations[i].sides.Find( playerSide ) ) {
                    //n_printf("player(%s).playerRelationship = %s\n", str(this), RELATIONSHIPS.getS( i ) );
                    playerRelationship.SetI( i );
                    break;
                }
            }
        }
    }
}
//-----------------------------------------------------------------------------//
