#define N_IMPLEMENTS nSoundEmitter
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "aspect/nsoundemitter.h"
#include "enkiduo/nopenalserver.h"

nNebulaScriptClass(nSoundEmitter, "noffsetaspect");

//------------------------------------------------------------------------------
/**
*/
nSoundEmitter::nSoundEmitter():
    playing( false ),
    emitterHandle( -1 ),
    gain( 1.0f ),
    pitch( 1.0f ),
    range( 1000.0f )
{
}

void nSoundEmitter::Initialize(){
    nOffsetAspect::Initialize();
}

void nSoundEmitter::InitVars(){
    nOffsetAspect::InitVars();
    vel.Connect( ent, "vel" );
    vel.DefaultV( 0,0,0 );

    camera.Connect( ent, "camera" );
    camera.DefaultRef("/game/camera");
    cameraPos.Connect( camera, "pos" );

    speed.Connect( ent, "speed");
    speed.DefaultF(1.0f);
}

nSoundEmitter::~nSoundEmitter(){
}
/* -----------------------------------------------------------------------------//
 * check my distance to the camera, if its greater then the threshold then stop playing sounds
 * otherwise pick a passive sound and play it
--------------------------------------------------------------------------------*/
void nSoundEmitter::Tick( float time, float dtime ){
    nOffsetAspect::Tick( time, dtime );
    if( emitterHandle == -1 ) {                                                 //if I dont have a sound emitter
        emitterHandle = soundServer->CreateEmitter();                           //get a new one from the soundServer
        //n_printf("Created emitter: %i\n", emitterHandle );
        soundServer->SetGain( emitterHandle, gain );
        soundServer->SetPitch( emitterHandle, pitch );
    }

    //calculate distance to the camera
    float distance;
    vector3 p = this->GetOffsetPos();
    vector3 cp;
    if( camera.isvalid() )
        cp = cameraPos.GetV();
    float d = (cp - p).len();
    if( playing ) { 			//i am currently playing a sound
        if( d > range * 1.1 /*|| fabs(speed.GetF()) > .1f*/) { 		//histerisis multiplier
            //n_printf("%s->stopping play -> %f > %f speed=%f\n", this->fn, d, range * 1.1, speed.GetF() );
            soundServer->StopPlaying( emitterHandle ); 	//i am out of range                     //its to far stop playing things
            this->playing = false;
        } else {
            vector3 offsetPos = this->GetOffsetPos();
            n_assert( offsetPos.x != NAN && offsetPos.y != NAN && offsetPos.z != NAN );
            soundServer->UpdateEmitter( emitterHandle, this->GetOffsetPos(), vel.GetV() );                              //update my emitters position to where I am
        }
    } else {
        if( d < range ) {			// i am within range - should start playing any idles
            if( passiveSounds.Size() > 0 ){
                int i = randI ( 0, passiveSounds.Size() -1 ); 	//the passive sound to play
                soundServer->SetLooping	( emitterHandle, true );                                                        //passive sounds loop forever
                soundServer->Queue	( emitterHandle, passiveSounds[i].Get() );                                      //put it in the tooplay queue
                //n_printf("%s->playing %s -> %f > %f\n", this->fn, passiveSounds[i].Get(), d, range * 1.1 );
                this->playing = true;
            }
        }
    }
}
/* -----------------------------------------------------------------------------//
 * play a sound a single time
 * good for gunfire / explosions and things
--------------------------------------------------------------------------------*/
void nSoundEmitter::Queue( const char* soundName ) {
    //n_printf("playing sound: %s @%s %f,%f \n", soundName, str(GetOffsetPos()), gain, pitch);
    soundServer->Queue( emitterHandle, soundName );
    this->playing = true;
}
/* -----------------------------------------------------------------------------//
 * add a passive sound to the list
 * passive sounds are played continously so long as the entity is within range
--------------------------------------------------------------------------------*/
void nSoundEmitter::AddPassiveSound( const char* soundName ) {
    passiveSounds.Append( soundName );
}
/* -----------------------------------------------------------------------------//
 * adjust the gain / volume of the sounds emitted by this entity
--------------------------------------------------------------------------------*/
void nSoundEmitter::SetGain( float gain ){
    this->gain = gain;
    if( emitterHandle > -1 )
        soundServer->SetGain( emitterHandle, gain );
}
/* -----------------------------------------------------------------------------//
 * adjust the pitch of the sounds emitted by this entity
--------------------------------------------------------------------------------*/
void nSoundEmitter::SetPitch( float pitch ){
    this->pitch = pitch;
    if( emitterHandle > -1 )
        soundServer->SetPitch( emitterHandle, pitch );
}
