#define N_IMPLEMENTS nWeapon
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "aspect/nweapon.h"
#include "enkiduo/ngameserver.h"
#include "physics/nphysics.h"
#include "physics/ncollisionnode.h"
#include "enkiduo/nfsm.h"
#include "enkiduo/nchannelholder.h"

nNebulaScriptClass(nWeapon, "noffsetaspect");

//------------------------------------------------------------------------------
/**
*/
nWeapon::nWeapon():
game( kernelServer, this )
{
    game = "/sys/servers/game";
    n_assert( game.isvalid() );
}

void nWeapon::Initialize(){
    nOffsetAspect::Initialize();
}

void nWeapon::InitVars(){
    nOffsetAspect::InitVars();
    fires.Connect( ent, "fires" );
    fires.DefaultS( "missle" );
    cooldown.Connect( ent, "cooldown" );
    cooldown.DefaultF(.1f);
    lastFired = -1.0f;
    vel.Connect( ent, "vel" );
    vel.DefaultV( 0,0,0 );
    firingForce.Connect( ent, "firingforce");
    firingForce.DefaultV( 0, 0, -.5 );
    collision = 0;
    side.Connect( ent, "side");

    nEnt* options = dynamic_cast<nEnt*>(kernelServer->Lookup("/options"));
    n_assert( options && "failed to find /options || its not an ent");
    firingEnabled.Connect( options, "firing" );
}
//-----------------------------------------------------------------------------//
nWeapon::~nWeapon(){
}
//------------------------------------------------------------------------------
void nWeapon::Tick( float time, float dtime ){
    nOffsetAspect::Tick( time, dtime );
    this->time = time;
}
//-----------------------------------------------------------------------------//
bool nWeapon::CanFire(){
    return ( firingEnabled.GetI() && (lastFired < 0 || roll(.1) && time - lastFired > cooldown.GetF()));
}
//-----------------------------------------------------------------------------//
nEnt* nWeapon::Fire( const char* target, float range ){
    if( this->CanFire() ) {
        lastFired = time;
        vector3 p = offsetPos;
        quaternion q = rot.GetQ();
        vector3 v = vel.GetV();
        nEnt* e = game->GenEnt( fires.GetS().Get() );
        if( e == 0 ) { 
            n_printf("nWeapon(%s) cannot create weapon ent:%s\n", this->fn, fires.GetS().Get() ); 
            return 0; 
        }
        //n_printf("%s creating weapon %s\n", this->fn, e->fn );
        //n_printf("setting new weapon ent:%s to side:%s\n", e->fn, side.GetRefPath() );
        e->CreateVar("side")->SetRef(side.GetRefPath());
        e->SetVarV("pos", p );
        e->SetVarQ("rot", q );
        e->SetVarV("vel", v );
        if( range > 0 ) {
            nChannelHolder* ch = (nChannelHolder*) e->GetVis()->Find("channelholder");
            if( ch ) 
                ch->Add("range", &range, 1 );
            else
                e->GetVis()->Sz( range );
        }
        nFSM* ai = (nFSM*) e->Find("ai");
        if( ai )	ai->SetState( "drop" );
        if( target )                    e->CreateVar("target")->SetRef( target );

        nPhysics* phy                   = (nPhysics*) e->Find("physics");
        if( phy )            	phy->ApplyAlignedForce( firingForce.GetV() );

        nCollisionNode* coll            = (nCollisionNode*) e->Find("collision");
        if( coll )            	coll->AddException( ent );
        if( collision )       	collision->AddException( e );
        else                  	collision = (nCollisionNode*) ent->Find("collision");
        return e;
    }
    return 0;
}
//-----------------------------------------------------------------------------//
nEnt* nWeapon::Fire( const char* target, quaternion q, float range, float damage, float impulse, float effectradius, int level ) {
    vector3 p = this->GetOffsetPos();
    if( this->CanFire() ) {
        lastFired = time;
        vector3 v = vel.GetV();

        q.norm();
        nEnt* e = game->GenEnt( fires.GetS().Get() );
        if( e == 0 ) { 
            n_printf("cannot find ent:%s\n", fires.GetS().Get() ); 
            return 0; 
        }

        //n_printf("%s creating weapon %s\n", this->fn, e->fn );
        //n_printf("setting new weapon ent:%s to side:%s\n", e->fn, side.GetRefPath() );
        e->CreateVar("side")->SetRef(side.GetRefPath());

        e->SetVarV("pos", p );
        e->SetVarQ("rot", q );
        e->SetVarV("vel", v );

        vector3 contactPoint = vector3(0,0,-range);
        contactPoint = q.rotate(contactPoint) + p;

        e->CreateVar("contactPoint")->SetV(contactPoint);
        e->CreateVar("weaponlevel")->SetI(level);

        //n_printf("nWeapon::fire: %s %s %s %f\n", target, str(p), str(q), range );
        if( range > 0 ) {
            nChannelHolder* ch = (nChannelHolder*) e->GetVis()->Find("channelholder");
            if( ch ) 	ch->Add("range", &range, 1 );
            else	e->GetVis()->Sz( range );
        }

        //see if the weapon has an ai, if so tell it to drop / launch
        nFSM* ai                        = (nFSM*) e->Find("ai");

        //attach any of the variables passed to me
        if( ai )	ai->SetState( "drop" );
        if( target )            	e->CreateVar("target")->SetRef( target );
        if( damage > -1.0f )    	e->CreateVar("damage")->SetF(damage);
        if( impulse > -1.0f )   	e->CreateVar("impulse")->SetF(impulse);
        if( effectradius > -1.0f )	e->CreateVar("effectradius")->SetF(effectradius);

        //if it has physics - apply the firing force
        nPhysics* phy                   = (nPhysics*) e->Find("physics");
        if( phy )	phy->ApplyAlignedForce( firingForce.GetV() );

        //if it has collision - tell it not to hit me
        nCollisionNode* coll            = (nCollisionNode*) e->Find("collision");
        if( coll )	coll->AddException( ent );
        if( collision )                 collision->AddException( e );
        else	collision = (nCollisionNode*) ent->Find("collision");

        return e;
    }
    return 0;
}
