#include <getopt.h>
#include <stdlib.h>
#include <time.h>
#include "kernel/nkernelserver.h"
#include "kernel/nroot.h"
#include "kernel/nenv.h"
#include "script/ntclserver.h"
#include "util/nstring.h"
#include "enkiduo/ngameserver.h"
/**
  THE main loop
  creates the kernelserver / scriptserver singletons, and tells the script server to run scripts/go.tcl
  @param argv - parameters to main are stored within nenvs under /sys/share/args
**/
int main( int argc, char** argv ){
    printf("********************************************************************************\n");
    printf("********************************************************************************\n");
    printf("********************************************************************************\n");
    srand(time(0));

    nKernelServer* ks = new nKernelServer();
    nScriptServer* tclServer = (nScriptServer*) ks->New("ntclserver", "/sys/servers/script");
    nScriptServer* pythonServer = (nScriptServer*) ks->New("npythonserver", "/sys/servers/python");
    
    enum ROLES {
        UNSPECIFIED,
        TCL,
        PYTHON,
    };
    const char* ROLES_STRINGS[] = {
        "UNSPECIFIED", 
        "TCL", 
        "PYTHON"
    };
    int role = UNSPECIFIED;
    int c;
    /*
    while (1) {
        int index;
        static struct option long_options[] = {
            {"tcl",         no_argument,       &role,     TCL},
            {"python",         no_argument,       &role,     PYTHON},
            {"ip",          required_argument, 0,         'i'},
            {0, 0, 0, 0}
        };
        c = getopt_long (argc, argv, "tpi:", long_options, &index);
        if( c == -1 )
            break;
    }
    printf("role:%s\n", ROLES_STRINGS[role] );
    */
    if( role == UNSPECIFIED ) {
        role = PYTHON;
        printf("setting base scripting language: %s\n", ROLES_STRINGS[role]);
    }
    
    char args[1024];
    memset(args, 0, 1024);
    for( int i = 1; i < argc; i ++ ){
        if( i > 1 )
            strcat( args, " ");
        strcat( args, argv[i] );
    }
    ((nEnv*) ks->New("nenv", "/sys/share/args"))->SetS( args );
    
    nEnv* home = (nEnv*) ks->Lookup( "/sys/share/assigns/home");
    nString h = home->GetS();
    char toRun[256];
    const char* err = 0;
    switch( role ) {
        case TCL:
            sprintf( toRun, "%s/data/scripts/go.tcl", h.Get() );
            tclServer->RunScript( toRun, err );
            if( err ) 
                printf("%s\n", err );
            break;
        case PYTHON:
            sprintf( toRun, "%s/code/scripts/common/go.py", h.Get() );
            pythonServer->RunScript( toRun, err );
            if( err ) 
                printf("%s\n", err );
            break;
    }

    nGameServer* gs = (nGameServer*) ks->Lookup("/sys/servers/game");
    if( gs )            
        gs->Start();
    else                
        printf("ERROR - Could not find Gameserver\n");

    switch( role ) {
        case TCL:
            sprintf( toRun, "%s/data/scripts/release.tcl", h.Get() );
            tclServer->RunScript( toRun, err );
            if( err ) printf("%s\n", err );
            printf("done with release script\n");
            break;
        case PYTHON:
            sprintf( toRun, "%s/code/scripts/common/release.py", h.Get() );
            pythonServer->RunScript( toRun, err );
            if( err ) printf("%s\n", err );
            printf("done with release script\n");
            break;
    }

    tclServer->Release();
    pythonServer->Release();
    //delete script; //#DO NOT PUT THIS IN DUMBASSES - I.E me, Christopher Eoin Miles
    delete ks;
    //n_sleep(10.0); //helps windows be less stupid
    return 0;
}
