#define N_IMPLEMENTS nAStar
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nastar.h"

nNebulaScriptClass(nAStar, "naspect");

//------------------------------------------------------------------------------
nAStar::nAStar():
prof (kernelServer, this ) {
    open.initialize(2048);
    prof = "/sys/servers/prof";
}
//------------------------------------------------------------------------------
void nAStar::Initialize(){
    nAspect::Initialize();
    //this->AddToDisplayList();
    estimatedCostCoef.Connect( ent, "estimatedCostCoef" );
    estimatedCostCoef.DefaultF( 1.100f );
    nEnt* options = LOOKUP(nEnt*, "/options");
    maxAStarSteps.Connect( options, "maxastarsteps" );
    maxAStarSteps.DefaultI( 10000 );
}
//------------------------------------------------------------------------------
void nAStar::InitVars(){
    nAspect::InitVars();
}
//------------------------------------------------------------------------------
nAStar::~nAStar(){
}
//------------------------------------------------------------------------------
void nAStar::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
}

/**-----------------------------------------------------------------------------//
 * weee
--------------------------------------------------------------------------------*/
AStarPath* nAStar::FindPath( AStarNode* start, AStarNode* goal ){
    prof->Begin("A*");
    //n_printf("nAStar::FindPath(%s,%s)\n", str(start), str(goal));

    AStarPath* path = new AStarPath();
    //reset all my info
    this->ResetNodes();
    open.clear(); //empty heap
    //closed.clear();

    //push the start node onto open
    start->SetCost( 0.0f );
    open.add( start->GetCost() + start->EstimateCostTo(goal) * estimatedCostCoef.GetF(), start );
    start->state = ASTARNODE_OPEN;

    int stepsTaken = 0;
    //while we have nodes to explore and we haven't found the goal
    while( open.length() > 0 && stepsTaken++ < maxAStarSteps.GetI() ){
        //get the most promising node
        float f;
        AStarNode* current;
        open.pop( f, current ); //
        current->SetExpanded(true);
        //n_printf("f:%f current:%s\n", f, str(current));

        //if it is the goal then build our return path
        if ( *current == *goal ) { //at goal -> push my node and all its parents onto the list
            //n_printf("found goal in %i steps\n", stepsTaken );
            AStarNode* n = current;
            path->Append(new AStarNode(*current) );
            while( n ) {
                //n_printf("path adding:%s\n", str(n));
                path->Append(new AStarNode(*n));
                n = n->GetParent();
            }
            path->Reverse(); //need to flip it around since this will be from goal to start
            computedPaths.Append(path);
            //path.Smooth();
            //path.Smooth();
            //computedPaths.Append(path);
            prof->End("A*");
            //path.Print();
            return path;
        }
        //its not the goal expand its neighbors
        else { 
            nArray<AStarNode*> neighbors;
            neighbors = current->GetNeighbors();
            //n_printf("has %i neighbors\n", neighbors.Size());
            AStarNode* neighbor;
            foreach( neighbor, neighbors ) { 
                //n_printf("neighbor:%s\n", str(neighbor));
                if( neighbor->state == ASTARNODE_OPEN || neighbor->state == ASTARNODE_UNUSED ) {
                    //figure out how much it costs to get there
                    float edgeCost = current->CalculateCostTo ( neighbor );
                    if( edgeCost < 0 ) {
                        n_printf("warning:edgeCost(%f) < 0\n");
                        edgeCost = 0;
                    }
                    //n_assert( edgeCost >= 0.0f );
                    float totalCost = edgeCost + current->GetCost();

                    if( neighbor->state == ASTARNODE_OPEN && neighbor->HasCost() && neighbor->GetCost() <= totalCost )  //neighbor has been previously discovered for less
                            //better has been previously discovered, we can skip this neighobr
                            continue;
                    neighbor->SetParent( current );
                    neighbor->SetCost( totalCost );
                    float estimatedCostToGoal = neighbor->GetCost() + neighbor->EstimateCostTo(goal) * estimatedCostCoef.GetF();
                    neighbor->SetEstimatedCost( estimatedCostToGoal );
                    
                    //okay so now we have found a new node, or a shorter path to an existing one -> push it on the open heap if its new
                    //otherwise fuck, take it off, and set it, nevermid, just move it around on the heap itself
                    //we shouldn't have to do this if our a* is any good anyways
                    if( neighbor->state == ASTARNODE_UNUSED ) {
                        open.add( neighbor->GetEstimatedCost(), neighbor );
                        neighbor->state = ASTARNODE_OPEN;
                    } else {
                        open.changeValue( neighbor->GetEstimatedCost(), neighbor );
                    }
                } else {
                    //n_printf("ignoring neighbor(%s) its closed already\n", str(neighbor));
                }
            }
        }
        current->state = ASTARNODE_CLOSED;
    }
    n_printf("Failed to find goal after %i steps\n", stepsTaken);
    prof->End("A*");
    return path;
}
//-----------------------------------------------------------------------------//
void nAStar::ResetNodes() {
    for( int i = 0; i < nodes; i ++ )
        nodes[i]->Reset();
}
//-----------------------------------------------------------------------------//
void nAStar::Display( nGfxServer* gfx ){
    nAspect::Display( gfx );
    //for( int i = 0; i < nodes; i ++ )
        //nodes[i]->Display(gfx);
    //for( int i = 0; i < computedPaths; i ++ )
        //computedPaths[i].Display(gfx);
    if( computedPaths.Size() > 0 )
        computedPaths[-1]->Display(gfx);
}
//-----------------------------------------------------------------------------//
int nAStar::GetPathLength(){
    return computedPaths[-1]->nodes.Size();
}
//-----------------------------------------------------------------------------//
vector3 nAStar::GetPathNode(int i){
    return computedPaths[-1]->nodes[i]->pos;
}
