#define N_IMPLEMENTS nCategoryServer
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/ncategoryserver.h"
#include "enkiduo/neventserver.h"

nNebulaScriptClass(nCategoryServer, "nthing");

//------------------------------------------------------------------------------
nCategoryServer::nCategoryServer():
    refEventServer( kernelServer, this )
{
}
//------------------------------------------------------------------------------
void nCategoryServer::Initialize(){
    nThing::Initialize();
    refEventServer = "/sys/servers/event";
}
//------------------------------------------------------------------------------
nCategoryServer::~nCategoryServer(){
}
/**-----------------------------------------------------------------------------//
 * create a new category / find an old one and return its handle
--------------------------------------------------------------------------------*/
int nCategoryServer::CreateCategory( const char* name, const char* path ) {
    //n_printf("nCategoryServer(%s).CreateCategory(%s,%s)=", str(this), name, path);
    for( int i = 0; i < categories; i ++ ) {
        if( categories[i].name == name ) {
            n_assert( categories[i].path == path && "trying to create two categories with the same name but different paths" );
            return i;
        }
    }
    int categoryHandle                  = categories.Size();
    Category& category                  = categories.Push();
    category.name                       = name;
    if( path && strlen(path) ) {
        category.path                   = path;
        category.hasVis                 = true;
    }
    //n_printf("%i\n", categoryHandle);
    //this->CategoryChanged(categoryHandle);
    return categoryHandle;
}
/**-----------------------------------------------------------------------------//
 * add an entity to a particular category
 * this should put it in the array, and setup any Categorys that need to be there
 * if the category does not exist -> create it
 * btw categories are never destroyed, since we used handled access
--------------------------------------------------------------------------------*/
bool nCategoryServer::AddEntToCategory( int handle, nEnt* e){
    //n_printf("nCategoryServer(%s).AddEntToCategory( %i:%s, %s )\n", str(this), handle, this->GetCategoryName( handle ), str(e) );
    bool changed = false;
    if ( categories[handle].members.Find( e ) ) {
        n_printf("warning trying to double add ent(%s) to category(%s,%s)\n", str(e), categories[handle].name.Get(), categories[handle].path.Get());
    }
    else {
        categories[handle].members.Append( e );
        e->AttachChildVis( categories[handle].name.Get(), categories[handle].path.Get() );
        //n_printf("categories[%i] = %i\n", handle, categories[handle].members.Size() );
        changed = true;
    }
    if( changed )
        this->CategoryChanged(handle);
    return changed;
}
/**-----------------------------------------------------------------------------//
 * remove a particular entity from a category
 * should be easy to just remv eit from the array and remove its link to the vis
--------------------------------------------------------------------------------*/
bool nCategoryServer::RemoveEntFromCategory  ( int handle, nEnt* e ){
    //n_printf("nCategoryServer(%s).RemoveEntFromCategory( %i:%s, %s )\n", str(this), handle, this->GetCategoryName(handle),  str(e) );
    bool changed = false;
    if ( categories[handle].members.Find( e ) == 0 ) {
        n_printf("warning trying to remove ent(%s) from category(%s,%s) that its not in\n", str(e), categories[handle].name.Get(), categories[handle].path.Get());
    }
    else {
        categories[handle].members.Remove( e );
        e->RemoveChildVis( categories[handle].name.Get() );
        changed = true;
    }
    if( changed )
        this->CategoryChanged(handle);
    return changed;
}
/**-----------------------------------------------------------------------------//
 * update an entire category in one sweep
 * walk through ents that are currently in there, and remove those not in the new group
 * then add entities not in the old group
--------------------------------------------------------------------------------*/
bool nCategoryServer::UpdateCategory( int handle, const EntArray& newEnts ){
    //n_printf("nCategoryServer(%s).UpdateCategory(%i,%s)\n", str(this), handle, str(newEnts));
    bool changed = false;
    Category& category = categories[handle];
    //remove entities not in the new group
    for( int i = 0; i < category.members; i++ ) {
        if( newEnts.Find( category.members[i] ) == 0 ) {
            this->RemoveEntFromCategory ( handle, category.members[i] );
            changed = true;
        }
    }
    for( int i =0; i < newEnts.Size(); i ++ ) {
        if( category.members.Find( newEnts[i] ) == 0 ) {
            this->AddEntToCategory      ( handle, newEnts[i] );
            changed = true;
        }
    }
    if( changed )
        this->CategoryChanged(handle);
    return changed;
}
/**-----------------------------------------------------------------------------//
 * remove all entities from a particular category
--------------------------------------------------------------------------------*/
bool nCategoryServer::ClearCategory( int handle ) {
    //n_printf("nCategoryServer(%s).ClearCategory(%i)\n", str(this), handle );
    bool clearedSomething = false;
    while( categories[handle].members.Size() > 0 ) {
        this->RemoveEntFromCategory( handle, categories[handle].members[0] );
        clearedSomething = true;
    }
    if( clearedSomething )
        this->CategoryChanged(handle);
    return clearedSomething;
}
/**-----------------------------------------------------------------------------//
 * just return the list of entities in a particular category, we should probably use a map or something to store the categories so we can get quick access here
 * well even better then that we can pull handles while its running! aha thats a good idea
 * okay so we'll use static integers passed by reference to store category info in the short term
--------------------------------------------------------------------------------*/
const EntArray* nCategoryServer::GetEntsInCategory ( int handle ){
    //n_printf("nCategoryServer(%s).GetEntsInCategory( %i )\n", str(this), handle );
    return &categories[handle].members;
}
/**-----------------------------------------------------------------------------//
 * called when a category chas changed
 * it should link this on to the eventserver, which can take it from there
 * anyone who wants can then link to these events
 * currently not linked in
--------------------------------------------------------------------------------*/
void nCategoryServer::CategoryChanged( int handle ) {
    return;
    nString eventName;
    eventName.PrintF("categoryChanged_%s", categories[handle].name.Get() );
    ifvalid( refEventServer )
        refEventServer->pull( categories[handle].event, eventName.Get(), categories[handle].name.Get(), /*override auto create*/ true);
}
