#define N_IMPLEMENTS nEventServer
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/neventserver.h"
#include "tnl/ntnl.h"

nNebulaScriptClass(nEventServer, "nthing");
PyObject* pythoncmd_addEvent( PyObject * /*self*/, PyObject *args);
PyObject* pythoncmd_pull    ( PyObject * /*self*/, PyObject *args);
PyObject* pythoncmd_pullLater( PyObject * /*self*/, PyObject *args);
PyObject* pythoncmd_register( PyObject * /*self*/, PyObject *args);
nEventServer* eventServer = 0;
//-----------------------------------------------------------------------------//
///python functions to register in the module
//-----------------------------------------------------------------------------//
///l/
static PyMethodDef nEventServerMethods[] = {
    {"addEvent",pythoncmd_addEvent,METH_VARARGS,NULL},
    {"pull",    pythoncmd_pull,    METH_VARARGS,NULL},
    {"pullLater",pythoncmd_pullLater,    METH_VARARGS,NULL},
    {"register",pythoncmd_register,METH_VARARGS,NULL},
    {NULL,NULL,0,NULL}    /* Sentinel */
};

//------------------------------------------------------------------------------
nEventServer::nEventServer():
tnl( kernelServer, this ){
    eventServer = this;
    autoCreate = false;
    tnl = "/net/tnl";
    this->AddToTriggerList();
}
//------------------------------------------------------------------------------
void nEventServer::Initialize(){
    nThing::Initialize();
    Py_InitModule("nEventServer",  nEventServerMethods);
}
//------------------------------------------------------------------------------
nEventServer::~nEventServer(){
    while( hooks.RemHead() );
}
//------------------------------------------------------------------------------
void nEventServer::Tick( float time, float dtime ){
    //n_printf("nEventServer.Tick()\n");
    nThing::Tick( time, dtime );
    while( pullLaters.Size() > 0 ) {
        pullLaterInfo pli = pullLaters.Pop();
        //n_printf("triggering pull later:%s,%s\n", pli.tag.Get(), pli.msg.Get() );
        this->pullInefficient( pli.tag.Get(), pli.msg.Get() );
    }
}
//-----------------------------------------------------------------------------//
void nEventServer::registerPythonFunction(const char* tag, PyObject* function){
    //n_printf("nEventServer::registerPythonFunction(%s,%p)\n", tag, function );
    Event* e = (Event*) hooks.Find(tag);
    if ( e == 0 && ( autoCreate || (strstr(tag, "key_") == tag)))                                                               //no event for that tag, create it
        e = newEvent(tag);
    if( e == 0 ) {
        n_printf("ERROR - trying to link a python function to a non existent event:%s\n", tag);
        return;
    }
    e->addPythonFunction(function);
}
/**-----------------------------------------------------------------------------//
 * Registers a c function call to be pulled when an event happens
 * this works through the nebula command system, which does a good job of dispatching this stuff
--------------------------------------------------------------------------------*/
void nEventServer::registerCFunction(const char* tag, nRoot* object, ulong cmdId ){
    //n_printf("nEventServr(%s).registerCFunction(%s,%s,%i)\n", str(this), tag, str(object), cmdId);
    Event* e = (Event*) hooks.Find(tag);
    if ( e == 0 && ( autoCreate || (strstr(tag, "key_") == tag)))                                                               //no event for that tag, create it
        e = newEvent(tag);
    if( e == 0 ) {
        n_printf("ERROR - trying to pull non existent event:%s\n", tag);
        return;
    }
    e->addCFunction(object, cmdId);
}
//-----------------------------------------------------------------------------//
void nEventServer::pull( void*& cachedEvent, const char* tag, const char* msg, bool overrideAutoCreate ){
    //n_printf("nEventServer::pull(%i,%s,%s)\n", index, tag, msg);
    nRoot* cwdBeforeEvent = kernelServer->GetCwd();
    if ( cachedEvent == 0 ) {
        Event* e = (Event*) hooks.Find(tag);
        if( e == 0 && (autoCreate || overrideAutoCreate || (strstr(tag, "key_") == tag)))
            e = newEvent( tag );
        if( e == 0 ) {
            n_printf("ERROR - trying to pull non existent event:%s,%s\n", tag,msg );
            return;
        }
        cachedEvent = e;
    }
    ((Event*)cachedEvent)->pull( tag, msg );
    //broadcast me
    //if( ((Event*)cachedEvent)->getCategory() == EVENT_CATEGORY_BROADCAST  )
        //ifvalid( tnl )
            //tnl->getConnection()->rpcBroadcastEvent( tag, msg );

    nRoot* cwdAfterEvent = kernelServer->GetCwd();
    if( cwdBeforeEvent != cwdAfterEvent )
        n_printf("people who responded to event: %s,%s failed to fix the selection stack\n", tag, msg);
}
//-----------------------------------------------------------------------------//
void nEventServer::pullInefficient( const char* tag, const char* msg, bool remotePull ){
    nRoot* cwdBeforeEvent = kernelServer->GetCwd();
    Event* e = (Event*) hooks.Find(tag);
    if( e == 0 && (autoCreate || (strstr(tag, "key_") == tag)))
        e = newEvent( tag );
    if( e == 0 ) {
        n_printf("ERROR - trying to pullIneffecient non existent event:%s,%s\n", tag,msg );
        return;
    }
    e->pull( tag, msg );
    //broad cast this event
    //n_printf("pullIneffecient(%s,%s,%i), %i\n", tag, msg, remotePull, e->getCategory() );
    //if( remotePull == false && e->getCategory() == EVENT_CATEGORY_BROADCAST  )
        //ifvalid( tnl )
            //tnl->getConnection()->rpcBroadcastEvent( tag, msg );
    nRoot* cwdAfterEvent = kernelServer->GetCwd();
    if( cwdBeforeEvent != cwdAfterEvent )
        n_printf("people who responded to event: %s,%s failed to fix the selection stack\n", tag, msg);
}
//-----------------------------------------------------------------------------//
void nEventServer::pullLater( const char* tag, const char* msg ) {
    //n_printf("nEventServer::pullLater(%s,%s)\n", tag, msg);
    for( int i = 0; i < pullLaters; i ++ ) {
        if (pullLaters[i].tag == tag) {
            //n_printf("pullLater(%s,%s) matches previous pullLater(%s,%s)\n", tag, msg, pullLaters[i].msg.Get(), pullLaters[i].msg.Get() );
            return;
        }
    }
    pullLaterInfo& pli = pullLaters.Push();
    pli.tag = tag;
    pli.msg = msg;
    //n_printf("pullLaters.Size(): %i\n", pullLaters.Size());
}
//-----------------------------------------------------------------------------//
void nEventServer::addEvent( const char* tag, const char* categoryString ){
    Event* e = (Event*) hooks.Find(tag);
    if( e == 0 )
        this->newEvent(tag, parseEventCategoryString( categoryString ) );
    this->autoCreate = false;
}
//-----------------------------------------------------------------------------//
Event* nEventServer::newEvent(const char* tag, EventCategory category){
    Event* e = new Event(category);
    e->SetName(tag);
    hooks.AddTail(e);
    return e;
}
//-----------------------------------------------------------------------------//
///Static function to add an event
//-----------------------------------------------------------------------------//
PyObject* pythoncmd_addEvent( PyObject * /*self*/, PyObject *args){
    n_assert(eventServer && "Trying to add python events, but no python hook server exists");
    const char* tag;
    const char* categoryString;
    if(PyArg_ParseTuple(args, "ss:addEvent", &tag, &categoryString)) {
        eventServer->addEvent( tag, categoryString );
    }
    else
        n_error("failed to parse args to addEvent\n");
    Py_INCREF(Py_None);
    return Py_None;
}
//-----------------------------------------------------------------------------//
///Static function for python to call when the module function is called
///finds the actual nEventServer out of the global variable and calls registerPythonFunction
//-----------------------------------------------------------------------------//
PyObject* pythoncmd_pull( PyObject * /*self*/, PyObject *args){
    n_assert(eventServer && "Trying to pull python events, but no event server exists");
    //printf("nEventServer::pull()\n" );
    const char* tag;
    const char* msg;
    if(PyArg_ParseTuple(args, "ss:pull", &tag, &msg)) {
        eventServer->pullInefficient( tag, msg );
    }
    else if (PyArg_ParseTuple(args, "s:pull", &tag)) {
        eventServer->pullInefficient( tag, "none" );
    }
    else
        n_error("failed to parse args to pull\n");
    Py_INCREF(Py_None);
    return Py_None;
}
//-----------------------------------------------------------------------------//
///Static function for python to call when the module function is called
///finds the actual nEventServer out of the global variable and calls registerPythonFunction
//-----------------------------------------------------------------------------//
PyObject* pythoncmd_pullLater( PyObject * /*self*/, PyObject *args){
    n_assert(eventServer && "Trying to pulllater python events, but no event server exists");
    //printf("nEventServer::pull()\n" );
    const char* tag;
    const char* msg;
    if(PyArg_ParseTuple(args, "ss:pullLater", &tag, &msg)) {
        eventServer->pullLater( tag, msg );
    }
    else if (PyArg_ParseTuple(args, "s:pullLater", &tag)) {
        eventServer->pullLater( tag, "none" );
    }
    else
        n_error("failed to parse args to pull\n");
    Py_INCREF(Py_None);
    return Py_None;
}

//-----------------------------------------------------------------------------//
///Static function for python to call when the module function is called
///finds the actual nEventServer out of the global variable and calls registerPythonFunction
//-----------------------------------------------------------------------------//
PyObject* pythoncmd_register( PyObject * /*self*/, PyObject *args){
    n_assert(eventServer && "Trying to register python hook events, but no python hook server exists");
    //printf("nEventServer::registerFunction()\n" );
    const char*tag;
    PyObject* function;
    if(PyArg_ParseTuple(args, "sO:register", &tag, &function)) {
        if (!PyCallable_Check(function))
            n_error("nEventServer registering non callable object\n");
        else {
            Py_INCREF(function);
            eventServer->registerPythonFunction( tag, function );
        }
    }
    else
        n_error("failed to parse args to register\n");
    Py_INCREF(Py_None);
    return Py_None;
}
