#define N_IMPLEMENTS nGameServer

//------------------------------------------------------------------------------
//  (C) 2004    Chris Miles
//------------------------------------------------------------------------------

#include "enkiduo/nent.h"
#include "collide/ncollideobject.h"
#include "gfx/ngfxtypes.h"
#include "enkiduo/ngameserver.h"
#include "misc/nquickwatch.h"
#include "enkiduo/neventserver.h"

nNebulaScriptClass(nGameServer, "nthing");

nGameServer::nGameServer():
    gfx         (kernelServer, this ),
    input       (kernelServer, this ),
    script      (kernelServer, this ),
    pythonServer(kernelServer, this ),
    graph       (kernelServer, this ),
    shadow      (kernelServer, this ),
    channel     (kernelServer, this ),
    console     (kernelServer, this ),
    math        (kernelServer, this ),
    particle    (kernelServer, this ),
    fx          (kernelServer, this ),
    trigger     (kernelServer, this ),
    camera      (kernelServer, this ),
    octree      (kernelServer, this ),
    prim        (kernelServer, this ),
    game        (kernelServer, this ),
    vis         (kernelServer, this ),
    ui          (kernelServer, this ),
    collide     (kernelServer, this ),
    prof        (kernelServer, this ),
    ref_nEventServer( kernelServer, this )
{
    shouldQuit = false;
    tickScript = 0;
    mouse.pressed[0] = 0;
    mouse.pressed[1] = 0;
    mouse.pressed[2] = 0;
    mouse.pos.set(.5f,.5f);
    holdMouse = false;
    lastNonBusyCounter = 0;
    simple = false;
    collideContext = 0;

    mouseCenter.found = false;
}

void nGameServer::Initialize()
{
    nThing::Initialize();
    gfx =       "/sys/servers/gfx";
    input =     "/sys/servers/input";
    script =    "/sys/servers/script";
    pythonServer= "/sys/servers/python";
    graph =     "/sys/servers/sgraph2";
    shadow =    "/sys/servers/shadow";
    channel =   "/sys/servers/channel";
    console =   "/sys/servers/console";
    math =      "/sys/servers/math";
    particle =  "/sys/servers/particle";
    fx =        "/sys/servers/specialfx";
    trigger=    "/sys/servers/trigger";
    camera =    "/game/camera/node";
    octree =    "/sys/servers/octree";
    prim =      "/sys/servers/primitive";
    collide =   "/sys/servers/collide";
    prof =      "/sys/servers/prof";
    ref_nEventServer = "/sys/servers/event";
    ifvalid( collide )
        collideContext = collide->GetDefaultContext();

    game =      "/game";
    vis =       "/game/vis";
    ui   =      "/ui";

    drawGrid = false;
    drawAxis = true;
    //drawScreenGrid = true;
    memset( keys, 0, sizeof(keys) );
    memset( inFrustum, 0, sizeof(nOctElement*) * 1024 );
    numInFrustum = 0;

    visualizeCollision = false;
    const char* err = 0;
    //pythonServer->RunScript("from common.app import *", err);
    //if( err )
        //n_printf("Error importing app - %s\n", err );
}

nGameServer::~nGameServer()
{
}
///Start the mainloop
void nGameServer::Start()
{
    this->loopCounter = 0;
    this->lastTime = -1.0f;
    n_assert( ui.isvalid() && game.isvalid() );
    ifvalid(ui)
        trigger->AddToTriggerList( ui.get() );
    trigger->AddToTriggerList( game.get() );

    kernelServer->ts->EnableFrameTime();
    kernelServer->ts->ResetTime();
    while( 1 ){
        if( shouldQuit ){
            n_printf("quitting: shouldQuit == false\n");
            break;
        }
        if( gfx.isvalid() == false ){
            //n_printf("quitting: gfx.isvalid == false\n");
            //break;
        }
        if( gfx.isvalid() && gfx->Trigger() == false ){
            n_printf("quitting: gfx->trigger() == false\n");
            break;
        }
        if( this->MainLoop() == false ){
            n_printf("quitting:: gameServer->MainLoop() == false\n");
            break;
        }
    }
    kernelServer->SetCwd(kernelServer->Lookup("/") );
    //RUNSCRIPT("/sys/servers/release.releasenow /");
}
///Stop the game
void nGameServer::Stop(){
    shouldQuit = true;
    static void* eventPtr = 0;
    ref_nEventServer->pull( eventPtr, "closing", this->fn );
}
///The main loop
bool nGameServer::MainLoop(){
    prof->BeginFrame();
    //-----------------------------------------------------------------------------//
    //trigger the kernelserver / timeserver
    //-----------------------------------------------------------------------------//
    kernelServer->Trigger();
    kernelServer->ts->Trigger();
    //-----------------------------------------------------------------------------//
    //calculate the time
    //-----------------------------------------------------------------------------//
    float time = (float) kernelServer->ts->GetFrameTime();
    float dtime = time - lastTime;
    //-----------------------------------------------------------------------------//
    //trigger the tcl server
    //-----------------------------------------------------------------------------//
    prof->Begin("Script");
    script->Trigger();
    prof->End("Script");
    //-----------------------------------------------------------------------------//
    //trigger the python server
    //-----------------------------------------------------------------------------//
    prof->Begin("Python");
    this->triggerWXPython(trigger->GetGameTime(), dtime);
    prof->End("Python");
    //-----------------------------------------------------------------------------//
    //input event handling
    //-----------------------------------------------------------------------------//
    prof->Begin("Input Handling");
    input->Trigger( time );
    for( nInputEvent* ie = input->FirstEvent(); ie; ie = input->NextEvent(ie))
        if( this->HandleDirectInput(ie, time) == false )
            n_printf("failed to handle input\n");
    input->FlushEvents();
    prof->End("Input Handling");
    //-----------------------------------------------------------------------------//
    //trigger game entities
    //-----------------------------------------------------------------------------//
    prof->Begin("Trigger");
    //n_printf("preTrigger\n");
    n_assert( game.isvalid() );
    trigger->TriggerThings( time );
    prof->End("Trigger");
    //-----------------------------------------------------------------------------//
    //check for collisions
    //-----------------------------------------------------------------------------//
    if( collideContext )
        collideContext->Collide();
    //-----------------------------------------------------------------------------//
    //set the time channel / nenv
    //-----------------------------------------------------------------------------//
    float gameTime = trigger->GetGameTime();
    channel->SetChannel1f( channel->GenChannel("time"), gameTime );
    static nQuickWatch w(kernelServer, "time");
    w->SetF( gameTime );
    //-----------------------------------------------------------------------------//
    //DEPRECATED: tick any tcl tick scripts
    //-----------------------------------------------------------------------------//
    prof->Begin("TickScript");
    if( this->tickScript ){
        RUNSCRIPT( tickScript );
    }
    prof->End("TickScript");
    //-----------------------------------------------------------------------------//
    //DEPRECATED: run any non busy scripts
    //-----------------------------------------------------------------------------//
    if( lastTime > 0 ) {
        if( loopCounter - lastNonBusyCounter > 5 && loopCounter > 10 ) {
            if( dtime < .2 ){
                if( nonBusyScripts.Size() > 0 ){
                    //n_printf("NONBUSY SCRIPTS GOING! size = %i\n", nonBusyScripts.Size());
                    nString script = nonBusyScripts.Pop();
                    //n_printf("script: %s\n", script.Get() );
                    RUNSCRIPT( script.Get() );
                    this->lastNonBusyCounter = loopCounter;
                }
            }
        }
    }
    //-----------------------------------------------------------------------------//
    //Render the scene
    //-----------------------------------------------------------------------------//
    prof->Begin("gfx");
    if( gfx.isvalid() && gfx->BeginScene() )
    {    
        prof->Begin("misc");
        gfx->Rgba( 1.0f,1.0f,1.0f,1.0f );
        matrix44 vwr;
        matrix44 vwr_inverted;
        matrix44 ident;
        ident.ident();
        if( camera.isvalid() ){
            vwr = camera->GetM();                                               //pull the viewer matrix
            float x, y, z;
            camera->GetT(x,y,z);
            channel->SetChannel1f( channel->GenChannel("camera_alt"), y);       //global camera altitude channel
            vector3 forward(0,0,-1);
            forward = vwr * forward;
            forward -= vector3(x,y,z);
            float theta = n_rad2deg(atan2(forward.z, forward.x));
            //n_printf("forward:%s theta:%f\n", str(forward), theta);
            channel->SetChannel1f( channel->GenChannel("camera_heading"), theta);       //global camera altitude channel
        }
        else{
            vwr.ident();                                                        //no camera, view is identity matrix
            once( n_printf("nGameServer::camera.isvalid() == false\n") );
        }
        //-----------------------------------------------------------------------------//
        //Set the viewer matrix
        //-----------------------------------------------------------------------------//
        gfx->SetMatrix( N_MXM_VIEWER, vwr );
        particle->Trigger();
        vwr_inverted = vwr;
        vwr_inverted.invert_simple();
        prof->End("misc");

        //-----------------------------------------------------------------------------//
        //Octree culling
        //-----------------------------------------------------------------------------//
        //prof->Begin("octree");
        //ifvalid( octree ) {
            //octree->BalanceTree();
            //numInFrustum = octree->CollectByFrustum( gfx.get(), inFrustum, 1024 );
            //FIXME - actually do culling
        //}
        //prof->End("octree");

        //-----------------------------------------------------------------------------//
        //Render the main scene
        //-----------------------------------------------------------------------------//
        prof->Begin("Scenegraph");
        if( graph->BeginScene( vwr_inverted ) ){
            if( fx.isvalid() )          fx->Begin();                                                                    //draw any special fx, not really used
            else                        once( n_printf("fx not valid()\n") );                                           
            if( vis.isvalid() )         graph->Attach( (nVisNode*) vis.get(), 0 );                                      //draw the world, just pass the main window
            else                        once( n_printf("vis not valid()\n") );
            ifvalid( octree )           octree->Visualize( gfx.get() );                                                 //if the octree wants to be drawn draw it
            if( fx.isvalid() )          fx->End( graph.get() );                                                         //stop with the fx
            else                        once( n_printf("fx not valid()\n") );
            graph->EndScene( true );
        }
        prof->End("Scenegraph");

        //-----------------------------------------------------------------------------//
        //Render any custom displays things have requested
        //-----------------------------------------------------------------------------//
        prof->Begin("Display");
        //set default flags
        nRState rs;
        rs.Set( N_RS_ZFUNC, N_CMP_LESS );
        gfx->SetState( rs );
        rs.Set( N_RS_ALPHABLENDENABLE, N_TRUE );
        gfx->SetState( rs );
        rs.Set( N_RS_ZFUNC, N_CMP_LESS );
        gfx->SetState( rs );
        rs.Set( N_RS_ALPHATESTENABLE, N_TRUE );
        gfx->SetState( rs );

        gfx->SetMatrix( N_MXM_MODELVIEW, vwr_inverted );
        prim->SetColor(1,1,1,1);
        this->RenderGrid( gfx.get() );                                          //draw the coord grid
        if( drawAxis )                  this->RenderAxis( gfx.get() );         // draw the coord axis
        //call registered display functions
        for( int i = 0; i < displayList.Size(); i ++ )
            displayList[i]->Display( gfx.get() );
        //draw the screen grid
        if( drawScreenGrid ){
            gfx->SetMatrix( N_MXM_MODELVIEW, ident );
            this->RenderScreenGrid( gfx.get() );
        }
        //visualize collision
        if( visualizeCollision )
            RenderCollideContext(gfx.get());
        //render the console text
        console->Render();
        prof->End("Display");
        gfx->EndScene();
    }
    prof->End("gfx");
    this->loopCounter++;

    //-----------------------------------------------------------------------------//
    //FPS style mouse - need to keep the mouse in a set position
    //-----------------------------------------------------------------------------//
    if( this->holdMouse )
        gfx->SetMousePosition( 200, 200 ); //problem is that this generates a mouse move event - which we have to cancel out elsewhere
    //gfx->HideCursor();

    lastTime = time;
    prof->EndFrame();
    return true;
}
//-----------------------------------------------------------------------------//
//Deal with direct input events
//-----------------------------------------------------------------------------//
#include "enkiduo/keys.h"
bool nGameServer::HandleDirectInput( nInputEvent* ie, float time ){
    const float maxPressedTime = .25f;
    if( ie->IsDisabled() ) return true;
    bool b = false;
    int id = ie->GetDeviceId();
    if( id > 150000 ) {
        if( roll( .0001 ) )
            n_printf("pissing away crap inputid:%i\n", id );
        return true;
    }
    int button = ie->GetButton();    
    vector2 np, cp, dp;
    nString eventName;
    int eventCategory; //plain, ctrl, alt, shift, ctrl_alt, ctrl_shift, ctrl_alt_shift
    bool found = false;
#ifdef __WIN32__
    if(button == 2 )
        button = 1;
    else if( button == 1 )
        button = 2;
#endif
    switch( ie->GetType() ){
        case N_INPUT_KEY_DOWN:
            //n_printf("%i.down @ %f\n", ie->GetKey(), time);
            keys[ie->GetKey()].pressed = 1;
            keys[ie->GetKey()].timeDown = time;
            b = true;
            break;
        case N_INPUT_KEY_UP:
            //n_printf("%i.up @ %f\n", ie->GetKey(), time);
            keys[ie->GetKey()].pressed = 0;
            if ( time - keys[ie->GetKey()].timeDown < maxPressedTime ) {
                //check modifier keys
                eventCategory = 0;
                //convert the state of the 3 modifier keys to an integer - some simple binary addition so that we can store a unique event for each
                if ( keys[N_KEY_CONTROL].pressed )                              eventCategory += 1;
                if ( keys[N_KEY_NONE].pressed )                                  eventCategory += 2;
                if ( keys[N_KEY_SHIFT].pressed )                                eventCategory += 4;
                //trigger pressed event
                if (keys[ie->GetKey()].pressedEvent[eventCategory] == 0){
                    eventName = "key_pressed_";
                    if ( keys[N_KEY_CONTROL].pressed )                          eventName += "ctrl_";
                    if ( keys[N_KEY_NONE].pressed )                              eventName += "alt_";
                    if ( keys[N_KEY_SHIFT].pressed )                            eventName += "shift_";

                    //n_printf("key:%i\n", ie->GetKey());
                    found = false;
                    for( int i = 0; key_remaptable[i].str != NULL; i ++ ) {
                        if (key_remaptable[i].key == ie->GetKey()) {
                            eventName += key_remaptable[i].str;
                            found = true;
                            break;
                        }
                    }
                    if ( found == false ) 
                        eventName += "alt";
                    ifvalid(ref_nEventServer)
                        ref_nEventServer->pull(keys[ie->GetKey()].pressedEvent[eventCategory], eventName.Get(), "1", /*overrideautocreate*/ true);
                    //n_printf("triggering event:%s\n", eventName.Get());
                }
                else
                    ifvalid(ref_nEventServer)
                        ref_nEventServer->pull(keys[ie->GetKey()].pressedEvent[eventCategory], "not looking up", "1");
            } 
            b = true;
            break;
        case N_INPUT_BUTTON_DOWN:
            mouse.pressed[button] = 1;            
            mouse.posPressed[button].set( ie->GetRelXPos(), ie->GetRelYPos() );
            b = true;
            break;
        case N_INPUT_BUTTON_UP:
            mouse.pressed[button] = 0;
            mouse.posReleased[button].set( ie->GetRelXPos(), ie->GetRelYPos() );
            b = true;
            break;
        case N_INPUT_MOUSE_MOVE:
            np.set( ie->GetAbsXPos(), ie->GetAbsYPos() );     //where the mouse is now
            cp = mouse.pos - mouse.corrections;            //where we were before
            dp = np - cp;            //how far we have moved

            if ( this->holdMouse ) {    
                if( mouseCenter.found == false ) {
                    mouseCenter.history.Append( np );
                    bool first = true;
                    vector2 mostCommon;
                    int mostFound;
                    for( int i = 0; i < mouseCenter.history; i ++ ) {
                        vector2 current = mouseCenter.history[i];
                        int numFound = 0;
                        for( int j = i; j < mouseCenter.history; j ++ )
                            if( mouseCenter.history[i].isequal(current, .01) )
                                numFound++;
                        //printf("v:(%f,%f) num:%i\n", current.x, current.y, numFound );
                        if( first || numFound > mostFound ) {
                            mostCommon = current;
                            mostFound = numFound;
                            first = false;
                        }
                    }
                    if( mostFound > 50 ) {
                        printf("most v:(%f,%f) num:%i\n", mostCommon.x, mostCommon.y, mostFound );
                        mouseCenter.pos = mostCommon;
                        mouseCenter.found = true;
                    }
                }
                else {
                    if ( np.isequal( mouseCenter.pos, .01 ) ) {
                        mouse.corrections -= dp;        //add this movement onto the corrections
                    } else {                //its a user initiated move
                        mouse.pos = np + mouse.corrections;        //set the new mouse pos
                        n_error("pos adjusted at too:%s\n", str(mouse.pos) );
                    }
                }
            } else {    //regular mouse    
                mouse.pos = vector2(ie->GetRelXPos(), ie->GetRelYPos() );        //set the new mouse pos
                //n_printf("mousepos:%s\n", str(mouse.pos) );
            }
            b = true;
            break;
        default:
            //n_printf("Unhandled input type\n");
            b = true;
            break;
    }
    return b;
}
//-----------------------------------------------------------------------------//
//Render the coord grid
//-----------------------------------------------------------------------------//
void nGameServer::RenderGrid( nGfxServer* ){
    prim->SetColor(.1f,.1f,0.4f, 0.1f);
    if( this->drawGrid )
        prim->WirePlane(50, 1);
    else 
        prim->WirePlane(0,1);
}
//-----------------------------------------------------------------------------//
//Draw the coordinate axis
//-----------------------------------------------------------------------------//
void nGameServer::RenderAxis( nGfxServer* gfx ){
    gfx->Begin( N_PTYPE_LINE_LIST );
    //X - RED
    gfx->Rgba(        1.0f,    0.0f,    0.0f,    1.0f );
    gfx->Coord(        0.0f,    .05f,    0.0f );
    gfx->Coord(  1000.0f,    .05f,    0.0f );
    //Y - GREEN
    gfx->Rgba(        0.0f,    1.0f,    0.0f,    1.0f );
    gfx->Coord(        0.0f,    .05f,    0.0f );
    gfx->Coord(        0.0f,1000.0f,    0.0f );
    //Z - BLUE
    gfx->Rgba(        0.0f,    0.0f,    1.0f,    1.0f );
    gfx->Coord(        0.0f,    .05f,    0.0f );
    gfx->Coord(        0.0f,    .01f,    1000.0f );
    gfx->End();
}
//-----------------------------------------------------------------------------//
//Draw the screen grid
//-----------------------------------------------------------------------------//
void nGameServer::RenderScreenGrid( nGfxServer* gfx ){
    gfx->Begin( N_PTYPE_LINE_LIST );
    gfx->Rgba( 1, 1, 1, 1 );
    for( float x = -1.0f; x <= 1.0f; x += .1f ){
        gfx->Coord( x,-1,-1 );
        gfx->Coord( x, 1,-1 );
        gfx->Coord(-1, x,-1 );
        gfx->Coord( 1, x,-1 );
    }
    gfx->End();
}
//-----------------------------------------------------------------------------//
//Draw the collide stuff
//-----------------------------------------------------------------------------//
void nGameServer::RenderCollideContext( nGfxServer* gfx ){
    if( collideContext )
        collideContext->Visualize( gfx, prim.get() );
}
//-----------------------------------------------------------------------------//
//All our silly inputs
//-----------------------------------------------------------------------------//
bool nGameServer::GetButton( const char* buttonName ){
    nString s = buttonName;
    if( s == "forward")                 return keys[N_KEY_Z].pressed;
    if( s == "backward")                return keys[N_KEY_X].pressed;
    if( s == "up")                      return keys[N_KEY_W].pressed;
    if( s == "down")                    return keys[N_KEY_S].pressed;
    if( s == "left")                    return keys[N_KEY_A].pressed;
    if( s == "right")                   return keys[N_KEY_D].pressed;
    if( s == "rollleft")                return keys[N_KEY_Q].pressed;
    if( s == "rollright")               return keys[N_KEY_E].pressed;
    if( s == "yawleft")                 return keys[N_KEY_LEFT].pressed;
    if( s == "yawright")                return keys[N_KEY_RIGHT].pressed;
    if( s == "pitchup")                 return keys[N_KEY_UP].pressed;
    if( s == "pitchdown")               return keys[N_KEY_DOWN].pressed;
    if( s == "boost")                   return keys[N_KEY_SPACE].pressed;
    if( s == "fireprimary")             return keys[N_KEY_CONTROL].pressed;
    if( s == "shift")                   return keys[N_KEY_SHIFT].pressed;
    if( s == "ctrl")                    return keys[N_KEY_CONTROL].pressed;
    if( s == "alt")                     return keys[N_KEY_MENU].pressed;
    if( s == "space")                   return keys[N_KEY_SPACE].pressed;
    n_error("failed to find button : %s\n", buttonName );
    return false;
}

MouseInfo nGameServer::GetMouseInfo(){    
    return this->mouse;
}
//-----------------------------------------------------------------------------//
//Create a new collision shape
//-----------------------------------------------------------------------------//
nCollideShape* nGameServer::NewCollideShape( const char* name, const char* file ){
    nCollideShape* shape = collide->NewShape(name);
    if( shape->IsInitialized() == false ){
        char filename[1024];
        sprintf( filename, "lib/%s/data/coll.n3d", file );
        //n_printf("loading collision file : %s\n", filename );
        shape->Load((nFileServer2*)kernelServer->Lookup("/sys/servers/file2"), filename );
    }
    return shape;
}
//-----------------------------------------------------------------------------//
//Create a new collision object
//-----------------------------------------------------------------------------//
nCollideObject* nGameServer::NewCollideObject( nRoot* ent ){
    if( collideContext == 0 ) return 0;
    nCollideObject* object = collideContext->NewObject();
    collideContext->AddObject( object );
    object->SetClientData( ent );
    return object;
}
//-----------------------------------------------------------------------------//
//Destroy a collision object
//-----------------------------------------------------------------------------//
void nGameServer::ReleaseCollideObject( nCollideObject* object ){
    if( collideContext == 0 ) return;
    if( object ){
        collideContext->RemObject( object );
        collideContext->ReleaseObject( object );
    }
}
/**-----------------------------------------------------------------------------//
 * Creates an entity
 * @param entType the type of entity to produce - this is taken from /ent/$type
 * @param path_s where the entity should be created - defaults to /game/ents/$type/
 * @param nameWanted_s the name of the entity - defaults to $type
 * genent guarantees the creation of a unique entity
 * it also links in the appropriate vis object from /vis/$type
 * mod it to create non-unique ents
--------------------------------------------------------------------------------*/
nEnt* nGameServer::GenEnt( const char* entType, const char* path_s, const char* nameWanted_s ) {
    //n_printf("nGameServer(%s).GenEnt(%s,%s,%s)\n", str(this), entType, path_s, nameWanted_s );
    n_assert( entType );
    nString path;
    bool unique = true;
    if( path_s )
        path = path_s;
    else
        path.PrintF("/game/ents/%s/", entType);

    nString nameWanted;
    if( nameWanted_s ) {
        nameWanted = nameWanted_s;
        unique = false;
    }
    else
        nameWanted = entType;
    //n_printf("GenEnt(%s,%s,%s,%i)\n", entType, path.Get(), nameWanted.Get(), unique );

    nRoot* p = kernelServer->Lookup( path.Get() );
    if( !p ) {
        //n_printf("ERROR: trying to generate ent:%s[%s] into nonexistant path:%s\n", nameWanted.Get(), entType, path.Get() );
        p = kernelServer->New("nthing", path.Get() );
    }
    int i = 0;
    nEnt* ent;
    nVisNode* vis;
    nString name;
    nString newPath;
    while ( 1 ) {
        if( unique ) 
            name.PrintF("%s%i", nameWanted.Get(), i );
        else
            name = nameWanted;
        newPath.PrintF("%s/%s", path.Get(), name.Get() );
        //n_printf("trying name:%s i:%i\n", name.Get(), i );
        if( p->Find(name.Get()) ) {
            //n_printf("found\n");
            if( !unique ) {
                return (nEnt*) p->Find( name.Get() );
            }
            i++;
        }
        else {
            //Step 1 - create ent
            nString parentPath;
            classVar(nent, entClass );
            parentPath.PrintF("/lib/ents/%s", entType );
            nRoot* parent = kernelServer->Lookup( parentPath.Get() );
            if( !parent ) {
                parentPath.PrintF("/ent/%s", entType );
                nRoot* parent = kernelServer->Lookup( parentPath.Get() );
                if( parent )
                    n_printf("using old style ent:%s\n", entType );
            }
            if( !parent ) {
                n_printf("ERROR::trying to create an ent of type:%s which was not found\n", parentPath.Get() );
                n_assert(0);
                return 0;
            } else {
                n_assert( parent->IsA(entClass ) && "trying to clone from a non-ent" );
                ent = (nEnt*) parent->Clone( newPath.Get() );
                break;
            }
        }
    }
    //Step 2 - create vis
    if( simple == false ) { 
        vis = this->GenVis( entType );
        if( ent && vis ) {
            char fullname[1024];
            vis->GetFullName(fullname, 1024);
            ent->SetVis( fullname );
        } else  {
            n_printf("ERROR: cannot set ent->vis, %i %i\n", ent, vis );
        }
    }

    foreachchild(nThing, aspect, ent){
        if( nString("engine") == aspect->GetName() ) //FIXME
            aspect->Tick(0,0);
    }
    if( ent->HasVar("creationEvent") ) {
        //static void* eventPtr = 0;
        ref_nEventServer->pullInefficient( ent->GetVarS("creationEvent"), ent->fn );
    }
    return ent;
}
///Generates and returns a visnode of the specified type
n3DNode* nGameServer::GenVis( const char* type ) {
    n3DNode* vis;
    nString path;
    path.PrintF("/game/vis/%s", type );
    nRoot* p = kernelServer->Lookup( path.Get() );
    if( !p ) {
        //n_printf("ERROR: trying to generate ent:%s[%s] into nonexistant vis path:%s\n", nameWanted.Get(), entType, path.Get() );
        p = kernelServer->New("nvisnode", path.Get() );
    }
    int i = 0;
    while( 1 ) {
        nString name;
        nString newPath;
        name.PrintF("%s%i", type, i );
        newPath.PrintF("%s/%s", path.Get(), name.Get() );
        //n_printf("trying name: %s\n", name.Get() );
        if( p->Find(name.Get()))
            i++;
        else {
            nString parentPath;
            parentPath.PrintF("/lib/vis/%s", type );
            nRoot* parent = kernelServer->Lookup( parentPath.Get() );
            if( !parent ) {
                parentPath.PrintF("/vis/%s", type );
                nRoot* parent = kernelServer->Lookup( parentPath.Get() );
                if( parent )
                    n_printf("using old style vis:%s\n", type );
            }
            if( !parent ){
                n_printf("ERROR: vis node not found for ent type: %s\n", type );
            } else {
                vis = (n3DNode*) parent->Clone( newPath.Get() );
            }
            break;
        }
    }
    return vis;
}
//-----------------------------------------------------------------------------//
void nGameServer::RunTCLScript( const char* fileName ){
    const char* err = 0;
    script->RunScript( fileName, err );
    if( err ) 
        printf("%s\n", err );
}
//-----------------------------------------------------------------------------//`
/**
 * @brief triggers the wxPython app
 * calls common.app.tickApp(0
 * if pythonServer is invalid just returns
**/
void nGameServer::triggerWXPython( float time, float dtime ){
    ifvalid( pythonServer ) {
        const char* err = 0;
        static int i = 0;
        if( i++ == 0 ) {
            pythonServer->Run("from common.app import *", err);
            if( err && err[0] != '\0')
                n_printf("Error Ticking python-%s\n", err );
        }
        nString cmd;
        cmd.PrintF("tickApp(%f,%f)", time, dtime );
        pythonServer->Run( cmd.Get(), err );
    }
}
//-----------------------------------------------------------------------------//
