#define N_IMPLEMENTS nOggStreamAL
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/noggstreamal.h"

nNebulaScriptClass(nOggStreamAL, "nthing");

//------------------------------------------------------------------------------
/**
*/
nOggStreamAL::nOggStreamAL()
{
}

void nOggStreamAL::Initialize(){
    nThing::Initialize();
}

nOggStreamAL::~nOggStreamAL(){
    alSourceStop(source);
    empty();
    alDeleteSources(1, &source);
    check();
    alDeleteBuffers(1, buffers);
    check();
    ov_clear(&oggStream);
}
//------------------------------------------------------------------------------
/**
*/

void nOggStreamAL::Tick( float time, float dtime ){
    nThing::Tick( time, dtime );
}

void nOggStreamAL::Open(const char* path)
{
    int result;
    
    if(!(oggFile = fopen(path, "rb")))
        throw string("Could not open Ogg file.");
    if((result = ov_open(oggFile, &oggStream, NULL, 0)) < 0)
    {
        fclose(oggFile);
        n_printf("Could not open ogg stream: %s\n", ErrorString( result ););
        return;
    }
    vorbisInfo = ov_info(&oggStream, -1);
    vorbisComment = ov_comment(&oggStream, -1);
    if(vorbisInfo->channels == 1)
        format = AL_FORMAT_MONO16;
    else
        format = AL_FORMAT_STEREO16;
        
        
    alGenBuffers(2, buffers);
    check();
    alGenSources(1, &source);
    check();
    
    alSource3f(source, AL_POSITION,        0.0, 0.0, 0.0);
    alSource3f(source, AL_VELOCITY,        0.0, 0.0, 0.0);
    alSource3f(source, AL_DIRECTION,       0.0, 0.0, 0.0);
    alSourcef (source, AL_ROLLOFF_FACTOR,  0.0          );
    alSourcei (source, AL_SOURCE_RELATIVE, AL_TRUE      );
}
void nOggStreamAL::info()
{
    nString s;
    s.PrintF("version:%s\nchannels:%s\nrate:%s\nbitrate upper:%s\n bitrate lower:%s\n bitratewindow:%s\nvendor:%s", vorbisInfo->version, vorbisInfo->channels, vorbisInfo->rate,  vorbisInfo->bitrate_upper, vorbisInfo->bitrate_nominal, vorbisInfo->bitrate_lower, vorbisInfo->bitrate_window, vorbisComment->vendor );
    for( int i = 0; i < vorbisComment->comments;  i++ ) {
        nString s2;
        s2.PrintF("\n%s", vorbisComment->user_comments[i] );
        s += s2;
    }
    return s;
}
bool nOggStreamAL::Playback()
{
    if(playing())
        return true;

    if(!stream(buffers[0]))
        return false;

    if(!stream(buffers[1]))
        return false;

    alSourceQueueBuffers(source, 2, buffers);
    alSourcePlay(source);

    return true;
}
bool nOggStreamAL::Playing()
{
    ALenum state;

    alGetSourcei(source, AL_SOURCE_STATE, &state);

    return (state == AL_PLAYING);
}
bool nOggStreamAL::Update()
{
    int processed;
    bool active = true;
    alGetSourcei(source, AL_BUFFERS_PROCESSED, &processed);
    while(processed--)
    {
        ALuint buffer;

        alSourceUnqueueBuffers(source, 1, &buffer);
        check();
        active = stream(buffer);
        alSourceQueueBuffers(source, 1, &buffer);
        check();
    }
    return active;
}
bool nOggStreamAL::Stream(ALuint buffer)
{
    char pcm[BUFFER_SIZE];
    int  size = 0;
    int  section;
    int  result;
    while(size < BUFFER_SIZE)
    {
        result = ov_read(&oggStream, pcm + size, BUFFER_SIZE - size, 0, 2, 1, &section);

        if(result > 0)
            size += result;
        else
            if(result < 0)
	throw errorString(result);
            else
	break;
    }

    if(size == 0)
        return false;

    alBufferData(buffer, format, pcm, size, vorbisInfo->rate);
    check();

    return true;
}
void nOggStreamAL::Empty()
{
    int queued;

    alGetSourcei(source, AL_BUFFERS_QUEUED, &queued);

    while(queued--)
    {
        ALuint buffer;

        alSourceUnqueueBuffers(source, 1, &buffer);
        check();
    }
}
void nOggStreamAL::Check()
{
    int error = alGetError();
    if(error != AL_NO_ERROR)
        throw string("OpenAL error was raised.");
}
string nOggStreamAL::ErrorString(int code)
{
    switch(code)
    {
        case OV_EREAD:
            return string("Read from media.");
        case OV_ENOTVORBIS:
            return string("Not Vorbis data.");
        case OV_EVERSION:
            return string("Vorbis version mismatch.");
        case OV_EBADHEADER:
            return string("Invalid Vorbis header.");
        case OV_EFAULT:
            return string("Internal logic fault (bug or heap/stack corruption.");
        default:
            return string("Unknown Ogg error.");
    }
}
