#define N_IMPLEMENTS nOpenALServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nopenalserver.h"
#include <unistd.h>
#include "enkiduo/nent.h"

nNebulaScriptClass(nOpenALServer, "nthing");

//------------------------------------------------------------------------------
ALfloat listenerPos[] = {0.0,0.0,4.0};
ALfloat listenerVel[] = {0.0,0.0,0.0};
ALfloat listenerOri[] = {0.0,0.0,1.0,0.0,1.0,0.0};
ALfloat sourcePos[] = {-2.0,0.0,0.0};
ALfloat sourceVel[] = {0.0,0.0,0.0};
//------------------------------------------------------------------------------

nOpenALServer::nOpenALServer():
camera( kernelServer, this ),
    dopplerCoef(1.0f),
    cameraVelCoef(1.0f)
{

    this->listenerPos                   = listenerPos;
    this->listenerVel                   = listenerVel;
    this->listenerOri                   = listenerOri;
    this->defaultSourcePos              = sourcePos;
    this->defaultSourceVel              = sourceVel;
    this->AddToTriggerList();

    //printf("consrructed listenerOri:%p\n", this->listenerOri);
    this->AddLogTag("openalserver");
}

void nOpenALServer::Initialize(){
    nThing::Initialize();

    alutInit( NULL, 0 );
    CheckError();

    alListenerfv( AL_POSITION, this->listenerPos );
    alListenerfv( AL_VELOCITY, this->listenerVel );
    alListenerfv( AL_ORIENTATION, this->listenerOri );
    CheckError();

    camera = "/game/camera";
    this->SetDopplerCoef( .01 );
}

nOpenALServer::~nOpenALServer(){
    alDeleteBuffers(buffer.AllocSize(), buffer);
    alDeleteSources(source.AllocSize(), source);
    alutExit();
}
//------------------------------------------------------------------------------
/**
 */

void nOpenALServer::Tick( float time, float dtime ){
    nThing::Tick( time, dtime );
    this->UpdateListener();
}

void nOpenALServer::CheckError() {
    int error = alGetError();
    if( error != AL_NO_ERROR ){
        log("ERROR: %i", error );
    }
}
void nOpenALServer::InitBuffers( int size ) {
    buffer.Reallocate( size,0 );
    alGenBuffers( size, buffer.Begin() );
    source.Reallocate( size,8 );
    alGenSources( size, source.Begin() );
}
void nOpenALServer::LoadWav( const char* name, const char* filename ) {
    void* data;
    log("nOpenALServer::LoadWav(%s,%s)", name, filename );
    Sound& s = sounds.Push();
    s.name = name;
    s.filename = filename;
    s.buffer = buffer.Push();
    s.source = source.Push();
    log("s.buffer:%i s.source:%i", s.buffer, s.source );
    //setup that source
    ALboolean al_bool;
    alutLoadWAVFile((ALbyte*) filename, &s.format, &data, &s.size, &s.freq, &al_bool ); 
    log("format:%i data:%p size:%i freq:%i al_bool:%i", s.format, data, s.size, s.freq, al_bool );
    alBufferData( s.buffer, s.format, data, s.size, s.freq );
    alutUnloadWAV( s.format, data, s.size, s.freq );
    CheckError();

    alSourcei(  s.source, AL_BUFFER, 	s.buffer );
    alSourcef( 	s.source, AL_PITCH, 	1.0 );	
    alSourcef( 	s.source, AL_GAIN, 	0.1 );
    alSourcefv( s.source, AL_POSITION, 	defaultSourceVel );
    alSourcefv( s.source, AL_VELOCITY, 	defaultSourceVel );
    alSourcei( 	s.source, AL_LOOPING, 	AL_FALSE );
    //alSourcef(	s.source, AL_ROLLOFF_FACTOR,	.1f );
    CheckError();
}
void nOpenALServer::Play( const char* sound ) {
    log("nOpenALServer::Play(%s)", sound );
    for( int i = 0; i < sounds; i ++ ) {
        if( sounds[i].name == sound ) {
            alSourcePlay( sounds[i].source );
            CheckError();
        }
    }
}
void nOpenALServer::Stop( const char* soundname ) {
    for( int i = 0; i < sounds; i ++ ) {
        static nString all("all");
        if( sounds[i].name == soundname || all == soundname ) {
            alSourceStop	( sounds[i].source );
            CheckError();
        }
    }
}
void nOpenALServer::StartLoop( const char* sound ) {
    log("nOpenALServer::StartLoop(%s)", sound );
    for( int i = 0; i < sounds; i ++ ) {
        if( sounds[i].name == sound ) {
            alSourcef( sounds[i].source, AL_GAIN, 	0.03 );
            alSourcei( sounds[i].source, AL_LOOPING, 	AL_TRUE );
            alSourcePlay( sounds[i].source );
            CheckError();
        }
    }
}

int nOpenALServer::CreateEmitter() {
    int handle = emitters.Size();
    Sound& s = emitters.Push();
    s.source = this->CreateSource();
    return handle;
}
int nOpenALServer::CreateEmitter( const char* name ) {
    for( int i = 0; i < sounds; i ++ ) {
        if( sounds[i].name == name ) {
            int handle = emitters.Size();
            Sound& s 	= emitters.Push();
            s.format 	= sounds[i].format;
            s.size 	= sounds[i].size;
            s.freq	= sounds[i].freq;
            s.buffer	= sounds[i].buffer;
            s.source	= source.Push();

            alSourcei(  	s.source, AL_BUFFER, 	s.buffer );
            alSourcef( 	s.source, AL_PITCH, 	1.0 );	
            alSourcef( 	s.source, AL_GAIN, 	1.0 );
            alSourcefv( 	s.source, AL_POSITION, 	defaultSourceVel );
            alSourcefv( 	s.source, AL_VELOCITY, 	defaultSourceVel );
            alSourcei( 	s.source, AL_LOOPING, 	AL_TRUE );
            //alSourcef(	s.source, AL_ROLLOFF_FACTOR,	.1f );

            alSourcePlay	( s.source );
            CheckError();
            log("created emitter : %i playing %s", i, name );
            return handle;
        }
    }
    return -1;
}
void nOpenALServer::UpdateEmitter( int handle, vector3 pos, vector3 vel ) {
    if( pos.len() > 1000000 );
    pos.norm();
    alSourcefv( emitters[handle].source, AL_POSITION, 	(float*)(&pos) );
    alSourcefv( emitters[handle].source, AL_VELOCITY, 	(float*)(&vel) );
    //log("relocated emitter %i to %s %s", handle, str(pos), str(vel) );
}
void nOpenALServer::UpdateListener() {
    vector3 pos = camera->GetVarV("pos");
    //n_printf("listener: %s\n", str(pos));
    quaternion rot = camera->GetVarQ("rot");
    vector3 vel = camera->GetVarV("vel");
    vel *= cameraVelCoef;
    alListenerfv( AL_POSITION, (float*)(&pos));
    alListenerfv( AL_VELOCITY, (float*)(&vel));
    vector3 forward (0,0,-1);
    vector3 up ( 0,1,0 );
    forward = rot.rotate( forward );
    up = rot.rotate( up );
    float orientation[6];
    orientation[0] = forward.x;
    orientation[1] = forward.y;
    orientation[2] = forward.z;
    orientation[3] = up.x;
    orientation[4] = up.y;
    orientation[5] = up.z;
    alListenerfv( AL_ORIENTATION, orientation );

    for ( int i = 0; i < sounds; i ++ ) {
        alSourcefv( sounds[i].source, AL_POSITION, 	(float*)(&pos) );
        //alSourcefv( sounds[i].source, AL_VELOCITY, 	(float*)(&vel) );
    }

}
int nOpenALServer::CreateSource() {
    int s = source.Push();
    alSourcef( 	s, AL_PITCH, 		1.0 );	
    alSourcef( 	s, AL_GAIN, 		1.0 );
    alSourcefv( s, AL_POSITION, 	defaultSourcePos );
    alSourcefv( s, AL_VELOCITY, 	defaultSourceVel );
    return s;
}
void nOpenALServer::Queue( int handle, const char* soundName ) {
    //n_printf("nOpenALServer.Queue(%i, %s)\n", handle, soundName);
    int i;
    for( i = 0; i < sounds; i ++ )
        if( sounds[i].name == soundName )
            break;
    if( i < sounds == false ) {
        log("failed to find sound:%s", soundName );
        return; //couldn't find the sound
    }
    alSourcei	( emitters[handle].source, AL_BUFFER, sounds[i].buffer );
    alSourcePlay	( emitters[handle].source );
}
void nOpenALServer::StopPlaying	( int emitterHandle ) {
    alSourceStop	( emitters[emitterHandle].source );
}
void nOpenALServer::SetLooping	( int emitterHandle , bool looping) {
    alSourcei( 	emitters[emitterHandle].source, AL_LOOPING, looping? AL_TRUE: AL_FALSE );
}
void nOpenALServer::SetGain	( int emitterHandle, float gain ) {
    alSourcef( emitters[emitterHandle].source, AL_GAIN, gain );
}
void nOpenALServer::SetPitch	( int emitterHandle, float pitch ) {
    alSourcef( emitters[emitterHandle].source, AL_PITCH, pitch );
}
