#define N_IMPLEMENTS nThing
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nthing.h"
#include "enkiduo/ntriggerserver.h"
#include "enkiduo/ngameserver.h"
#include "enkiduo/nmessageserver.h"
#include "node/n3dnode.h"
#include "enkiduo/nopenalserver.h"
#include "enkiduo/nreleaseserver.h"
#include "enkiduo/colours.h"

nNebulaScriptClass(nThing, "nroot");

nProfServer* nThing::prof = 0;
//------------------------------------------------------------------------------
/**
*/
nThing::nThing():
    game	( kernelServer, this ),
    soundServer	( kernelServer, this ),
    linesToDraw(0,10),
    displayed                           ( displayInfo.onList )
{
    triggerInfo.onList = false;
    triggerInfo.initialized = false;
    displayInfo.onList = false;
    displayInfo.initialized = false;
    releaseInfo.onList = false;
    releaseInfo.initialized = false;

    logName = "all";
    logging = true;
    loggingLevel = 0;

    clearLines = true;
}
void nThing::Initialize()
{
    game = "/sys/servers/game";
    soundServer = "/sys/servers/audio";
    ticksPerFrame = TICKS_ONCE;
    nRoot::Initialize();
    alive = true;
    messageServer = (nMessageServer*) kernelServer->Lookup("/sys/servers/message");
}
//------------------------------------------------------------------------------
/**
 */
nThing::~nThing()
{
    //n_printf("<nThing::~nThing>\n");
    if( triggerInfo.onList ){
        nTriggerServer* t = (nTriggerServer*) kernelServer->Lookup("/sys/servers/trigger");
        if( t )
            t->RemoveFromTriggerList( this );
    }
    if( displayInfo.onList ){
        if( game.isvalid() && (this != game.get()) )
            game->RemoveFromDisplayList( this );
    }
    if( releaseInfo.onList ) {
        if( game.isvalid() && (this != game.get()) )
            game->RemoveFromReleaseList( this );
    }
    //n_printf("</nThing::~nThing>\n");
}

void nThing::Trigger( nTriggerServer* triggerServer ) {
    UpdateLists();
    static nClass* thingClass = kernelServer->FindClass("nthing");
    //n_printf("%s::Trigger()\n", GetName() );
    triggerServer->Enqueue( this, ticksPerFrame );
    nRoot* childToErase = 0;
    for ( nRoot* child = GetTail(); child; child = child->GetPred() ){
        if( childToErase ) {
            //n_printf("%s->Release()\n", ((nThing*)childToErase)->fn);
            nReleaseServer* releaseServer = (nReleaseServer*) kernelServer->Lookup("/sys/servers/release");
            releaseServer->ReleaseNow( childToErase );
            //childToErase->Release();
        }
        childToErase = 0;
        if( child->IsA( thingClass ) ){
            if( ((nThing*)child)->IsAlive() )
	((nThing*)child)->Trigger( triggerServer );
            else 
	childToErase = child;
        }
    }
    if( childToErase ) {
        nReleaseServer* releaseServer = (nReleaseServer*) kernelServer->Lookup("/sys/servers/release");
        releaseServer->ReleaseNow( childToErase );
    }
        //childToErase->Release();
}
void nThing::Tick( float , float ) {
    if( clearLines ) 
        linesToDraw.Clear();
}
void nThing::UpdateLists(){
    if( triggerInfo.onList && ( triggerInfo.initialized == false) ){
        nTriggerServer* t = (nTriggerServer*) kernelServer->Lookup("/sys/servers/trigger");
        ifnonzero( t ){
            //n_printf("Added %s to triggerlist\n", fn );
            t->AddToTriggerList( this );
            triggerInfo.initialized = true;
        }
    }
    if( displayInfo.onList && (displayInfo.initialized == false) ) {
        ifvalid( game ) {
            game->AddToDisplayList( this );
            displayInfo.initialized = true;
        }
    }
    if( releaseInfo.onList && ( releaseInfo.initialized == false) ){
        ifvalid( game ) {
            game->AddToReleaseList( this );
            releaseInfo.initialized = true;
        }
    }
}
void nThing::Send( nMessage* m ){
    messageServer->Send( m );
}
void nThing::Message( nMessage* m ){
    nRoot* child;
    classVar(nthing, nThingClass );
    for( child = GetHead(); child; child = child->GetSucc() )
        if( child->IsA( nThingClass ) )
            ((nThing*)child)->Message( m );
}

/*
   void nThing::Up()
   {
   nRoot* par = parent; // need this because your going to remove your own link
   int i, n;
   int currentpos;;
   nRoot* c;
   i = 0;
   for( c = par->GetHead(); c; c = c->GetSucc() )
   i++;
   n = i;
   nRoot ** arr = new nRoot*[n];

   for(    i = 0, c = par->GetHead();
   i < n && c; 
   i++, c = c->GetSucc() ){
   arr[i] = c;
   if( arr[i] == this ) 
   currentpos = i;
   }
   if( currentpos == 0 )
   n_printf("Can't move up already first\n");
   else {
   nRoot* temp = arr[currentpos-1];
   arr[currentpos-1] = arr[currentpos];
   arr[currentpos] = temp;
   for( i = 0; i < n; i ++ ){
   arr[i]->Remove();
   par->AddTail( arr[i] );
   }
   }
   delete[] arr;
   }

   void nThing::Down()
   {
   nRoot* par = parent;
   int i, n;
   int currentpos;
   nRoot* c;
   i = 0;
   for(    c = par->GetHead(); c; c = c->GetSucc() )
   i++;
   n = i;
   nRoot ** arr = new nRoot*[n];
   for(    i = 0, c = par->GetHead();
   i < n && c; 
   i++, c = c->GetSucc() ){
   arr[i] = c;
   if( arr[i] == this ) 
   currentpos = i;
   }
   if( currentpos >= n-1 )
   n_printf("Can't move down already last\n");
   else {
   nRoot* temp = arr[currentpos+1];
   arr[currentpos+1] = arr[currentpos];
   arr[currentpos] = temp;
   for( i = 0; i < n; i ++ ){
   arr[i]->Remove();
   par->AddTail( arr[i] );
   }
   }
   delete[] arr;
   }
 */

void nThing::SetTicksPerFrame( int i ){
    switch( i ){
        case 1:
            ticksPerFrame = TICKS_ONCE;
            break;
        case 2:
            ticksPerFrame = TICKS_TWICE;
            break;
        case 4:
            ticksPerFrame = TICKS_FOUR;
            break;
        case 8:
            ticksPerFrame = TICKS_EIGHT;
            break;
        case 16:
            ticksPerFrame = TICKS_SIXTEEN;
            break;
        default:
            n_printf("CANNOT set ticks per frame to %i\n", i );
    }
}

void nThing::AddToTriggerList(){
    triggerInfo.onList = true;
    UpdateLists();
}
void nThing::AddToDisplayList(){
    //n_printf("%s->AddToDisplayList()\n", fn );
    displayInfo.onList = true;
    UpdateLists();
}
void nThing::RemoveFromDisplayList(){
    if( displayInfo.onList ) {
        game->RemoveFromDisplayList( this );
        displayInfo.onList = false;
        displayInfo.initialized = false;
    }
}
void nThing::AddToReleaseList(){
    releaseInfo.onList = true;
    UpdateLists();
}
void nThing::Display( nGfxServer* gfx ){
    gfx->Begin( N_PTYPE_LINE_LIST );
    for( int i = 0; i < linesToDraw.Size(); i ++ ){
        gfx->Rgba(  colorRGBValues[linesToDraw[i].colour][0] / 256.0f, 
                    colorRGBValues[linesToDraw[i].colour][1] / 256.0f, 
                    colorRGBValues[linesToDraw[i].colour][2] / 256.0f, 
                    1.0f );
        vector3 a = linesToDraw[i].line.b;
        vector3 b = linesToDraw[i].line.m;
        b += a;
        gfx->Coord( a );
        gfx->Coord( b );
    }
    gfx->End();
}
void nThing::DrawAngle( vector3 p, float t, uint colour, float length ){
    quaternion q;
    q.set_rotate_y( t );
    vector3 v(0,0,length);
    v = q.rotate(v);
    linesToDraw.Append( LineInfo( line3(p, p+v), colour));
}

nString nThing::CreateVis( const char* nodePath, const char* destinationPath ){
    //just needs to build the appropriate new call
    //n_printf("nodePath:%s\n", nodePath );
    nRoot* node = kernelServer->Lookup( nodePath );
    classVar(n3dnode, n3dnodeClass );
    nString dest = nString("/game/vis/") + destinationPath;
    if( node ){
        n_assert( node->IsA( n3dnodeClass ) );
        nRoot* destination = kernelServer->Lookup( dest.Get() );
        ifnonzero( destination ){
            char* name = new char[strlen(nodePath+20)];
            int counter = 0;
            while(1){
	sprintf( name, "%s%i", node->GetName(), counter++ );
	//n_printf("checking name : %s\n", name );
	if( destination->Find( name ) == 0 ){
	    dest = dest + "/" + name;
	    //n_printf("cloning to %s\n", dest.Get() );
	    node->Clone( dest.Get() );
	    return dest;
	}
            }
            delete name;
        }
    }
    return "none";
}

void nThing::Up(){
    if( this->pred== 0 ){
        n_printf("%s Cannot move up - Already First\n", fn );
    } else {
        if( this->pred->pred == 0 ){
            n_printf("%s is in second spot,  will need to changes parents head\n");
        } else {
            n_printf("moving up a spot\n");
            nNode* a = this->pred->pred;
            nNode* b = this->pred;
            nNode* c = this;
            nNode* d = this->succ;

            a->succ = c;
            c->pred = a;

            c->succ = b;
            b->pred = c;

            b->succ = d;
            d->pred = b;
        }
    }
}
void nThing::Down(){


}

void nThing::DrawPoint( vector3 p, uint colour, float w ) {
    this->DrawPoint( p, colour, w, w, w );
}
void nThing::DrawPoint( vector3 p, uint colour, float w, float h, float d ){
    this->DrawLine( p - vector3( w,0,0 ), p + vector3(w,0,0 ), colour);
    this->DrawLine( p - vector3( 0,h,0 ), p + vector3(0,h,0 ), colour);
    this->DrawLine( p - vector3( 0,0,d ), p + vector3(0,0,d ), colour);
}
const int numPoints = 16;
vector3* GenOffsets(int axis=0){
    vector3* offsets = new vector3[numPoints];
    int i = 0;
    for( float theta = 0.0f; theta <= 3.1415 * 2.0f; theta += (3.1415 * 2.0f / (numPoints-1)) ) {
        n_assert( i < numPoints );
        switch(axis){
            case 0:
                offsets[i++].set(cos(theta), 0.0f, sin(theta));
                break;
            case 1:
                offsets[i++].set(cos(theta), sin(theta), 0.0f);
                break;
            case 2:
                offsets[i++].set(0.0f, sin(theta), cos(theta));
                break;
        }
    }
    return offsets;
}
void nThing::DrawCircle( vector3 p, uint colour, float r ) {
    static vector3 *offsets = GenOffsets();
    for( int i = 0; i < numPoints-1; i ++ )
        this->DrawLine( p + offsets[i] * r, p + offsets[i+1] * r, colour);
    this->DrawLine( p + offsets[numPoints-1] * r, p + offsets[0] * r, colour );
}
void nThing::DrawSphere( vector3 p, uint colour, float r ) {
    static vector3 *offsets[3] = {GenOffsets(0), GenOffsets(1), GenOffsets(2)};
    for( int j = 0; j < 3; j ++ ) { 
        for( int i = 0; i < numPoints-1; i ++ )
            this->DrawLine( p + offsets[j][i] * r, p + offsets[j][i+1] * r, colour);
        this->DrawLine( p + offsets[j][numPoints-1] * r, p + offsets[j][0] * r, colour );
    }
}

/// replacement printf -> dumps to log
void nThing::logTo( const char* logName, const char* format, ... ) {
    if( this->logging == false ) return;
    va_list arglist;
    va_start(arglist, format);
    char s [1024];
    vsprintf( s, format, arglist );
    char s2 [1024];
    sprintf( s2, "logto {%s} {%s}", logName, s );
    RUNSCRIPT( s2 );
    va_end(arglist);
}

void nThing::log( const char* format, ... ) {
    if( this->logging == false ) return;
    va_list arglist;
    va_start(arglist, format);
    char s [1024];
    vsprintf( s, format, arglist );
    if( s[strlen(s)-1] == '\n')
        s[strlen(s)-1] = '\0';
    char s2 [1024];
    sprintf( s2, "logto {%s} {%s}", logName.Get(), s );
    //RUNSCRIPT( s2 );
    va_end(arglist);
}

void nThing::log( int level, const char* format, ... ) {
    if( this->logging == false ) return;
    if( this->loggingLevel < level ) return;
    va_list arglist;
    va_start(arglist, format);
    char s [1024];
    vsprintf( s, format, arglist );
    char s2 [1024];
    sprintf( s2, "logto {%s} {%s}", logName.Get(), s );
    RUNSCRIPT( s2 );
    va_end(arglist);
}

bool nThing::Simple(){ return game->GetSimple(); } 

nRoot* nThing::PushChild( const char* className){
    static char name[1024];
    int i = 0;
    while(1){
        sprintf( name, "%s%i", className, i );
        if( this->Find(name) )
            i++;
        else
            break;
    }
    nRoot* t = kernelServer->_newUnnamedObject(className);
    t->SetName( name );
    this->AddHead( t );
    t->Initialize();
    return t;
}
nRoot* nThing::QueueChild( const char* className){
    static char name[1024];
    int i = 0;
    while(1){
        sprintf( name, "%s%i", className, i );
        if( this->Find(name) )
            i++;
        else
            break;
    }
    nRoot* t = kernelServer->_newUnnamedObject(className);
    t->SetName( name );
    this->AddTail( t );
    t->Initialize();
    return t;
}

nRoot* nThing::ReplaceChildren( const char* className ) {
    for( nRoot* r = this->GetHead(); r; r = r->GetSucc() ) {
        classVar( nthing, thingClass );
        if( r->IsA( thingClass ) )
            ((nThing*)r)->Die();
    }
    return this->QueueChild( className );
}
