#define N_IMPLEMENTS nTriggerServer
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/ntriggerserver.h"

nNebulaScriptClass(nTriggerServer, "nthing");

//------------------------------------------------------------------------------
/**
*/
nTriggerServer::nTriggerServer():
    channel	( kernelServer, this ),
    prof	( kernelServer, this )
{
    queues.SetFixedSize( 5 ); //once, twice, third, fourth, fifth
    gameTime                            = 0.0f;
    lastTime                            = 0.0f;
    scaledTime                          = 0.0f;
    timeStep                            = 0.01f;
    timeScale                           = 1.0f;
    timeLost 	= 0.0f;
    maxTimeStepsPerFrame                = 50;
    channel 	= "/sys/servers/channel";
    prof 	= "/sys/servers/prof";
}

void nTriggerServer::Initialize()
{
    nThing::Initialize();
}
//------------------------------------------------------------------------------
/**
*/
nTriggerServer::~nTriggerServer()
{
}

void nTriggerServer::TriggerThings( float time ){
    static float timeTriggering 	= 0.0f;
    static float timeNotTriggering 	= 0.0f;
    static nTimeServer* timeServer      = (nTimeServer*) kernelServer->Lookup("/sys/servers/time");
    this->actualTime                    = time;

    static float notTriggeringStart     = timeServer->GetTime();
    float notTriggeringStop             = timeServer->GetTime();
    float nottrigger                    = notTriggeringStop - notTriggeringStart;
    timeNotTriggering                  += nottrigger;

    float triggerStart                  = timeServer->GetTime();
    //n_printf("nTriggerServer::TriggerThings()\n" );
    float dtime                         = time - lastTime;
    //n_printf("Triggering:%f nottriggering:%f\n", timeTriggering, timeNotTriggering );
    //n_printf("time: %f lastTime: %f dtime: %f scaledTime: %f gameTime: %f timeLost: %f\n", time, lastTime, dtime, scaledTime, gameTime, timeLost );
    lastTime                            = time;
    scaledTime                         += dtime * timeScale;
    int numSteps = 0;
    while( (numSteps++ < maxTimeStepsPerFrame) && ((gameTime + timeStep) < (scaledTime - timeLost)) ){
        for( int i = 0; i < toTrigger.Size(); i ++ )
            this->TickThings( toTrigger[i] );
        gameTime += timeStep;
    }
    if( gameTime + timeStep < scaledTime )
        timeLost = scaledTime - (gameTime + timeStep);
    float triggerStop = timeServer->GetTime();
    float triggering = triggerStop - triggerStart;
    timeTriggering += triggering;

    //n_printf("trigger:%f not:%f\n", nottrigger, triggering );
    notTriggeringStart = timeServer->GetTime();
}
void nTriggerServer::TickThings( nThing* things ){
    //n_printf("nTriggerServer::TickThings( %s )\n", things->GetName() );
    n_assert( things );
    things->Trigger( this );
    int i;
    for( i = 0; i < 16; i ++ ){
        if( i==0 ) {
            //prof->Begin("TickOnce");
            TickQueue(TICKS_ONCE, timeStep );
            //prof->End("TickOnce");
        }
        if( i%8 == 0 ){ 
            //prof->Begin("TickTwice");
            TickQueue(TICKS_TWICE, timeStep / 2.0f);
            //prof->End("TickTwice");
        }
        if( i%4 == 0 ) {
            //prof->Begin("TickFour");
            TickQueue(TICKS_FOUR, timeStep / 4.0f);
            //prof->End("TickFour");
        }
        if( i%2 == 0 ) {
            //prof->Begin("TickEight");
            TickQueue(TICKS_EIGHT, timeStep / 8.0f);
            //prof->End("TickEight");
        }
        //prof->Begin("TickSixteen");
        TickQueue(TICKS_SIXTEEN, timeStep / 16.0f);
        //prof->End("TickSixteen");
    }
    for( i = 0; i < queues.Size(); i ++ )
        queues[i].Clear();
}

void nTriggerServer::TickQueue( int name, float dtime ){
    for( int i = 0; i < queues[name].Size(); i ++ ){
        //n_printf("queue %i %i = %f %f\n", name, i, gameTime, dtime );
        queues[name][i]->Tick(gameTime, dtime);
    }
}

void nTriggerServer::Enqueue( nThing* thing, TICKS_PER_FRAME ticksPerFrame ){
    if(!( ticksPerFrame >= 0 && ticksPerFrame < queues.Size() )){
        n_printf("%s has %i ticks per frame\n", thing->GetName(), ticksPerFrame );
        n_assert(0);
    }
    queues[ticksPerFrame].Append( thing );
}
