#define N_IMPLEMENTS nVar
//------------------------------------------------------------------------------
//  (C) 2003	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nvar.h"
#include "enkiduo/var.h"
#include "kernel/nkernelserver.h"
#include "enkiduo/nthing.h"
#include "enkiduo/nvarserver.h"
#include "enkiduo/nent.h"
char* VarTypeNames[] = {
    "VOID",
    "INT",
    "FLOAT",
    "VECTOR3",
    "QUATERNION",
    "STRING",
    "REF",
    "RC",
    "OBJECT",
};

nVarServer* nVar::varServer = 0;
nNebulaScriptClass(nVar, "nroot");

void nVar::InheritFrom( nVar* v ){
    SetType( v->GetType() );
    switch( type ){
        case VARTYPE_INT:
            SetI(v->GetI());
            break;
        case VARTYPE_FLOAT:
            SetF(v->GetF());
            break;
        case VARTYPE_VECTOR3:
            SetV(v->GetV());
            break;
        case VARTYPE_QUATERNION:
            SetQ(v->GetQ());
            break;
        case VARTYPE_STRING:
            SetS(v->GetS());
            break;
        case VARTYPE_REF:
            SetRef(v->GetRef());
            break;
        default:
            break;
    }
}
void nVar::AddChildVar( var* childVar, const char* varName ){
    CheckType(VARTYPE_REF);
    if( childVars == 0 )	n_error( "%s(nVar)::AddChildVar - array of child vars it not initialized", GetName() );
    ChildVarInfo* c 	= new ChildVarInfo();
    c->childVar 	= childVar;
    c->childName 	= varName;
    childVar->Attach( this );
    childVars->Append(c);
}
/**
  One of my children has been deleted - remove him from my list
*/
void nVar::RemoveChildVar( var* childVar ){
    CheckType(VARTYPE_REF);
    if( childVars == 0 )		n_error( "%s(nVar)::AddChildVar - array of child vars it not initialized", GetName() );
    for( int i = 0; i < childVars->Size(); i ++ ){
        ChildVarInfo* c = (*childVars)[i];
        if( c->childVar == childVar ){
            c->childVar->Detach( this );
            childVars->Erase( i );
            delete c;
        }
    }
}
void nVar::ToString( char* s ){
    char val[128];
    switch( type ){
        case VARTYPE_VOID:
            sprintf( val, "VOID");
            break;
        case VARTYPE_INT:
            sprintf( val, "%i", intVal );
            break;
        case VARTYPE_FLOAT:
            sprintf( val, "%f", floatVal );
            break;
        case VARTYPE_VECTOR3:
            sprintf( val, "%6.4f %6.4f %6.4f          l:%.4f", vectorVal->x, vectorVal->y, vectorVal->z, vectorVal->len() );
            break;
        case VARTYPE_QUATERNION:
            sprintf( val, "%6.4f %6.4f %6.4f %6.4f       p:%.4f r:%.4f y:%.4f", quatVal->x, quatVal->y, quatVal->z, quatVal->w, 
                    n_rad2deg(quatVal->ToPitch()), 
                    n_rad2deg(quatVal->ToRoll()),
                    n_rad2deg(quatVal->ToYaw()));
            break;
        case VARTYPE_STRING:
            sprintf( val, "%s", stringVal->Get() );
            break;
        case VARTYPE_REF:
            sprintf( val, "%s", ref->getname() );
            break;
        case VARTYPE_RC:
            sprintf( val, "rc" );
            break;
        default:
            break;
    }
    sprintf( s, "%-12s %s", GetName(), val );
}

void nVar::SetType( VarType varType ){
    //if (varType == VARTYPE_REF)
        //n_printf("Settype(%s, %s)\n", VarTypeNames[varType], VarTypeNames[this->type]);
    if( type != varType ){
        if( type != VARTYPE_VOID && type != varType)
            n_printf("%s just redefined type from %s -> %s\n", this->fn, VarTypeNames[type], VarTypeNames[varType] );
        this->ClearMem();
        type = varType;
        switch( type ){
            case VARTYPE_VECTOR3:
	//if( vectorVal )
	    //delete vectorVal;
                vectorVal = new vector3();
                break;
            case VARTYPE_QUATERNION:
	//if( quatVal )
	    //delete quatVal;
                quatVal = new quaternion();
                break;
            case VARTYPE_STRING:
	//if( stringVal )
	    //delete stringVal;
                stringVal = new nString();
                break;
            case VARTYPE_REF:
	//if( ref != 0 )
	    //delete ref;
                ref = new nDynAutoRef<nEnt>(kernelServer, this );
	//if( childVars != 0 ) 
	    //delete childVars;
                childVars = new nArray<ChildVarInfo*>();
                break;
            case VARTYPE_RC:
	n_assert( rc_ptr == 0 );
                rc_ptr = new rc();
                break;
            default:
                break;
        }
    }
}

void nVar::CheckType( VarType varType ){
    if( type != varType ){
        char fullname[1024];
        GetFullName( fullname, 1024 );
        n_error("check type differs on %s : is->%s shouldbe->%s\n", fullname, VarTypeNames[type], VarTypeNames[varType] );
        //n_assert( 0 );
        //return false;
    }
    //return true;
}

void nVar::ClearMem(){
    int i;
    switch( type ){
        case VARTYPE_VECTOR3:
            delete vectorVal;
            break;
        case VARTYPE_QUATERNION:
            delete quatVal;
            break;
        case VARTYPE_STRING:
            delete stringVal;
            break;
        case VARTYPE_REF:
            delete ref;
            for( i = 0; i < childVars->Size(); i ++ ) {
	(*childVars)[i]->childVar->Detach( this );
	delete (*childVars)[i];
            }
            break;
        case VARTYPE_RC:
            if( rc_ptr )
	delete rc_ptr;
            break;
        default:
            break;
    }
    switch( category ) {
        case VARCAT_SERVER:
            varServer->RemoveVar( this );
            break;
        default:
            break;
    }
}

bool nVar::isvalid(){
    if( this->parent == 0 )  {
        n_printf("%s returning false no parent\n", this->GetName());
        return false;
    }
    if( this->GetName() == 0 ) {
        n_printf("%s returning false no name\n", this->GetName());
        return false;
    }
    switch( type ){
        case VARTYPE_VOID:
            return false;
        case VARTYPE_FLOAT:
        case VARTYPE_INT:
        case VARTYPE_VECTOR3:
        case VARTYPE_QUATERNION:
        case VARTYPE_STRING:
            return true;
        case VARTYPE_OBJECT:
            return ( objectVal == 0 );
        case VARTYPE_REF:
            //n_printf("%s got into ref check\n", this->GetName());
            if( ref->isvalid() ){
	if( childState == -1 || childState == 0 ) //children need to be reconnect / connected
	    this->UpdateChildren();
	return true;
            } else if( childState == 1 && ref->isvalid() == false ){ 
	this->UpdateChildren();
            }
            return false;
        case VARTYPE_RC:
            return rc_ptr != 0;
    }
    n_printf("escaped switch on nVar::isvalid() type: %i\n", type );
    return false;
}
/**
  Updates all its childrens.
  IE this is a ref var pointing to an entity, which has changed.
  Since it is pointing to something else, things linked to me, IE pointing to something attached to that entity, need to update themselves
*/
void nVar::UpdateChildren(){
    CheckType( VARTYPE_REF );
    if( ref->isvalid() ) {
        //foreach child - connect them
        for( int i = 0; i < childVars->Size(); i ++ ){
            ChildVarInfo* c = (*childVars)[i];
            n_assert( c );
            c->childVar->Connect( (nEnt*) ref->get(), c->childName.Get() );
        }
        this->childState = 1;
    } else {
        for( int i = 0; i < childVars->Size(); i ++ ){
            ChildVarInfo* c = (*childVars)[i];
            n_assert( c );
            c->childVar->Disconnect();
        }
        this->childState = 0;
    }
}
void nVar::SetCategory( VARCAT category ) {
    if( category ) 
        this->category = category;
    switch( category ) {
        case VARCAT_SERVER:
        case VARCAT_CLIENT:
            //if( ent == 0 )
                //this->FindEnt();
            //ent->AddServerVar( this );
            if( varServer )
                varServer->AddVar( this );
            break;
        default:
            break;
    }
}

int nVar::ReleasePriority(){ return -10; } //release after aspects
void nVar::FindEnt(){
    n_assert( ent == 0 );
    classVar(nent, entClass );
    nRoot* t = this->GetParent();
    while( t ) {
        if( t->IsA( entClass ) ) {
            this->ent = (nEnt*) t;
            return;
        }
        t = t->GetParent();
    }
    n_assert( 0 && "var could not find its ent parent\n");
}

///called when vars are being deleted
bool nVar::Release(){ 
    //printf("%s.Release()\n", GetName()); 
    return nRoot::Release(); 
}

void nVar::SetRef( nEnt* t ){
    if( t ) {
        if (t->IsAlive()) { 
            this->SetRef(t->fn);
            counter++;
            //this->UpdateChildren();
        } else {
            char name[1024];
            this->GetFullName( name, 1024 );
            n_printf ( "%s set to an ent which is not alive\n", name );
        }
    } else {
        n_printf("invalidating ref\n");
    } 
}
