#define N_IMPLEMENTS nWaver
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "enkiduo/nwaver.h"
#include "gfx/ngfxtypes.h"
#include "gfx/nvertexbuffer.h"
#include "gfx/nindexbuffer.h"
#include "gfx/nchannelset.h"
#include "gfx/nchannelcontext.h"
#include "gfx/nscenegraph2.h"
#include "node/nmeshnode.h"
#include "enkiduo/ntriggerserver.h"
#include "enkiduo/nent.h"

nNebulaScriptClass(nWaver, "nmeshmixer");

//------------------------------------------------------------------------------
nWaver::nWaver():
    camera        ( kernelServer, this ),
    triggerServer ( kernelServer, this ),
    waveRoot      ( kernelServer, this ),
    refMap        ( kernelServer, this )
{
    camera = "/game/camera";
    n_assert( camera.isvalid());
    triggerServer = "/sys/servers/trigger";
    n_assert( triggerServer.isvalid() );
    waveRoot = "/map/waves";
    lastWaveChecked = 0;
    refMap = "/map/map";
}
//------------------------------------------------------------------------------
void nWaver::Initialize(){
    nMeshMixer::Initialize();
}
//------------------------------------------------------------------------------
nWaver::~nWaver(){
}
//-----------------------------------------------------------------------------//
void nWaver::mix(nVertexBuffer **vb_src,
        float *w,
        int num,
        nSceneGraph2* sceneGraph)
{
    n_assert(vb_src && w && sceneGraph);
    if( lastWaveChecked != waveRoot->GetTail()){ //we have new waves
        foreachchild(nEnt, wave, waveRoot.get() ) {
            //n_printf("new wave:%s\n", str(wave));
            WaveInfo& newWave = waves.Push();
            newWave.typeVar.Connect( wave, "type");
            newWave.typeVar.DefaultS("directional");
            newWave.periodVar.Connect( wave, "period");
            newWave.periodVar.DefaultF(1.0f);
            newWave.amplitudeVar.Connect( wave, "amplitude");
            newWave.amplitudeVar.DefaultF(0.0f);
            newWave.directionVar.Connect( wave, "direction");
            newWave.directionVar.DefaultF(0.0f);
            newWave.wavelengthVar.Connect( wave, "wavelength");
            newWave.wavelengthVar.DefaultF( 10.0f );
        }
        lastWaveChecked = waveRoot->GetTail();
    }
    for( int i = 0; i < waves; i ++ ) {
        if ( waves[i].typeVar.isdirty()){
            nString newType = waves[i].typeVar.GetS();
            if (newType == "directional"){
                waves[i].type = WAVE_DIRECTIONAL;
            }
            //n_printf("read wavetype:%s\n", newType.Get());
        }
        if( waves[i].directionVar.isdirty()){
            float d = waves[i].directionVar.GetF();
            d = n_deg2rad(d);
            waves[i].xCoef = cos(d);
            waves[i].zCoef = sin(d);
            n_printf("direction is dirty val:%f,%f\n", waves[i].xCoef, waves[i].zCoef);
        }
    }

    // get index buffer 
    nIndexBuffer* indexBuffer = sceneGraph->GetIndexBuffer();
    n_assert(indexBuffer);

    // get destination vertex buffer
    nVertexBuffer *vb_dest = this->dyn_vb.Begin(
            indexBuffer,
            sceneGraph->GetPixelShader(),
            sceneGraph->GetTextureArray());

    // get pointers to source vertex components
    float *coord[N_MAXSOURCES+1];
    float *norm[N_MAXSOURCES+1];
    ulong *color[N_MAXSOURCES+1];
    float *uv[N_MAXNUM_TEXCOORDS][N_MAXSOURCES+1];
    this->time = triggerServer->GetGameTime();
    vector3 cameraPos = camera->GetVarV("pos");
    int i,j,k;
    for (i=0; i<num; i++) 
    {
        vb_src[i]->LockVertices();
        coord[i] = vb_src[i]->coord_ptr;
        norm[i]  = vb_src[i]->norm_ptr;
        color[i] = vb_src[i]->color_ptr;
        for (j=0; j<N_MAXNUM_TEXCOORDS; j++) {
            uv[j][i] = vb_src[i]->uv_ptr[j];
        }
    }
    coord[i] = vb_dest->coord_ptr;
    norm[i]  = vb_dest->norm_ptr;
    color[i] = vb_dest->color_ptr;
    for (j=0; j<N_MAXNUM_TEXCOORDS; j++) 
    {
        uv[j][i] = vb_dest->uv_ptr[j];
    }

    // get update bools
    bool do_coord = this->GetUpdateCoord();
    bool do_norm  = this->GetUpdateNorm();
    bool do_color = this->GetUpdateColor();
    bool do_uv[N_MAXNUM_TEXCOORDS];
    do_uv[0] = this->GetUpdateUv0();
    do_uv[1] = this->GetUpdateUv1();
    do_uv[2] = this->GetUpdateUv2();
    do_uv[3] = this->GetUpdateUv3();

    // get 16 bit fixed point weight values for color lerping
    ushort usweight[N_MAXSOURCES];
    for (i=0; i<num; i++) 
    {
        usweight[i] = (short) (256.0f * w[i]);
    }

    // the destination buffer must be written continously, because
    // it may be placed in uncached memory (AGP or video mem)
    // it may actually be better to keep coords, norms, etc 
    // in separate arrays -> DX8!
    int num_v = vb_src[0]->GetNumVertices();
    int dest = num;
    int src_stride  = vb_src[0]->stride4;
    int dest_stride = vb_dest->stride4;
    for (i=0; i<num_v; i++) 
    {
        if (coord[0]) 
        {
            if (do_coord) 
            {
                register float x = coord[0][0] * w[0];
                register float y = coord[0][1] * w[0];
                register float z = coord[0][2] * w[0];
                for (j=1; j<num; j++) 
                {
                    x += coord[j][0] * w[j];
                    y += coord[j][1] * w[j];
                    z += coord[j][2] * w[j];
                }
                x += cameraPos.x;
                z += cameraPos.z;
                y += this->displacementAt( x,z );
                y = forceAboveLand(x, y, z);
                //y += 0.2f *  (cos( time *  2.5f + ((z * 5.5 + x * 10 ) / 50)* N_PI * 2.0f ));
                coord[dest][0] = x;
                coord[dest][1] = y;
                coord[dest][2] = z;
            } 
            else 
            {
                //n_printf("default setting coord to %f %f %f\n", coord[0][0], coord[0][1], coord[0][2]);
                coord[dest][0] = coord[0][0];
                coord[dest][1] = coord[0][1];
                coord[dest][2] = coord[0][2];
            }
            for (j=0; j<num; j++) 
            {
                coord[j] += src_stride;
            }
            coord[dest] += dest_stride;
        }

        if (norm[0]) 
        {
            if (do_norm) 
            {
                register float x = norm[0][0] * w[0];
                register float y = norm[0][1] * w[0];
                register float z = norm[0][2] * w[0];
                for (j=1; j<num; j++) 
                {
                    x += norm[j][0] * w[j];
                    y += norm[j][1] * w[j];
                    z += norm[j][2] * w[j];
                }
                norm[dest][0] = x;
                norm[dest][1] = y;
                norm[dest][2] = z;
            } 
            else 
            {
                norm[dest][0] = norm[0][0];
                norm[dest][1] = norm[0][1];
                norm[dest][2] = norm[0][2];
            }
            for (j=0; j<num; j++) 
            {
                norm[j] += src_stride;
            }
            norm[dest] += dest_stride;
        }

        if (color[0]) 
        {
            if (do_color) 
            {
                ulong c = color[0][0];
                ushort r = (ushort((c>>24) & 0xff) * usweight[0]) >> 8;
                ushort g = (ushort((c>>16) & 0xff) * usweight[0]) >> 8;
                ushort b = (ushort((c>>8)  & 0xff) * usweight[0]) >> 8;
                ushort a = (ushort(c       & 0xff) * usweight[0]) >> 8;
                for (j=1; j<num; j++) 
                {
                    c = color[j][0];
                    r += (ushort((c>>24) & 0xff) * usweight[j]) >> 8;
                    g += (ushort((c>>16) & 0xff) * usweight[j]) >> 8;
                    b += (ushort((c>>8)  & 0xff) * usweight[j]) >> 8;
                    a += (ushort(c       & 0xff) * usweight[j]) >> 8;
                }
                r&=0xff; g&=0xff; b&=0xff; a&=0xff;
                color[dest][0] = (r<<24)|(g<<16)|(b<<8)|(a);
            } 
            else 
            {
                color[dest][0] = color[0][0];
            }
            for (j=0; j<num; j++) 
            {
                color[j] += src_stride;
            }
            color[dest] += dest_stride;
        }

        for (k=0; k<N_MAXNUM_TEXCOORDS; k++) 
        {
            if (uv[k][0]) 
            {
                if (do_uv[k]) 
                {
                    register float u = uv[k][0][0] * w[0];
                    register float v = uv[k][0][1] * w[0];
                    for (j=1; j<num; j++) {
                        u += uv[k][j][0] * w[j];
                        v += uv[k][j][1] * w[j];
                    }
                    uv[k][dest][0] = u + cameraPos.x / ( 100000.0f / 2.0f );
                    uv[k][dest][1] = v - cameraPos.z / ( 100000.0f / 2.0f );
                } 
                else 
                {
                    uv[k][dest][0] = uv[k][0][0];
                    uv[k][dest][1] = uv[k][0][1];
                }
                for (j=0; j<num; j++) 
                {
                    uv[k][j] += src_stride;
                }
                uv[k][dest] += dest_stride;
            }
        }
    }

    // weight bounding box
    const bbox3 *src_bb = &(vb_src[0]->GetBBox());
    bbox3 dest_bb;
    dest_bb.vmin = src_bb->vmin * w[0];
    dest_bb.vmax = src_bb->vmax * w[0];
    for (j=1; j<num; j++) 
    {
        src_bb = &(vb_src[j]->GetBBox());
        dest_bb.vmin += src_bb->vmin * w[j];
        dest_bb.vmax += src_bb->vmax * w[j];
    }
    vb_dest->SetBBox(dest_bb);

    // unlock everything
    for (j=0; j<num; j++) 
    {
        vb_src[j]->UnlockVertices();
    }
    this->dyn_vb.End(num_v, indexBuffer->GetNumIndices());

    // if readonly, hand 
    if (this->dyn_vb.GetReadOnly())
    {
        sceneGraph->SetVertexBuffer(vb_dest);
    }
    else
    {
        sceneGraph->SetVertexBuffer(0);
    }
}

float nWaver::displacementAt( float x, float z ) {
    float totalDisplacement = 0.0f;
    for( int i = 0; i < waves; i ++ ) {
        float displacement = 0.0f;
        float posDisp = 0.0f;
        float timeDisp = 0.0f;
        switch( waves[i].type){
            case WAVE_DIRECTIONAL:
                timeDisp  = time * waves[i].periodVar.GetF();
                posDisp   = x * waves[i].xCoef + z * waves[i].zCoef;
                posDisp  /= waves[i].wavelengthVar.GetF();
                posDisp  *= N_PI * 2.0f;
                displacement = cos( timeDisp + posDisp );
                displacement *= waves[i].amplitudeVar.GetF();
                totalDisplacement += displacement;
                break;
            default:
                n_printf("crap wavetype:%i\n", waves[i].type);
                //n_assert( 0 && "crap wave type");
                break;
        }
    }
    return totalDisplacement;
}
/**-----------------------------------------------------------------------------//
 * okay so we need to make sure our points dont go below land + some displacement based upon camera alt
--------------------------------------------------------------------------------*/
float nWaver::forceAboveLand( float x, float y, float z) {
    float alt = refMap->GetHeight( x, z );
    if ( alt < 0.0f ) {
        vector3 cameraPos = camera->GetVarV("pos");
        float minAlt = cameraPos.y / 150.0f;
        minAlt = 0;
        //if( roll(.001))
            //n_printf("y:%f alt:%f minAlt:%f\n", y, alt, minAlt );
        if( y < alt + minAlt )
            y = alt + minAlt;
    } 
    return y;
}
