#define N_IMPLEMENTS nAllocatorGA
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "ga/nallocatorga.h"
#include "ga/ngene.h"
#include "ga/ngeneint.h"
#include "ga/ngenefloat.h"
#include "ga/ngenome.h"
#include "im/nallocatorastar.h"

nNebulaScriptClass(nAllocatorGA, "nga");

//------------------------------------------------------------------------------
nAllocatorGA::nAllocatorGA():allocator( kernelServer, this ) {
    slaveClassName = "nallocatorga";
}
//------------------------------------------------------------------------------
void nAllocatorGA::Initialize(){
    nGA::Initialize();
}
//------------------------------------------------------------------------------
void nAllocatorGA::InitVars(){
    nGA::InitVars();
    allocator                           = ent->ChildPath("allocator");
    allocatorAStar                      = dynamic_cast<nAllocatorAStar*>( this->GetParent()->Find("allocatorastar") );
    n_assert( allocatorAStar && "nAllocator GA failed to find its allocator A*");
    this->AddToDisplayList();
}
//------------------------------------------------------------------------------
nAllocatorGA::~nAllocatorGA(){
}
//------------------------------------------------------------------------------
void nAllocatorGA::Tick( float time, float dtime ){
    nGA::Tick( time, dtime );
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::Run(nAllocator* allocator){
    //n_printf("nAllocatorGA(%s).Run(%s)\n", str(this), str(allocator));
    this->ClearPopulation();
    nGA::Run();
}
//-----------------------------------------------------------------------------//
/**
 * okay so what does the genome do
 * enumerates each group to an objective
 * so what we need is a bloc kof bits for ecah group, that should be long enough to hold the objectives
 */
void nAllocatorGA::CreateGenome( nGenome* g ){
    //n_printf("nAllocatorGA(%s).CreateGenome(%s) %i groups mapped to %i objectives\n", str(this), str(g), allocator->groups.Size(), allocator->objectives.Size());
    for( int i = 0; i < allocator->groups; i ++ ) {
        ((nGeneInt*) g->CreateChild("ngeneint", allocator->groups[i].name.Get()))->InitI(0, allocator->objectives.Size());
    }
    cachedSearches.Clear();
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::StartEvaluation( nGenome* g ){
    //n_printf("nAllocatorGA(%s).StartEvaluation(%s)\n", str(this), str(g));
}
//-----------------------------------------------------------------------------//
bool nAllocatorGA::EvaluationFinished(){
    //n_printf("nAllocatorGA(%s)::EvaluationFinished()\n", str(this));
    //return true;
    return roll(.5f);
}
//-----------------------------------------------------------------------------//
/**
 * okay so how do we compute fitness
 * okay well we just check the allocation
 **/
float nAllocatorGA::GetFitness(){
    //n_printf("//-----------------------------------------------------------------------------/\n");
    //n_printf("nAllocatorGA(%s)::Getfitness(%s)\n", str(this), str(this->slaveGenome));
    //n_printf("//-----------------------------------------------------------------------------/\n");
    //now i want to walk the genome
    //n_printf("assignment=\n");
    float rewardSum = 0.0f;
    int groupIndex = 0;
    for( int i = 0; i < allocator->objectives; i ++ )
        allocator->objectives[i].unitsAssigned = 0;

    //foreach boat / group - check how  much it benefits the allocation
    foreachchild(nGeneInt, parameter, this->slaveGenome ){
        int assignment = parameter->GetI();
        nAllocator::GroupInfo& group         = allocator->groups[groupIndex];
        nAllocator::ObjectiveInfo& objective = allocator->objectives[assignment];
        //n_printf("group(%i.%s.%s) assigned to %i.%s\n", groupIndex, parameter->GetName(), group.name.Get(), assignment, objective.name.Get());

        //if objective is already full - no credit for adding more too it
        float totalReward;
        float powerReward               = objective.powerCoef * group.powerSum;
        float valueReward               = objective.valueCoef * group.valueSum;
        float speedReward               = objective.speedCoef * group.speedAvg;
        int cachedSearchIndex = allocator->groups.Size() * groupIndex + assignment;
        while( cachedSearchIndex >= cachedSearches.Size() )
            cachedSearches.Append( -1.0f );
        //n_printf("cachedSearchIndex:%i cachedSearch:%f\n", cachedSearchIndex, cachedSearches[cachedSearchIndex] );
        float routeCost;
        if( cachedSearches[cachedSearchIndex] <= 0.0f ) {
            AStarPath* path = allocatorAStar->FindPath( group.centroid, objective.point );
            routeCost = path->GetCost();
            //n_printf("%s creating path\n", str(this));
            //

            //n_printf("%s created path from %s->%s totalPaths:%i\n", str(this), str(group.centroid), str(objective.point), this->paths.Size());
            if( this->paths.Size() > 100 )
                this->paths.Erase(0);
            this->paths.Append( path );
            cachedSearches[cachedSearchIndex] = routeCost;
        } else {
            //n_printf("already cached\n");
            routeCost = cachedSearches[cachedSearchIndex];
        }
        //n_printf("routeCost:%f\n", routeCost);
        //n_printf("accessibility from %s -> %s = %f\n", str(group.centroid), str(objective.point), accessibility );
        //float distancePenalty           = (objective.point - group.centroid).len();
        //distancePenalty /= 10000.0f;
        //this->DrawLine( objective.point, group.centroid, blue );
    
        routeCost /= 250.0f;
        totalReward                     = objective.reward * (powerReward + valueReward + speedReward) - routeCost;

        nAllocator::ObjectiveInfo* rootObjective = &objective;
        if( objective.rootObjectiveInfo ) {
            rootObjective = objective.rootObjectiveInfo;
            //n_printf("rootObjective:%p\n", rootObjective);
        }
        if ( rootObjective->maxUnits > 0 && rootObjective->unitsAssigned >= rootObjective->maxUnits )
            totalReward = 0;
        rootObjective->unitsAssigned    += group.numUnits;
        //n_printf("unitsAssigned:%i\n", rootObjective->unitsAssigned);
        rewardSum                       += totalReward;
        //n_printf("objective.reward:%f powerReward:%f valueReward:%f speedReward:%f totalReward:%f rewardSum:%f routeCost:%f\n", objective.reward, powerReward, valueReward, speedReward, totalReward, rewardSum, routeCost );
        groupIndex++;
        //n_printf("//-----------------------------------------------------------------------------/\n");
        //n_printf("//-----------------------------------------------------------------------------/\n");
    }
    return rewardSum;
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::StopEvaluation( nGenome* g ){
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::InitializeSlave( nGA* slave ){
    //n_printf("InitializeSlave(%s, %s)\n", str(this), str(slave));
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::OnRunDone(){
    //n_printf("nAllocatorGA(%s).OnRunDone\n", str(this));
    nArray<nGenome*> sortedGenomes = this->GetSortedGenomes();
    nGenome* best = sortedGenomes[0];
    int groupIndex = 0;
    foreachchild(nGeneInt, parameter, best ){
        int assignment = parameter->GetI();
        nAllocator::GroupInfo& group         = allocator->groups[groupIndex];
        nAllocator::ObjectiveInfo& objective = allocator->objectives[assignment];
        group.assignment = assignment;
        groupIndex++;
    }
    allocator->OnAllocationGAFinished();
    cachedSearches.Clear();
    paths.Clear();
}
//-----------------------------------------------------------------------------//
void nAllocatorGA::InitializeWorld(){
    //n_printf("nAllocatorGA(%s).InitializeWorld()\n", str(this));
    this->paths.Clear();
}
//-----------------------------------------------------------------------------//
