#define N_IMPLEMENTS nGABlue
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ngablue.h"
#include "ga/nga.h"

nNebulaScriptClass(nGABlue, "ngaproblem");

//------------------------------------------------------------------------------
/**
*/
nGABlue::nGABlue():
    platformRoot    	( kernelServer, this ),
    assetRoot       	( kernelServer, this ),
    targetRoot      	( kernelServer, this ),
    threatRoot      	( kernelServer, this ),
    platformEntRoot 	( kernelServer, this ),
    targetEntRoot   	( kernelServer, this ),
    pathfinder      	( kernelServer, this )
{
}
//-----------------------------------------------------------------------------//
void nGABlue::Initialize(){
    nGAProblem::Initialize();
    platformRoot            	= "/scenario/blue/platforms";
    assetRoot               	= "/scenario/blue/assets";
    targetRoot              	= "/scenario/red/targets";
    threatRoot              	= "/scenario/red/threats";
    platformEntRoot         	= "/game/ents/platform";
    targetEntRoot           	= "/game/ents/target";
    pathfinder              	= "/sys/servers/pathfinder";
}
//-----------------------------------------------------------------------------//
void nGABlue::InitVars(){
    nGAProblem::InitVars();
}
//-----------------------------------------------------------------------------//
nGABlue::~nGABlue(){
}
//-----------------------------------------------------------------------------//
void nGABlue::Tick( float time, float dtime ){
    nGAProblem::Tick( time, dtime );
}
//-----------------------------------------------------------------------------//
///Construct the genome
//-----------------------------------------------------------------------------//
nGenome* nGABlue::BuildEncoding(){
    log("<nGABlue::BuildEncoding()>");
    targetNames.Clear();
    threatNames.Clear();
    nRoot* target, *asset, *threat;
    nGenome* sampleEncoding = (nGenome*) kernelServer->New("ngenome", "/ga/sampleencoding");
    nGenome* alloc = sampleEncoding->AddV("allocation"); //name
    alloc->SetInjectedValue( 1.0f );
    if( targetRoot.isvalid() ){
        for( target = targetRoot->GetHead(); target; target = target->GetSucc() ){
            targetNames.Append( target->GetName() );
        }
        ifvalid( threatRoot )
        for( threat = threatRoot->GetHead(); threat; threat = threat->GetSucc() ){
            threatNames.Append( threat->GetName() );
        }
        ifvalid( assetRoot ) 
        for( asset = assetRoot->GetHead(); asset; asset = asset->GetSucc() ){
            alloc->AddE( asset->GetName(), &targetNames );
        }
    } else {
        log("nGA Cannot build encoding - target root is invalid()\n");
    }
    nGenome* rc = sampleEncoding->AddF("rc", 0.0, 3.0 ); //name, min, max, bits
    rc->SetInjectedValue( 9.0f );
    sampleEncoding->Randomize();
    log("</nGA::BuildEncoding()>");
    return sampleEncoding;
}
//-----------------------------------------------------------------------------//
/// Load / Reset the mission
//-----------------------------------------------------------------------------//
void nGABlue::LoadMission(){
    log("<nGA::LoadMission()>");
    double missionLoadingStarted = time->GetTime();
    
    nString script;
    script.PrintF("GALog::loadMission {%s}", ga->GetSituation() );
    RUNSCRIPT( script.get() );

    double missionLoadingFinished = time->GetTime();
    ga->times.loadingMission = missionLoadingFinished - missionLoadingStarted;
    log("</nGA::LoadMission()>");
}
//-----------------------------------------------------------------------------//
/// Load the plan to play
//-----------------------------------------------------------------------------//
void nGABlue::LoadGenomePlan( nString genomePlan ) {
    log("<nGA::LoadGenomePlan>");
    double planLoadingStarted  = time->GetTime();
    
    nString script;
    script.PrintF("GALog::loadGenomePlan \"%s\"", genomePlan.Get() );
    RUNSCRIPT(script.Get());

    double planLoadingFinished = time->GetTime();
    ga->times.loadingPlan = planLoadingFinished - planLoadingStarted;
    log("</nGA::LoadGenomePlan>");
}
//-----------------------------------------------------------------------------//
/// Is the evaluation finished?
//-----------------------------------------------------------------------------//
bool nGABlue::IsEvaluationDone(){
    nRoot *platform, *aiRoot, *ai;
    int numRunning = 0;
    ifvalid( platformEntRoot ) 
    for ( platform = platformEntRoot->GetHead(); platform; platform = platform->GetSucc() ){
        aiRoot = platform->Find("ai");
        if( aiRoot ) for ( ai = aiRoot->GetHead(); ai; ai = ai->GetSucc() )
            if( ((nThing*)ai)->IsAlive() )
	numRunning++;
    }
    return ( numRunning == 0 );
}

//-----------------------------------------------------------------------------//
///Stop a running evalation
//-----------------------------------------------------------------------------//
void nGABlue::StopEvaluation(){
    log("nGABlue::StopEvaluation()>");
    pathfinder->Clear();			//clear your routing data 
}
//-----------------------------------------------------------------------------//
//begin an evaluation
//-----------------------------------------------------------------------------//
void nGABlue::StartEvaluation( nGenome* g ){
    log("<nGABlue::StartEvaluation()>");
    this->ResetWorld(g);
    this->LoadGenomePlan( this->GenomeToPlan( g ) );	//load the plan for this mission
    //this->individualBeingEvaluated = nextIndividualToEvaluate;
    log("</nGABlue::StartEvaluation()>");
}
//-----------------------------------------------------------------------------//
///Reloads the world - clearing everything and placing it again
//-----------------------------------------------------------------------------//
void nGABlue::ResetWorld( nGenome* g ){
    RUNSCRIPT("GA::clearMission");		//erase all existing things
    this->LoadMission();			//load the situation
}
//-----------------------------------------------------------------------------//
///  Converts a genome into a plan
//-----------------------------------------------------------------------------//
nString nGABlue::GenomeToPlan( nGenome* g ){
    log ("<nGA::GenomeToPlan()>");
    double genomeToPlanStarted = time->GetTime();
    ga->times.routing = 0.0;
    nGenome* alloc = g->GetChild(0);
    float rc = g->GetChild(1)->GetF();
    pathfinder->SetRC( rc );
    XMLString plan;
    nRoot* platform, *asset, *target;
    //foreach platform
    //Create plans one platform at a time
    plan.open("SIDE", "blue" );
    if( platformRoot.isvalid() ) for( platform = platformRoot->GetHead(); platform; platform = platform->GetSucc() ){
        plan.open( "PLATFORM", platform->GetName() );
        char platformPath[1024];
        platform->GetFullName( platformPath, 1024 );
        //log("starting platform : %s", platformPath );
        vector3 wp[2];
        wp[0] = ((nEnt*)platform)->GetVarV("pos");
        plan.open( "AI" );
        plan.wrap( "TAKEOFF" );
        nRoot* assets = platform->Find("assets");
        //foreach ASSET
        int i = 0; //index into allocation
        if( assets ) 
        for ( asset = assets->GetHead(); asset; asset = asset->GetSucc() ){
            if ( i >= alloc->NumChildren() ) { //make sure alloc size = true number of assets
	log("i >= alloc->NumChildren --- %i >= %i", i, alloc->NumChildren() );
	continue;
            }
            const char* assetName = alloc->GetChild(i)->GetName();
            asset = assetRoot->Find( assetName );
            const char* assetType = ((nEnt*)asset)->GetVarS("type");
            n_assert( asset != 0 );
            const char* assetsPlatform = ((nEnt*)asset)->GetVarS("platform");
            if( strcmp( platformPath, assetsPlatform ) == 0 ){ //if the asset belongs to you
	const char* targetName = alloc->GetChild(i)->GetE(); //pull the target enum out of the chromosome
	target = kernelServer->Lookup((nString("/scenario/red/targets/") + targetName).Get());
	ifnonzero( target ) {
	    wp[1] = ((nEnt*)target)->GetVarV("pos");
	    double routingStarted = time->GetTime();
	    pathfinder->FindPath( wp[0], wp[1] );
	    double routingStopped = time->GetTime();
	    ga->times.routing += routingStopped - routingStarted;
	    //log("routed from %f,%f,%f to %f,%f,%f", wp[0].x,wp[0].y,wp[0].z, wp[1].x,wp[1].y,wp[1].z );
	    for( int i = 0; i < pathfinder->NumWP(); i ++ ){
	        char txt[128];
	        vector3 v = pathfinder->GetWP(i);
	        sprintf( txt, "%f %f %f", v.x, v.y, v.z );
	        plan.wrap("MOVE", txt );
	    }
	    wp[0] = wp[1];
	    plan.open("ATTACK", targetName );
	    plan.wrap("WITHASSET", assetType );
	    plan.close("ATTACK");
	}
            }
            i++;
        }
        //route the platform back home
        wp[1] = ((nEnt*)platform)->GetVarV("startpos");
        pathfinder->FindPath( wp[0], wp[1] );
        for( int j = 0; j < pathfinder->NumWP(); j ++ ){
            vector3 v = pathfinder->GetWP(j);
            char txt[128];
            sprintf( txt, "%f %f %f", v.x, v.y, v.z );
            plan.wrap("MOVE", txt );
        }
        plan.close("AI");
        plan.close("PLATFORM");
    }
    else 
        once (n_printf("platform root not valid\n") );
    plan.close("SIDE");
    double genomeToPlanStopped = time->GetTime();
    ga->times.genomeToPlan = genomeToPlanStopped - genomeToPlanStarted;
    log ("</nGA::GenomeToPlan()>");
    return plan;
}
//-----------------------------------------------------------------------------//
///Determine the fitness from a finished evaluation
//-----------------------------------------------------------------------------//
float nGABlue::ParseFitness() {
    log ("<nGABlue::DoneEvaluation>" );
    //return plan.toString();
    ga->times.runningStopped = time->GetTime();
    ga->times.running = ga->times.runningStopped - ga->times.runningStarted;
    double fitnessParsingStart = time->GetTime();
    float fitness = 0;
    float damageBlue = 0.0f;
    float damageRed = 0.0f;

    float totalDistanceTraveled = 0;
    ifvalid( targetEntRoot ) for ( nEnt* target = (nEnt*) targetEntRoot->GetHead(); target; target = (nEnt*) target->GetSucc() ){
        float value = target->GetVarF("value");
        float hp = target->GetVarF("hp");
        float damageDone = (1.0f - hp) * value;
        log("target  : %20s v:%f h:%f d:%f", target->fn, value, hp, damageDone );
        damageRed += damageDone;
    }
    ifvalid( platformEntRoot) for ( nEnt* platform= (nEnt*) platformEntRoot->GetHead(); platform; platform = (nEnt*) platform->GetSucc() ){
        float value = platform->GetVarF("value");
        float hp = platform->GetVarF("hp");
        float distanceTraveled= platform->GetVarF("distancetraveled");
        totalDistanceTraveled += distanceTraveled;
        float damageDone = (1.0f - hp) * value;
        log("platform: %20s v:%f h:%f d:%f dist:%f", platform->fn, value, hp, damageDone, distanceTraveled );
        damageBlue += damageDone;
    }
    double fitnessParsingStop = time->GetTime();
    ga->times.parsingFitness = fitnessParsingStop - fitnessParsingStart;

    ga->times.evaluationStopped = time->GetTime();
    ga->times.evaluation = ga->times.evaluationStopped - ga->times.evaluationStarted;
    float distancePenalty = totalDistanceTraveled * .01f;
    fitness = damageRed - damageBlue - distancePenalty;
    log("fitness: %f damageBlue: %f damageRed: %f distancePenalty: %f\n", fitness, damageBlue, damageRed, distancePenalty );
    log ("</nGABlue::DoneEvaluation>" );
    return fitness;
}
