#define N_IMPLEMENTS nGACar
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ngacar.h"
#include "ga/nga.h"
#include "gfx/nchannelserver.h"

nNebulaScriptClass(nGACar, "ngaproblem");

//------------------------------------------------------------------------------
/**
*/
nGACar::nGACar():
car( kernelServer, this )
{
    car = "/game/ents/car/car0";
    this->AddToTriggerList();
}

void nGACar::Initialize(){
    nGAProblem::Initialize();
    vars.Append("drivetrain");
    vars.Append("width");
    vars.Append("height");
    vars.Append("depth");
    vars.Append("wheelradiusf");
    vars.Append("wheelwidthf");
    vars.Append("wheeldensityf");
    vars.Append("wheelradiusr");
    vars.Append("wheelwidthr");
    vars.Append("wheeldensityr");
    vars.Append("enginepower");
    vars.Append("engineresistance");
    vars.Append("maxspeed");
}

void nGACar::InitVars(){
    nGAProblem::InitVars();
    start.Connect( ent, "start");
    goal.Connect( ent, "goal");
}

nGACar::~nGACar(){
}
//------------------------------------------------------------------------------
/**
*/

///construct the default encoding -> the structure will be copied to all others
nGenome* nGACar::BuildEncoding(){
    log("<nGACar::BuildEncoding()>");
    nGenome* sampleEncoding = (nGenome*) kernelServer->New("ngenome", "/ga/sampleencoding");
    sampleEncoding->AddI("drivetrain", 	0, 	2 );
    sampleEncoding->AddF("width", 	0.10f, 	3.0f, .1 );
    sampleEncoding->AddF("height", 	0.10f, 	1.0f, .1 );
    sampleEncoding->AddF("depth", 	0.10f, 	3.0f, .1 );
    sampleEncoding->AddF("wheelradiusf", 	0.10f, 	5.0f, .1 );
    sampleEncoding->AddF("wheelwidthf", 	0.10f, 	1.0f, .1 );
    sampleEncoding->AddF("wheeldensityf", 	0.01f, 	0.5f, .01 );
    sampleEncoding->AddF("wheelradiusr", 	0.10f, 	5.0f, .1 );
    sampleEncoding->AddF("wheelwidthr", 	0.10f, 	1.0f, .1 );
    sampleEncoding->AddF("wheeldensityr", 	0.01f, 	0.5f, .01 );
    sampleEncoding->AddF("enginepower", 	10.0f, 	1000.0f, 10 );
    sampleEncoding->AddF("engineresistance", 	0.0f, 	100.0f, 10 );
    sampleEncoding->AddF("maxspeed", 	10.0f, 	100.0f, 1   );
    sampleEncoding->Randomize();
    log("</nGACar::BuildEncoding()>");
    RUNSCRIPT("Car::import");
    return sampleEncoding;
}
///begins the evaluation
void nGACar::StartEvaluation( nGenome* g ){
    startTime = time;
    log("<nGACar::StartEvaluation()>");
    car->SetVarI( "drivetrain",  g->GetChild(0)->GetI() );
    for( int i = 1; i < vars; i ++ ) {
        float f = g->GetChild(i)->GetF();
        log("car->SetVarF(%s,%f)", vars[i].Get(), f );
        car->SetVarF( vars[i].Get(), f );
    }
    car->SetVarV("pos", start.GetV() );
    car->SetVarV("vel", vector3(0,0,0) );
    car->SetVarV("force", vector3(0,0,0) );
    car->SetVarV("rvel", vector3(0,0,0) );
    car->SetVarV("torque", vector3(0,0,0) );
    RUNSCRIPT("Car::pop");
    RUNSCRIPT("Car::flip");
    RUNSCRIPT("Car::import");
    log("</nGACar::StartEvaluation()>");
    initialCheck = false;
    disqualified = false;
}
/// Is the evaluation finished?
bool nGACar::IsEvaluationDone(){
    vector3 p = car->GetVarV("pos");
    if( initialCheck == false )
    if( time - startTime > 10.0 ) {
        initialCheck = true;
        float dist  = (car->GetVarV("pos") - start.GetV() ).len();
    }
    vector3 up(0,1,0);
    quaternion q = car->GetVarQ("rot");
    up = q.rotate( up );
    if( up.y < 0 ) {
        disqualified = true;
        return true;
    }
    if( (p - goal.GetV()).len() <  25 )
        return true;
    if( time - startTime > 100 )
        return true;
    return false;
}
///called to stop an evaluation -> clear things up
void nGACar::StopEvaluation(){
    log("nGACar::StopEvaluation()>");
    log("/nGACar::StopEvaluation()>");
}
///begin an evaluation
float nGACar::ParseFitness() {
    log ("<nGACar::DoneEvaluation>" );
    if( disqualified ) {
        log("disqualified");
        return 0.0f;
    }
    //return plan.toString();
    float timeTaken = time - startTime;
    log("timeTaken:%f", timeTaken );
    float dist = (car->GetVarV("pos") - goal.GetV()).len();
    float fitness = (200.0f - timeTaken) * (1000.0f - dist);
    log("timeTaken:%f dist:%f fit:%f",   timeTaken, dist, fitness );
    log ("</nGACar::DoneEvaluation>" );
    return fitness;
}
