#define N_IMPLEMENTS nGenome
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "ga/ngenome.h"
#include "ga/ngene.h"
#include "enkiduo/neventserver.h"
#include "ga/nga.h"

nNebulaScriptClass(nGenome, "nfsm");

//------------------------------------------------------------------------------
nGenome::nGenome():
ref_event( kernelServer, this ) {
    stateNames.Append("UNEVALUATED");
    stateNames.Append("EVALUATED");
    stateNames.Append("BEING_EVALUATED");
    this->complainAboutDoubleStateSets = false;
    ref_event = "/sys/servers/event";
    this->fitness                       = 0;
    this->sharedFitness                 = 0;
    this->normalizedFitness             = 0;
    this->fitnessPercentile             = 0.0f;
    this->rank                          = 0;
    this->generation                    = 0;
    this->similarity                    = 0.0f;
}
//------------------------------------------------------------------------------
void nGenome::Initialize(){
    nFSM::Initialize();
    this->triggerSelf                   = true;
    this->geneRoot                      = dynamic_cast<nGene*> (this->CreateChild                   ("ngene", "gene"));
    n_assert( this->geneRoot );
    bitsAllocated                       = 256;
    this->bits                          = new bool[bitsAllocated];
    this->bitsBeingUsed                 = 0;
}
//------------------------------------------------------------------------------
void nGenome::InitVars(){
    nFSM::InitVars();
    crossoverProb.Connect( ent, "crossoverProb" );
    crossoverProb.DefaultF( .75f );
    mutationProb.Connect( ent, "mutationProb" );
    mutationProb.DefaultF( .01f );
    evaluationsDone.Connect( ent, "evaluationsDone" );
    evaluationsDone.DefaultI( 0 );
    mostRecentFitness.Connect( ent, "mostRecentFitness");
    mostRecentFitness.DefaultF( 0.0f );
    //numEvaluations.Connect( ent, "numEvaluations" );
    //numEvaluations.DefaultI( 0 );
}
//------------------------------------------------------------------------------
nGenome::~nGenome(){
    //n_printf("~nGenome(%s)\n", str(this));
}
//------------------------------------------------------------------------------
void nGenome::Tick( float time, float dtime ){
    //n_printf("nGenome::Tick(%f,%f)\n", time, dtime );
    nFSM::Tick( time, dtime );
}
//-----------------------------------------------------------------------------//
bool nGenome::States( StateMachineEvent event, uint state, float time, float dtime ){
    nGA* ga = dynamic_cast<nGA*>(this->GetParent());
    n_assert( ga && "ngenome failed to find its ga parent");
    if( ga->RecordData() == false ) 
        return true;
    BeginStateMachine
        State(UNEVALUATED)
            OnEnter
                //n_printf("%s.UNEVALUATED\n", this->fn);
                static void * eventPtr = 0;
                ref_event->pull( eventPtr, "gaGenomeChanged", this->fn);
        State(EVALUATED)
            OnEnter
                //n_printf("%s.EVALUATED\n", this->fn);
                static void * eventPtr = 0;
                ref_event->pull( eventPtr, "gaGenomeChanged", this->fn);
        State(BEING_EVALUATED)
            OnEnter
                //n_printf("%s.LOCAL_EVALUATING\n", this->fn);
                static void * eventPtr = 0;
                ref_event->pull( eventPtr, "gaGenomeChanged", this->fn);
    EndStateMachine
}
//-----------------------------------------------------------------------------//
//one of my children wants some bits to store data in
//-----------------------------------------------------------------------------//
int nGenome::GetBits(nGene* gene, int numBits){
    //n_printf("nGenome(%s).GetBits(%s,%i)\n", this->fn, gene->fn, numBits);
    int index = this->bitsBeingUsed;
    bitsBeingUsed += numBits;
    n_assert( bitsBeingUsed < bitsAllocated && "ngenome cannot grow its bit array at the moment -> its dumb");
    return index;
}
//-----------------------------------------------------------------------------//
const char* nGenome::GetBits(){
    static nString bits;
    bits = "";
    nString bit;
    for( int i = 0; i < this->bitsBeingUsed; i ++ ) {
        bit.PrintF("%i", this->bits[i]);
        bits += bit;
    }
    return bits.Get();
}
//-----------------------------------------------------------------------------//
void nGenome::Randomize(){
    for( int i = 0; i < bitsBeingUsed; i ++ ) {
        bits[i] = roll(.5f);
    }
}
//-----------------------------------------------------------------------------//
/**
 * copy this genome from another one
 * to do this just copy the bitstring
 * and other parameters
 **/
void nGenome::CopyFrom( nGenome* genome ){
    //n_printf("nGenome(%s).copyFrom(%s)\n", this->fn, genome->fn);
    n_assert( this->bits != genome->bits && "copying from myself");
    n_assert( this->bitsAllocated == genome->bitsAllocated && this->bitsBeingUsed == genome->bitsBeingUsed && "all genomes must be the same size at the moment" );
    this->fitness                       = genome->fitness;
    this->normalizedFitness             = genome->normalizedFitness;
    this->fitnessPercentile             = genome->fitnessPercentile;
    this->rank                          = genome->rank;
    for( int i = 0; i < bitsBeingUsed; i ++ )
        this->bits[i] = genome->bits[i];
    //n_printf("copy from setstate:%s\n", this->stateNames[genome->GetStateI()]);
    this->SetState(genome->GetStateI());
}
/**
 * cross me over with another genome
 * lets just do one point crossover for now
 * !!this will modify both genomes!! not just this one
 **/
void nGenome::CrossOverWith( nGenome* genome ){
    this->SetState(UNEVALUATED);
    if( roll( crossoverProb.GetF() ) == false) {
        return;
    }
    //n_printf("nGenome(%s).CrossOverWith(%s)\n", this->fn, genome->fn);
    int crossoverPoint = randI( 0, this->bitsBeingUsed -1 );
    //n_printf("crossoverPoint:%i\n", crossoverPoint );
    for( int i = crossoverPoint; i < bitsBeingUsed; i ++ )
        swap(bits[i], genome->bits[i]);
    //n_printf("crossover setstate:%s\n", this->stateNames[UNEVALUATED]);
    this->SetState(UNEVALUATED);
}
/**
 * mutate me -> bitwise over mutationprob
 */
void nGenome::Mutation(){
    bool dirty = false;
    float mprob = mutationProb.GetF();
    for( int i = 0; i < bitsBeingUsed; i ++ ) {
        if ( roll(mprob) ) {
            dirty = true;
            bits[i] = !bits[i];
        }
    }
    if( dirty ) {
        //n_printf("mutation setstate:%s\n", this->stateNames[UNEVALUATED]);
        this->SetState(UNEVALUATED);
    }
}
//-----------------------------------------------------------------------------//
TNL::Vector<bool> nGenome::GetTNLBitString(){
    TNL::Vector<bool> bitString;
    for( int i = 0; i < bitsBeingUsed; i ++ )
        bitString.push_back( bits[i] );
    return bitString;
}
