#define N_IMPLEMENTS nIMGA
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "ga/nimga.h"
#include "ga/ngene.h"
#include "ga/ngeneint.h"
#include "ga/ngenefloat.h"
#include "ga/ngenome.h"
#include "im/nim.h"
#include "im/niment.h"
#include "im/nimradial.h"
#include "enkiduo/neventserver.h"
#include "aspect/nside.h"

nNebulaScriptClass(nIMGA, "nga");

//------------------------------------------------------------------------------
nIMGA::nIMGA():nGA(){
    this->slaveClassName = "nimga";
}
//------------------------------------------------------------------------------
void nIMGA::Initialize(){
    nGA::Initialize();
}
//------------------------------------------------------------------------------
void nIMGA::InitVars(){
    nGA::InitVars();
    n_printf("nIMGA(%s).initVars()\n", str(this));
    imEvolving.Connect( ent, "imEvolving" );
    imEvolving.DefaultRef("");
    evaluationTime.Connect              ( ent, "evaluationTime" );
    evaluationTime.DefaultF             ( 60.0f * 10.0f ); //6 minutes
    evaluationTimeRemaining.Connect     ( ent, "evaluationTimeRemaining" );
    evaluationTimeRemaining.DefaultF    ( 0.0f );
    timeEvaluationStarted.Connect       ( ent, "timeEvaluationStarted" );
    timeEvaluationStarted.DefaultF      ( 0.0f );

}
//------------------------------------------------------------------------------
nIMGA::~nIMGA(){
}
//------------------------------------------------------------------------------
void nIMGA::Tick( float time, float dtime ){
    //n_printf("nIMGA::Tick(%f,%f)\n", time, dtime );
    nGA::Tick( time, dtime );
    evaluationTimeRemaining.SetF( evaluationTime.GetF() - (time - timeEvaluationStarted.GetF()));
    //n_printf("nIMGA::Tick(%f,%f) state=:%s\n", time, dtime, this->GetState() );
    
    //n_printf("IMEVOLVING:%p\n", imEvolving.GetRef());
    //n_printf("state:%s time:%f isvalid:%i\n", this->GetState(), time, imEvolving.isvalid());
    if( this->GetStateI() == MASTER_WAITING_FOR_RUN && time > 5.0f && imEvolving.isvalid() ) {
        n_printf("nIMGA(%s) evolving %s\n", this->fn, imEvolving.GetRef()->fn );
        this->Run();
    }
    //n_printf("%i %i %i\n", this->GetStateI() == MASTER_WAITING_FOR_RUN, time > 5.0f, imEvolving.isvalid() );
}
//-----------------------------------------------------------------------------//
//create the genome
//-----------------------------------------------------------------------------//
void nIMGA::CreateGenome( nGenome* g ) {
    //n_printf("nIMGA(%s)::CreateGenome(%p,%s)\n", str(this), g, g?g->fn:"");
    n_assert(imEvolving.isvalid());
    nRoot* im = imEvolving.GetRef();
    //nRoot* im = kernelServer->Lookup(imEvolving.Get());
    this->RecursivePullCoefs(im, g->GetGeneRoot() );
}
//-----------------------------------------------------------------------------//
void nIMGA::RecursivePullCoefs( nRoot* im, nGene* gene ) {
    //n_printf("nIMGA::RecursivePullCoefs(%s,%s)\n", im->fn, gene->fn);
    foreachchild(nIM, node, im){
        nGene* childGene = (nGene*) gene->CreateChild("ngene", node->GetName());
        n_assert( childGene && "failed to initialize child gene\n");
        classVar(niment, imentClass);
        if( node->IsA( imentClass )){
            //childGene->CreateChild("ngenefloat", "func");
            //childGene->CreateChild("ngenefloat", "radius");
            //childGene->CreateChild("ngenefloat", "ipol");
            //childGene->CreateChild("ngenefloat", "who");
            ((nGeneInt*) childGene->CreateChild("ngeneint",           "func"))->InitI(0, numFuncStrings);
            ((nGeneInt*) childGene->CreateChild("ngeneint",           "ipol"))->InitI(0, numIpolStrings);
            ((nGeneInt*) childGene->CreateChild("ngeneint",         "radius"))->InitI(0, numRadiiStrings);
            ((nGeneInt*) childGene->CreateChild("ngeneint",            "who"))->InitI(0, numWhoStrings);
            ((nGeneFloat*) childGene->CreateChild("ngenefloat",       "coef"))->InitF(-1.0f,1.0f,.1f);
            ((nGeneFloat*) childGene->CreateChild("ngenefloat", "radiusCoef"))->InitF(+0.0f,10.0f,.1f);
        }
        classVar(nimradial, imradialClass);
        if ( node->IsA( imradialClass ) ) {
            ((nGeneInt*) childGene->CreateChild("ngeneint",             "to"))->InitI(0, numWhoStrings + 2);
            ((nGeneFloat*) childGene->CreateChild("ngenefloat","thetaOffset"))->InitF(-N_PI,N_PI,.1f);
            ((nGeneFloat*) childGene->CreateChild("ngenefloat", "thetaWidth"))->InitF(0.0f,180.0f,.1f);
        }
        this->RecursivePullCoefs( node, childGene );
    }
}
//-----------------------------------------------------------------------------//
//hook pulled to start the evalution of an individual
//-----------------------------------------------------------------------------//
void nIMGA::InitializeWorld(){
    n_printf("nIMGA(%s).InitializeWorld()\n", this->fn );
    //store the world state
    static void * storeWorldPtr = 0;
    ref_event->pull( storeWorldPtr, "storeWorld", "/game/ents/oilplatform/oilplatform0");
    this->SendInitializeWorldMessage( gamestate.GetS().Get() );
}
//-----------------------------------------------------------------------------//
//parse out the appropriate coefficients and assign them to my im
//-----------------------------------------------------------------------------//
void nIMGA::StartEvaluation( nGenome* g ){
    ////n_printf("nIMGA::StartEvaluation(%s)\n", g?g->fn:"0x0");

    //reset the world state
    static void * resetWorldPtr = 0;
    ref_event->pull( resetWorldPtr, "resetWorld", "/game/ents/oilplatform/oilplatform0");

    //go through the gene tree and set the appropriate coefficients
    nGene* geneRoot                     = g->GetGeneRoot();
    nRoot* imRoot                       = imEvolving.GetRef();
    //nRoot* imRoot                       = kernelServer->Lookup(imEvolving.Get());
    //n_printf("geneRoot:%s, imRoot:%s\n", str(geneRoot), str(imRoot) );
    this->RecursiveGeneParse( geneRoot, imRoot );                               //assign coefficients
    //find all the clones of this im and assign to them as well
    foreachchild( nRoot, im,imRoot->GetParent()){                               //look at each sibling to the im
        if ( im == imRoot )                                                     //dont parse me again
            continue;
        const char* myName = imRoot->GetName();
        const char* otherName = im->GetName();
        //n_printf("myName:%s otherName:%s\n", myName, otherName);
        if( strstr( otherName, myName ) == otherName) { //othername starts with my name
            //n_printf("is a clone\n");
            this->RecursiveGeneParse( geneRoot, im );
        }
    }
    timeEvaluationStarted.SetF(time);
}
//-----------------------------------------------------------------------------//
void nIMGA::RecursiveGeneParse( nGene* geneRoot, nRoot* imRoot ){
    foreachchild(nIM, imChild, imRoot ){
        if( strcmp(imChild->GetName(), "riskyness") == 0 ) { 
            n_printf("skipping riskyness(%s)\n", str(imChild));
            continue;
        }
        nGene* geneChild = dynamic_cast<nGene*>(geneRoot->Find(imChild->GetName()));
        //n_printf("imChild:%s geneChild:%s\n", imChild->fn, geneChild?geneChild->fn:"none");
        n_assert( geneChild && "failed to find geneChild matching, or it is the wrong class");

        classVar(niment, imentClass);
        if( imChild->IsA( imentClass )){
            nGeneInt* funcGene          = dynamic_cast<nGeneInt*>(geneChild->Find("func"));
            nGeneInt* ipolGene          = dynamic_cast<nGeneInt*>(geneChild->Find("ipol"));
            nGeneInt* radiusGene        = dynamic_cast<nGeneInt*>(geneChild->Find("radius"));
            nGeneInt* whoGene           = dynamic_cast<nGeneInt*>(geneChild->Find("who"));
            nGeneFloat* coefGene        = dynamic_cast<nGeneFloat*>(geneChild->Find("coef"));
            nGeneFloat* radiusCoefGene  = dynamic_cast<nGeneFloat*>(geneChild->Find("radiusCoef"));
            n_assert( coefGene && radiusCoefGene && "failed to find coefGene or the radiusCoefGene" );
            //n_printf("coefGene:%s radiusCoefGene:%s\n", coefGene?coefGene->fn:"none", radiusCoefGene?radiusCoefGene->fn:"none");
            int func                    = funcGene->GetI();
            int ipol                    = ipolGene->GetI();
            int radius                  = radiusGene->GetI();
            int who                     = whoGene->GetI();
            float coef                  = coefGene->GetF();
            float radiusCoef            = radiusCoefGene->GetF();
            printf("im:%50s func:%20s ipol:%10s radius:%10s who:%i-%10s coef:%10f radiusCoef:%10f\n", str(imChild), funcStrings[func], ipolStrings[ipol], radiiStrings[radius], who, whoStrings[who], coef, radiusCoef );
            ((nIMEnt*)imChild)->SetFunc( func );
            ((nIMEnt*)imChild)->SetIpol( ipol );
            ((nIMEnt*)imChild)->SetRadius( radius );
            ((nIMEnt*)imChild)->SetWho( who );
            ((nIMEnt*)imChild)->SetCoef( coef );
            ((nIMEnt*)imChild)->SetRadiusCoef( radiusCoef );
        } 
        classVar(nimradial, imradialClass);
        if ( imChild->IsA( imradialClass ) ) {
            nGeneInt* toGene            = dynamic_cast<nGeneInt*>(geneChild->Find("to"));
            nGeneFloat* thetaOffsetGene = dynamic_cast<nGeneFloat*>(geneChild->Find("thetaOffset"));
            nGeneFloat* thetaWidthGene  = dynamic_cast<nGeneFloat*>(geneChild->Find("thetaWidth"));

            int to                      = toGene->GetI();
            float thetaOffset           = thetaOffsetGene->GetF();
            float thetaWidth            = thetaWidthGene->GetF();

            ((nIMRadial*)imChild)->SetTo( to );
            ((nIMRadial*)imChild)->setThetaOffset( thetaOffset );
            ((nIMRadial*)imChild)->setThetaWidth( thetaWidth );
            printf("im:%50s to:%22s thetaOffset:%10f thetaWidth:%10f\n", str(imChild), ((nIMRadial*)imChild)->GetTo(), thetaOffset, thetaWidth );
        }
        this->RecursiveGeneParse( geneChild, imChild );
    }
}
//-----------------------------------------------------------------------------//
//check to see if the evaluation has finished
bool nIMGA::EvaluationFinished(){
    //n_printf("nIMGA::EvaluationFinished()\n");
    return evaluationTimeRemaining.GetF() < 0.0f;
    //return true;
}
//-----------------------------------------------------------------------------//
void nIMGA::StopEvaluation( nGenome* g ) {
    //n_printf("nIMGA::StopEvaluation(%s)\n", g?g->fn:"0x0");
    static void * jumpTimePtr = 0;
    ref_event->pull( jumpTimePtr, "jumpTime", "600");
}
//-----------------------------------------------------------------------------//
/**
 * determine fitness for how we did, well how should we do this?
 * well we can look at damage*value to opponents - damage*value for us
 * okay so i look at the parent of the im in order to determine my side, sum up damage to its units and sm up damage to hostiles 
 * vwallah ive got fitness
 */
//-----------------------------------------------------------------------------//
float nIMGA::GetFitness(){
    float fitness;
    //fitness = this->RecursiveSumCoefficients( imEvolving.GetRef() );
    //return fitness;

    static nRoot* neutralSide = kernelServer->Lookup("/game/sides/neutral");
    const EntArray* friendlies;
    nSide* mySide                       = dynamic_cast<nSide*>(imEvolving.GetRef()->GetParent()->Find("side"));
    n_assert                            ( mySide && "nIMGA failed to find the sides its im is evolving");
    friendlies                          = mySide->GetUnits();
    float friendlyDamageTaken           = this->SumDamageValues( friendlies );
    const SideArray* hostileSides       = mySide->GetRelatedSides(HOSTILE);
    const EntArray* hostileUnits;
    float totalHostileDamage            = 0.0f;
    for( int i = 0; i < hostileSides->Size(); i ++ ) {
        hostileUnits                    = hostileSides->Get(i)->GetUnits();
        float hostileDamage             = this->SumDamageValues( hostileUnits );
        totalHostileDamage += hostileDamage;
    }
    fitness = totalHostileDamage - friendlyDamageTaken;
    n_printf("fitness: %f totalHostileDamage:%f friendlyDamageTaken:%f\n", fitness, totalHostileDamage, friendlyDamageTaken );
    return fitness;
    
    /*
    //const char* imGroupPath = imEvolving.GetRef()->GetVarRef("myGroup");
    //if( strcmp(imGroupPath, "none") == 0 ) {
        //n_printf("warning evolving im:%s which is not assigned to any group\n", imEvolving.GetRef()->fn);
        //return 0.0f;
    //}
    //nRoot* imGroup = kernelServer->Lookup( imGroupPath );
    const char* entPath = 0;
    const char* sidePath = 0;
    nEnt* mySide;
    foreachchild( nEnt, groupUnit, imGroup ){
        entPath = groupUnit->GetVarS("path");
        if( strcmp(imGroupPath, "none") == 0 ) {
            n_printf("warning evolving im:%s which has group:%s which has an invalid unit(%s)/path in it\n", imEvolving.GetRef()->fn, imGroup->fn, groupUnit->fn);
            return 0.0f;
        }
        nEnt* e = (nEnt*) kernelServer->Lookup( entPath );
        sidePath = e->GetVarRef("side");
        mySide = (nEnt*) kernelServer->Lookup(sidePath);
        break;
    }
    n_printf("my im:%s has group:%s which has a unit:%s with side:%s\n", imEvolving.GetRef()->fn, imGroupPath, entPath, sidePath);

    fitness = 0.0f;
    {
    foreachchild(nRoot, ents, kernelServer->Lookup("/game/ents")){
        foreachchild(nEnt, ent, ents){
            nRoot* side = kernelServer->Lookup(ent->GetVarRef("side"));
            n_assert( side && "ent has no side");
            if ( side == neutralSide ) //ignore neutral boats for now
                continue;
            n_printf("ent:%s side:%s\n", ent->fn, side->fn);
            if( ent->HasVar("value") == 0 ) {
                n_printf("ent:%s has no value\n", ent->fn);
                continue;
            }
            float value = ent->GetVarF("value");

            if( ent->HasVar("hp") == 0 ) {
                n_printf("ent:%s has no hp\n", ent->fn);
                continue;
            }
            float hp = ent->GetVarF("hp");

            if( ent->HasVar("maxhp") == 0 ) {
                n_printf("ent:%s has no maxhp\n", ent->fn);
                continue;
            }
            float maxhp = ent->GetVarF("maxhp");

            n_printf("value:%f hp:%f maxhp:%f\n", value, hp, maxhp);
            float damageTaken = maxhp - hp;

            if( side == mySide ) {
                fitness -= value * damageTaken;
                n_printf("losing: %f points because of damage taken -> %f\n", value*damageTaken, fitness);
            } else {
                fitness += value * damageTaken;
                n_printf("gaining:%f points because of damage  done -> %f\n", value*damageTaken, fitness);
            }
        }
    }
    }
    //n_printf("parsed fitness: %f\n", fitness);
    return fitness;
    */
}
//-----------------------------------------------------------------------------//
float nIMGA::RecursiveSumCoefficients( nRoot* imRoot ){
    float answer = 0.0f;
    foreachchild(nIM, imChild, imRoot ){
        classVar(niment, imentClass);
        if( imChild->IsA( imentClass )){
            float coef = ((nIMEnt*)imChild)->GetCoef();
            float radiusCoef = ((nIMEnt*)imChild)->GetRadiusCoef();
            //n_printf("read coef:%f radiusCoef: %f\n", coef, radiusCoef );
            answer += coef;
            answer += radiusCoef;
        }
        answer += this->RecursiveSumCoefficients( imChild );
    }
    return answer;
}
//-----------------------------------------------------------------------------//
void nIMGA::EvolveIM(const char* imPath){
    n_printf("nIMGA(%s).EvolveIM(%s)\n", str(this), imPath);
    this->Tick(0,0); //be sure our variables are initted
    //this->InitVars();
    this->imEvolving.SetRef(imPath);
    if( this->imEvolving.GetRef() == 0 )
        n_printf("failed to find im to evolve\n");
    //this->Run();
    //this->SetState( MASTER_WAITING_FOR_RUN );
}
//-----------------------------------------------------------------------------//
void nIMGA::InitializeSlave( nGA* slave ){
    n_printf("nIMGA(%s).InitializeSlave(%s)\n", str(this), str(slave));
    n_assert( imEvolving.isvalid() );
    nIMGA* imslave = dynamic_cast<nIMGA*>(slave);
    n_assert( imslave && "failed to cast imga's slave to an imga");
    imslave->EvolveIM( this->imEvolving.GetRef()->fn );
}
//-----------------------------------------------------------------------------//
float nIMGA::SumDamageValues( const EntArray* ents ) {
    float sum;
    for( int i= 0; i < ents->Size(); i ++ ) {
        nEnt* ent = ents->Get(i);
        if( ent->HasVar("value") == 0 ) {
            n_printf("ent:%s has no value\n", ent->fn);
            continue;
        }
        float value = ent->GetVarF("value");

        if( ent->HasVar("hp") == 0 ) {
            n_printf("ent:%s has no hp\n", ent->fn);
            continue;
        }
        float hp = ent->GetVarF("hp");

        if( ent->HasVar("maxhp") == 0 ) {
            n_printf("ent:%s has no maxhp\n", ent->fn);
            continue;
        }
        float maxhp = ent->GetVarF("maxhp");
        float damageTaken = maxhp - hp;
        sum += value * damageTaken;
    }
    return sum;
}
