#define N_IMPLEMENTS nAllocator
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "im/nallocator.h"
#include "im/nim.h"
#include "aspect/nside.h"
#include "im/nimobjective.h"
#include "ga/nallocatorga.h"
#include "im/nimmanager.h"

nNebulaScriptClass(nAllocator, "naspect");

//------------------------------------------------------------------------------
nAllocator::nAllocator():
    imManager( kernelServer, this ),
    allocatorGA( kernelServer, this )
{
}
//------------------------------------------------------------------------------
void nAllocator::Initialize(){
    //n_printf("nAllocator.Initialize(%s)\n", str(this));
    nAspect::Initialize();
}
//------------------------------------------------------------------------------
void nAllocator::InitVars(){
    nAspect::InitVars();
    side = dynamic_cast<nSide*>(ent->Find("side"));
    n_assert(side && "nAllocator failed to find its corresponding side" );
    allocatorGA                         = ent->ChildPath("allocatorga");
    imManager                           = ent->ChildPath("manager");
}
//------------------------------------------------------------------------------
nAllocator::~nAllocator(){
}
//------------------------------------------------------------------------------
void nAllocator::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
}
//-----------------------------------------------------------------------------//
/**
 * okay now the fun part
 * we need to go through all the units allocated to my side, and figure out which im they should follow
 * well or we can allocate them by group? okay lets do that...
 * so we go through the groups, and see if we can match them up with the allocation requested by the different ims
 * i'd like to say this should wait though so I can prepare my biometrics talk.. eww
 **/
//-----------------------------------------------------------------------------//
/**
 * okay so how do we do the allocation
 * alright so we need to figure out which units / groups I have and tell them to go do there thing
 * so we can pull a list of units from the side, or we can just walk the groups.
 * lets do this --- lets walk the groups and identify groups of units we have
 * then we can try to allocate them best along the influece maps lines
 *
 * okay so now we have this group info metric which tells us what each group is good for
 * now we want to setup a ga to search for the optimal group -> target allocation
 * so how do we do this?
 *
 * okay so we can create the ga easy enough
 * so we tell the ga to reset its population run against these groups/objectives
 * and when its done it can alert us
 * well what do we want to do..
 * okay so once the allocation is done, we need to pass it on to the units, so we make another function
 **/
//-----------------------------------------------------------------------------//
void nAllocator::Allocate( nIM* imDone ) {
    //n_printf("nAllocator::Allocate(%s)\n", str(imDone) );
    if( imDone == this->GetTail() ){
        groups = this->FindMyGroups();                                          //figure out my groups
        objectives = this->FindMyObjectives();                                  //figure out my objectives
        if( imManager.isvalid() ) {
            if( allocatorGA.isvalid() ) {
                allocatorGA->Run(this);
                imManager->SetActive(false);
            }
        }
    }
}
//-----------------------------------------------------------------------------//
/**
 * okay the ga has finished, we figure out what it finished with? and then we actually set it
 */
//-----------------------------------------------------------------------------//
void nAllocator::OnAllocationGAFinished(){
    //n_printf("nAllocator(%s).OnAllocationGAFinished()\n", str(this));
    for( int i = 0; i < groups; i ++ ) {
        //n_printf("assigning group:%i.%s to %i.%s.%s\n", i, groups[i].name.Get(), groups[i].assignment, objectives[groups[i].assignment].name.Get(), str(objectives[groups[i].assignment].point) );
        nRoot* group = kernelServer->Lookup( groups[i].path.Get() );
        foreachchild(nEnt, entProxy, group ){
            const char* path = entProxy->GetVarS("path");
            //n_printf("path:%s\n", path);
            nEnt* e = (nEnt*) kernelServer->Lookup(path);
            n_assert( e );
            //if (e->Find("moveto") == 0 )
                //kernelServer->New("nboatmoveto", e->ChildPath("moveto").Get());
            if( e->HasVar("destination" ))
                e->SetVarV("destination", objectives[groups[i].assignment].point );
        }
    }
    if( imManager.isvalid() )
        imManager->SetActive( true );
    else
        n_printf("failed to find find/reactivate immanager after allocatorga finished");
}
//-----------------------------------------------------------------------------//
/**
 * determine which groups I have to allocate, and the inofmration I need about them to do the allocation
 */
//-----------------------------------------------------------------------------//
nArray<nAllocator::GroupInfo> nAllocator::FindMyGroups(){
    nArray<GroupInfo> groups;
    //reassign units
    //n_printf("%s allocating\n", str(this));
    nString mySide      = side->GetParent()->FullNameString();
    nRoot* groupRoot = kernelServer->Lookup("/comm/groups");
    foreachchild( nEnt, group, groupRoot ) {
        GroupInfo info;
        info.name = group->GetName();
        info.path = group->fn;
        info.numUnits = 0;
        //n_printf("group:%s\n", str(group));
        bool hasMoveableUnits = false;
        foreachchild( nEnt, entProxy, group ) {
            if( entProxy->HasVar("path")) {
                const char* path = entProxy->GetVarS("path");
                nEnt* ent           = LOOKUP(nEnt*, path);
                const char* entSide = ent->GetVarRef("side");

                if( mySide == entSide ) {
                    float power, speed, value;
                    vector3 position;
                    if( ent->Find("engine"))
                        hasMoveableUnits = true;
                    if( ent->HasVar("imPower") )                                power = ent->GetVarF("imPower");
                    else                                                        power = 0.0f;
                    if( ent->HasVar("imSpeed") )                                speed = ent->GetVarF("imSpeed");
                    else                                                        speed = 0.0f;
                    if( ent->HasVar("imValue") )                                value = ent->GetVarF("imValue");
                    else                                                        value = 0.0f;
                    position = ent->GetVarV("pos");
                    //n_printf("p:%f s:%f f:%f\n", power, speed, value );
                    //n_printf("position:%s\n", str(position));
                    info.powers.Append( power );
                    info.values.Append( value );
                    info.speeds.Append( speed );
                    info.positions.Append( position );
                    info.numUnits++;
                }
            }
        }
        n_assert( info.numUnits == info.powers.Size() );
        n_assert( info.numUnits == info.values.Size() );
        n_assert( info.numUnits == info.speeds.Size() );
        info.powerSum = calculateSum( info.powers );
        info.valueSum = calculateSum( info.values );
        info.speedSum = calculateSum( info.speeds );
        info.powerAvg = calculateAverage( info.powers );
        info.valueAvg = calculateAverage( info.values );
        info.speedAvg = calculateAverage( info.speeds );
        info.centroid = calculateAverage( info.positions );
        //n_printf("centroid:%s\n", str(info.centroid));
        //n_printf("group:%s numUnits:%i power:%f-%f value:%f-%f speed:%f-%f\n", info.path.Get(), info.numUnits, info.powerSum, info.powerAvg, info.valueSum, info.valueAvg, info.speedSum, info.speedAvg );
        if( info.numUnits > 0 && hasMoveableUnits ) {
            //n_printf("new group:%s with numunits:%i\n", info.name.Get(), info.numUnits);
            groups.Append( info );
        }
    }
    return groups;
}
//-----------------------------------------------------------------------------//
/**
 * determine which objectives I have, and gather the information I need to do the allocation
 */
//-----------------------------------------------------------------------------//
nArray<nAllocator::ObjectiveInfo> nAllocator::FindMyObjectives(){
    nArray<ObjectiveInfo> objectives;
    const nArray<vector3>* points;
    foreachchild( nIMObjective, objective, this ) {
        if( strcmp(objective->GetName(), "riskyness") == 0 ) 
            continue;
        if( strcmp(objective->GetName(), "collisionAvoidance") == 0 ) 
            continue;
        points = objective->getObjectivePoints();
        ObjectiveInfo* rootObjectiveInfo = 0;
        for( int i = 0; i < points->Size(); i ++ ){
            vector3 point = points->Get(i);
            ObjectiveInfo info;
            int x, y;
            objective->getCoord( point, x, y );
            info.reward                 = objective->getVal( x, y );
            info.name                   = objective->GetName();
            info.powerCoef              = objective->GetPowerCoef();
            info.valueCoef              = objective->GetValueCoef();
            info.speedCoef              = objective->GetSpeedCoef();
            info.maxUnits               = objective->GetMaxUnits();
            info.point                  = point;
            //n_printf("name:%s powerCoef:%f valueCoef:%f speedCoef:%f maxUnits:%i\n", info.name.Get(), info.powerCoef, info.valueCoef, info.speedCoef, info.maxUnits);
            objectives.Append( info );
            if( i == 0 ) {
                rootObjectiveInfo = &objectives[-1];
                //n_printf("rootObjectiveInfo:%p\n", rootObjectiveInfo );
            }
            else {
                objectives[-1].rootObjectiveInfo = rootObjectiveInfo;
                //n_printf("%p.rootObjectiveInfo:%p\n", &objectives[-1],  rootObjectiveInfo );
            }
        }
    }
    return objectives;
}
