#define N_IMPLEMENTS nAllocatorAStar
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "im/nallocatorastar.h"
#include "im/nim.h"

nNebulaScriptClass(nAllocatorAStar, "nastar");

//------------------------------------------------------------------------------
nAllocatorAStar::nAllocatorAStar():
    riskynessIM( kernelServer, this ) 
{
}
//------------------------------------------------------------------------------
void nAllocatorAStar::Initialize(){
    nAStar::Initialize();
    this->clearLines = false;
}
//------------------------------------------------------------------------------
void nAllocatorAStar::InitVars(){
    nAStar::InitVars();
    //this->riskynessIM = this->GetParent()->ChildPath("allocator/riskyness");
    //this->AddToDisplayList();
    this->riskynessIM = this->GetParent()->ChildPath("allocator/collisionAvoidance");
    n_printf("riskynessIM:%s\n", this->riskynessIM.getname());
    kernelServer->PopCwd();
    if( this->riskynessIM.isvalid() == false )
        n_printf("nAllocatorAStar(%s) failing to find its riskyness im\n");
    int res = riskynessIM->getResolution();
    cachedNodes = new AllocatorAStarNode**[res];
    for( int i = 0; i < res; i ++ ) {
        cachedNodes[i] = new AllocatorAStarNode*[res];
        for( int j = 0; j < res; j ++ ) {
            cachedNodes[i][j] = 0;
        }
    }
}
//------------------------------------------------------------------------------
nAllocatorAStar::~nAllocatorAStar(){
}
//------------------------------------------------------------------------------
void nAllocatorAStar::Tick( float time, float dtime ){
    nAStar::Tick( time, dtime );
}
/**-----------------------------------------------------------------------------//
 * find a path from some entity to a destination point
 * right of way offset specifies which side of the channel you watn to be on, well could we auto calculate this, so you walk through all your points except the first and last and offset them by an angle to the right of the current heading
--------------------------------------------------------------------------------*/
AStarPath* nAllocatorAStar::FindPath( vector3 startPos, vector3 goalPos ) {
    n_printf("nAllocatorAStar::FindPath(%s)->%s)\n", str(startPos), str(goalPos));
    this->linesToDraw.Clear();

    AllocatorAStarNode* start = new AllocatorAStarNode( this );
    start->Set( startPos );
    nodes.Append( start );

    AllocatorAStarNode* goal = new AllocatorAStarNode( this );
    goal->Set( goalPos );
    nodes.Append( goal );

    AStarPath* path = nAStar::FindPath( (AStarNode*) start, (AStarNode*) goal );
    vector3 shift(100.0f, 0.0f, 0.0f);
    //path->shift( vector3(5,0,0) );
    for ( int i = 0; i < path->nodes.Size() - 1 ; i ++ ) {
        int lookAhead = path->nodes.Size() - i - 1;
        if( lookAhead > 10 )
            lookAhead = 10;
        float skewRatio = 1.0f;
        if( i < 10 )
            skewRatio = i / 10.0f;
        else if ( lookAhead < 10 )
            skewRatio = lookAhead / 10.0f;
        vector3 forwardVector           = path->nodes[i + lookAhead]->pos - path->nodes[i]->pos;
        forwardVector.norm();
        vector3 rightVector             ( -forwardVector.z, forwardVector.y, forwardVector.x );
        vector3 shiftVector             = rightVector * shift.x * skewRatio;
        if( this->displayed ) {
            this->DrawLine                  ( path->nodes[i]->pos, path->nodes[i]->pos + shiftVector, white );
            this->DrawLine                  ( path->nodes[i]->pos, path->nodes[i]->pos + forwardVector, white );
        }
        path->nodes[i]->pos             += shiftVector;
    }
    return path;
}
//-----------------------------------------------------------------------------//
void AllocatorAStarNode::Set( vector3 pos ) {
    this->pos = pos;
    nAllocatorAStar* allocatorAStarServer = dynamic_cast<nAllocatorAStar*>(this->server);
    n_assert( allocatorAStarServer  );
    nIM* riskynessIM = allocatorAStarServer->getRiskynessIM();
    n_assert( riskynessIM && "AllocatorAStarNode::Set failed to find riskynessIM");
    riskynessIM->getCoord( pos, this->x, this->y );
}
/**-----------------------------------------------------------------------------//
 * check if two states are equal -> generally this versus the goal state
 * since we have a nice fuzzy world we need some kinda threshold here
 --------------------------------------------------------------------------------*/
bool AllocatorAStarNode::operator == ( const AStarNode& aneighbor ) {
    const AllocatorAStarNode* neighbor = dynamic_cast<const AllocatorAStarNode*>(&aneighbor);
    return ( this->x == neighbor->x && this->y == neighbor->y );
    //float distanceCost = (neighbor->pos - this->pos).len();
    //n_printf("distanceCost:%f\n", distanceCost);
    //return distanceCost < 10.0f;
}
/**-----------------------------------------------------------------------------//
 * figure out howm much it will cost to get from here to a neighbor
 * this cost should include actual travel time, fuel costs as well as undesireable behavior penalties
 * including: going near other boats, particularily in front of them, as well as going near land
 --------------------------------------------------------------------------------*/
float AllocatorAStarNode::CalculateCostTo( AStarNode* aneighbor ) {
    AllocatorAStarNode* neighbor = dynamic_cast<AllocatorAStarNode*>(aneighbor);
    //AllocatorAStarNode* neighbor = (AllocatorAStarNode*)(aneighbor);
    float distanceCost = (neighbor->pos - this->pos).len();
    nAllocatorAStar* allocatorAStarServer = dynamic_cast<nAllocatorAStar*>(this->server);
    n_assert( allocatorAStarServer  );
    nIM* riskynessIM = allocatorAStarServer->getRiskynessIM();
    n_assert( riskynessIM && "AllocatorAStarNode::CalculateCostTo failed to find riskynessIM");
    float riskyness = riskynessIM->getVal( neighbor->x, neighbor->y );
    float cost = distanceCost * ( 1.0 - riskyness );
    //n_printf("CalculateCostTo(%s,%s) = %f + %f = %f\n", str(this), str(neighbor), distanceCost, riskyness, cost );
    return cost;
}
/* -----------------------------------------------------------------------------//
 * calcaulte distance between two nodes taht are possibly not neighbors -> required to do smoothing
 * to do this we can walk the length of the thing and figure it out
 * okay so how do we do this? well we walk the thing every 1 unit, and check where it lies witihn the 4 points around it and do a weighted average
--------------------------------------------------------------------------------*/
float AllocatorAStarNode::CalculateLongCostTo( AStarNode * to) { 
    //AllocatorAStarNode* to = dynamic_cast<AllocatorAStarNode*>(ato);
    nAllocatorAStar* allocatorAStarServer = dynamic_cast<nAllocatorAStar*>(this->server);
    n_assert( allocatorAStarServer  );
    nIM* riskynessIM = allocatorAStarServer->getRiskynessIM();
    n_assert( riskynessIM && "AllocatorAStarNode::CalculateLongCostTo failed to find riskynessIM");

    vector3 dx = to->pos - this->pos;
    //printf("CalculateLongCostTo(%s->%s)\n", str(this->pos), str(to->pos));
    int numSegments                     = (int) ceil( dx.len() );
    dx /= numSegments;
    vector3 p                           = this->pos;
    float totalCost                     = 0.0f;
    for( int i = 1; i < numSegments+1; i ++ ) {
        //printf("p:%s\n", str(p) );
        p += dx;
        totalCost += dx.len() * riskynessIM->getVal( p );
    }
    return totalCost;

} 
/**-----------------------------------------------------------------------------//
 * give a rough estimate of how close this state is to some other state, usually the goal state
 * just for ageneral idea which way is towards the goal
--------------------------------------------------------------------------------*/
float AllocatorAStarNode::EstimateCostTo( AStarNode* anode ) {
    AllocatorAStarNode* node = dynamic_cast<AllocatorAStarNode*>(anode);
    n_assert( node );

    float distanceCost = (node->pos - this->pos).len();
    //n_printf("EstimateCost(%s,%s)=%f\n", str(this), str(anode), distanceCost );
    return distanceCost;
}
//-----------------------------------------------------------------------------//
/**
 * draw me to the screen -> I only have the gfx server pointer so this will be a lil crude
 **/
//-----------------------------------------------------------------------------//
void AllocatorAStarNode::Display( nGfxServer* gfx ){
    gfx->Begin( N_PTYPE_LINE_LIST );
    if( this->parent ) {
        gfx->Rgba( 0,0,1,1 );
        gfx->Coord( pos );
        gfx->Rgba( .1,.1,1,1 );
        gfx->Coord( ((AllocatorAStarNode*) (parent))->pos );
    }
    gfx->End();
}

/* -----------------------------------------------------------------------------//
 * find my neighbors
 * for this all we need to do is generate new nodes with the x,y values
--------------------------------------------------------------------------------*/
nArray<AStarNode*> AllocatorAStarNode::GetNeighbors(){
    nArray<AStarNode*> neighbors;
    nAllocatorAStar* allocatorAStarServer = dynamic_cast<nAllocatorAStar*>(this->server);
    n_assert( allocatorAStarServer  );
    nIM* riskynessIM = allocatorAStarServer->getRiskynessIM();
    for( int dx = -1; dx <= 1; dx ++ ) {
        for( int dy = -1; dy <= 1; dy ++) {
            if ( dx == 0 && dy == 0 )  {
                continue;
            } else {
                int nx = this->x + dx;
                int ny = this->y + dy;
                if ( nx >= 0 && nx < riskynessIM->getResolution()) {
                    if ( ny >= 0 && ny < riskynessIM->getResolution()) {
                        AllocatorAStarNode* node;
                        if ( allocatorAStarServer->cachedNodes[nx][ny] ) {
                            node = allocatorAStarServer->cachedNodes[nx][ny];
                        } else {
                            node = new AllocatorAStarNode(this->server);
                            node->Set( riskynessIM->getPoint(nx, ny));
                            server->AppendNode( node );
                            allocatorAStarServer->cachedNodes[nx][ny] = node;
                        }
                        neighbors.Append(node);
                    }
                }
            }
        }
    }
    return neighbors;
}
