#define N_IMPLEMENTS nBoatMoveTo
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/nboatmoveto.h"
#include "im/nallocatorastar.h"

nNebulaScriptClass(nBoatMoveTo, "nboatai");

//------------------------------------------------------------------------------
nBoatMoveTo::nBoatMoveTo():
    astar( kernelServer, this ) {
    this->path                           = 0;
}
//------------------------------------------------------------------------------
void nBoatMoveTo::Initialize(){
    nBoatAI::Initialize();
    this->AddToDisplayList();
}
//------------------------------------------------------------------------------
void nBoatMoveTo::InitVars(){
    nBoatAI::InitVars();
    destination.Connect                 ( ent, "destination");
    destination.DefaultV                ( vector3(0,0,0));
    subDestination.Connect              ( ent, "subDestination");
    subDestination.DefaultV             ( vector3(0,0,0));
    astarSearchFrequency.Connect        ( ent, "astarSearchFrequency");
    astarSearchFrequency.DefaultF       ( 30.0f );
    lastAStarSearch.Connect             ( ent, "lastAStarSearch" );
    lastAStarSearch.DefaultF            ( randF( 0, astarSearchFrequency.GetF() ));
    pathIndex.Connect                   ( ent, "pathIndex" );
    pathIndex.DefaultI                  ( 0 );
    //astar                               = nString(ent->GetVarRef("side")) + "/allocatorastar";
    astar                               = "/game/sides/blue/allocatorastar";
    maxDistanceToNextPoint.Connect       ( ent, "maxDistanceToNextPoint" );
    maxDistanceToNextPoint.DefaultF      ( 1000.0f );
    maxSpeed.Connect                    ( ent, "maxSpeed" );
    maxSpeed.DefaultF                   ( 1.0f );
}
//------------------------------------------------------------------------------
nBoatMoveTo::~nBoatMoveTo(){
}
//------------------------------------------------------------------------------
void nBoatMoveTo::Tick( float time, float dtime ){
    nBoatAI::Tick( time, dtime );
    vector3 p = pos.GetV();
    vector3 d = destination.GetV();
    if( d.len() < 1.0f )                //destinaton = 0,0,0 -> stop
        return;
    float distanceToGoal = (p-d).len();
    if ( distanceToGoal < 200.0f ) {
        destination.SetV            (0,0,0);
        desiredSpeed.SetF           (0.0f);
        return;
    }
    if( astar.isvalid() ) {
        if ( (destination.isdirty() && destination.GetV().len() > 1.0f ) || ( time - lastAStarSearch.GetF() > astarSearchFrequency.GetF()) ){ //if we need to rerun our astar search
            if ( path )
                delete path;
            path                        = astar->FindPath( p, d );
            //n_printf("ent(%s).FindPath(%s->%s)=%s\n", str(this), str(p), str(d), str(path));
            pathIndex.SetI              ( 0 );
            lastAStarSearch.SetF( time );
        }
        int pi                          = pathIndex.GetI();
        if( pi == path->nodes.Size()) {  //out of points to traverse
            d.set                       (0,0,0);
            destination.SetV            (0,0,0);
            desiredSpeed.SetF           (0.0f);
        }
        else {
            vector3 dx                  = path->nodes[pi]->pos - p;
            float len                   = dx.len();
            if ( pi == path->nodes.Size() - 1 )
                d                       = path->nodes[-1]->pos;
            else {
                if( len < 200.0f ) {
                    pi                 += 1;
                    pathIndex.SetI      ( pi );
                    maxDistanceToNextPoint.SetF( (path->nodes[pi]->pos - p).len() );
                }
                float distanceToNextPoint = (path->nodes[pi]->pos - p).len();
                float ipolRatio         = n_clamp(distanceToNextPoint / maxDistanceToNextPoint.GetF(), 0.0f, 0.9f);
                if( pi == path->nodes.Size() -1 )
                    subDestination.SetV ( path->nodes[pi]->pos );
                else {
                    //printf("ipolRatio:%f\n", ipolRatio);
                    subDestination.SetV ( path->nodes[pi]->pos * ipolRatio + path->nodes[pi+1]->pos * (1.0f - ipolRatio));
                }
                d                       = subDestination.GetV();
            }
        }
    }
    else
        n_printf("%s.astar(%s) is invalid\n", str(this), astar.getname());

    vector3 dx = d - p;
    dx.y = 0.0f;
    float dist = dx.len();
    float heading = atan2(dx.x, dx.z);
    desiredHeading.SetF(heading);
    desiredSpeed.SetF(maxSpeed.GetF() );
    if( ent->Selected() )
        this->DrawLine( p, d, white );
}
//-----------------------------------------------------------------------------//
void nBoatMoveTo::Display( nGfxServer* gfx ){
    nBoatAI::Display( gfx );
    //if ( this->path )
        //this->path->Display( gfx );
}
//-----------------------------------------------------------------------------//
