#define N_IMPLEMENTS nOilBeaconAspect
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/noilbeaconaspect.h"
#include "enkiduo/ngameserver.h"
#include "enkiduo/colours.h"

nNebulaScriptClass(nOilBeaconAspect, "naspect");

//------------------------------------------------------------------------------
nOilBeaconAspect::nOilBeaconAspect():
    refGameServer( kernelServer, this ) {
    buoyPlacementThetaOffset = randF( 0, N_PI * 2.0f );

}
//------------------------------------------------------------------------------
void nOilBeaconAspect::Initialize(){
    nAspect::Initialize();
    this->AddToDisplayList();
    refGameServer = "/sys/servers/game";
}
//------------------------------------------------------------------------------
void nOilBeaconAspect::InitVars(){
    nAspect::InitVars();
    pos.Connect                         ( ent, "pos" );
    maxBuoys.Connect                    ( ent, "maxBuoys" );
    maxBuoys.DefaultI                   ( 5 );
    captureDistanceThreshold.Connect    ( ent, "captureDistanceThreshold" );
    captureDistanceThreshold.DefaultF   ( 250.0f );
    timeSinceLastScan.Connect           ( ent, "timeSinceLastScan" );
    timeSinceLastScan.DefaultF          ( -1.0f );
    timeBetweenScans.Connect            ( ent, "timeBetweenScans" );
    timeBetweenScans.DefaultF           ( 5.0f );
    sideCapturedBy.Connect              ( ent, "sideCapturedBy" );
    sideCapturedBy.DefaultS             ( "/game/sides/neutral" );
    buoyPlacementThetaRadius.Connect    ( ent, "buoyPlacementThetaRadius");
    buoyPlacementThetaRadius.DefaultF   ( 50.0f );
    buoyPlacementRandomization.Connect  ( ent, "buoyPlacementRandomization");
    buoyPlacementRandomization.DefaultF ( 2 );
    captureR.Connect                    ( ent, "captureR" );
    captureR.DefaultF                   ( 0.0f );
    side.Connect                        ( ent, "side" );
}
//------------------------------------------------------------------------------
nOilBeaconAspect::~nOilBeaconAspect(){
}
//------------------------------------------------------------------------------
//-----------------------------------------------------------------------------
/* -----------------------------------------------------------------------------//
 * the oilbeacon aspect occasionally scans for nearby units
 * if it finds any then it should start being captured
 * if it rescans and only finds units of a particular side, and it is short a buoy it creates a new one
 * the buoys float around near the oilslick and produces resources
--------------------------------------------------------------------------------*/
void nOilBeaconAspect::Tick( float time, float dtime ){
    //n_printf("nOilBeaconAspect::Tick(%f,%f)\n", time, dtime );
    nAspect::Tick                 ( time, dtime );
    float progressTowardsNextBuoy = ( time - timeSinceLastScan.GetF() ) / timeBetweenScans.GetF();
    float newCaptureRatio         = ( (buoys.Size() + progressTowardsNextBuoy) / float(maxBuoys.GetI()));
    newCaptureRatio               = n_clamp( newCaptureRatio, 0.0f, 1.0f );
    captureR.SetF                 ( newCaptureRatio );
    if ( (timeSinceLastScan.GetF() < 0 && roll(.01)) || (time - timeSinceLastScan.GetF() > timeBetweenScans.GetF()) ){
        //n_printf("nOilBeaconAspect(%s).scanning for nearby units\n", str(this));
        timeSinceLastScan.SetF( time );
        vector3 p                       = pos.GetV();
        nRoot* entRoots                 = LOOKUP(nRoot*, "/game/ents");
        //if we can add another buoy
        if( buoys.Size() < maxBuoys.GetI() ) {
            foreachchild(nThing, entType, entRoots){
                foreachchild(nEnt, otherEnt, entType){
                    if( nString("/game/sides/neutral") == otherEnt->GetVarRef("side") ) {
                        //printf("skipping ent:%s its neutral\n", str(ent));
                        continue;
                    }
                    vector3 entPos = otherEnt->GetVarV("pos");
                    float dist = (entPos - p).len();
                    //n_printf("ent:%s entPos;%s dist:%f\n", str(ent), str(entPos), dist);
                    if( dist < captureDistanceThreshold.GetF()){
                        //n_printf("ent(%s) within range\n", str(ent));
                        this->DrawLine( p, entPos, white );
                        //create a new buoy
                        if( refGameServer.isvalid() == false ) {
                            n_printf("nOilBeaconAspect(%s) failed to find gameserver at :%s\n", str(this), refGameServer.getname() );
                            return;
                        }

                        nEnt* buoy                  = refGameServer->GenEnt("buoy");
                        buoy->SetVarRef             ( "side", otherEnt->GetVarRef("side"));
                        //n_printf                  ( "nOilBeaconAspect %s created buoy %s\n", str(this), str(buoy) );
                        float dtheta                = PI2 / maxBuoys.GetI();                   //theta between buoys
                        float theta                 = dtheta * buoys.Size() + buoyPlacementThetaOffset;
                        vector3 dp                  ( sin(theta), 0, cos(theta));                  //normalized direction of placement
                        dp                          = dp * buoyPlacementThetaRadius.GetF();              //offset from my position
                        float r                     = buoyPlacementRandomization.GetF();
                        dp                         += vector3(randF(-r,r), 0, randF(-r,r));
                        vector3 buoyPos             = p + dp ;
                        buoy->SetVarV               ( "pos", buoyPos);
                        nDynAutoRef<nEnt> & refBuoy = buoys.Push();
                        refBuoy.initialize          ( kernelServer, this);
                        refBuoy                     = buoy;

                        if( buoys.Size() == maxBuoys.GetI() )
                            side.SetRef( otherEnt->GetVarRef("side" ) );
                        return;
                    }
                }
            }
        }
    }
}
