#define N_IMPLEMENTS nPathfindMap
//------------------------------------------------------------------------------
//  (C) 2006	Ryan
//------------------------------------------------------------------------------
#include "lagoon/npathfindmap.h"

nNebulaScriptClass(nPathfindMap, "naspect");

//------------------------------------------------------------------------------
nPathfindMap::nPathfindMap() : terrainMap(kernelServer, this),
			       ref_eventServer(kernelServer, this),
			       zones(10, 5)

{
}
//------------------------------------------------------------------------------
void nPathfindMap::Initialize(){
    nAspect::Initialize();
    this->map = 0;
    this->workingMap = 0;
    this->res = 0;
    this->working = false;
    this->zonenum = 0;
    this->currentzone = 0;
}
//------------------------------------------------------------------------------
void nPathfindMap::InitVars(){
    nAspect::InitVars();
    worldMin.Connect(ent, "worldMin");
    worldMax.Connect(ent, "worldMax");
    nEnt* mapEnt = LOOKUP(nEnt*, "/map");
    worldSize.Connect(mapEnt, "worldsize");
    resolution.Connect(ent, "resolution");
    
    worldMin.DefaultV(0,0,0);
    worldMax.DefaultV(1000,0,1000);
    worldMin.SetV(0,0,0);
    worldMax.SetV(worldSize.GetF(), 0, worldSize.GetF());

    resolution.DefaultI(256);
}
//------------------------------------------------------------------------------
nPathfindMap::~nPathfindMap(){
}
//------------------------------------------------------------------------------
void nPathfindMap::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    if(currentzone++ >= zones.Size()) {
      currentzone = 0;
      //swap grids
      //clear old grid
      //if resolution is dirty
      //  reallocate working map
      //  calcPoints
    }
}
//------------------------------------------------------------------------------
template <class T>
bool deallocArray(T**& arr, int width, int height){
  if(arr == 0)
    return false;
  n_assert(width > 0 && height > 0);
  for(int i = 0; i < width; i++)
    delete [] arr[i];
  delete[] arr;
  arr = 0;
  return true;
}
//------------------------------------------------------------------------------
template <class T>
bool allocArray(T**& arr, int width, int height, T* defaultValue = NULL) {
  n_assert(arr==0);
  n_assert(width > 0 && height > 0);
  arr = new T*[width];
  n_assert(arr && "cannot allocate array");
  for(int i = 0; i < width; i++) {
    arr[i] = new T[height];
    n_assert( arr[i] && "cannot allocate subarray" );
  }
  if(defaultValue)
    for(int i = 0; i < width; i++)
      for(int j = 0; j < height; j++)
	arr[i][j] = *defaultValue;
  return true;
}
//------------------------------------------------------------------------------
void nPathfindMap::deallocArrays() {
  deallocArray(map, res, res);
  deallocArray(workingMap, res, res);
  deallocArray(points, res+1, res+1);
}
//------------------------------------------------------------------------------
void nPathfindMap::allocArrays() {
  static float defaultVal = 0.0f;
  static vector3 defaultPoint(0,0,0);
  allocArray(map, res, res, &defaultVal);
  allocArray(workingMap, res, res, &defaultVal);
  allocArray(points, res+1, res+1, &defaultPoint);
}
//------------------------------------------------------------------------------
void nPathfindMap::calcPoints(){
  vector3 min = worldMin.GetV();
  const float height = 0.1f;
  this->squareSize = (worldMax.GetV() - worldMin.GetV()) / float(resolution.GetI());
  for(int x = 0; x <= res; x++)
    for(int z = 0; z <= res; z++)
      points[x][z] = vector3(min.x + squareSize.x * x, height, min.z + squareSize.z * z);

  float h = 0.0f;
  ifvalid(terrainMap) {
    for(int x = 0; x <= res; x++)
      for(int z = 0; z <= res; z++) {
	h = terrainMap->GetHeight(points[x][z].x, points[x][z].z);
	if(h > 0.0f)
	  points[x][z].y = 1.0f;
      }
  }
}
//------------------------------------------------------------------------------
float nPathfindMap::getVal(vector3 v) {
  int xi, yi;
  getCoord(v, xi, yi);
  float rx = v.x - points[xi][yi].x;
  float ry = v.z - points[xi][yi].z;
  float val;
  val += map[xi+0][yi+0] * (1.0f-rx) * (1.0f-ry);
  val += map[xi+1][yi+0] * (     rx) * (1.0f-ry);
  val += map[xi+0][yi+1] * (1.0f-rx) * (     ry);
  val += map[xi+1][yi+1] * (     rx) * (     ry);
  return val;
}
//------------------------------------------------------------------------------
void nPathfindMap::getCoord( vector3 point, int& x, int& y ){
    vector3 min = worldMin.GetV();
    vector3 max = worldMax.GetV();
    this->squareSize = (worldMax.GetV() - worldMin.GetV()) / float(resolution.GetI());
    x = (point.x - min.x) / squareSize.x;
    y = (point.z - min.z) / squareSize.z;
    n_assert( x >= 0 && x < res && y >= 0 && y < res && "got coord for some value which was invalid" );
}
//------------------------------------------------------------------------------
void nPathfindMap::calcMap(){
  if( this->map ) {
    this->calculateValues();
  }
  this->working = false;
}
//------------------------------------------------------------------------------
void nPathfindMap::calculateValues() {
  this->Tick(0,0);
  for(int x = 0; x < res; x++) {
    for(int y = 0; y < res; y++) {
      ; //Cycle through all the zones and combine
    }
  }
}
//------------------------------------------------------------------------------
int nPathfindMap::registerZone(nPathfindZone* zone) {
  zones.Add(zonenum++, zone);
  return zonenum;
}
//------------------------------------------------------------------------------
bool nPathfindMap::turn(int zoneID) {
  return (zoneID == zones.GetKeyAt(currentzone));
}
//------------------------------------------------------------------------------
int nPathfindMap::numZones() {
  return zones.Size();
}
//------------------------------------------------------------------------------
void nPathfindMap::setZone(nPathfindZone* zone, int x, int y) {
  float zoneval, mapval;
  int w = zone->rw/2;
  int h = zone->rh/2;
  int sx = x - w;
  int sy = y - h;
  int ex = x + w;
  int ey = y + h;
  
  vector3 min = worldMin.GetV();
  vector3 max = worldMax.GetV();
  sx = ((sx <= min.x) ? sx : min.x);
  sy = ((sy <= min.z) ? sy : min.z);
  ex = ((ex >= max.x) ? ex : max.x);
  ey = ((ey >= max.z) ? ey : max.z);
  
  for(int i = sx; i < ex; i++) {
    for(int j = sy; j < ey; j++) {
      zoneval = zone->grid[i][j];
      mapval = workingMap[i][j];
      workingMap[i][j] = ((zoneval > mapval) ? zoneval : mapval);
    }
  }
}
//------------------------------------------------------------------------------
void nPathfindMap::unregisterZone(int zoneID) {
  zones.Rem(zoneID);
}
