#define N_IMPLEMENTS nPathfindZone
//------------------------------------------------------------------------------
//  (C) 2006	Ryan
//------------------------------------------------------------------------------
#include "lagoon/npathfindzone.h"
#include <math.h>
#include "lagoon/npathfindmap.h"

nNebulaScriptClass(nPathfindZone, "naspect");

//------------------------------------------------------------------------------
nPathfindZone::nPathfindZone() {
}
//------------------------------------------------------------------------------
void nPathfindZone::Initialize(){
  nAspect::Initialize();
  //find the ent and get x, y, heading
  //find the map
  lastangle = 0.0f;
  grid = 0;
  baseGrid = 0;
  maxradius = 150.0f;
  maxdiag = maxradius * 1.414f;
  gridID = map->registerZone(this);
  makeBase();
}
//------------------------------------------------------------------------------
void nPathfindZone::InitVars(){
    nAspect::InitVars();
}
//------------------------------------------------------------------------------
nPathfindZone::~nPathfindZone(){
  deallocArrays();
  map->unregisterZone(gridID);
}
//------------------------------------------------------------------------------
void nPathfindZone::Tick( float time, float dtime ){
  nAspect::Tick( time, dtime );
  //update x, y, heading
  if(squareSize.x != map->squareSize.x or squareSize.z != map->squareSize.x) {
    makeBase();
  }
  if(map->turn(gridID)) {
    //need to convert posx and posy to scaled units
    map->setZone(this, posx, posy);
  } else {
    rotate(heading);
  }
}
//------------------------------------------------------------------------------
void nPathfindZone::makeBase() {
  deallocArrays();
  squareSize = map->squareSize;
  w = (int)(maxradius/squareSize.x);
  h = (int)(maxradius/squareSize.z);
  rw = (int)(maxradius/squareSize.x);
  rh = (int)(maxradius/squareSize.z);
  allocArrays();
  rotate(lastangle);
}
//------------------------------------------------------------------------------
void nPathfindZone::rotate(float angle) {
  lastangle = angle;
}
//------------------------------------------------------------------------------
template <class T>
bool deallocArray(T**& arr, int width, int height){
  if(arr == 0)
    return false;
  n_assert(width > 0 && height > 0);
  for(int i = 0; i < width; i++)
    delete [] arr[i];
  delete[] arr;
  arr = 0;
  return true;
}
//------------------------------------------------------------------------------
template <class T>
bool allocArray(T**& arr, int width, int height, T* defaultValue = NULL) {
  n_assert(arr==0);
  n_assert(width > 0 && height > 0);
  arr = new T*[width];
  n_assert(arr && "cannot allocate array");
  for(int i = 0; i < width; i++) {
    arr[i] = new T[height];
    n_assert( arr[i] && "cannot allocate subarray" );
  }
  if(defaultValue)
    for(int i = 0; i < width; i++)
      for(int j = 0; j < height; j++)
	arr[i][j] = *defaultValue;
  return true;
}
//------------------------------------------------------------------------------
void nPathfindZone::allocArrays() {
  static float defaultVal = 0.0f;
  allocArray(grid, w, h, &defaultVal);
  allocArray(baseGrid, rw, rh, &defaultVal);
}

void nPathfindZone::deallocArrays() {
  deallocArray(grid, w, h);
  deallocArray(grid, rw, rh);
}
