#define N_IMPLEMENTS nRadarServer
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/nradarserver.h"
#include "lagoon/nboatradarsensor.h"
#include "aspect/nside.h"

nNebulaScriptClass(nRadarServer, "nthing");

//------------------------------------------------------------------------------
nRadarServer::nRadarServer() {
    nextRadarToSweep = 0;
}
//------------------------------------------------------------------------------
void nRadarServer::Initialize(){
    nThing::Initialize();
    //this->AddToDisplayList();
}
//------------------------------------------------------------------------------
nRadarServer::~nRadarServer(){
}
//------------------------------------------------------------------------------
void nRadarServer::Tick( float time, float dtime ){
    nThing::Tick( time, dtime );
    if( radars.Size() == 0 ) 
        return;
    nextRadarToSweep %= radars.Size();
    nBoatRadarSensor* radar = radars[nextRadarToSweep];
    nEnt* radarEnt = radar->GetEnt();
    int fromSideHandle = radarEnt->GetSideHandle();
    vector3 p = radar->GetEnt()->GetVarV("pos");

    nEnt* sideEnt = (nEnt*) kernelServer->Lookup( radarEnt->GetVarRef("side") );
    n_assert( sideEnt );
    //n_printf("sideEnt:%s\n", str(sideEnt) );
    nSide* sidePtr = (nSide*) sideEnt->Find("side");                            n_assert( sidePtr );
    const SideArray* hostiles = sidePtr->GetRelatedSides(HOSTILE);              n_assert( hostiles );
    nRoot* entTypes = kernelServer->Lookup("/game/ents");
    foreachta( nSide*, hs, hostiles ) {
        const EntArray* units = hs->GetUnits();
        foreachta( nEnt*, toEnt, units ) {
            if( toEnt->IsAlive() && toEnt->Find("stealthyness") ) {
                vector3 tp = toEnt->GetVarV("pos");
                vector3 tdx = tp - p;
                float toDist = tdx.len();
                float targetTheta = atan2( tdx.x, tdx.z );
                bool detected = true;
                //n_printf("radarEnt:%s toEnt:%s\n", str(radarEnt), str(toEnt));
                if ( toDist < radarEnt->GetVarF("radarRange")) { //target is in range
                    foreachchild( nThing, entType, entTypes ) {
                        foreachchild( nEnt, middleEnt, entType ) {
                            if( middleEnt != radarEnt && middleEnt != toEnt ) {
                                if( middleEnt->GetSelectable()){
                                    vector3 mep = middleEnt->GetVarV("pos");
                                    quaternion mer = middleEnt->GetVarQ("rot");
                                    vector3 medx = mep - p;
                                    float meDist = medx.len();
                                    if ( meDist < toDist ) {
                                        if ( middleEnt->HasVar("dimension") ) {
                                            float minTheta = 0;
                                            float maxTheta = 0;
                                            bool first = true;
                                            vector3 dimensions = middleEnt->GetVarV("dimension");
                                            float theta;
                                            vector3 dx;
                                            for( int k = 0; k < 4; k ++ ) {
                                                vector3 point = vector3(dimensions.x * (k / 2 == 0 ? 1:-1), 0, dimensions.z * (k % 2 == 0 ? 1 : -1));
                                                point = mep + mer.rotate( point );
                                                dx = point - p;
                                                theta = atan2( dx.x, dx.z );
                                                if ( first || theta < minTheta ) {
                                                    minTheta = theta;
                                                }
                                                if ( first || theta > maxTheta ) { 
                                                    maxTheta = theta;
                                                }
                                                first = false;
                                            }
                                            //this->DrawAngle( p, minTheta, green, radarEnt->GetVarF("radarRange"));
                                            //this->DrawAngle( p, maxTheta, red,   radarEnt->GetVarF("radarRange"));
                                        }
                                        float meTheta = atan2( medx.x, medx.z );
                                        //this->DrawAngle( p, meTheta - .1f, blue );
                                        //this->DrawAngle( p, meTheta,       green );
                                        //this->DrawAngle( p, meTheta + .1f, blue );
                                        float dtheta = fabs(meTheta - targetTheta);
                                        if ( dtheta > 180 )
                                            dtheta -= 360.0f;
                                        dtheta = fabs( dtheta );
                                        if ( dtheta < .1f )
                                            detected = false;
                                    }
                                }
                            }
                        }
                    }
                }
                else
                    detected = false;
                if ( detected && radar->CanDetect( toEnt ) == false) {
                    radar->DetectedEntity( toEnt );
                    toEnt->SetVarI("exposed", toEnt->GetVarI("exposed") + 1);
                }
                if ( detected == false && radar->CanDetect( toEnt ) == true) {
                    radar->EscapedEntity( toEnt );
                    toEnt->SetVarI("exposed", toEnt->GetVarI("exposed") - 1);
                }
                //this->DrawLine( p, tp, detected ? white:black, 5);
            }
        }
    }
    nextRadarToSweep++;
}
