#define N_IMPLEMENTS nTankerAI
//------------------------------------------------------------------------------
//  (C) 2006	Chris Miles
//------------------------------------------------------------------------------
#include "lagoon/ntankerai.h"
#include "util/nheap.h"
#include "lagoon/nboatmoveto.h"

nNebulaScriptClass(nTankerAI, "nfsm");

//------------------------------------------------------------------------------
nTankerAI::nTankerAI():
    ref_moveto( kernelServer, this )
{
}
//------------------------------------------------------------------------------
void nTankerAI::Initialize(){
    nFSM::Initialize();

    pos.Connect                         ( ent, "pos" );
    nextOilBeacon.Connect               ( ent, "nextOilBeacon");
    nextOilBeacon.DefaultRef            ( "" );
    oilBeaconPos.Connect                ( nextOilBeacon, "pos" );
    destination.Connect                 ( ent, "destination" );
    maxSpeed.Connect                    ( ent, "maxSpeed" );
    desiredSpeed.Connect                ( ent, "desiredSpeed" );
    desiredHeading.Connect              ( ent, "desiredHeading" );
    stateNames.Append                   ("init");
    stateNames.Append                   ("findDestination");
    stateNames.Append                   ("followPath");
    stateNames.Append                   ("passBoat");
    stateNames.Append                   ("dock");
    stateNames.Append                   ("wait");
    this->AddToDisplayList();

    ref_moveto                          = ent->ChildPath( "moveto" );
    if ( ref_moveto.isvalid() == false )
        ent->CreateChild("nboatmoveto", "moveto");
    n_assert( ref_moveto.isvalid() );
    this->clearLines = false;
}
//------------------------------------------------------------------------------
void nTankerAI::InitVars(){
    nFSM::InitVars();
}
//------------------------------------------------------------------------------
nTankerAI::~nTankerAI(){
}
//------------------------------------------------------------------------------
void nTankerAI::Tick( float time, float dtime ){
    nFSM::Tick( time, dtime );
}

bool nTankerAI::States( StateMachineEvent event, uint state, float time, float dtime ){
    BeginStateMachine
        State(Init)                     //hold position
            OnEnter
                this->SetState(FindDestination);
        State(FindDestination)
            OnTick
                nHeap<nEnt*> possibleDestinations(1024);
                nRoot* oilBeacons = LOOKUP(nRoot*, "/game/ents/oilbeacon");
                if ( oilBeacons == 0 ) {
                    n_printf("failed to find oilbeacons\n");
                    return true;
                }
                foreachchild( nEnt, oilbeacon, oilBeacons ){
                    vector3 obPos = oilbeacon->GetVarV("pos");
                    vector3 dx    = obPos - pos.GetV();
                    float d       = dx.len();
                    possibleDestinations.add( -d, oilbeacon );
                }
                nEnt* beacon = possibleDestinations.pop();
                n_printf("%s.nextOilBeacon:%s\n", str(this), str(beacon));
                nextOilBeacon.SetRef( beacon );
                this->SetState(FollowPath);
        State(FollowPath)
            OnEnter
                n_printf("%s.FollowPath\n", str(this));
                ifvalid( nextOilBeacon ) {
                    destination.SetV( oilBeaconPos.GetV() );
                }
            OnTick
                //just pass our destination on to nboatmoveto
                bool check = roll(.01);
                if ( check )
                    this->linesToDraw.Clear();
                //if we have reached our destination find a new one
                if( (oilBeaconPos.GetV() - pos.GetV()).len() < 200.0f )
                    this->SetState( FindDestination );
                /**
                 * okay so now we need to search for other boats that could be in our way, and enter into pass mode with them
                 * if they have a moveto or a tankerai then see if we are passing close to them
                 * how do we check that? well we can do some kind of spherical test or something based upon their course
                 * okay so we walk the path both of us have created
                 * if points come to close, then we can enter pass mode
                 * okay so check if the points in the paths match in reverse order
                 * lets just check the next 10 / 20 points
                 */
                bool firstRightMostHeading = true;
                float rightmostHeading = 0;
                nRoot* ents = LOOKUP(nRoot*, "/game/ents");
                if( check ){
                    foreachchild( nRoot, entType, ents ){
                        foreachchild( nEnt, ent, entType ){
                            if( ent == this->ent ) 
                                continue;
                            if( ent->Find("moveto") == 0 || ent->GetVarV("destination").len() < 1.0f ) 
                                continue;
                            vector3 p = pos.GetV();
                            vector3 op = ent->GetVarV("pos");
                            //this->DrawLine( pos.GetV(), ent->GetVarV("pos"), green );
                            //n_printf("nEnt(%s) considering avoiding (%s)\n", str(this), str(ent));
                            const AStarPath* myPath    = ref_moveto->GetPath();
                            const AStarPath* otherPath = ((nBoatMoveTo*)ent->Find("moveto"))->GetPath();
                            //n_printf("myPath:\n%s\notherPath:\n%s\n", str(*myPath), str(*otherPath));
                            int nodesToLookAhead = 10;
                            if( myPath->nodes.Size() > nodesToLookAhead && otherPath->nodes.Size() > nodesToLookAhead ) {
                                vector3 myHeading    = myPath->nodes[nodesToLookAhead]->pos;
                                vector3 otherHeading = otherPath->nodes[nodesToLookAhead]->pos;
                                myHeading    = myHeading - p;
                                otherHeading = otherHeading - op;
                                myHeading.norm();
                                otherHeading.norm();
                                vector3 dp = op - p;
                                float l = dp.len();
                                dp.norm();
                                if ( dp % myHeading > .90 ) { //roughly in front of me consider passing it
                                    float dotProduct = myHeading % otherHeading;
                                    //n_printf("%s %% %s = %f\n", str(myHeading), str(otherHeading), dotProduct);
                                    if( dotProduct < -.90 ) { //heading the other way
                                        this->DrawLine( p, op, red );
                                        float headingToPass = atan2(dp.x, dp.z);
                                        while ( headingToPass < 0 )
                                            headingToPass += N_PI * 2.0f;
                                        if ( firstRightMostHeading || headingToPass < rightmostHeading ) {
                                            rightmostHeading = headingToPass;
                                            firstRightMostHeading = false;
                                        }
                                    }
                                    if( dotProduct > .7f ) {
                                        this->DrawLine( p, op, blue );
                                    }
                                }
                            }
                        }
                    }
                    /*
                    if ( firstRightMostHeading == false ) {
                        destination.SetV( 0,0,0 );
                        desiredHeading.SetF( rightmostHeading - .15f );
                        n_printf("desiredHeading to pass:%f\n", desiredHeading.GetF() );
                        this->DrawAngle( pos.GetV(), desiredHeading.GetF(), green );
                        desiredSpeed.SetF( maxSpeed.GetF() );
                    } else if( nextOilBeacon.isvalid() && destination.GetV().len() < 1.0f ) {
                        destination.SetV( oilBeaconPos.GetV() );
                    }
                    */
                }
                //desiredHeading.SetF( desiredHeading.GetF() + .3f );
    EndStateMachine
}
