#define N_IMPLEMENTS nInfluenceMap
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "map/ninfluencemap.h"
#include "map/nmap.h"

nNebulaScriptClass(nInfluenceMap, "nthing");

//------------------------------------------------------------------------------
/**
*/
nInfluenceMap::nInfluenceMap():
    map( kernelServer, this ),
    coef( 1.0f ),
    size( 0 ),
    minVal(0.0f),
    maxVal(0.0f),
    A(0.0f),
    B(0.0f),
    C(0.0f)
{
}

void nInfluenceMap::Initialize(){
    nThing::Initialize();
    map = "/map";
    this->AddLogTag("ninfluencemap");
}

nInfluenceMap::~nInfluenceMap(){
}
//------------------------------------------------------------------------------
/**
*/

void nInfluenceMap::Tick( float time, float dtime ){
    nThing::Tick( time, dtime );
}

void nInfluenceMap::SetSize( int s ) { 
    this->size = s; 
    init(grid,0.0f,size,size);
    init(normGrid,0.0f,size,size);

    float maxHeight = 20.0;
    float mapsize = 50000.0;
    squareSize = mapsize / (float)(size-1);
    init(points,vector3(0,0,0),size+1,size+1);
    for( int i = 0; i < size+1; i++ ) {
        for( int j = 0; j < size+1; j ++ ) {
            points[i][j].x = i * squareSize;
            points[i][j].z = j * squareSize;
            if( map.isvalid() )
                points[i][j].y = map->GetHeight( points[i][j].x, points[i][j].z ) + 1.0;
            else
                points[i][j].y = maxHeight;
        }
    }
}
void nInfluenceMap::SetVal( int x, int z, float val) {
    //log(10, "grid[%i][%i] = %f", x, z, val);
    grid[x][z] = val;
}

float nInfluenceMap::GetVal( vector2 v ) const { 
    int x = (int) (v.x / squareSize);
    int z = (int) (v.y / squareSize);
    return GetVal( x,z );
}
float nInfluenceMap::GetVal( int x, int z ) const { 
    if( x < 0 ) 	x = 0;
    if( x >= size ) 	x = size-1;
    if( z < 0 ) 	z = 0;
    if( z >= size ) 	z = size-1;
    return grid[x][z]; 		
}
float nInfluenceMap::GetNormVal( vector2 v ) const { 
    int x = (int)( v.x / squareSize);
    int z = (int)( v.y / squareSize);
    return GetNormVal( x,z );
}
float nInfluenceMap::GetNormVal( int x, int z ) const{ 
    if( x < 0 ) 	x = 0;
    if( x >= size ) 	x = size-1;
    if( z < 0 ) 	z = 0;
    if( z >= size ) 	z = size-1;
    return normGrid[x][z]; 		
}
void nInfluenceMap::Normalize(){
    log("%s->Normalize", this->fn);
    if( size == 0 ) return;
    for( int x = 0; x < size; x ++ ) {
        for( int z = 0; z < size; z ++ ) {
            //normGrid[x][z] = (grid[x][z] - minVal) / range;
            normGrid[x][z] = grid[x][z] * coef;
        }
    }
    this->CalcBest();
}
void nInfluenceMap::CalcBest(){
    minVal = maxVal = normGrid[0][0];
    for( int x = 0; x < size; x ++ ) {
        for( int z = 0; z < size; z ++) {
            if( normGrid[x][z] < minVal ) {
	minVal = normGrid[x][z];
	minPoint.set(x,z);
            }
            if( normGrid[x][z] > maxVal ) {
	maxVal = normGrid[x][z];
	maxPoint.set(x,z);
            }
        }
    }
    float range = maxVal-minVal;
    log("%s->minVal:%f maxVal:%f minp:%.0f,%.0f maxp:%.0f,%.0f range:%f", this->fn, minVal, maxVal, minPoint.x, minPoint.y, maxPoint.x, maxPoint.y, range );
}
void nInfluenceMap::GetColor( float val, nGfxServer* gfx ) {
    if( val >= 0.0f )
        gfx->Rgba( val, val, 0.2, 0.25f ); //goes to yellow
    else if( val < 0.0f && val > -1.0f)
        gfx->Rgba( -val, 1.0f + val, 0, .4f );
    else
        gfx->Rgba( 1.0, 0, 0, .4f );
}
void nInfluenceMap::Display( nGfxServer* gfx ) {
    //n_printf("nInfluenceMap.Display\n");
    int x, y;
    static float lineBrightness = .2f;
    for( y = 0; y < size-1; y ++ ) {
        gfx->Begin( N_PTYPE_TRIANGLE_STRIP );
        for( x = 0; x < size; x ++ ) {
            this->GetColor( normGrid[x][y], gfx );
            gfx->Coord( points[x][y] );
            this->GetColor( normGrid[x][y+1], gfx );
            gfx->Coord( points[x][y+1] );
        }
        gfx->End();
    }
    for( x = 0; x < size; x ++ ) {
        gfx->Begin(N_PTYPE_LINE_STRIP);
        for( y = 0; y < size; y ++ ) {
            gfx->Rgba( normGrid[x][y], normGrid[x][y], 0.2 + lineBrightness, 0.25f );
            gfx->Coord( points[x][y] + vector3(0,.1,0) );
        }
        gfx->End();
    }
    for( y = 0; y < size; y ++ ) {
        gfx->Begin(N_PTYPE_LINE_STRIP);
        for( x = 0; x < size; x ++ ) {
            gfx->Rgba( normGrid[x][y], normGrid[x][y], 0.2 + lineBrightness, 0.25f 		);
            gfx->Coord( points[x][y] + vector3(0,.1,0) );
        }
        gfx->End();
    }
}

void nInfluenceMap::Clear(){
    for( int i = 0; i < size; i ++ )
        for( int j = 0; j < size; j ++ )
            grid[i][j] = 0.0f;
}
    
void nInfluenceMap::Set ( const nInfluenceMap& rhs ) {
    this->size 	= rhs.size;
    this->squareSize 	= rhs.squareSize;
    this->grid 	= rhs.grid;
    this->normGrid 	= rhs.normGrid;
    this->points 	= rhs.points;
    this->minPoint 	= rhs.minPoint;
    this->maxPoint 	= rhs.maxPoint;
    this->maxVal 	= rhs.maxVal;
    this->minVal 	= rhs.minVal;
}
void nInfluenceMap::Add( const nInfluenceMap& rhs ) {
    for( int i = 0; i < size; i ++ )
        for( int j = 0; j < size; j ++ )
            normGrid[i][j] += rhs.GetNormVal( i, j );
    this->CalcBest();
    //this->Normalize();
}
