#define N_IMPLEMENTS nCollisionNode
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/ncollisionnode.h"
#include "collide/ncollideobject.h"
#include "enkiduo/colours.h"

nNebulaScriptClass(nCollisionNode, "nphysicalaspect");

//------------------------------------------------------------------------------
/**
*/
nCollisionNode::nCollisionNode():
    //refVis		( kernelServer, this ),
    gameServer		( kernelServer, this ),
    collideServer	( kernelServer, this )
{
    AddLogTag("collision");
}

void nCollisionNode::Initialize(){
    nPhysicalAspect::Initialize();
    gameServer 		= "/sys/servers/game";
    collideServer 	= "/sys/servers/collide";
    classInitialized 	= false;
    shapeInitialized 	= false;
    collideObject 	= 0;
}

void nCollisionNode::InitVars(){
    nPhysicalAspect::InitVars();
    /*
    nVisNode* vis = ent->GetVis();
    if( vis ) {
        char vis[1024];
        ent->GetVis()->GetFullName( vis, 1024 );
        //const char* vis = ent->GetVis();
        refVis = vis;
    }*/

    pos.	 Connect( ent, "pos" );
    rot.	 Connect( ent, "rot" );
    vel.	 Connect( ent, "vel" );
    mass.	 Connect( ent, "mass" );
    collideClass.Connect( ent, "collideclass" );
    collideClass.DefaultS("default");
    collideShape.Connect( ent, "collideshape" );
    collideShape.DefaultS("default");

    collideObject = gameServer->NewCollideObject(this);
    collideObject->SetClientData( ent );
    //AddToDisplayList();
}

nCollisionNode::~nCollisionNode(){
    if( collideObject ) {
        //log(10, "%s->ReleaseCollideObject", this->fn);
        gameServer->ReleaseCollideObject( collideObject );
    }
}
//------------------------------------------------------------------------------
/**
*/

void nCollisionNode::Tick( float time, float dtime ){
    nPhysicalAspect::Tick( time, dtime );
    collidedLastTick = false;
    if( collideObject == 0 ){
        //log(10, "%s: collideObject == 0", fn);
        return;
    }
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    quaternion qi = q;
    qi.invert();

    if( classInitialized == false || collideClass.isdirty() ){
        classInitialized = true;
        const char* collClass = collideClass.GetS().Get();
        //log(10, "%s->Initializing class = %s", this->fn, collClass );
        collideObject->SetCollClass( collideServer->QueryCollClass( collClass ) );
    }
    if( shapeInitialized == false || collideShape.isdirty() ){
        shapeInitialized = true;
        const char* collShape = collideShape.GetS().Get();
        nCollideShape* shape = gameServer->NewCollideShape( collShape, collShape );
        //log(10, "%s->Initializing collideShape = %s", this->fn, collShape );
        collideObject->SetShape(shape);
    }
    if( classInitialized && shapeInitialized ) {
        nCollideReport** report;
        int numColls = collideObject->GetCollissions( report );
        if ( numColls > 0 ) {
            vector3 v = vel.GetV();
            for( int i = 0; i < numColls; i ++ ){
                nEnt* other = (nEnt*) report[i]->co2->GetClientData();
	//bug - we are getting crappy others inside this
	//i.e. -> indvidiuals are not being correctly removed
                if( other != 0 && other->IsAlive() ) {
	    collidedLastTick = true;
                    vector3 contact = report[i]->contact;
	    this->DrawPoint( contact, blue );
                    vector3 normal, otherNormal;
                    if( ent == other ){
                        other = (nEnt*) report[i]->co1->GetClientData();
                        normal = report[i]->co2_normal;
	        otherNormal = report[i]->co1_normal;
                    } else {
                        normal = report[i]->co1_normal;
	        otherNormal = report[i]->co2_normal;
                    }
	    if( exceptions.Find( other ) ) {
	        continue;
	    } 
                    if( collisionResponse.isvalid() ){
                        collisionResponse->Collision( other, contact, normal, otherNormal );
                    }
                    else {
                        vector3 v = vel.GetV();
                        float nv = v % normal;
                        if( nv > 0.0 ) { //moving towards other
                            float cr = 1.0f;
                            vector3 rel_velocity = v - other->GetVarV("vel");;
                            float m = mass.GetF();
                            float m0 = other->GetVarF("mass");
                            float j = (-(1+cr) * (rel_velocity%normal) / (( normal%normal)*(1.0f/m+1.0f/m0)));
                            vector3 mf = normal * j;
                            vector3 p = pos.GetV();
                            physics->ApplyForce( mf );
                            physics->ApplyTorque( mf * qi.rotate(contact-p));
	            //n_printf("physics->ApplyForce( %f,%f,%f )\n", mf.x, mf.y, mf.z );
                            //pos.SetV( pos.GetV() - normal*rel_velocity.len()*dtime);
                        }
                    }
                }
            }
        }
        if( numColls == 0 )
            lastPos = p;
        matrix44 mx;
        mx.ident();
        mx.mult_simple(matrix44(rot.GetQ()));
        mx.translate(p);
        collideObject->Transform( time, mx );
    }
}

bool nCollisionNode::BlockDeath( bool immed ){
    if( immed ) 
        return false;
    if( collideObject ) {
        //log(10, "%s->changing to ignore mode", this->fn );
        collideObject->SetCollClass( collideServer->QueryCollClass( "ignore" ) );
    }
    if( collidedLastTick ) {
        //log(10, "%s->blocking death", this->fn);
        return true;
    }
    else 
        return false;
}
