#define N_IMPLEMENTS nGroundColl
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/ngroundcoll.h"
#include "enkiduo/colours.h"

nNebulaScriptClass(nGroundColl, "nphysicalaspect");

//------------------------------------------------------------------------------
/**
*/
nGroundColl::nGroundColl():
    map( kernelServer, this )
{
}

void nGroundColl::Initialize(){
    nPhysicalAspect::Initialize();
    map = "/map/map";
    this->ticksPerFrame = TICKS_FOUR;
    AddToDisplayList();
}

void nGroundColl::InitVars(){
    nPhysicalAspect::InitVars();
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    vel.Connect( ent, "vel" );
    mass.Connect( ent, "mass" );

}

nGroundColl::~nGroundColl(){
}
//------------------------------------------------------------------------------
/**
*/

void nGroundColl::Tick( float time, float dtime ){
	int i;
    nPhysicalAspect::Tick( time, dtime );
    vector3 p = pos.GetV();
    quaternion r = rot.GetQ();
    vector3 v = vel.GetV();
    float m = mass.GetF();
    nArray<int> collIndexes;
    nArray<float> collAlts;
    float totalAlt = 0.0f;

    float biggestAltDiff;
    for( i = 0; i < points.Size(); i ++ ){
        vector3 tp = p + r.rotate(points[i]);
        translatedPoints0[i] = translatedPoints[i];
        translatedPoints[i] = tp;
        float h = map->GetHeight( tp.x, tp.z );
        float a = tp.y - h;
        if( i == 0 || a < biggestAltDiff )
            biggestAltDiff = a;
        if( a < 0.0f ){
            collIndexes.Append( i );
            collAlts.Append( a );
            totalAlt += a;
        }
        //DrawLine( tp, tp + r.rotate(normals[i]), white );
    }

    float contactForce = ent->GetVarV("constantforce").y * dtime;
    float cr = .50f; //coefficent of restitution
    float ratio;
        float damping = .1f;
    for( i = 0; i < collIndexes.Size(); i ++ ){
        int index = collIndexes[i];
        vector3 normal = normals[index];
        if( collIndexes.Size() == 1 )               
            ratio = 1.0f;
        else                                        
            ratio = collAlts[i] / totalAlt;
        normal = r.rotate( normal );
        map->GetNormal( points[index].x, points[index].z, normal );
        normal.set(0,1,0);
        float vn= v%normal;
        if( vn < 0.0f ) {
            float j = (-(1+cr) * vn / (( normal%normal)*(1.0f/m)));
            j += contactForce;
            vector3 mf = normal * j * ratio;
            physics->ApplyForce( mf );
            vector3 tpv = translatedPoints[index] - translatedPoints0[index];
            float tpvv = tpv % points[index];
            mf -= normal * tpvv * damping;
            vector3 torque = -mf * points[index];
            physics->ApplyTorque( torque*.2f  );
            //if( rand()%1000 == 0 ){
                //n_printf("tpvv:%f ", tpvv );
                //n_printv( tpv );
            //}
            //n_printf("vn:%5.2f j:%5.2f, cf:%5.2f, mf:%5.2f,%5.2f,%5.2f, torque:%5.2f,%5.2f,%5.2f\n", vn, j, contactForce, mf.x, mf.y, mf.z, torque.x, torque.y, torque.z );
            DrawLine( p + r.rotate(points[index]), p + r.rotate(points[index]) + mf * 100.0, green );
            DrawLine( p + r.rotate(points[index]), p + r.rotate(points[index]) + mf * points[i] * 100.0, red );
        }
    }
    if( collIndexes.Size() > 0 ) {
        p.y += -biggestAltDiff + .01;
        pos.SetV( p );
    }
}

void nGroundColl::Sum(){
    for( int i = 0; i < points.Size(); i ++ )
        n_printf("(%.2f,%.2f,%.2f)\n", points[i].x, points[i].y, points[i].z );
}
