#define N_IMPLEMENTS nGroundFriction
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/ngroundfriction.h"

nNebulaScriptClass(nGroundFriction, "nphysicalaspect");

//------------------------------------------------------------------------------
/**
*/
nGroundFriction::nGroundFriction():
    map( kernelServer, this )
{
}

void nGroundFriction::Initialize(){
    nPhysicalAspect::Initialize();
    map = "/map/map";
    this->ticksPerFrame = TICKS_FOUR;
    AddToDisplayList();
}

void nGroundFriction::InitVars(){
    nPhysicalAspect::InitVars();
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    vel.Connect( ent, "vel" );
    mass.Connect( ent, "mass" );
    cuz = .1f;
    cux = .5f;
}

nGroundFriction::~nGroundFriction(){
}
//------------------------------------------------------------------------------
/**
*/

void nGroundFriction::Tick( float time, float dtime ){
    nPhysicalAspect::Tick( time, dtime );
    vector3 p = pos.GetV();
    float h = map->GetHeight(p.x, p.z);
    float a = p.y - h;
    quaternion r = rot.GetQ();
    if( a < 2.0f && r.ToRoll() < .3 ){
        vector3 v = vel.GetV();
        vector3 forward = r.rotate(vector3(0,0,-1));
        vector3 right = r.rotate(vector3(1,0,0));
        float vz = v % forward;
        float vx = v % right;
        n_assert( vz < v.len() );
        n_assert( vx < v.len() );
        vector3 f = vector3(0,0,1) * vz * cuz + vector3(-1,0,0) * vx * cux;
        //f /= mass.GetF();
        physics->ApplyAlignedForce( f * dtime);
        //DrawLine( p, p + v*100.0, white );
        //DrawLine( p, p + forward*10.0, red );
        //DrawLine( p, p + right*10.0, blue );
        //DrawLine( p, p + f*100.0f, yellow );
    }
}

