#define N_IMPLEMENTS nPhysics
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/nphysics.h"
#include "enkiduo/nprofserver.h"

nNebulaScriptClass(nPhysics, "naspect");

//------------------------------------------------------------------------------
/**
*/
nPhysics::nPhysics()
{
}

void nPhysics::Initialize()
{
    nAspect::Initialize();
    pos.                    Connect(ent, "pos");
    vel.                    Connect(ent, "vel");
    force.                  Connect(ent, "force");
    rot.                    Connect(ent, "rot");
    rvel.                   Connect(ent, "rvel");
    torque.                 Connect(ent, "torque");
    alignedTorque.          Connect(ent, "alignedtorque");
    mass.                   Connect(ent, "mass");
    inertia.                Connect(ent, "inertia");
    friction.               Connect(ent, "friction");
    rfriction.              Connect(ent, "rfriction");
    vel.                    DefaultV(0,0,0);
    force.                  DefaultV(0,0,0);
    rvel.                   DefaultQ(0,0,0,0);
    torque.                 DefaultV(0,0,0);
    alignedTorque.          DefaultV(0,0,0);
    mass.                   DefaultF(1.0f);
    inertia.                DefaultF(1.0f);
    friction.               DefaultF(1.0f);
    rfriction.              DefaultF(1.0f);
    this->ticksPerFrame = TICKS_ONCE;
}
//------------------------------------------------------------------------------
/**
*/
nPhysics::~nPhysics()
{
}

void nPhysics::Tick( float , float dtime ){
    if( ent->Held() ) return;
    //prof->Begin("nPhysics");
    vector3 t = pos.GetV();
    vector3 p0 = t;
    vector3 dt = vel.GetV();
    vector3 ddt = force.GetV();
    bool setForce = ddt.len() > 0.0;

    quaternion q = rot.GetQ();
    quaternion dq = rvel.GetQ();
    vector3 ddq = torque.GetV();
    bool setTorque = ddq.len() > 0.0;
    vector3 addq = alignedTorque.GetV();
    bool setAlignedTorque = addq.len() > 0.0;

    //n_printv( ddq );
    //n_printv( addq );
    float m = mass.GetF();
    float i = inertia.GetF();
    //float f = 1.0f - 1.0f / friction.GetF();
    float f = friction.GetF();
    float rf = rfriction.GetF();

    this->DoMath( t, dt, ddt, q, dq, ddq, addq, m, i, f, rf, dtime );

    pos     .SetV(t);
    vel     .SetV(dt);
    if(setForce)
        force.SetV( 0,0,0 );
    rot     .SetQ(q);
    rvel    .SetQ(dq);
    if(setTorque)
        torque.SetV(0,0,0);
    if(setAlignedTorque)
        alignedTorque.SetV(0,0,0);

    //n_printf("Physics(%s)::setPos = %s\n", this->fn, str(pos.GetV()));
    //prof->End("nPhysics");
}

void nPhysics::DoMath( vector3& t, vector3& dt, const vector3& ddt, quaternion& q, quaternion& dq, vector3 ddq, vector3 addq, const float& m, const float& i, const float& f, const float& rf, float dtime ) {

    ddq /= i;
    addq /= i;
    
    // f / m - dt * friction * dtime
    vector3 accel = ddt/m - dt * f * dtime;
    dt += accel;
    t  += dt * dtime;

    dq += quaternion( ddq.x, ddq.y, ddq.z, 0 );
    addq = q.rotate( addq );
    dq += quaternion( addq.x, addq.y, addq.z, 0 );

    //rvel -= rfriction
    dq -= dq * rf * dtime;
    //rot += rvel * dtime
    q += (dq * q) / 2.0f * dtime;

    q.normalize();
}
    

void nPhysics::ApplyForce         ( const vector3& f ){
    //n_assert( f.len() < 1000.0 );
    vector3 v = force.GetV();
    v += f;
    force.SetV(v);
}
void nPhysics::ApplyAlignedForce  ( const vector3& f ){
    //n_printv( f );
    //n_assert( f.len() < 1000.0 );
    vector3 v = force.GetV();
    v += rot.GetQ().rotate( f );
    force.SetV(v);
}
void nPhysics::ApplyTorque        ( const vector3& f ){
    //n_assert( f.len() < 1000.0 );
    static int i = 0;
    //n_printf("%i %f ", i++, rot.GetQ().ToRoll() );
    //n_printv( f );
    vector3 t = torque.GetV();
    t += f;
    torque.SetV(t);
}
void nPhysics::ApplyAlignedTorque ( const vector3& f, bool count ){
    vector3 at = alignedTorque.GetV();
    at += f;
    alignedTorque.SetV(at);
    if (!count)
        alignedTorque.DontCount();
}
