#define N_IMPLEMENTS nShipEngine
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "physics/nshipengine.h"

nNebulaScriptClass(nShipEngine, "nphysicalaspect");

//------------------------------------------------------------------------------
/**
*/
nShipEngine::nShipEngine()
{
}

void nShipEngine::Initialize(){
    nPhysicalAspect::Initialize();
}

void nShipEngine::InitVars(){
    nPhysicalAspect::InitVars();
    heading.Connect( ent, "heading" );
    heading.DefaultF( 0.0f );
    speed.Connect( ent, "speed" );
    speed.DefaultF( 0.0f );
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    enginepower.Connect( ent, "enginepower" );
    enginepower.DefaultF( 100.0f );
    thrusterpower.Connect( ent, "thrusterpower" );
    thrusterpower.DefaultF( 10.0f );
}

nShipEngine::~nShipEngine(){
}
//------------------------------------------------------------------------------
/**
*/

void nShipEngine::Tick( float time, float dtime ){
    nPhysicalAspect::Tick( time, dtime );

    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();

    float ep = enginepower.GetF();
    float tp = thrusterpower.GetF();
    float h = heading.GetF();
    float s = speed.GetF();

    vector3 forward(0,0,-1);
    forward = q.rotate( forward );
    float theta = atan2( forward.x, forward.z );

    if( s > 0.0f ) {
        float dtheta = h - theta;
        while( dtheta > N_PI )
            dtheta -= N_PI * 2.0f;
        while( dtheta < -N_PI )
            dtheta += N_PI * 2.0f;

        if( fabs(dtheta) > .10 ) {
            if( dtheta > 0.0 )
	dtheta = 1.0f;
            else 
	dtheta = -1.0f;
            physics->ApplyAlignedTorque( vector3(0.0f, tp * dtheta, 0.0f)*dtime );
        }
        if( fabs(dtheta) < .5f ) {
            physics->ApplyAlignedForce( vector3(0, 0, -dtime * ep ) * s);
        }
    }
}
