#define N_IMPLEMENTS nSpring
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "physics/nspring.h"

nNebulaScriptClass(nSpring, "noffsetaspect");

//------------------------------------------------------------------------------
/**
*/
nSpring::nSpring():
    map( kernelServer, this )
{
}

void nSpring::Initialize(){
    nOffsetAspect::Initialize();
    map = "/map/map";
    this->ticksPerFrame = TICKS_FOUR;
    //AddToDisplayList();
}

void nSpring::InitVars(){
    nOffsetAspect::InitVars();
    springLength.Connect( ent, "springlength" );
    springLength.DefaultF( 1.0f );
    springForce.Connect ( ent, "springforce" );
    springForce.DefaultF( 1.0f );
    springDamping.Connect( ent, "springdamping" );
    springDamping.DefaultF( 1.0f );
    h0 = -1.0f;
}

nSpring::~nSpring(){
}
//------------------------------------------------------------------------------
/**
*/

void nSpring::Tick( float time, float dtime ){
    nOffsetAspect::Tick( time, dtime );
    float gh;
    if( map.isvalid() ) 
        gh = map->GetHeight( offsetPos.x, offsetPos.z );
    else
        gh = 0.0f;
    float k = springForce.GetF();
    float dc = springDamping.GetF();

    float h = offsetPos.y - gh;

    //this->DrawPoint( offsetPos, white );
    if( h < 0 ) {
        float r;            //ratio of power being exerted
        r = -h;

        float f = r * k * dtime;
        float df = -(h-h0) * dc;
        float tf = f + df;
        
        vector3 force(0,tf,0);
        //this->DrawLine( offsetPos, offsetPos + force, red );
        this->ApplyPointForce( force );
    }
}

