#define N_IMPLEMENTS nHoverEngine
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nhoverengine.h"

nNebulaScriptClass(nHoverEngine, "nphysicalaspect");

//------------------------------------------------------------------------------
/**
*/
nHoverEngine::nHoverEngine()
{
}

void nHoverEngine::Initialize(){
    nPhysicalAspect::Initialize();
    this->AddToDisplayList();
}

void nHoverEngine::InitVars(){
    nPhysicalAspect::InitVars();
    veldesired.Connect(ent, "veldesired");
    veldesired.DefaultV(0,0,0);
    bearingdesired.Connect(ent, "bearingdesired");
    bearingdesired.DefaultF(-1.0f);
    enginepower.Connect   (ent, "enginepower");
    enginepower.DefaultF(1.0f);
    thrusterpower.Connect   (ent, "thrusterpower");
    thrusterpower.DefaultF(1.0f);
    pos.Connect( ent, "pos" );
    rot.Connect( ent, "rot" );
    overdrive.Connect( ent, "overdrive");
    overdrive.DefaultI( 0 );
    overdriveAmount.Connect( ent, "overdriveamount");
    overdriveAmount.DefaultF( .2f );
}

nHoverEngine::~nHoverEngine(){
}
//------------------------------------------------------------------------------
/**
*/

////////////////////////////////////////////////////////////////////////////////
///Apply forces and torques congruent with hovercraft physics to reach the desired velocity and bearing
////////////////////////////////////////////////////////////////////////////////
void nHoverEngine::Tick( float time, float dtime ){
    nPhysicalAspect::Tick( time, dtime );
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();

    //apply force
    float ep = enginepower.GetF();
    if( overdrive.GetI() == 1 )
        ep *= 1.0f + overdriveAmount.GetF();

    float tp = thrusterpower.GetF();
    vector3 vd = veldesired.GetV();
    vd.y = 0.0f;
    float b = bearingdesired.GetF();
    physics->ApplyForce( vd * ep * dtime);
    veldesired.SetV( 0,0,0 );

    //apply torque
    float desiredTheta;
    if( b == -1.0f) 	desiredTheta = atan2( vd.x, vd.z ); //default bearing - no ones voting
    else	        	desiredTheta = b;
    //figure out my bearing
    vector3 forward(0,0,1);
    forward = q.rotate( forward );
    float theta = atan2( forward.x, forward.z );

    if( vd.len() > 0.5f ) {
        float dtheta = desiredTheta - theta;	//direction i need to turn to find desired
        while( dtheta > N_PI )	            dtheta -= N_PI * 2.0f;
        while( dtheta < -N_PI )	            dtheta += N_PI * 2.0f;

        if( fabs(dtheta) > 0.05 ) {
            if( dtheta < .3 && dtheta > 0 )		dtheta = .3;
            if( dtheta > -.3 && dtheta < 0 )		dtheta = -.3;
            physics->ApplyAlignedTorque( -1.0f * vector3(0.0f, tp * dtheta, 0.0f)*dtime );
        }
    }
}

