#define N_IMPLEMENTS nHoverStandoff
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nhoverstandoff.h"

nNebulaScriptClass(nHoverStandoff, "nai");

//------------------------------------------------------------------------------
/**
*/
nHoverStandoff::nHoverStandoff():
    arbiter( kernelServer, this )
{
    //printf("nHoverStandoff()\n");
    //this->AddToDisplayList();
}

void nHoverStandoff::Initialize(){
    //printf("nHoverStandoff::Initialize()\n");
    nAI::Initialize();
}

void nHoverStandoff::InitVars(){
    nAI::InitVars();
    //n_printf("nHoverStandoff::InitVars()\n");
    pos.Connect		( ent, "pos" );
    target.Connect	( ent, "target" );
    target.DefaultRef("");
    mytarget.Connect	( ent, "mytarget" );
    mytarget.DefaultRef("");
    targetPos.Connect	( target, "pos" );
    standoffRange.Connect	( ent, "standoffrange" );
    standoffRange.DefaultF	( 20.0f );
    arbiter = ent->ChildPath("arbiter");

    parentlegion.Connect( ent, "parentlegion" );
    formationpos.Connect( ent, "formationpos" );

    overdrive.Connect( ent, "overdrive" );
}

nHoverStandoff::~nHoverStandoff(){
}
//------------------------------------------------------------------------------
/**
*/

void nHoverStandoff::Tick( float time, float dtime ){
    nAI::Tick( time, dtime );
}

bool nHoverStandoff::States( StateMachineEvent event, uint state, float time, float dtime ){
    BeginStateMachine
        State(Approach)
            OnEnter
	if( mytarget.isvalid() )
	    target.SetRef( mytarget.GetRef()->fn );
	else {
	    n_printf("target invalid from the start dying\n");
	    this->Die();
	}
            OnTick
	if( target.isvalid() ) {
	    vector3 p = pos.GetV();
	    vector3 tp = targetPos.GetV();
	    vector3 dx = tp - p;
	    float tl = dx.len();
	    dx.norm();
	    float theta = atan2( dx.x, dx.z );
	    
	    vector3 veldesired = vector3( sin(theta), 0, cos(theta) );

	    vector3 d = formationpos.GetV();
	    dx = d - p;
	    float l = dx.len();
	    if( l > 1 ) {
	        dx.norm();
	        if( veldesired % dx > 0 ) //both in the same line - we are falling behind
	            overdrive.SetI ( 1 );
	        else
	            overdrive.SetI( 0 );
	        veldesired += dx * .2f;
	    }

	    arbiter->VoteVelocity	(veldesired, 10, 1 );
	    arbiter->VoteBearing	(theta, 10, 1 );
	    if( tl < standoffRange.GetF() ){
	        SetState( Standoff );
	    }
	} else {
	    //n_printf("%s->target invalid\n", ent->fn);
	    target.SetRef("");
	    this->Die();
	}
        State(Standoff)
            OnTick
	if(target.isvalid() ) {
	    vector3 p = pos.GetV();
	    vector3 tp = targetPos.GetV();
	    vector3 dx = tp - p;
	    float tl = dx.len();
	    if( tl > standoffRange.GetF() * 1.0f ){
	        SetState( Approach );
	    } else if( tl < standoffRange.GetF() * 0.9f ) {
	        if( parentlegion.isvalid() ) { //we are within range - we should try to stay in formation
	            vector3 d = formationpos.GetV();
	            dx = d - p;
	            float l = dx.len();
	            if( l > 4 ) {
		dx.norm();
		arbiter->VoteVelocity( dx, 10, 1 );
	            } else
		arbiter->VoteVelocity( vector3(), 10, 1 );
	        }
	    }
	} else
	    this->Die();
        EndStateMachine
}
