#define N_IMPLEMENTS nLegionAspect
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nlegionaspect.h"
#include "util/nheap.h"
#include "physics/nphysics.h"
#include "enkiduo/nprofserver.h"
#include "enkiduo/ngameserver.h"
#include "rts/nlegionmove.h"
#include "rts/nlegionattacklegion.h"
#include "rts/nlegionlegionmove.h"
#include "rts/nlegionlegionattacklegion.h"
#include "rts/nlegionassault.h"

nNebulaScriptClass(nLegionAspect, "ncontroller");
nLegionAspect::nLegionAspect():
physics( kernelServer, this ),
allocationTactic( NTO1 ),
N(1),
myController( kernelServer, this ),
timeWithoutUnits( -1.0f ),
pureLegion( false ) {
}
void nLegionAspect::Initialize() {
    nController::Initialize();
    this->AddToDisplayList();
    formationCalcNum = 0;
}
void nLegionAspect::InitVars() {
    nController::InitVars();

    pos.        Connect     ( ent, "pos" );
    rot.        Connect     ( ent, "rot" );
    formationPos.   Connect ( ent, "formationpos" );
    formationPos.   DefaultV    ( 0,0,0 );
    formationRot.   Connect ( ent, "formationrot" );
    formationRot.   DefaultQ    ( 0,0,0,1 );
    formationSize.  Connect ( ent, "formationsize" );
    formationSize.  DefaultV    ( 1,1,1 );
    width.      Connect ( ent, "width" );
    width.      DefaultF    ( 1.0f );
    depth.      Connect ( ent, "depth");
    depth.      DefaultF    ( 1.0f );
    parentLegion.   Connect ( ent, "parentlegion" );
    parentLegion.   DefaultRef  ("");
    formation.      Connect ( ent, "formation" );
    formation.      DefaultV    ( 1, 0, 0 );
    numRowsColumns. Connect ( ent, "numrowscolumns" );
    numRowsColumns. DefaultI    ( 1 );    
    numRowsColumns. GetI();                                                                        //eat up the first dirty call
    legionRank.Connect( ent, "legionrank" );
    legionRank.DefaultI( 0 );
    minalt.Connect( ent, "minalt");
    maxalt.Connect( ent, "maxalt");

    //legion 0 variables
    legion0.Connect( ent, "legion0" );
    legion0.DefaultRef("");    
    l0pos.  Connect( legion0, "pos" );
    l0rot.  Connect( legion0, "rot" );
    l0width.Connect( legion0, "width" );
    l0depth.Connect( legion0, "depth" );
    side.Connect( ent, "side" );

    entname     = ent->FullNameString();
    physics     = ent->ChildPath("physics");
    myController    = ent->ChildPath("ai");    

    legionLocked.Connect( ent, "legionlocked" );
    legionLocked.DefaultI( 0 );

    static nRoot* legionRoot = kernelServer->Lookup("/game/ents/legion");
    //n_printf("legionRoot:%s parent:%s\n", legionRoot->fn, ent->GetParent()->fn );
    if ( ent->GetParent() == legionRoot )
        this->pureLegion = true;

    int numResources = 3;		//FIXME
    nString name;
    for( int i = 0; i < numResources; i ++ ) {
        name.PrintF("resource%i", i );
        resources.Push().Connect( side, name.Get() );
    }
}

nLegionAspect::~nLegionAspect() {
}
///get rid of any bad links in the unit list - just incase
void nLegionAspect::CleanUnits() {
    classVar(nent, entclass);
    for ( int i = 0; i < units; i ++ ) {
        if ( units[i].ent->IsA( entclass ) == false )
            n_error("hand an ent in the unit list that is not an ent\n");
        if ( units[i].ent->IsAlive() == false )
            units.Erase(i--);
    }
}
///i am dying - remove me from my parent legion
void nLegionAspect::DetachFromLegion() {
    if ( legionAddedToo.isvalid() ) {                                                              //already in a legion - regroup me
        legionAddedToo->RemoveUnit( ent );
        legionAddedToo.invalidate();
    } else {
        //n_printf("%s wants to detach but isn't\n", this->fn );
    }
}
///adds a new unit to the legion
void nLegionAspect::AddUnit( nEnt* e ) {
    UnitInfo& u = units.Push();
    u.ent = e;
    u.pos.Connect( e, "pos" );
    u.formationPos. Connect( e, "formationpos"    );
    u.formationRot. Connect( e, "formationrot"    );
    u.formationPoint.   Connect( e, "formationpoint"  );
    u.formationSize.    Connect( e, "formationsize"   );
    u.formationHandle.  Connect( e, "formationhandle" );
    CalculateFormation();
}
///removes a child unit from this legion
void nLegionAspect::RemoveUnit( nEnt* e ) {
    for ( int i = 0; i < units; i ++ )
        if ( units[i].ent == e )
            units.Erase(i--);
    CalculateFormation();
}
///display stuff to screen + propogate info on formation position down
void nLegionAspect::Tick( float time, float dtime ) {
    nController::Tick( time, dtime );
    if ( formationCalcNum > 0 && numRowsColumns.isdirty() ) {                                      //someone has realigned the formation 
        //n_printf("numRowsColumns.isdirty()\n");
        this->CalculateFormation();                                                                //rebuild the points
    }
    if ( parentLegion.isvalid() ) {                                                                //UPDATE MY PARENT LEGION
        if ( parentLegion.isdirty() ) {
            //n_printf("%s has a new legion parent of %s\n", this->fn, parentLegion.GetRef()->fn );
            DetachFromLegion();
            nLegionAspect* la = (nLegionAspect*) parentLegion.GetRef()->Find("legion");
            ifnonzero( la ) {
                n_assertisa( la, nlegionaspect );
                la->AddUnit( ent );
                legionAddedToo = la;
            }
        }
    }
    if ( units.Size() > 0 ) {                                                                      //I am a real legion - not just a unit
        this->CleanUnits();                                                                        //toss any crap units in my list
        if ( this->timeWithoutUnits > -1.0f )
            this->timeWithoutUnits = -1.0f;
        //create legion0    
        if ( legion0.isvalid() == false ) {
            legion0.SetRef( game->GenEnt("legion0") );
        }
        //set widths / depths
        vector3 fs = formationSize.GetV();
        width.SetF( fs.x );
        depth.SetF( fs.z );    

        if ( legion0.isvalid() ) {
            l0width.SetF( fs.x );
            l0depth.SetF( fs.y );
        }
        //set my units formation pos's
        //apply the force on me from their location            
        this->SetFormationPoints();
        this->ApplyAlignmentForce();

        if ( legionLocked.GetI() == 1 ) {                                                          //we are locked
            ifvalid( parentLegion ) {
                pos.SetV( formationPos.GetV() );
                rot.SetQ( formationRot.GetQ() );
            }
            l0pos.SetV( formationPos.GetV() );                                                     //set our formation specifically to our formation0
            l0rot.SetQ( formationRot.GetQ() );
        } else if ( myController->GetHead() == 0 ) {                                               //we have nothing active too do
            if ( parentLegion.isvalid() ) {
                pos.SetV( formationPos.GetV() );
                rot.SetQ( formationRot.GetQ() );
            }
            l0pos.SetV( pos.GetV() );
            l0rot.SetQ( rot.GetQ() );
        }
    } else if ( this->pureLegion ) {                                                               ///if we are just a legion, and we have no children, die after 30 seconds
        if ( timeWithoutUnits < 0 )
            timeWithoutUnits = 0.0f;
        timeWithoutUnits += dtime;
        if ( timeWithoutUnits > 30.0f )
            ent->Die();
    }
}
///sets each child's formationpos variable
void nLegionAspect::SetFormationPoints() {
    vector3 p;
    quaternion q;
    n_assert( legion0.isvalid() );
    p = l0pos.GetV();
    q = l0rot.GetQ();
    vector3 forward( 0,0,1.0f );
    forward = q.rotate( forward );
    float bearing = atan2( forward.x, forward.z );
    for ( int i = 0; i < units; i ++ ) {
        vector3 fp = q.rotate(formationPoints[units[i].formationHandle.GetI()]) + p;
        units[i].formationPoint.SetV( formationPoints[units[i].formationHandle.GetI()] );
        units[i].formationPos.SetV( fp );        
        units[i].formationRot.SetQ( rot.GetQ() );        
    }    
}
///apply the forces neccessary to make this legion follow its kids
void nLegionAspect::ApplyAlignmentForce() {
    static float ForceSumThreshold = 2.0f;
    static float TorqueSumThreshold = 10.0f;
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    vector3 forceSum;
    vector3 torqueSum;
    for ( int i = 0; i < units; i ++ ) {
        vector3 fp = q.rotate(formationPoints[units[i].formationHandle.GetI()]) + p;
        vector3 dist = (units[i].pos.GetV()) - fp;
        dist.y = 0;
        forceSum += dist;
        torqueSum += dist * q.rotate( formationPoints[units[i].formationHandle.GetI()] );
    }
    forceSum = forceSum / (float) units.Size();
    torqueSum = torqueSum / (float) units.Size();
    if ( forceSum.len() > ForceSumThreshold ) {
        physics->ApplyForce( forceSum );                 
    }
    torqueSum.x = 0.0f;
    torqueSum.z = 0.0f;

    if ( torqueSum.len() > TorqueSumThreshold ) {
        //n_printf("torqueSum:%s --- %f\n", str( torqueSum ), torqueSum.len() );
        if ( torqueSum.y < -10 )
            torqueSum.y = -90;
        else if ( torqueSum.y > 10 )
            torqueSum.y = 90;
        else
            torqueSum.y = 0;
        if ( legionLocked.GetI() == 0 )
            physics->ApplyTorque( -torqueSum );
        else
            ifvalid(parentLegion) {
            //((nPhysics*)(parentLegion.GetRef()->Find("physics")))->ApplyTorque( -torqueSum / units.Size() );
        }
    }
}
///determine where in space the units shuld go
void nLegionAspect::CalculateFormation() {
    nAspect::Tick(0,0);                                                                            //initialze variables incase we miss them
    //n_printf("%s->CalculateFormation(%i)\n", this->fn, formationCalcNum++);
    formationPoints.Clear();
    vector3 fs;
    int nrc = numRowsColumns.GetI();
    int rank = 0;
    for ( int i = 0; i < units; i++ ) {
        vector3 ofs = units[i].formationSize.GetV();
        //n_printf("ofs:%s\n", str(ofs));
        if ( nrc > 0 ) {
            fs.x += ofs.x;
            if ( ofs.y > fs.y )
                fs.y = ofs.y;
            if ( ofs.z > fs.z )
                fs.z = ofs.z;
        } else {
            fs.z += ofs.z;
            if ( ofs.y > fs.y )
                fs.y = ofs.y;
            if ( ofs.x > fs.x )
                fs.x = ofs.x;
        }
    }
    vector3 l, r;
    formationSize.SetV( fs );
    if ( units.Size() > 0 ) {
        rank = 1;
        this->CleanUnits();
        if ( units.Size() %2 == 0 ) {                                                              //even number of units - start them left-right
        } else {
            if ( nrc > 0 ) {
                l.x -= units[0].formationSize.GetV().x;
                r.x -= units[0].formationSize.GetV().x;
            } else {
                l.z -= units[0].formationSize.GetV().z;
                r.z -= units[0].formationSize.GetV().z;
            }
        }
        for ( int i = 0; i < units; i ++ ) {
            if ( units[i].ent->HasVar("legionrank")) {
                int r = units[i].ent->GetVarI("legionrank");
                //n_printf("child: %s has rank: %i\n", units[i].ent->fn, r );
                if ( rank <= r ) {
                    //n_printf("increasing rank to : %i\n", rank );
                    rank = r +1;
                }
            }
            if ( i % 2 == 0 ) {                                                                    //evens go right
                if ( nrc > 0 )   r.x += units[i].formationSize.GetV().x;
                else    r.z += units[i].formationSize.GetV().z;
                formationPoints.Append( r );
                if ( nrc > 0 )   r.x += units[i].formationSize.GetV().x;
                else    r.z += units[i].formationSize.GetV().z;
            } else {                                                                               //odds go left
                if ( nrc > 0 )   l.x -= units[i].formationSize.GetV().x;
                else    l.z -= units[i].formationSize.GetV().z;
                formationPoints.Append( l );
                if ( nrc > 0 )   l.x -= units[i].formationSize.GetV().x;
                else    l.z -= units[i].formationSize.GetV().z;
            }
        }
    }
    this->AssignPositions();
    if ( parentLegion.isvalid() ) {
        nEnt* e = (nEnt*) parentLegion.GetRef();
        ifnonzero(e) {
            nLegionAspect* la = (nLegionAspect*) e->Find("legion");
            n_assert( la != this );
            ifnonzero ( la )
            la->CalculateFormation();
        }
    }
    //n_printf("legionrank: %i\n", rank );
    legionRank.SetI ( rank );
    minalt.SetF( 10 * rank );
    maxalt.SetF( 10 * rank );
}
///figure out which units should go to which locations - right now push all n^2 possible combinations - pop them off a heap and take the shortest ones - simple greedy allocation algorithm
void nLegionAspect::AssignPositions() {
    this->CleanUnits();
    vector3 p = pos.GetV();
    quaternion q = rot.GetQ();
    nHeap<vector2> assignmentsHeap;
    //n_printf("assignmentsHeap.Initialize(%i)\n", units.Size() * formationPoints.Size() );
    if ( units.Size() == 1 ) {
        units[0].formationHandle.SetI( 0 );
        return;
    }
    assignmentsHeap.initialize( units.Size() * formationPoints.Size() );
    //n_printf("units:%i points:%i\n", units.Size(), formationPoints.Size() );
    for ( int i = 0; i < units; i++ ) {
        vector3 up = units[i].pos.GetV();
        for ( int j = 0; j < formationPoints; j++ ) {
            vector3 d = q.rotate( formationPoints[j] ) + p;
            float val = (d-up).len();
            //n_printf("%s->%s val:%f\n", str(up), str(d), val );
            assignmentsHeap.add( val, vector2(i,j));
        }
    }
    bool* assignedUnits = new bool[units.Size()];
    bool* assignedPoints = new bool[formationPoints.Size()];
    memset( assignedUnits, false, units.Size() * sizeof(bool) );
    memset( assignedPoints, false, formationPoints.Size() * sizeof(bool) );
    vector2 a;
    float val;
    while ( assignmentsHeap.length() > 0 ) {
        assignmentsHeap.pop( val, a );
        int unit = (int) a.x;
        int point = (int) a.y;
        //n_printf("%f->%i %f->%i\n", a.x, unit, a.y, point );
        if ( assignedUnits[unit] == false && assignedPoints[point] == false ) {
            //n_printf("Assigning unit:%i to point:%i %s->%s val:%f\n", unit, point, str(units[unit].pos.GetV()),str(formationPoints[point]), val );
            assignedUnits[unit] = true;
            assignedPoints[point] = true;
            units[unit].formationHandle.SetI( point );
        }
    }
    delete[] assignedUnits;
    delete[] assignedPoints;
}
///we have recieved a message
///if we have children - then push a nlegion command + nlegion proxy commands on all the children
void nLegionAspect::Message( nMessage* message ) {
    //prof->Begin("nLegionAspect::Message()");	  
    if ( message->to == entname ) {                                                                //the message is too me
        if ( units.Size() == 0 ) {                                                                 //i am just a unit
            if ( message->title == "command" ) {                                                   //i have beeen right clicked on something
                if ( message->data != "none" ) {                                                   //it was not over a blank area - clicked on a unit	    
                    nEnt* e = (nEnt*) kernelServer->Lookup( message->data.Get() );
                    if ( e && e->GetSideHandle() == ent->GetSideHandle() ) {                       //on my side
                        if ( nString("legion") == e->GetParent()->GetName() ) {                    //commanded to a friendly legion - i.e. join that legion
                            ent->SetVarRef("parentlegion", e->fn);             
                        }
                    }
                }
            }
        } else {                                                                                   //we are a real legion - have children
            float heading;
            heading = message->f0;
            if ( heading == -1.0f ) {                                                              //we were not told a new heading to follow
                vector3 forward( 0, 0, 1 );
                forward = rot.GetQ().rotate( forward );
                heading = (float) atan2( forward.x, forward.z );
            } else {                                                                               //we have a new heading to abide by 
                quaternion q;
                q.set_rotate_y(heading);
                rot.SetQ(q);
            }           
            if ( message->title == "command" ) {                                                   //i have beeen right clicked on something
                if ( message->data != "none" ) {                                                   //it was not over a blank area - clicked on a unit	    
                    nEnt* e = (nEnt*) kernelServer->Lookup( message->data.Get() );
                    if ( e && e->GetSideHandle() == ent->GetSideHandle() ) {                       //on my side
                        if ( nString("legion") == e->GetParent()->GetName() ) {                    //commanded to a friendly legion - i.e. join that legion
                            ent->SetVarRef("parentlegion", e->fn);             
                        }
                    }
                }
            }
            if ( message->title == "command" || message->title == "attack move" ) {                //i have beeen right clicked on something
                quaternion q = rot.GetQ();
                if ( message->data == "none" ) {                                                   //it was over a blank area - just move
                    nLegionMove* lm;
                    if ( message->string == "push" )
                        lm = (nLegionMove*) myController->PushChild("nlegionmove");
                    else if ( message->string == "queue" )
                        lm = (nLegionMove*) myController->QueueChild("nlegionmove");
                    else
                        lm = (nLegionMove*) myController->ReplaceChildren("nlegionmove");

                    //n_printf("pushed nlegionmove:%s\n", lm->fn );
                    lm->CreationMessage( message );
                    for ( int i = 0; i < units; i ++ ) {
                        nMessage* m = new nMessage();
                        //*m = *message;
                        m->to = units[i].ent->fn;
                        m->title = "legionmove";
                        m->data = lm->fn;
                        vector3 dx = message->v - pos.GetV();           
                        m->v = message->v;
                        m->f0 = atan2( dx.x, dx.z );
                        Send( m );  
                    }
                } else {                                                                           //over an entity
                    nEnt* e = (nEnt*) kernelServer->Lookup(message->data.Get() );
                    static nRoot* legionRoot = kernelServer->Lookup("/game/ents/legion");
                    static nRoot* legionRootN = kernelServer->Lookup("/game/ents/legionn");
                    if ( legionRoot == e->GetParent() ) {                                          //its a legion
                        if ( e->GetSideHandle() != ent->GetSideHandle() ) {                        ///ATTACKING AN ENEMY LEGION
                            nEnt* legion = e;
                            while ( legion->HasVar("parentlegion") ) { //find the enemy super legion - target them all
                                const char* s = legion->GetVarRef("parentlegion");
                                //n_printf("pulled parentlegion: %s\n", s );
                                if ( s && strcmp( s, "none" ) != 0 ) {
                                    legion = (nEnt*) kernelServer->Lookup( s );
                                    n_assertisa( legion, nent );
                                } else
                                    break;
                            }

                            nLegionAspect* enemyLegion = (nLegionAspect*) legion->Find("legion");
                            n_assert( enemyLegion );

                            nLegionAttackLegion* lal = (nLegionAttackLegion*) myController->PushChild("nlegionattacklegion");
                            //n_printf("pushed nLegionAttackLegion: %s\n", lal->fn );
                            message->data = legion->fn;
                            lal->CreationMessage( message );

                            for ( int i = 0; i < units; i ++ ) {
                                nMessage* m = new nMessage();
                                //*m = *message;
                                m->to = units[i].ent->fn;
                                m->title = "legionattacklegion";
                                m->data = lal->fn;
                                vector3 dx = message->v - pos.GetV();           
                                m->v = message->v;
                                m->f0 = atan2( dx.x, dx.z );
                                this->Send( m );  
                            }
                        }
                    } else if ( legionRoot == e->GetParent() ) {                                   //its a legionn
                        //dunno how this happens, but ignore it
                    } else {                                                                       //otherwise propogate the message down to my units
                        for ( int i = 0; i < units; i ++ ) {
                            nMessage* m = new nMessage();
                            *m = *message;
                            m->to = units[i].ent->fn;
                            m->f0 = heading;
                            //n_printf("m->to:%s\n", m->to.Get() );
                            this->Send( m );
                        }
                    }
                }
            } else if ( message->title == "assault" ) {
                nLegionAssault* la;
                if      ( message->string == "push"  )              la = (nLegionAssault*) myController->PushChild("nlegionassault");
                else if ( message->string == "queue" )              la = (nLegionAssault*) myController->QueueChild("nlegionassault");
                else			    la = (nLegionAssault*) myController->ReplaceChildren("nlegionassault");

	//message->data = legion->fn;
	la->CreationMessage( message );

	for ( int i = 0; i < units; i ++ ) {
	    nMessage* m = new nMessage();
	    //*m = *message;
	    m->to = units[i].ent->fn;
	    m->title = "legionattacklegion";
	    m->data = la->fn;
	    vector3 dx = message->v - pos.GetV();           
	    m->v = message->v;
	    m->f0 = atan2( dx.x, dx.z );
	    this->Send( m );  
	}
            } else if ( message->title == "legionmove" ) {                                         //we are a sub legion commanded to do a legion move
	//n_printf("%s commanded to do a legionlegionmove\n", this->fn );
	nLegionLegionMove* llm = (nLegionLegionMove*) myController->PushChild( "nlegionlegionmove" );
	llm->SetProxyParent( message->data.Get() );
	for ( int i = 0; i < units; i ++ ) {
	    nMessage* m = new nMessage();
	    *m = *message;
	    m->to = units[i].ent->fn;
	    this->Send( m );
	}
            } else if ( message->title == "legionattacklegion" ) {                                 //for now just pass down to kids
	//n_printf("%s command to do a legionlegionattacklegion\n", this->fn );
	nLegionLegionAttackLegion* llal = (nLegionLegionAttackLegion*) myController->PushChild( "nlegionlegionattacklegion" );
	llal->SetProxyParent( message->data.Get() );
	for ( int i = 0; i < units; i ++ ) {
	    nMessage* m = new nMessage();
	    *m = *message;
	    m->to = units[i].ent->fn;
	    Send( m );
	}
	/*
	   nHoverLegionAttackLegion* hlal = (nHoverLegionAttackLegion*) this->PushChild( "nhoverlegionattacklegion" );
	   hlal->SetProxyParent( message->data.Get() );
	 */
            } else {
	n_printf("unknown command: %s\n", message->title.get() );
            }
        }
    }
}
///make a list with all my units
void nLegionAspect::BuildMyUnitsList( EntArray* myUnits ) {
    this->CleanUnits();
    nHeap<nEnt*> toCheck;
    toCheck.initialize( 10 );
    for ( int i = 0; i < units; i++ ) {
        toCheck.add( 0, units[i].ent );
    }
    while ( toCheck.length() > 0 ) {
        nEnt* e = toCheck.pop();
        static nRoot* legionRoot = kernelServer->Lookup("/game/ents/legion");
        nRoot* parent = e->GetParent();
        n_assert( parent && legionRoot );
        if ( parent == legionRoot ) {                                                              //my kid is a legion
            nLegionAspect* childLegion = (nLegionAspect*) e->Find("legion");
            for ( int i = 0; i< childLegion->units; i ++ )
	toCheck.add( 0, childLegion->units[i].ent );
        } else
            myUnits->Append( e );
    }
    //n_printf("my units size: %i\n", myUnits->Size() );
}
///map attacking units to targets
void nLegionAspect::AllocateAttackers( EntArray* myUnits, EntArray* enemyUnits, nMessage* message ) {
    int i;
    vector3 p = pos.GetV();
    nHeap<nEnt*> closestEnemies;
    closestEnemies.initialize( enemyUnits->Size() );
    for ( i = 0; i < enemyUnits->Size(); i ++ ) {
        nEnt* e = enemyUnits->Get(i);
        closestEnemies.add((e->GetVarV("pos") - p).len(), e );
    }
    int allocated;
    nEnt* enemyTargetting = 0;
    switch ( this->allocationTactic ) {
        case NTO1:
            allocated = 0;
            for ( i = 0; i < myUnits->Size(); i ++ ) {
	allocated++;
	if ( enemyTargetting == 0 || allocated > N ) {
	    enemyTargetting = closestEnemies.pop();
	    allocated = 0;
	}
	//tell that child to attack
	nMessage* m = new nMessage();
	*m = *message;
	m->from = this->fn;
	m->to = myUnits->Get(i)->fn;
	m->data = enemyTargetting->fn;
	//n_printf("allocating %s to %s\n", m->to.Get(), enemyTargetting->fn );
	this->Send( m );
            }
            break;
        case CLOSEST:
            break;
    }
}

///request an upgrade for this guys units
void nLegionAspect::Upgrade( const char* varName, int level ) {
    if( strcmp( varName, "laserlevel") == 0) {
        this->Upgrade( "sbeamlevel", level );
        this->Upgrade( "mbeamlevel", level );
        this->Upgrade( "lbeamlevel", level );
    }
    else if( strcmp( varName, "misslelevel") == 0) {
        this->Upgrade( "smisslelevel", level );
        this->Upgrade( "mmisslelevel", level );
        this->Upgrade( "lmisslelevel", level );
    }
    else {
        EntArray allUnits;
        this->BuildMyUnitsList( &allUnits );
        for( int i = 0; i < allUnits; i ++ ) {
            n_assert( allUnits[i]->HasVar(varName) );
            int currentLevel = allUnits[i]->GetVarI(varName);
            //if( currentLevel < level )
            if( currentLevel != level ) {
	if( resources[level-1].GetF() > 50 ) {
	    allUnits[i]->SetVarI( varName, level );
	    resources[level-1].SetF( resources[level-1].GetF() - 50 );
	}
            }
        }
    }
}
