#define N_IMPLEMENTS nRTSArmor
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "rts/nrtsarmor.h"
#include "enkiduo/neventserver.h"

nNebulaScriptClass(nRTSArmor, "naspect");

//------------------------------------------------------------------------------
/**
*/
nRTSArmor::nRTSArmor():
    physics( kernelServer, this )
{
}

void nRTSArmor::Initialize(){
    nAspect::Initialize();
}

void nRTSArmor::InitVars(){
    nAspect::InitVars();
    hp.Connect( ent, "hp" );
    hp.DefaultF( 10.0f );
    maxhp.Connect( ent, "maxhp" );
    maxhp.DefaultF( 10.0f );
    hpRatio.Connect( ent, "hpRatio" );
    hpRatio.DefaultF( 1.0f );
    pos.Connect( ent, "pos" );
    regenerationRate.Connect( ent, "regenerationrate" );
    regenerationRate.DefaultF(0.0f);
    nEnt* options = dynamic_cast<nEnt*>(kernelServer->Lookup("/options"));
    n_assert( options && "failed to find /options || its not an ent");
    damageOn.Connect( options, "damage" );
    //damageOn.DefaultI(1);


    entname = ent->FullNameString();
    physics = ent->ChildPath("physics");
}

nRTSArmor::~nRTSArmor(){
}
//------------------------------------------------------------------------------
/**
*/

void nRTSArmor::Tick( float time, float dtime ){
    nAspect::Tick( time, dtime );
    float h = hp.GetF();
    float mh = maxhp.GetF();
    h += maxhp.GetF() * regenerationRate.GetF() * dtime;
    if( h > mh )
        h = mh;
    if( h <= 0.0 ) { //we are outta hps
        //trigger an event
        //nEventServer* eventServer = dynamic_cast<nEventServer*> (kernelServer->Lookup("/sys/servers/event") );
        //if (eventServer)
            //eventServer->pullInefficient("entityDied", ent->fn);
        //else
            //n_printf("%s killed but no eventserver found\n", this->fn);
        ent->Die();
    }
    hp.SetF( h );
    hpRatio.SetF( h / mh );
}
void nRTSArmor::Message( nMessage* message ) {
    if( message->to == entname ){
        if( message->title == "damage" ) {                                      //hit with a direct fire weapon
            float h                     = hp.GetF();
            float damage                = message->f0;
            float impulse               = message->f1;

            if( impulse > 0 ) {                                                 //weapon moves us
                if( physics.isvalid() ) {                                       //track down my physics and apply the force
                    vector3 p = pos.GetV();
                    vector3 dx = p - message->v;
                    dx.norm();
                    vector3 force = dx * impulse;
                    //force.y *= 10.0f;                                         //use this to make things tend to bounce up
                    physics->ApplyForce( force );
                }
            } 
            nString from                = message->from;                        //the bastard who shot me

            nString deflectionVarName;
            deflectionVarName.PrintF("%sdeflection", message->data.Get() );
            nVar* deflectionVar         = ent->FindVar( deflectionVarName.Get() );
            float deflection;
            if ( deflectionVar )        { deflection = deflectionVar->GetF(); }
            else                        { deflection = 0; }
            float deflected             = damage * (deflection / 100.0f);

            if( damageOn.GetI() ) {
                h                          -= damage - deflected;
                //n_printf("%s.damage:%f deflection:%f h:%f\n", this->fn, damage, deflected, h );
                hp.SetF( h );
                hpRatio.SetF( h / maxhp.GetF() );
            }
        }
    }
    if( message->title == "areadamage" ) {
        float damage = message->f0;
        float impulse = message->f1;
        float effectradius = message->f2;
        vector3 ep = message->v;
        vector3 p = pos.GetV();
        vector3 dx = p - ep;
        float d = dx.len();
        if( d < effectradius ) {                                                //move me from the impact
            if( impulse > 0 ) {                                                 //apply impact force
                if( physics.isvalid() ) {
                    if( d < 1 )
                        d = 1.0f;
                    dx.norm();
                    //direction of force * max force / 1 + distance
                    vector3 force = dx * ( impulse * ( 1.0f - (d / effectradius) ));
                    force.y *= 100.0f;
                    physics->ApplyForce( force );
                }
            } 
            if( damage > 0 ) {                                                  //apply damage
                float h = hp.GetF();
                nString deflectionVarName;
                deflectionVarName.PrintF("%sdeflection", message->data.Get() );
                nVar* deflectionVar         = ent->FindVar( deflectionVarName.Get() );
                float deflection;
                if ( deflectionVar )        { deflection = deflectionVar->GetF(); }
                else                        { 
                    deflection = 0; n_printf("warning failed to find deflection var:%s\n", deflectionVarName.Get());
                }
                float deflected             = damage * (deflection / 100.0f);

                if( damageOn.GetI() ) {
                    h -= damage - deflected;
                    //n_printf("%s.damage:%f deflection:%f h:%f\n", this->fn, damage, deflected, h );
                    hp.SetF( h );
                    hpRatio.SetF( h / maxhp.GetF() );
                }
            }
        }
    }
}
