#include "tnl/gaconnection.h"
#include "ga/nga.h"
#include "ga/ngenome.h"
#include "tnl/ganetinterface.h"
using namespace TNL;
TNL_IMPLEMENT_NETCONNECTION    (GAConnection, NetClassGroupGame, true); 
//------------------------------------------------------------------------------
//        TNLCONNECTION STUFF    
//------------------------------------------------------------------------------
GAConnection::GAConnection()
{
    myNetInterface = 0;
    static int ids = 0;
    this->ga = 0;
    this->id = ids++;
    this->state = 0;
    this->setIsAdaptive();
    this->genomeEvaluating = 0;
}
//-----------------------------------------------------------------------------//
GAConnection::~GAConnection(){
    if( myNetInterface )
        delete myNetInterface;
}

//------------------------------------------------------------------------------
/****NETWORKING RPCS*******************************************************************/
//------------------------------------------------------------------------------
TNL_IMPLEMENT_RPC    (GAConnection, rpcInitializeWorldMessage, (StringTableEntry s), (s), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirServerToClient, 0) { 
    printf("Got initialize world message from server: %s\n", s.getString()); 
} 
//-----------------------------------------------------------------------------//
TNL_IMPLEMENT_RPC    (GAConnection, rpcStartEvaluation, (TNL::Vector<bool> vec), (vec), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirServerToClient, 0) { 
    nArray<bool> bits;
    for( int i = 0; i < vec.size(); i ++ )
        bits.Append( vec[i] );
    if( this->ga ) {
        ga->StartEvaluation( bits );
    }
}
//-----------------------------------------------------------------------------//
TNL_IMPLEMENT_RPC    (GAConnection, rpcSetState, (TNL::U8 i), (i), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirAny, 0) { 
    //printf("rpcSetState(%i)\n", i );
    this->state = i;
} 
//-----------------------------------------------------------------------------//
TNL_IMPLEMENT_RPC    (GAConnection, rpcSetFitness, (TNL::F32 fitness), (fitness), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirAny, 0) { 
    //printf("rpcSetFitness(%f)\n", fitness);
    if ( this->genomeEvaluating ) {
        this->genomeEvaluating->EvaluationFinished( fitness );
        this->genomeEvaluating = 0;
    }
} 
//-----------------------------------------------------------------------------//
TNL_IMPLEMENT_RPC    (GAConnection, rpcClearPopulation, (TNL::U8 flag), (flag), NetClassGroupGameMask, RPCGuaranteedOrdered, RPCDirAny, 0) { 
    //printf("rpcClearPopulation(%i)\n", flag);
    if( ga ) 
        ga->ClearPopulation();
} 
//-----------------------------------------------------------------------------//
void GAConnection::Tick( float time, float dtime ) {
    if( myNetInterface ) {
        myNetInterface->checkIncomingPackets(); 
        myNetInterface->processConnections(); 
    } else
        printf("%i.GAConnection - no net interface\n", id);
}
//------------------------------------------------------------------------------
void GAConnection::openServer( const char* ip){
    printf("%i.GAConnection::OpenServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    myNetInterface = new GANetInterface(this, cmdAddress); 
    //myNetInterface->addNetInterfaceAddConnectionEventHandler( this );
    myNetInterface->setAllowsConnections(true); 
    //tnl->SetState("server");
}
//-----------------------------------------------------------------------------//
void GAConnection::connectToLocalServer( const char* ip, GAConnection* server ) {
    printf("%i.GAConnection::ConnectToLocalServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    Address bindAddress(IPProtocol, Address::Any, 0); 
    myNetInterface = new GANetInterface(this, bindAddress); 
    //myNetInterface->addNetInterfaceAddConnectionEventHandler( this );
    this->connectLocal( myNetInterface, server->myNetInterface );
}
//------------------------------------------------------------------------------
void GAConnection::connectToServer( const char* ip ){
    printf("%i.GAConnection::ConnectToServer(%s)\n", id, ip);
    Address cmdAddress(ip);
    Address bindAddress(IPProtocol, Address::Any, 0); 
    myNetInterface = new GANetInterface(this, bindAddress); 
    //myNetInterface->addNetInterfaceAddConnectionEventHandler( this );
    this->connect(myNetInterface, cmdAddress); 
    //tnl->SetState("client");
}
//-----------------------------------------------------------------------------//
/*
void GAConnection::connect( const TNL::Address& address ){
    printf("%i.GAConnection::ConnectToServer(%s)\n", id, address.toString());
    //Address cmdAddress(ip);
    Address bindAddress(IPProtocol, Address::Any, 0); 
    myNetInterface = new GANetInterface(bindAddress); 
    myNetInterface->addNetInterfaceAddConnectionEventHandler( this );
    this->connect(myNetInterface, address); 
}
*/
//------------------------------------------------------------------------------
void GAConnection::packetDropped    ( PacketNotify* notify ){
    //printf("%i.GAConnection::packetDropped()\n", id);
    Parent::packetDropped( notify );
}
void GAConnection::packetReceived    ( PacketNotify* notify ){
    //printf("%i.GAConnection::packetReceived\n", id);
    Parent::packetReceived( notify );
}
void GAConnection::writePacket    ( BitStream* stream, PacketNotify* notify){
    //printf("GAConnection::writePacket\n");
    Parent::writePacket( stream, notify );
}
void GAConnection::readPacket    ( BitStream* stream ){
    //printf("GAConnection::readPacket\n");
    Parent::readPacket( stream );
}
void GAConnection::processEvent    ( NetEvent* event ){
    //printf("%i.GAConnection::processEvent\n", id);
    Parent::processEvent( event );
}
void GAConnection::writeConnectRequest    ( BitStream* stream ){
    //printf("%i.GAConnection::writeConnectRequest\n", id);
    Parent::writeConnectRequest( stream );
}
bool GAConnection::readConnectRequest    ( BitStream* stream, const char** errorString ){
    //printf("%i.GAConnection::readConnectRequest %s\n", id, errorString ? *errorString : "none" );
    return Parent::readConnectRequest( stream, errorString );
}
void GAConnection::writeConnectAccept    ( BitStream* stream ){
    //printf("%i.GAConnection::writeConnectAccept\n", id);
    Parent::writeConnectAccept( stream );
}
bool GAConnection::readConnectAccept    ( BitStream* stream, const char** errorString ){
    //printf("%i.GAConnection::readConnectAccept %s\n", id, errorString ? *errorString : "none" );
    return Parent::readConnectAccept( stream, errorString );
}
void GAConnection::onConnectionEstablished() {
    Parent::onConnectionEstablished();
    printf("%i.GAConnection::onConnectionEstablished()\n", id );
}
void GAConnection::onAddConnection( NetConnection* netConnection ) {
    n_printf("%i.GAConnection::addConnection(%p)\n", id, netConnection);
    GAConnection* connection = dynamic_cast<GAConnection*> (netConnection);
    if( connection ) {
        n_printf("connection: %i\n", connection->id);
        if( this->ga )
            this->ga->AddSlaveConnection( connection ); 
        else
            printf("ERROR -> onAddConnection cannot find parent nga\n");
    } else
        printf("ERROR -> onAddConnection with something thats not a gaconnection\n");
}
