#define N_IMPLEMENTS nTurret
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "turret/nturret.h"
#include "node/nmeshnode.h"
#include "gfx/nvertexbuffer.h"
#include "node/nlinknode.h"
#include "util/nheap.h"
#include "aspect/nside.h"
#include "aspect/nsoundemitter.h"
#include "enkiduo/nprofserver.h"
//-----------------------------------------------------------------------------//
nNebulaScriptClass(nTurret, "noffsetaspect");
//------------------------------------------------------------------------------
nTurret::nTurret():
    game( kernelServer, this),
    emitter	( kernelServer, this ) {
    game = "/sys/servers/game";
}
//------------------------------------------------------------------------------
void nTurret::Initialize(){
    nOffsetAspect::Initialize();
}
//------------------------------------------------------------------------------
void nTurret::InitVars(){
    nOffsetAspect::InitVars();
    mother.Connect                      ( ent, "mother" );                      mother.DefaultRef                       ("");
    side.Connect                        ( mother, "side" );
    fires.Connect                       ( ent, "fires" );                       fires.DefaultS                          ("");  
    firing.Connect                      ( ent, "firing" );                      firing.DefaultI                         (1);
    cooldown.Connect                    ( ent, "cooldown" );                    cooldown.DefaultF                       (1.0f);
    autotarget.Connect                  ( ent, "autotarget" );                  autotarget.DefaultI                     (1);
    autoTargetFreq.Connect              ( ent, "autoTargetFreq" );              autoTargetFreq.DefaultF                 (0.1f);
    target.Connect                      ( ent, "target" );                      target.DefaultRef                       ("");
    targetPos.Connect                   ( target, "pos" );
    lastFired = -1.0f;

    nEnt* options = dynamic_cast<nEnt*>(kernelServer->Lookup("/options"));
    n_assert( options && "failed to find /options || its not an ent");
    firingEnabled.Connect( options, "firing" );
    emitter = ent->ChildPath("emitter");
    this->lastAutoTarget = -1.0f;
}
//------------------------------------------------------------------------------
nTurret::~nTurret(){
}
//------------------------------------------------------------------------------
void nTurret::Tick( float time, float dtime ){
    nOffsetAspect::Tick( time, dtime );
    if( ent->Held() ) return;
    this->time = time;
    if ( firing.GetI() > 0 && this->CanFire() ) {
        if ( this->target.isvalid() == false && this->autotarget.GetI() > 0 ) {
            if( lastAutoTarget < 0 || time - lastAutoTarget > autoTargetFreq.GetF() ) {
                this->AutoTarget();
                lastAutoTarget = time;
            }
        }
        this->Fire();
    }
}
//-----------------------------------------------------------------------------//
void nTurret::Fire(){
    if ( this->CanFire() == false ) return;
    //n_printf("nTurret::Fire()\n");
    ifvalid( emitter ) {
        nString soundName = fires.GetS() + "fire";
        //n_printf("playing: %s\n", soundName.Get());
        emitter->Queue(soundName.Get());
    }
    this->lastFired = this->time;
}
//-----------------------------------------------------------------------------//
bool nTurret::CanFire(){ 
    return firingEnabled.GetI() && (lastFired < 0 || time - lastFired > cooldown.GetF());
}
//TODO - explore optomizations - in particular not locking / unlocking the buffer
vector3 nTurret::GetTargetPoint( nEnt* target ){
    if ( target == 0 ) {                n_printf("no target\n");                return vector3();                       }
    n3DNode* vis                        = target->GetVis();
    if ( vis == 0 ) {                   n_printf("no vis\n");                   return vector3();                       }
    nLinkNode* link                     = (nLinkNode*) vis->Find("link");
    if ( link == 0 ) {                  n_printf("no link\n");                  return vector3();                       }
    nVisNode* node                       = link->GetTargetPtr();
    if ( node == 0 ) {                  n_printf("no node\n");                  return vector3();                       }
    nMeshNode* mesh                     = (nMeshNode*) node->Find("mesh");
    if ( mesh == 0 ) {                  n_printf("no mesh\n");                  return vector3();                       }
    nVertexBuffer* buffer               = mesh->GetVertexBuffer();
    if ( buffer == 0 ) {                n_printf("no buffer\n");                return vector3();                       }
    int numCoords                       = buffer->GetNumVertices();
    int coordIndex                      = randI( 0, numCoords-1);
    buffer->LockVertices();
    vector3 coord = buffer->GetCoord(coordIndex);
    buffer->UnlockVertices();
    coord.x *= -1.0f;
    coord.z *= -1.0f;
    return coord;
};
//-----------------------------------------------------------------------------//
void nTurret::AutoTarget(){
    //n_printf("nTurret::AutoTarget()\n");
    prof->Begin("Turret.AutoTarget");
    static nHeap<nEnt*> heap(128);
    heap.clear();

    float dist;
    vector3 p = pos.GetV();
    //new way..pull the list of enemy factions
    //then pull their list of units, and scan them
    nEnt* sideEnt = (nEnt*) side.GetRef();                                      n_assert( sideEnt );
    nSide* sidePtr = (nSide*) sideEnt->Find("side");                            n_assert( sidePtr );
    const SideArray* hostiles = sidePtr->GetRelatedSides(HOSTILE);              n_assert( hostiles );
    nSide* hs = 0;
    nEnt* e = 0;
    //n_printf("se:%s sp:%s hostiles:%i\n", sideEnt->fn, sidePtr->fn, hostiles->Size() );
    for( int i = 0; i < hostiles->Size(); i ++ ) {
        hs = hostiles->Get(i);
        const EntArray* units = hs->GetUnits();
        //n_printf("units:%i\n", units->Size() );
        for( int i = 0; i < units->Size(); i ++ ) {
            e = units->Get(i);
            //static nRoot* legionRoot = kernelServer->Lookup("/game/ents/legion");
            //if( e->GetParent() != legionRoot ) {
            //if( e->HasVar("stealthPoints") && e->GetVarF("stealthPoints") > 0.0f )
                //continue;               //skip me im hidden
            if( e->IsAlive() ) {
                vector3 tp = e->GetVarV("pos");
                vector3 dx = tp - p;
                dist = dx.len();
                dist *= randF(.9f, 1.1f );
                //n_printf("p:%s tp:%s dist:%f ", str(p), str(tp), dist );
                //dist = 1.0f / dist;
                //n_printf("1/d:%f\n", dist );
                heap.add( dist, e );
            }
            //}
        }
        e = 0;
    }
    if( heap.length() > 0 ) {
        heap.pop( dist, e );
        //n_printf("%s AutoTargetting %s\n", ent->fn, e->fn);
        target.SetRef( e );
    }
    prof->End("Turret.AutoTarget");
}
