#define N_IMPLEMENTS nPlannerUI
//------------------------------------------------------------------------------
//  (C) 2004	Chris Miles
//------------------------------------------------------------------------------
#include "ui/nplannerui.h"
#include "ui/nuielement.h"
#include "enkiduo/nfsmqueue.h"
#include "enkiduo/neventserver.h"
#include "ui/nplanner.h"
#include "ui/ninterfaceserver.h"
#include "util/nheap.h"
#include "enkiduo/colours.h"

nNebulaScriptClass(nPlannerUI, "nfsm");

//------------------------------------------------------------------------------
/**
 */
nPlannerUI::nPlannerUI():
    gameServer        ( kernelServer, this ),
    octree            ( kernelServer, this ),
    eventServer       ( kernelServer, this ),
    interface         ( kernelServer, this ),
    refCategoryServer ( kernelServer, this )
{
}

void nPlannerUI::Initialize(){
    nFSM::Initialize();
    stateNames.Append("disabled");
    stateNames.Append("normal");
    stateNames.Append("leftmousebuttondown");
    stateNames.Append("middlemousebuttondown");
    stateNames.Append("rightmousebuttondown");
    stateNames.Append("entplacementpre");
    stateNames.Append("entplacement");
    stateNames.Append("multipleEntPlacementPre");
    stateNames.Append("multipleEntPlacement");
    stateNames.Append("entRotation");
    stateNames.Append("boxSelection");
    stateNames.Append("RMenu");

    gameServer = "/sys/servers/game";
    octree = "/sys/servers/octree";
    eventServer = "/sys/servers/event";
    interface = "/ui/interface";
    AddToDisplayList();

    selectionEvent = 0;
    mouseOverEvent = 0;
    rightClickEvent = 0;
    rmenu                               = 0;

    this->createVisNode(this->fn);

    multiSelect                         = false;

    refCategoryServer                   = "/sys/servers/category";
    n_assert                            ( refCategoryServer.isvalid() && "nPlannerUI cannot find categoryServer");
    categoryOver                        = refCategoryServer->CreateCategory("mouseOver",     "/lib/node/overlay/mouseOver");
    categoryOverGroup                   = refCategoryServer->CreateCategory("overGroup",     "/lib/node/overlay/mouseOverGroup");
    categorySelected                    = refCategoryServer->CreateCategory("selected",      "/lib/node/overlay/selected");
    categorySelectedGroup               = refCategoryServer->CreateCategory("selectedGroup", "/lib/node/overlay/selectedGroup");
    categoryTargetted                   = refCategoryServer->CreateCategory("targetted",     "/lib/node/overlay/targetted");
    categoryCatSelected                 = refCategoryServer->CreateCategory("catSelected",   "/lib/node/overlay/catSelected");
    //nEventServer* eventServer           = LOOKUP(nEventServer*, "/sys/servers/event" );
    //eventServer->registerCFunction      ( "entityDestroyed", this, 'RDEL' );
    //
    ticksSinceLastBoxSelection = 0;
}

void nPlannerUI::InitVars(){
    nFSM::InitVars();
    
    selected.Connect( ent, "selected" );
    selected.DefaultRef("");
    mousePos.Connect( ent, "mousepos" );
    selectionRadius.Connect( ent, "selectionradius");
    selectionRadius.DefaultF(10);
    selectionThreshold.Connect( ent, "selectionthreshold" );
    selectionThreshold.DefaultF( 0.995f );
    rotateYSensitivity.Connect( ent, "rotateysensitivity");
    rotateYSensitivity.DefaultF( 360.0f );

    camera.Connect( ent, "camera" );
    camera.DefaultRef ("/game/camera");
    cameraPos.Connect( camera, "pos" );
    cameraRot.Connect( camera, "rot" );

    entbeingplaced.Connect( ent, "entbeingplaced");
    entbeingplaced.DefaultRef("");
    entbeingplacedpos.Connect( entbeingplaced, "pos");

    windowSettings.Connect( ent, "windowSettings");
    windowSettings.DefaultRef("/sys/share/windowsettings");
    aspectRatio.Connect( windowSettings, "aspectRatio" );

    rightClickTime.Connect(ent, "rightClickTime");
    rightClickTime.DefaultF(.05f);
}

nPlannerUI::~nPlannerUI(){
}
/**-----------------------------------------------------------------------------//
 * watches the mouse for any selection / command events
--------------------------------------------------------------------------------*/
bool nPlannerUI::States( StateMachineEvent event, uint state, float time, float ){   
    MouseInfo mouseInfo;
    ifvalid( gameServer )
        mouseInfo = gameServer->GetMouseInfo();
    vector2 mouseDiff = mouseInfo.pos - lastMouseInfo.pos;
    lastMouseInfo = mouseInfo;
    this->UpdateObjectsOver();
    if( camera.isvalid() == false ) 
        return false;
    float height = cameraPos.GetV().y;
    height /= 100.0f;

    if( state == Disabled ) 
        return false;

    /*
    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();
        */
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    //-----------------------------------------------------------------------------//
    BeginStateMachine
        //-----------------------------------------------------------------------------//
        State ( Disabled )
            n_error("cannot get here");
        //-----------------------------------------------------------------------------//
        State( Normal )
            OnTick
                if( mouseInfo.pressed[0] )
                    SetState( LeftMouseButtonDown );
                else if( mouseInfo.pressed[1] )
                    SetState( MiddleMouseButtonDown );
                else if( mouseInfo.pressed[2] )
                    SetState( RightMouseButtonDown );
        //-----------------------------------------------------------------------------//
        State( LeftMouseButtonDown )
            OnEnter
                this->MouseEvent( 0 );
                timedown[0] = time;
            OnTick
                if ( mouseInfo.pressed[0] == false ) {
                    SetState( Normal );
                    printf("ticksSinceLastBoxSelection:%i\n", ticksSinceLastBoxSelection++);
                }
                else if (time - timedown[0] > .2f)
                    this->SetState(BoxSelection);
            //-----------------------------------------------------------------------------//
        State( MiddleMouseButtonDown )
            OnEnter
                //this->MouseEvent( 1 );
                timeRightMouseButtonDown = time;
            OnTick
                //if( mouseInfo.pressed[1] == false )
                    //SetState( Normal );
                /*
                float ry = mouseDiff.y * rotateYSensitivity.GetF();
                for( i = 0; i < objectsSelected; i ++ ) {
                    quaternion q = objectsSelected[i]->GetVarQ("rot");
                    quaternion q1;
                    q1.set_rotate_y( n_deg2rad(ry) );
                    q = q1 * q;
                    objectsSelected[i]->SetVarQ("rot", q );
                }
                */
                /*
                float timeDown = time - timeMiddleMouseButtonDown;
                if (timeDown > rightClickTime.GetF() ) {  //popupmenu
                    this->catMenu = true;
                    this->SetState(RMenu);
                }
                */
                if( mouseInfo.pressed[1] == false ) { //middle click
                    this->MouseEvent( 1 );
                    SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( RightMouseButtonDown )
            //the right mouse button is down, if they raise it quickly then pull a right click event, otherwise go into an open popup menu state
            OnEnter
                timeRightMouseButtonDown = time;
            OnTick
                float timeDown = time - timeRightMouseButtonDown;
                if (timeDown > rightClickTime.GetF() ) {  //popupmenu
                    //this->catMenu = false;
                    this->SetState(RMenu);
                }
                else if( mouseInfo.pressed[2] == false ) { //right click
                    this->MouseEvent( 2 );
                    SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( EntPlacementPre )		//otherwise it will pull the same click that activated the button thats trying to place the ent
            OnEnter
            OnTick
                if( mouseInfo.pressed[0] == false ) {
                    this->SetState( EntPlacement );
                }
        //-----------------------------------------------------------------------------//
        State( MultipleEntPlacementPre )
            OnEnter
            OnTick
                if( mouseInfo.pressed[0] == false )
                    this->SetState( MultipleEntPlacement );
        //-----------------------------------------------------------------------------//
        State( EntPlacement )
            OnEnter
                n_assert( entbeingplaced.isvalid() );
            OnExit
                entbeingplaced.SetRef("");
            OnTick
                ifvalid( entbeingplaced )
                    entbeingplacedpos.SetV( mousePos.GetV() );
                if( mouseInfo.pressed[0] == true ) {                            //place it
                    this->SetState( Normal );
                    static void* v;
                    eventServer->pull(v, "entPlaced", str(mousePos.GetV()));
                }
                else if( mouseInfo.pressed[2] == true ) {                       //destroy it
                    entbeingplaced.GetRef()->Die();
                    this->SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( BoxSelection )
            OnTick
                if ( mouseInfo.pressed[0] == false ) {
                    this->BoxSelect();
                    SetState( Normal );
                }
        //-----------------------------------------------------------------------------//
        State( MultipleEntPlacement )
            OnExit
            OnTick
                //n_printf("MultipleEntPlacement(%i,%i,%i)\n", mouseInfo.pressed[0], mouseInfo.pressed[1], mouseInfo.pressed[2]);
                for( int i = 0; i < entsBeingPlaced; i ++ ) {
                    entsBeingPlaced[i].pos.SetV(mousePos.GetV() + entsBeingPlaced[i].offset);
                    entRotationCenter = mousePos.GetV();
                }
                if( mouseInfo.pressed[0] == true ) {
                    n_printf("finished placing ent\n");
                    this->SetState(Normal);
                    static void* v;
                    eventServer->pull(v, "entPlaced", str(mousePos.GetV()));
                    entsBeingPlaced.Clear();
                }
                if( mouseInfo.pressed[2] == true )
                    this->SetState(EntRotation);
        //-----------------------------------------------------------------------------//
        State( EntRotation )
            OnTick
                //n_printf("EntRotation(%i,%i,%i)\n", mouseInfo.pressed[0], mouseInfo.pressed[1], mouseInfo.pressed[2]);
                float ry = (mouseDiff.x * .1f + mouseDiff.y) * rotateYSensitivity.GetF();
                for( int i = 0; i < entsBeingPlaced; i ++ ) {
                    quaternion q = entsBeingPlaced[i].ent->GetVarQ("rot");
                    quaternion q1;
                    q1.set_rotate_y( n_deg2rad(ry) );
                    q = q1 * q;
                    entsBeingPlaced[i].ent->SetVarQ("rot", q );
                    entsBeingPlaced[i].offset = q1.rotate( entsBeingPlaced[i].offset );
                    entsBeingPlaced[i].pos.SetV(entRotationCenter + entsBeingPlaced[i].offset);
                }
                if ( mouseInfo.pressed[2] == false ) {
                    this->SetState(MultipleEntPlacement);
                }
        //-----------------------------------------------------------------------------//
        State( RMenu )
            ///so the user has open a right click menu -> what we need to do here is find the actual menu they want, and then open it
            ///okay so we need to check if the user has rightclicked on a unit, or in the open world or something
            OnEnter
                //n_printf("Rmenu.enter\n");
                n_assert (this->rmenu == 0);
                nString rmenuPath = "/ui/rmenus/command/";

                enum {
                    RMENU_CATEGORY_WORLD,
                    RMENU_CATEGORY_UNIT,
                    RMENU_CATEGORY_UNIT_TO_WORLD,
                    RMENU_CATEGORY_UNIT_TO_UNIT,
                } category;

                ///first of all determine teh general context of the menu event -> is it over a single entity, the world, or between entities
                if ( objectsSelected.Size() > 0 ) { 
                    if ( objectsOver.Size() > 0  ) {  //we have something selected, and our mouse is over something
                        category = RMENU_CATEGORY_UNIT_TO_UNIT;
                        for( int i = 0; i < objectsOver.Size(); i ++ ) { //search to see if the object over is one of the selected entities
                            for( int j = 0; j < objectsSelected.Size(); j ++ ) {
                                if( objectsOver[i] == objectsSelected[j] ) {
                                    category = RMENU_CATEGORY_UNIT;
                                }
                            }
                        }
                    } else {
                        n_assert( objectsOver.Size() == 0 );
                        category = RMENU_CATEGORY_UNIT_TO_WORLD;
                    }
                } else {
                    n_assert( objectsSelected.Size() == 0 );
                    if( objectsOver.Size() > 0 ) {
                        category = RMENU_CATEGORY_UNIT;
                    } else {
                        n_assert( objectsOver.Size() == 0 );
                        category = RMENU_CATEGORY_WORLD;
                    }
                }
                nString unitRelationship, unitType, possibleRMenuPath, fromUnitType, fromUnitRelationship, toUnitType, toUnitRelationship;
                //-----------------------------------------------------------------------------//
                ///find the path to the appropriate menu to display
                //-----------------------------------------------------------------------------//
                switch( category ){
                    //-----------------------------------------------------------------------------//
                    case RMENU_CATEGORY_WORLD:
                        rmenuPath += "world";                   //right clicking on the world
                        break;
                    //-----------------------------------------------------------------------------//
                    ///menu for a particular unit or group
                    //-----------------------------------------------------------------------------//
                    case RMENU_CATEGORY_UNIT:
                        ///most specific menu -> relationship + type
                        unitRelationship                                = "friendly";
                        unitType                                        = objectsOver[0]->GetParent()->GetName();
                        possibleRMenuPath                               = rmenuPath + unitRelationship + unitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get())){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        ///next most specific menu -> type
                        unitType                                        = objectsOver[0]->GetParent()->GetName();
                        possibleRMenuPath                               = rmenuPath + unitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get() )){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        //most general
                        rmenuPath += "unit";
                        break;
                    //-----------------------------------------------------------------------------//
                    ///ordering a unit to the world 
                    //-----------------------------------------------------------------------------//
                    case RMENU_CATEGORY_UNIT_TO_WORLD:
                        unitRelationship                                = "friendly";

                        ///most specific menu -> relationship + type
                        unitRelationship                                = "friendly";
                        unitType                                        = objectsSelected[0]->GetParent()->GetName();
                        possibleRMenuPath                               = rmenuPath + unitRelationship + unitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get())){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        ///next most specific menu -> type
                        unitType                                        = objectsSelected[0]->GetParent()->GetName();
                        possibleRMenuPath                               = rmenuPath + unitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get() )){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        ///most general 
                        rmenuPath += "unitToWorld";
                        break;
                    //-----------------------------------------------------------------------------//
                    ///ordering a unit to another unit
                    //-----------------------------------------------------------------------------//
                    case RMENU_CATEGORY_UNIT_TO_UNIT:
                        ///most specific menu -> relationship + type
                        fromUnitRelationship                            = "friendly";
                        fromUnitType                                    = objectsSelected[0]->GetParent()->GetName();
                        toUnitRelationship                              = "hostile";
                        toUnitType                                      = objectsOver[0]->GetParent()->GetName();

                        possibleRMenuPath                               = rmenuPath + fromUnitRelationship + fromUnitType + "to" + toUnitRelationship + toUnitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get())){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        ///next most specific menu -> type
                        fromUnitType                                    = objectsSelected[0]->GetParent()->GetName();
                        toUnitType                                      = objectsOver[0]->GetParent()->GetName();

                        possibleRMenuPath                               = rmenuPath + fromUnitType + "to" + toUnitType;
                        if ( LOOKUP(nEnt*, possibleRMenuPath.Get())){
                            n_printf("using specific rmenu:%s\n", possibleRMenuPath.Get());
                            rmenuPath = possibleRMenuPath;
                            break;
                        }

                        ///most general 
                        rmenuPath += "unitToUnit";
                        break;
                    //-----------------------------------------------------------------------------//
                    default:
                        n_assert( 0 && "unknown RMENU_CATEGORY");
                        break;
                }

                ///find the approriate rmenu and display it
                rmenu = (nEnt*) kernelServer->Lookup(rmenuPath.Get());
                if ( rmenu == 0 )
                    n_printf("failed to find appropriate rmenu to show: %i %i %s\n", objectsSelected.Size(), objectsOver.Size(), rmenuPath.Get());
                else {
                    //n_printf("rmenu:%p->%s\n", rmenu, rmenu->fn);
                    rmenu->SetVarI("menuActive", 1);

                    rmenu->CreateVar("mousePos")->SetV(mousePos.GetV());
                    rmenu->CreateVar("objectsOver")->SetS(this->getObjectsOverString());
                    rmenu->CreateVar("objectsSelected")->SetS(this->getObjectsSelectedString());
                    vector3 ringCenter = interface->MousePosTo0Centered( mouseInfo.posPressed[2] );

                    /*ringCenter.x            = mouseInfo.posPressed[2].x;
                    ringCenter.y            = mouseInfo.posPressed[2].y;
                    ringCenter.x            = (ringCenter.x - .5) * 2.0f * aspectRatio.GetF();
                    ringCenter.y            = (ringCenter.y - .5) * -2.0f;
                    */
                    rmenu->SetVarV          ("ringCenter", ringCenter.x, ringCenter.y, 0);
                }
            OnExit
                if ( rmenu ) {
                    rmenu->SetVarI          ("menuActive", 0);
                    rmenu = 0;
                } else {
                    n_warn("exiting out of non open rmenu\n");
                }
            OnTick
                if ( mouseInfo.pressed[2] == false ) {
                    if( this->changeState == false ) //just incase the rmenu event changed my own state
                        SetState("normal");
                }
            //-----------------------------------------------------------------------------//
    EndStateMachine
}
/**-----------------------------------------------------------------------------//
* makes a list of all ents that exist in the world.
--------------------------------------------------------------------------------*/
void GatherEnts( nArray<nEnt*>& nodes, nRoot* r, nClass* entClass ){
    for( nRoot* node = r->GetHead(); node; node = node->GetSucc() ){
        if( node->IsA( entClass ) )
            nodes.Append( (nEnt*)node );
        GatherEnts( nodes, node, entClass );
    }
}
/**-----------------------------------------------------------------------------//
 * walk through all the entites and determine which of them are under the mouse
 * when an entity moves under the mouse we should create a link node to it, which we then bind to the /lib/node/overlay/over
 * we should probably create quick functions to add remove entities from the categories
--------------------------------------------------------------------------------*/
void nPlannerUI::UpdateObjectsOver() {
    //classVar( nent, entClass );
    float aspect = 1.0f;
    ifvalid( windowSettings )
        aspect = aspectRatio.GetF();
    //bool pullHook = false;
    bool mouseOverChanged = false;

    //const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    ///plain old left click selection
    if( this->currState.localState != BoxSelection ) {
        float bestVal = -1.0f;
        nEnt* bestEnt = 0;
        ifvalid( camera ) {
            nRoot* r = kernelServer->Lookup("/game/ents");
            if( r ) {
                vector3 p                   = cameraPos.GetV();
                quaternion q                = cameraRot.GetQ();
                //vector3 forward 	(0,0,-1);

                ///calculate where the mouse is located in 3d space
                MouseInfo mi                = gameServer->GetMouseInfo();
                vector3 forward = interface->MousePosTo0Centered( mi.pos );
                //vector2 mp;
                //mp.x = (1-mi.pos.x) * 2.0f - 1.0f;
                //mp.x *= aspect;
                //mp.y = mi.pos.y  * 2.0f - 1.0f;

                //forward.set( -mp.x, -mp.y, -1.0);
                forward.z = -1;
                forward.norm();

                forward = q.rotate( forward );
                classVar( nent, entClass );
                nArray<nEnt*> nodes;
                GatherEnts( nodes, r, entClass );
                nEnt* node;
                foreach( node, nodes ) {
                    if( node->IsA( entClass ) ) {
                        nEnt* e = ( nEnt* ) node;
                        if( e->IsAlive() && e->GetSelectable() ) {
                            vector3 tp = e->GetVarV("pos");
                            ///compare the ray from eye to mouse pos versus eye to other entities
                            ///if they are similar enough then the mouse is over them
                            vector3 dp = tp - p;
                            float dot = dp % forward;
                            dot /= dp.len();
                            float val = dot;
                            if( bestEnt == 0 || val > bestVal ) {
                                bestVal = val;
                                bestEnt = e;
                            }
                        }
                    }
                }
            }
        }
        if( bestEnt ) {                                                         ///if we atleast found something
            if( bestVal < 1 - ((1 - selectionThreshold.GetF()) * interface->GetFieldOfView())) {                         ///but its too far from the mouse to qualify toss it out
                bestEnt = 0;                                                    ///doesn't qualify
            }
        }

        if( objectsOver.Size() == 0 && bestEnt == 0 ) {                         ///do nothing, mouse was and is over nothing
        } else if( objectsOver.Size() > 0 && objectsOver[0] == bestEnt ) {     ///do nothing the mouse is over the same entity it was over last tick
        } else if( objectsOver.Size() == 0 && bestEnt ){                        ///mouse has moved over something
            //n_printf("was over nothing now over%s\n", str(bestEnt));
            mouseOverChanged |= refCategoryServer->AddEntToCategory( categoryOver, bestEnt);
        } else if( objectsOver.Size() > 0 && bestEnt == 0 ) {                  ///mouse was over something, but not anymore
            //n_printf("was over %s now over nothing\n", str(objectsOver) );
            EntArray newObjectsOver;
            mouseOverChanged |= refCategoryServer->UpdateCategory( categoryOver, newObjectsOver );
        } else if ( objectsOver.Size() > 0 && bestEnt && objectsOver[0] != bestEnt ) { ///mouse was over an entity, but now its over a different entity
            //n_printf("was over %s now over %s\n", str(objectsOver), bestEnt );
            EntArray newObjectsOver;
            newObjectsOver.Push() = bestEnt;
            mouseOverChanged |= refCategoryServer->UpdateCategory( categoryOver, newObjectsOver );
        } else {
            //n_printf("objectsOver:%s bestEnt:%s\n", str(objectsOver), str(bestEnt));
            n_assert( 0 && "a mouse over case ihad not anticipated has happened");
        }
    } else {
        EntArray newObjectsOver;
        nRoot* r = kernelServer->Lookup("/game/ents");
        if( r ) {
            vector3 p                   = cameraPos.GetV();
            quaternion q                = cameraRot.GetQ();
            vector3 forward 	(0,0,-1);
            forward                     = q.rotate(forward);
            //float dot1                  = forward % (p - boxSelectionPoints[0]);
            //float dot2                  = forward % (p - boxSelectionPoints[1]);
            matrix44 vrw                = camera.GetRef()->GetM();
            matrix44 vrw_inverted       = vrw;
            vrw_inverted.invert_simple();

            MouseInfo mi                = gameServer->GetMouseInfo();
            vector3 p0 = interface->MousePosTo0Centered( mi.posPressed[0] );
            vector3 p1 = interface->MousePosTo0Centered( mi.pos );
            /*
            vector2& p0 = mi.posPressed[0];
            vector2& p1 = mi.pos;
            //n_printf("originalpos:%s,%s\n", str(p0), str(p1));
            p0.x = (1-p0.x) * 2.0f - 1.0f;
            p0.x *= aspect;
            p0.y = p0.y  * 2.0f - 1.0f;
            p1.x = (1-p1.x) * 2.0f - 1.0f;
            p1.x *= aspect;
            p1.y = p1.y  * 2.0f - 1.0f;
            p0 = p0 * -1.0f;
            p1 = p1 * -1.0f;
            */
            //n_printf("manipulated:%s,%s\n", str(p0), str(p1));
            //vector2 v[2];
            #define draw(p0,p1) this->DrawLine(p + q.rotate(p0), p+q.rotate(p1), white);
            draw(vector3(p0.x,p0.y,-1), vector3(p0.x,p1.y,-1));
            draw(vector3(p0.x,p0.y,-1), vector3(p1.x,p0.y,-1));
            draw(vector3(p0.x,p1.y,-1), vector3(p1.x,p1.y,-1));
            draw(vector3(p1.x,p0.y,-1), vector3(p1.x,p1.y,-1));
            #undef draw

            foreachchild(nThing, enttype, r){
                foreachchild(nEnt, ent, enttype){
                    if (ent->GetSelectable()) {
                        //n_printf("ent:%s  ", ent->fn);
                        vector3 ep = ent->GetVarV("pos");
                        ep = vrw_inverted * ep;
                        if (ep.z < 0 ){ 
                            ep.x /= -ep.z;
                            ep.y /= -ep.z;
                            ep.z /= -ep.z;
                            //n_printf("%s->%s %s %s\n", ent->fn, str(ep), str(p0), str(p1));
                            bool xbetween = ep.x >= p0.x && ep.x <= p1.x || ep.x >= p1.x && ep.x <= p0.x;
                            bool ybetween = ep.y >= p0.y && ep.y <= p1.y || ep.y >= p1.y && ep.y <= p0.y;
                            if ( xbetween && ybetween ) {
                                //this->DrawPoint( p + q.rotate(ep), white, .01);
                                newObjectsOver.Append( ent );
                                //objectsOver.Append(ent);
                            }
                            //else if( xbetween || ybetween )
                                //this->DrawPoint( p + q.rotate(ep), green, .01);
                            //else
                                //this->DrawPoint( p + q.rotate(ep), red, .01);
                        }
                    }
                }
            }
        }
        mouseOverChanged |= refCategoryServer->UpdateCategory( categoryOver, newObjectsOver );
    }
    ///Old bbox method
    /*
       float r = selectionRadius.GetF();
       vector3 p = mousePos.GetV();
       bbox3 bbox(p-vector3(r,r,r), p+vector3(r,r,r));
       nOctreeElement* elems[1024];
       int numElems = octree->CollectByBBox( bbox, (nOctElement**) elems, 1024 );
       */
    if ( mouseOverChanged ) {
        EntArray overGroupEntities;
        this->FindGroupMembers              ( objectsOver,       overGroupEntities );
        refCategoryServer->UpdateCategory   ( categoryOverGroup, overGroupEntities );
        eventServer->pull( mouseOverEvent, "mouseOverChanged", getObjectsOverString().Get());
    }
}
/**-----------------------------------------------------------------------------//
  called when a mouse event happens - ie a button is pressed
--------------------------------------------------------------------------------*/
void nPlannerUI::MouseEvent( int button ) {
    //n_printf("MouseEvent(%i)\n", button);
    bool blocked = blockedCounter > 0;
    //n_printf("blocked:%i\n", blocked);
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    int numSelected                 = objectsSelected.Size();
    bool* matched;
    EntArray newObjects;
    bool selectionChanged = false;
    for( int i = 0; i < planners.Size(); i ++ ) {
        blocked = blocked || planners[i]->MouseEvent( button, &objectsSelected, &objectsOver, mousePos.GetV() );
        //if( blocked )
        //n_printf("%s blocked\n", planners[i]->GetName() );
    }
    int offset = 0;
    if( blocked )
        return;
    switch( button ) {
        case 0:		//left - new objects selected
            ///to do selection determine which entities were selecteed before, and if the entities selected / unselected changes then unselect / select them appropriately

            matched                     = new bool [numSelected];
            memset                      (matched, 0, sizeof(bool)*numSelected);
            for( int i = 0; i < objectsOver; i ++ ) {
                bool found              = false;
                for( int j = 0; j < numSelected; j ++ )
                    if( objectsSelected[j] == objectsOver[i] ) {
                        found           = true;
                        matched[j]      = true;
                    }
                //n_printf("search for ent:%s == %i\n", objectsOver[i]->fn, found );
                if( !found )
                    newObjects.Append( objectsOver[i]);
            }
            for( int i = 0; i < numSelected; i ++ ) {
                if( matched[i] == false ) {
                    //n_printf("unselecting: %s\n", objectsSelected[i+offset]->fn);
                    objectsSelected[i + offset]->Deselect();                             //remove it from objects selected
                    selectionChanged = true;
                    offset--;
                }
            }
            delete[] matched;

            this->multiSelect = true;
            for ( int i = 0; i < newObjects; i ++ ) {
                //n_printf("selecting: %s\n", newObjects[i]->fn);
                newObjects[i]->Select();
                selectionChanged = true;
            }
            this->multiSelect = false;

            if ( selectionChanged )
                this->UpdateSelection();
            break;
        case 1:		//middle
            break;
        case 2:		//right
            //n_printf("right button event\n");
            if( objectsSelected.Size() == 0 ) {
                nMessage* m = new nMessage();    
                m->from     = ent->fn;
                m->to       = "unselected";
                m->title    = "command";
                m->v        = mousePos.GetV();
                if( game->GetButton("shift") )
                    m->string = "queue";
                else if( game->GetButton("ctrl"))
                    m->string = "push";
                printf("string:%s\n", m->string.Get() );
                Send( m );                
            }
            for( int i = 0; i < objectsSelected.Size(); i ++ ){
                if( objectsSelected[i]->Find("ai") ) {
                    nMessage* m = new nMessage();    
                    m->from     = ent->fn;
                    m->to       = objectsSelected[i]->fn;
                    m->title    = "command";
                    m->v        = mousePos.GetV();
                    if( game->GetButton("shift") )
                        m->string = "queue";
                    else if( game->GetButton("alt"))
                        m->string = "push";
                    //printf("string:%s\n", m->string.Get() );
                    if( objectsOver.Size() > 0 )
                        m->data = objectsOver[i]->fn;
                    Send( m );                
                } else {
                }
            }
            nString command;
            command.PrintF("%s", str(mousePos.GetV()));
            for( int i = 0; i < objectsOver; i ++ )
                command += " " + objectsOver[i]->FullNameString();
            //n_printf("eventServer->pull(rightClick\n");
            eventServer->pull( rightClickEvent, "rightClick", command.Get() );
            break;
    }
}
/**-----------------------------------------------------------------------------//
  adds a particular planner to my list of planners
  those planners will be notified when something is selected / commanded
  if they return true - then they block me doing anything with that command
--------------------------------------------------------------------------------*/
void nPlannerUI::AddPlanner( nRoot* r ) {
    n_printf("AddPlanner( %s )\n", r->GetName() );
    classVar(nplanner, plannerclass);
    if( r->IsA( plannerclass )) {
        planners.Push() = (nPlanner*) r;
    } else {
        char fullname[1024];
        r->GetFullName( fullname, 1024 );
        n_printf("WARNING: nPlanner::AddPlanner( %s ) trying to add a non planner node\n", fullname );
    }
}
/**-----------------------------------------------------------------------------//
* clears the list of ents i am over
--------------------------------------------------------------------------------*/
/*void nPlannerUI::ClearObjectsOver(){
    for( int i = 0; i < objectsOver; i ++ ) {
        objectsOver[i]->RemoveChildVis("mouseOver");
    }
    objectsOver.Clear();
}*/
//-----------------------------------------------------------------------------//
/**-----------------------------------------------------------------------------//
 * update which entities have the mouse over them
--------------------------------------------------------------------------------*/
/*void nPlannerUI::UpdateOverLinks(){
    n_printf("nPlannerUI::UpdateOverLinks()\n");
    for( int i = 0; i < objectsOver; i ++ ) 
        objectsOver[i]->AttachChildVis("mouseOver", "/lib/node/overVis");
}*/
/**-----------------------------------------------------------------------------//
 * hide whatever rmenu was active
--------------------------------------------------------------------------------*/
void nPlannerUI::HideSelectedMenus() {
    //n_printf("HideSelectedMenus()\n");
    static nHeap<nUIElement*> 	elements(32);
    static nHeap<nRoot*> 	possib(32);
    nRoot* selectedMenusRoot = kernelServer->Lookup("/ui/menus/selected/");
    if( selectedMenusRoot ) {
        possib.add( 0, selectedMenusRoot );
        while( possib.length() > 0 ) {
            nRoot* r = possib.pop();
            classVar( nuielement, uielementClass );
            if( r->IsA( uielementClass ) )
                elements.add( 0, (nUIElement*) r );
            for( nRoot* r2 = r->GetHead(); r2; r2 = r2->GetSucc() )
                possib.add( 0, r2 );
        }
        while( elements.length() > 0 ) {
            //n_printf("%s->Hide()\n", elements.top()->fn );
            elements.pop()->Hide();
        }
    }
}
void nPlannerUI::ShowSelectedMenus(){
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    //const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    nRoot* toDisplay = 0;
    nRoot* selectedMenusRoot = kernelServer->Lookup("/ui/menus/selected/");
    if( selectedMenusRoot ) {
        const char* s;
        if( objectsSelected.Size() == 0 ) {
            s = "default";
        }
        else {
            s = objectsSelected[0]->GetParent()->GetName();
        }
        toDisplay = selectedMenusRoot->Find(s);
        if( toDisplay ) {
            //n_printf("opening selected menu:%s\n", s );
            foreachchild( nUIElement, uie, toDisplay ){
                uie->Show();
            }
            ((nFSMQueue*)selectedMenusRoot)->SetState( s );
        }
    }
}

void nPlannerUI::PlaceEnt( nEnt* ent ) {
    this->PlaceMultipleEnts( str(ent) );
    /*
    ifnonzero (ent) {
        //n_printf("%s placing %s\n", this->fn, ent->fn );
        if( entbeingplaced.isvalid() ) {
            //printf("killing ent being placed to place new: %s\n", entbeingplaced.GetRef()->fn );
            entbeingplaced.GetRef()->Die();
        }
        entbeingplaced.SetRef( ent );
        this->SetState( EntPlacementPre );
    }
    */
}
void nPlannerUI::AddSelectedEnt( nEnt* e ){
    //n_printf("nPlannerUI(%s)::AddSelectedEnt(e:%s)\n", str(this), str(e));
    refCategoryServer->AddEntToCategory( categorySelected, e );
    if ( multiSelect == false ) {
        this->UpdateSelection();
    }
}
void nPlannerUI::RemoveSelectedEnt( nEnt* e ){
    refCategoryServer->RemoveEntFromCategory( categorySelected, e );
    this->UpdateSelection();
}
void nPlannerUI::ClearSelection(){
    //EntArray newObjectsSelected;
    //refCategoryServer->UpdateCategory( categorySelected, newObjectsSelected );
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    for( int i = 0; i < objectsSelected; i ++ )
        objectsSelected[i--]->Deselect();
}
nString nPlannerUI::getObjectsSelectedString(){
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    nString answer;
    nString line;
    if (objectsSelected.Size() == 0 ) {
        answer = "";
    }
    else {
        answer.PrintF("%s", objectsSelected[0]->fn );
        for( int i = 1; i < objectsSelected; i ++ ) {
            line.PrintF(" %s", objectsSelected[i]->fn );
            answer += line;
        }
    }
    return answer;
}
nString nPlannerUI::getObjectsOverString(){
    nString answer;
    nString line;
    const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    if (objectsOver.Size() == 0 ) {
        answer = "";
    }
    else {
        answer.PrintF("%s", objectsOver[0]->fn );
        for( int i = 1; i < objectsOver; i ++ ) {
            line.PrintF(" %s", objectsOver[i]->fn );
            answer += line;
        }
    }
    return answer;
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AddSelectedEnt( const char* path ) {
    //n_printf("nPlannerUI(%s).AddSelectedEnt(%s)\n", str(this), path);
    nEnt* e = (nEnt*) kernelServer->Lookup(path);
    this->AddSelectedEnt( e );
}
//-----------------------------------------------------------------------------//
void nPlannerUI::PlaceMultipleEnts( const char* paths ) {
    if( entsBeingPlaced.Size() != 0 ) {
        n_printf(" warning should finish multiple placements before starting another\n");
        entsBeingPlaced.Clear();
    }
    n_printf("nPlannerUI::PlaceMultipleEnts(%s)\n", paths);
    char* s = n_strdup(paths);
    char* path;
    while( (path = tok(s, " \n") )){
        //n_printf("path:%s\n", path);
        nEnt* e = dynamic_cast<nEnt*>(kernelServer->Lookup(path));
        n_assert(e && "trying to place a non ent");
        entsBeingPlacedInfo& i = entsBeingPlaced.Push();
        i.ent = e;
        //n_printf("i.ent:%s\n", i.ent->fn);
        i.pos.Connect( i.ent, "pos" );
        //n_printf("pos:%s\n", str(i.pos.GetV()) );
    }
    //n_printf("calculating centroid\n");
    vector3 centroid;
    for( int i = 0; i < entsBeingPlaced; i ++ )
        centroid += entsBeingPlaced[i].pos.GetV() / float(entsBeingPlaced.Size());
    //n_printf("centroid:%s\n", str(centroid) );
    for( int i = 0; i < entsBeingPlaced; i ++ )
        entsBeingPlaced[i].offset = entsBeingPlaced[i].pos.GetV() - centroid;

    this->SetState(MultipleEntPlacementPre);
}
/* -----------------------------------------------------------------------------//
 * user has trigger a box selection
 * what should we do?
 * just update our objects selected, and then trigger a box selection event
--------------------------------------------------------------------------------*/
void nPlannerUI::BoxSelect(){
    const EntArray& objectsOver     = *(refCategoryServer->GetEntsInCategory( categoryOver ));
    const EntArray& objectsSelected = *(refCategoryServer->GetEntsInCategory( categorySelected ));
    int i;
    int numSelected = objectsSelected.Size();
    this->multiSelect = true;

    bool* matched = new bool[numSelected];
    for( i = 0; i < numSelected; i ++ )
        matched[i] = false;
    for( i = 0; i < objectsOver; i ++ ) {
        bool found = false;
        for( int j = 0; j < objectsSelected; j ++ )
            if( objectsSelected[j] == objectsOver[i] ) {
                found = true;
                matched[j] = true;
            }
        //n_printf("search for ent:%s == %i\n", objectsOver[i]->fn, found );
        if( !found )
            objectsOver[i]->Select();
    }
    for( i = 0; i < numSelected; i ++ ) {
        if( matched[i] == false ) {
            //n_printf("unselecting: %s\n", objectsSelected[i]->fn);
            objectsSelected[i]->Deselect();                             //remove it from objects selected
            i--;                                                        //make sure we dont go past our list of selected objects into newly selected things
            numSelected--;
        }
    }
    delete[] matched;

    for( i = 0; i < planners; i ++ ) 
        planners[i]->SelectionEvent( &objectsSelected );
    this->HideSelectedMenus();
    this->ShowSelectedMenus();
    eventServer->pull( selectionBoxEvent, "selectionChangedBox", getObjectsSelectedString().Get() );
    this->multiSelect = false;
}
//-----------------------------------------------------------------------------//
/**-----------------------------------------------------------------------------//
 * instead of worrying abouut this here want i want to do is attach a linknode to the vis child of any entity that is selected, has the mouse over it, etc..
--------------------------------------------------------------------------------*/
/*bool nPlannerUI::Attach(nSceneGraph2* sceneGraph){
    //n_printf("nPlannerUI::Attach()\n");
    //AttachRings(objectsOver,            overVis,                                sceneGraph);
    //AttachRings(objectsSelected,        selectedVis,                            sceneGraph);
    //AttachRings(overGroupEntities,      overGroupVis,                           sceneGraph);
    //AttachRings(selectedGroupEntities,  selectedGroupVis,                       sceneGraph);
    //AttachRings(catSelectedEntities,    catSelectedVis,                         sceneGraph);
    //AttachRings(targettedEntities,      targettedVis,                           sceneGraph);
    return true;
}*/
//-----------------------------------------------------------------------------//
void nPlannerUI::AttachRings(const EntArray& ents, nAutoRef<n3DNode>& node, nSceneGraph2* sceneGraph ){
    nEnt* e;
    float r;
    if( node.isvalid() == false && roll(.1)) {
        once(n_printf("node:%s is invalid\n", node.getname()));
    }
    ifvalid(node)
    foreach( e, ents ) {
        vector3 p = e->GetVarV("pos");
        //n_printf("attaching %s to :%s @%s\n", node.getname(), e->fn, str(p));
        if ( e->HasVar("selectionCircleRadius"))
            r = e->GetVarF("selectionCircleRadius");
        else
            r = 5.0f;
        node->Txyz(p.x, p.y + 1, p.z);
        node->Sxyz(r, r, r);
        node->Attach( sceneGraph );
    }
}
//-----------------------------------------------------------------------------//
void nPlannerUI::FindGroupMembers(const EntArray& ents, EntArray& groupMembers){
    groupMembers.Clear(); 
    nEnt* e;
    nEnt* group;
    //calculate which entities are in groups with entities i am over
    EntArray groups;
    foreach( e, ents){
        if (e->HasVar("groupable") && e->GetVarI("groupable") == 1) {
            if (e->HasVar("group") == false ){
                //n_printf("%s has no group\n", e->fn);
                static void* eventPtr = 0;
                eventServer->pull(eventPtr, "unitWithoutGroup", e->fn);
            } else {
                const char* groupPath;
                groupPath = e->GetVarRef("group");
                n_assert( groupPath );
                group = (nEnt*) kernelServer->Lookup(groupPath);
                n_assert( group );
                if ( groups.Find(group) == 0 ) {
                    groups.Append(group);
                }
            }
        }
    }
    foreach( group, groups) {
        foreachchild(nEnt, member, group){
            const char* path = member->GetVarS("path");
            e = (nEnt*) kernelServer->Lookup(path);
            if( ents.Find(e) == 0 ){
                groupMembers.Append(e);
            } 
        }
    }
}
//-----------------------------------------------------------------------------//
void nPlannerUI::AddCatSelectedEnt(const char* s ) {
    refCategoryServer->AddEntToCategory( categoryCatSelected, LOOKUP(nEnt*, s) );
}
//-----------------------------------------------------------------------------//
void nPlannerUI::ClearCatSelection(){
    refCategoryServer->ClearCategory( categoryCatSelected );
}
//-----------------------------------------------------------------------------//
//void nPlannerUI::AddTarget(const char* s ) {
    //refCategoryServer->AddEntToCategory( categoryTargetted, LOOKUP(nEnt*, s));
//}
//-----------------------------------------------------------------------------//
void nPlannerUI::UpdateSelection(){
    for( int i = 0; i < planners; i ++ ) 
        planners[i]->SelectionEvent( refCategoryServer->GetEntsInCategory( categorySelected ));
    this->HideSelectedMenus();
    this->ShowSelectedMenus();
    //RUNSCRIPT ("Selection::updateSelected");
    //n_printf("updateSelection:%s\n", getObjectsSelectedString().Get() );
    eventServer->pull( selectionEvent, "selectionChanged", getObjectsSelectedString().Get() );
    EntArray newSelectedGroupEntities;
    this->FindGroupMembers( *refCategoryServer->GetEntsInCategory( categorySelected ), newSelectedGroupEntities );
    refCategoryServer->UpdateCategory( categorySelectedGroup, newSelectedGroupEntities );
}
//-----------------------------------------------------------------------------//
/*void nPlannerUI::RemoveDeletedEntFromLists( const char* entPath ) {
    n_printf("nPlannerUI(%s).RemoveDeletedEntFromLists(%s)\n", str(this), entPath);
    nEnt* e = LOOKUP(nEnt*, entPath);
    while( objectsSelected.Remove( e ));
    while( objectsOver.Remove( e ));
    while( selectedGroupEntities.Remove( e ));
    while( overGroupEntities.Remove( e ));
    while( catSelectedEntities.Remove( e ));
    while( targettedEntities.Remove( e ));
}*/
//-----------------------------------------------------------------------------//
