#define N_IMPLEMENTS nRMenuItem
//------------------------------------------------------------------------------
//  (C) 2005	Chris Miles
//------------------------------------------------------------------------------
#include "ui/nrmenuitem.h"
#include "ui/ntextdisplayer.h"
#include "ui/ninterfaceserver.h"
#include "enkiduo/ngameserver.h"
#include "enkiduo/neventserver.h"

nNebulaScriptClass(nRMenuItem, "nfsm");

//------------------------------------------------------------------------------
nRMenuItem::nRMenuItem():
    ref_text                            (kernelServer, this),
    ref_eventServer                     (kernelServer, this),
    ref_interfaceServer                 (kernelServer, this)
{

    ref_text                            = "/lib/node/text/text";
    ref_eventServer                     = "/sys/servers/event";
    ref_interfaceServer                 = "/ui/interface";
    width                               = 60.0f;
    width                              *= LOOKUP(nEnt*, "/sys/share/windowsettings")->GetVarF("fieldOfView");
    this->AddToTriggerList();

    stateNames.Append("Waiting");
    stateNames.Append("Expandable");
    stateNames.Append("Back");
    stateNames.Append("MouseOver");
    stateNames.Append("Expanded");
    stateNames.Append("Passive");

    for( int i= 0; i < stateNames; i ++ ) {
        DrawInfo& info = drawInfo.Push();
        info.ref_slice.initialize       (kernelServer, this);
        info.ref_fan.initialize         (kernelServer, this);
        info.ref_backgroundfan.initialize(kernelServer, this);
    }

    drawInfo[Waiting].ref_slice           = "/lib/node/slice";
    drawInfo[Expandable].ref_slice        = "/lib/node/expandableslice";
    drawInfo[Back].ref_slice              = "/lib/node/backslice";
    drawInfo[MouseOver].ref_slice         = "/lib/node/activeslice";
    drawInfo[Expanded].ref_slice          = "/lib/node/expandedslice";
    drawInfo[Passive].ref_slice           = "/lib/node/passiveslice";

    drawInfo[Waiting].ref_fan             = "/lib/node/fan";
    drawInfo[Expandable].ref_fan          = "/lib/node/expandablefan";
    drawInfo[Back].ref_fan                = "/lib/node/backfan";
    drawInfo[MouseOver].ref_fan           = "/lib/node/activefan";
    drawInfo[Expanded].ref_fan            = "/lib/node/passivefan";
    drawInfo[Passive].ref_fan             = "/lib/node/passivefan";

    drawInfo[Waiting].ref_backgroundfan   = "/lib/node/backgroundfan";
    drawInfo[Expandable].ref_backgroundfan= "/lib/node/backgroundfan";
    drawInfo[Back].ref_backgroundfan      = "/lib/node/backgroundfan";
    drawInfo[MouseOver].ref_backgroundfan = "/lib/node/backgroundfan";
    drawInfo[Expanded].ref_backgroundfan  = "/lib/node/passivebackgroundfan";
    drawInfo[Passive].ref_backgroundfan   = "/lib/node/passivebackgroundfan";

    drawInfo[Waiting].drawText          = true;
    drawInfo[Expandable].drawText       = true;
    drawInfo[Back].drawText             = true;
    drawInfo[MouseOver].drawText        = true;
    drawInfo[Expanded].drawText         = false;
    drawInfo[Passive].drawText          = false;
    this->button                        = 1;
}
//------------------------------------------------------------------------------
void nRMenuItem::Initialize(){
    nFSM::Initialize();
    this->createVisNode(this->fn);
}
//------------------------------------------------------------------------------
void nRMenuItem::InitVars(){
    nFSM::InitVars();
    windowSettings.Connect              ( ent, "windowSettings");
    windowSettings.DefaultRef           ("/sys/share/windowsettings");
    aspectRatio.Connect                 ( windowSettings, "aspectRatio" );
    menuActive.Connect                  ( ent, "menuActive");
    menuActive.DefaultI                 (0);
    ringCenter.Connect                  ( ent, "ringCenter");
    ringCenter.DefaultV                 (0,0,0);
    expansionTime.Connect               ( ent, "expansionTime");
    expansionTime.DefaultF              ( .00f );
    ringRadius.Connect                  ( ent, "ringRadius");
    ringRadius.DefaultF                 ( 0.30 );

    rmenuRoot.Connect                   ( ent, "rmenuRoot");
    rmenuRoot.DefaultRef                ( this->findRMenuRoot()->fn );
    mousePos.Connect                    ( rmenuRoot, "mousePos" );
    objectsOver.Connect                 ( rmenuRoot, "objectsOver" );
    objectsSelected.Connect             ( rmenuRoot, "objectsSelected" );

}

//-----------------------------------------------------------------------------//
nEnt* nRMenuItem::findRMenuRoot(){
    nRoot* a;
    nRoot* b;
    nEnt* root = 0;
    classVar(nent, entClass);
    classVar(nrmenuitem, rmenuItemClass);
    a = this->GetParent();
    b = this;
    while( a && b ) {
        if ( a->IsA(entClass) && b->IsA(rmenuItemClass)) {
            root = (nEnt*) a;
        }
        a = a->GetParent();
        b = b->GetParent();
    }
    n_assert( root );
    //n_printf("found root : %s\n", root->fn );
    return root;
}
//------------------------------------------------------------------------------
nRMenuItem::~nRMenuItem(){
}
//------------------------------------------------------------------------------
void nRMenuItem::Tick( float time, float dtime ){
    nFSM::Tick( time, dtime );
}

bool nRMenuItem::States( StateMachineEvent event, uint state, float time, float dtime ){
    if ( menuActive.isvalid() == false )  //sometimes we get attached before being properly initialized
        return true;
    if( (event == EVENT_UPDATE || event == EVENT_ATTACH) && menuActive.GetI() == 0 )  { //our ring has been deactivated
        if (state != Waiting)  //reset to waiting state when deactivated
            this->SetState(Waiting);
        return true;
    }
    MouseInfo info = game->GetMouseInfo();
    BeginStateMachine
        State(Waiting)
            OnEnter
            OnTick
                if ( this->CheckMouseLoc() )
                    this->SetState(MouseOver);
                else if ( this->back )
                    this->SetState(Back);
                else if ( this->GetHead() != 0 )
                    this->SetState(Expandable);
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
            OnAttach
                this->Draw();
        State(Expandable)
            OnEnter
            OnTick
                if ( this->CheckMouseLoc() )
                    this->SetState(MouseOver);
                else if ( this->back )
                    this->SetState(Back);
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
            OnAttach
                this->Draw();
        State(Back)
            OnEnter
            OnTick
                if ( this->CheckMouseLoc() )
                    this->SetState(MouseOver);
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
            OnAttach
                this->Draw();
        State(MouseOver)
            OnEnter
                this->timeMouseOver = time;
            OnTick
                if ((time - timeMouseOver > expansionTime.GetF()) && (this->GetHead() != NULL))
                    this->SetState(Expanded);
                if ((time - timeMouseOver > expansionTime.GetF()) && (this->back)) {
                    menuActive.SetI(0);
                    nRoot* parent1 = this->GetParent();
                    n_assert( parent);
                    nRoot* parent2 = parent->GetParent();
                    n_assert( parent2);
                    nEnt* e = (nEnt*) parent2->GetParent();
                    n_assert( e );

                    vector3 childCenter = ref_interfaceServer->MousePosTo0Centered( info.pos );
                    /*
                    vector3 childCenter;
                    childCenter.x = info.pos.x;
                    childCenter.y = info.pos.y;
                    childCenter.x = (childCenter.x - .5) * 2.0f * aspectRatio.GetF();
                    childCenter.y = (childCenter.y - .5) * -2.0f;
                    */
                    e->SetVarI("menuActive", 1);
                    e->SetVarV("ringCenter", childCenter);
                }
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
                else if ( this->CheckMouseLoc() == false )
                    this->SetState(Waiting);
            OnExit
                if (info.pressed[button] == false && CheckMouseLoc()) {
                    ifvalid(ref_eventServer)
                        if (eventName.Get() != "") {
                            nString msg;
                            msg.PrintF("%s;%s;%s;%s;(%s);(%s)", this->GetName(), text.Get(), this->fn, str(mousePos.GetV()), objectsOver.GetS().Get(), objectsSelected.GetS().Get());
                            ref_eventServer->pull( eventPtr, eventName.Get(), msg.Get() );
                        }
                    //n_printf("USER SELECTED MENU ITEM: %s/%s %s %s %s\n", text.Get(), this->GetName(), str(mousePos.GetV()), objectsOver.GetS().Get(), objectsSelected.GetS().Get());
                    //n_printf("pull( %p, %s, %s)\n", eventPtr, eventName.Get(), text.Get());
                }
            OnAttach
                this->Draw();
        State(Expanded)
            OnEnter
                //activate all my child trees
                if ( this->GetHead() ) {
                    vector3 childCenter = ref_interfaceServer->MousePosTo0Centered( info.pos );
                    /*
                    vector3 childCenter;
                    childCenter.x = info.pos.x;
                    childCenter.y = info.pos.y;
                    childCenter.x = (childCenter.x - .5) * 2.0f * aspectRatio.GetF();
                    childCenter.y = (childCenter.y - .5) * -2.0f;
                    */
                    foreachchild(nEnt, e, this) {
                        e->SetVarI("menuActive", 1);
                        e->SetVarV("ringCenter", childCenter);
                    }
                }
                //set all my siblings to a passive state
                foreachchild(nRMenuItem, b, this->GetParent() ) {
                    if (b != this) {
                        b->SetState("Passive");
                    }
                }
                menuActive.SetI(2);
            OnTick
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
                if ( menuActive.GetI() == 1 )
                    this->SetState(Waiting);
            OnAttach
                this->Draw();
        State(Passive)
            OnEnter
                //n_printf("nRMenuItem(%s).Passive.Enter\n", this->fn);
                menuActive.SetI(2);
            OnTick
                if (info.pressed[button] == false )
                    menuActive.SetI(0);
                if ( menuActive.GetI() == 1 )
                    this->SetState(Waiting);
            OnAttach
                this->Draw();
    EndStateMachine
}
/**-----------------------------------------------------------------------------//
 * place the angled lines, they should center around the center
 * scale them by the radius of the thing, rotate them by theta
--------------------------------------------------------------------------------*/
void nRMenuItem::Draw(){
    float fov                           = ref_interfaceServer->GetFieldOfView();
    float ringRadius                  = this->ringRadius.GetF();
    ringRadius                       *= fov;
    DrawInfo& info                    = drawInfo[this->GetStateI()];

    vector3 center                    = ringCenter.GetV();
    //vector2 center;
    //center.set(center3.x, center3.y);
    float rtheta                      = n_deg2rad(theta);
    float rwidth                      = n_deg2rad(width*.5);
    float textSize                    = 18.5 * .3 / ringRadius;

    //if( info.ref_fan.isvalid() == false )
        //n_printf("fan:%s is not valid\n", info.ref_fan.getname());
    ifvalid(info.ref_backgroundfan)
    if( this->GetSucc() == 0 ) {
        info.ref_backgroundfan->Txyz(center.x, center.y, -1.00f );
        info.ref_backgroundfan->Sxyz(1.9f * ringRadius, 1.0f, 1.9f * ringRadius);
        info.ref_backgroundfan->Rxyz(90,0,0);
        info.ref_backgroundfan->Attach(sceneGraph);
    }
    ifvalid(info.ref_fan){
        info.ref_fan->Txyz(center.x, center.y, -1.01f );
        info.ref_fan->Sxyz(2.2f * ringRadius * sin(rwidth), 1.0f, 2.2f * ringRadius*cos(rwidth) );
        info.ref_fan->Rxyz(90,0,-theta);
        info.ref_fan->Attach(sceneGraph);
    }
    ifvalid(info.ref_slice) {
        info.ref_slice->Txyz(center.x, center.y,-1);                       
        info.ref_slice->Sxyz(.005 * fov, 1, ringRadius * 1.8);
        info.ref_slice->Rxyz(90,0,-theta+width*.5);
        info.ref_slice->Attach(sceneGraph);
        info.ref_slice->Rxyz(90,0,-theta-width*.5);
        info.ref_slice->Attach(sceneGraph);
    }

    if (info.drawText)
    ifvalid(ref_text) {
        vector3 textPosition = center + vector3(ringRadius*sin(rtheta), ringRadius*cos(rtheta), 0);
        ref_text->SetString(text.get(), ALIGN_CENTER);
        ref_text->SetOffset(vector3(textPosition.x * textSize, textPosition.y * textSize, -textSize));
        ref_text->Attach(sceneGraph);
    }
}

bool nRMenuItem::CheckMouseLoc(){
    ///calculate if it is within our area
    MouseInfo info      = game->GetMouseInfo();
    bool mouseInSection = false;
    vector3 center      = ringCenter.GetV();
    vector3 pos         = ref_interfaceServer->MousePosTo0Centered( info.pos );
    pos                 = pos - center;
    pos.z               = 0;
    if ( pos.len() > .1 * ref_interfaceServer->GetFieldOfView())  {
        float mtheta = n_rad2deg(atan2(pos.x, pos.y));
        float dtheta = fabs(mtheta - this->theta);
        while( dtheta > 180.0f )
            dtheta -= 360.0f;
        dtheta = fabs(dtheta);
        if( dtheta < width / 2.0f ) //our section width goes half to the left half to the right
            mouseInSection = true;
    }
    return mouseInSection;
}
