#!/usr/bin/python
import os
import sys
from xml.dom import minidom
from xml.dom.minidom import getDOMImplementation
import wx
floatFormat = '%.4f'
#/-----------------------------------------------------------------------------//
def getText(root):
    nodelist = root.childNodes
    rc = ''
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc = rc + node.data
    return str(rc.strip())
#/-----------------------------------------------------------------------------//
def convertMsToKnots(ms):
    return ms * 1.94384449
#/-----------------------------------------------------------------------------//
def convertKnotsToMs(knots):
    return knots / 1.94384449
#/-----------------------------------------------------------------------------//
def printExcept(module=None):
    import sys
    print '--------------------------------------------------------------------------------'
    if module:
        print 'Exception thrown in module:', module
        print '--------------------------------------------------------------------------------'
    sys.excepthook(sys.exc_info()[0], sys.exc_info()[1], sys.exc_info()[2])
    print '--------------------------------------------------------------------------------'
#/-----------------------------------------------------------------------------//
class EntEditorPanel(wx.Panel):
    #/-----------------------------------------------------------------------------//
    row = 0
    col = 0
    #/-----------------------------------------------------------------------------//
    def addLabelAndText(self, labelText, text, newLabel=True):
        label                           = wx.StaticText(self, -1, labelText)
        self.sizer.Add                  ( label, (self.row, self.col))
        self.col                       += 1
        ctrl                            = wx.TextCtrl(self,-1, str(text), style=wx.TE_READONLY)
        self.sizer.Add                  ( ctrl, (self.row, self.col), flag=wx.EXPAND)
        self.col                       += 1
        newCtrl = None
        if(newLabel):
            newCtrl                     = wx.TextCtrl(self,-1, str(text))
            self.sizer.Add              ( newCtrl, (self.row, self.col), flag=wx.EXPAND)
        self.col                       -= 2
        self.row                       += 1
        if newCtrl:
            return newCtrl
        else:
            return ctrl
    #/-----------------------------------------------------------------------------//
    def calcPerformance(self, setCtrls=True):
        print 'calcPerformance'
        if self.noCalcPerformance:
            return
        print self.enginePower, self.rudderTorque, self.inertia, self.rfriction
        self.maxspeed                   = self.enginePower / (self.mass * self.friction)
        self.maxspeedInKnots            = convertMsToKnots(self.maxspeed)
        self.turningSpeed               = (self.enginePower * self.rudderTorque ) / (self.inertia * self.rfriction)
        self.accelTime                  = 2.0 / self.friction
        self.turnAccelTime              = 2.0 / self.rfriction
        print 'turningSpeed', self.turningSpeed

        if setCtrls:
            self.maxSpeedCtrl.SetValue       (floatFormat % self.maxspeed)
            self.maxSpeedInKnotsCtrl.SetValue(floatFormat % self.maxspeedInKnots)
            self.turningSpeedCtrl.SetValue   (floatFormat % self.turningSpeed)
            self.accelTimeCtrl.SetValue      (floatFormat % self.accelTime)
            self.turnAccelTimeCtrl.SetValue  (floatFormat % self.turnAccelTime)
    #/-----------------------------------------------------------------------------//
    def calcParameters(self):
        print 'calcParameters'
        self.inertia                    = self.mass
        self.friction                   = 2.0 / self.accelTime
        self.rfriction                  = 2.0 / self.turnAccelTime
        self.enginePower                = self.maxspeed * self.mass * self.friction
        print self.turningSpeed, self.inertia, self.rfriction, self.enginePower
        self.rudderTorque               = (self.turningSpeed * self.inertia * self.rfriction ) / self.enginePower

        self.noCalcPerformance          = True
        self.inertiaCtrl.SetValue       (floatFormat % self.inertia)
        self.frictionCtrl.SetValue      (floatFormat % self.friction)
        self.rfrictionCtrl.SetValue     (floatFormat % self.rfriction)
        self.enginePowerCtrl.SetValue   (floatFormat % self.enginePower)
        self.rudderTorqueCtrl.SetValue  (floatFormat % self.rudderTorque)
        self.noCalcPerformance          = False
    #/-----------------------------------------------------------------------------//
    def parseVars(self, ented):
        self.vars = {}
        for ed in ented.getElementsByTagName('var'):
            name  = ed.getAttribute('name')
            type  = ed.getAttribute('type')
            value = ed.getAttribute('value')
            if type == 'float':
                self.vars[name] = float(value)
    #/-----------------------------------------------------------------------------//
    def resetParameters(self, event=None):
        self.mass                        = self.vars['mass']
        self.inertia                     = self.vars['inertia']
        self.friction                    = self.vars['friction']
        self.rfriction                   = self.vars['rfriction']
        self.enginePower                 = self.vars['enginePower']
        self.rudderTorque                = self.vars['rudderTorque']

        if event:
            self.calcPerformance           (event != None)
            self.massCtrl.SetValue         (floatFormat % self.mass)
            self.inertiaCtrl.SetValue      (floatFormat % self.inertia)
            self.frictionCtrl.SetValue     (floatFormat % self.friction)
            self.rfrictionCtrl.SetValue    (floatFormat % self.rfriction)
            self.enginePowerCtrl.SetValue  (floatFormat % self.enginePower)
            self.rudderTorqueCtrl.SetValue (floatFormat % self.rudderTorque)
    #/-----------------------------------------------------------------------------//
    def __init__(self, parent, id, document, ented, filename):
        wx.Panel.__init__(self,parent,id)
        self.document = document
        self.ented    = ented
        self.filename = filename
        self.sizer = wx.GridBagSizer(0,0)
        self.sizer.AddGrowableCol(1)
        self.sizer.AddGrowableCol(2)
        self.noCalcPerformance          = False

        self.addLabelAndText            ( 'Filename :', self.filename, False)
        self.entName                    = getText(ented)
        self.addLabelAndText            ( 'Ent Name :', self.entName, False)

        self.parseVars(ented)
        self.resetParameters()

        self.massCtrl               = self.addLabelAndText( 'Mass:',                                      floatFormat % self.mass)
        self.inertiaCtrl            = self.addLabelAndText( 'Inertia:',                                   floatFormat % self.inertia)
        self.frictionCtrl           = self.addLabelAndText( 'Friction:',                                  floatFormat % self.friction)
        self.rfrictionCtrl          = self.addLabelAndText( 'RFriction:',                                 floatFormat % self.rfriction)
        self.enginePowerCtrl        = self.addLabelAndText( 'Engine Power:',                              floatFormat % self.enginePower)
        self.rudderTorqueCtrl       = self.addLabelAndText( 'Rudder Torque:',                             floatFormat % self.rudderTorque)

        self.Bind                   (wx.EVT_TEXT, self.setMass,             self.massCtrl         )
        self.Bind                   (wx.EVT_TEXT, self.setInertia,          self.inertiaCtrl      )
        self.Bind                   (wx.EVT_TEXT, self.setFriction,         self.frictionCtrl     )
        self.Bind                   (wx.EVT_TEXT, self.setRFriction,        self.rfrictionCtrl    )
        self.Bind                   (wx.EVT_TEXT, self.setEnginePower,      self.enginePowerCtrl  )
        self.Bind                   (wx.EVT_TEXT, self.setRudderTorque,     self.rudderTorqueCtrl )

        self.calcPerformance        ( False )
        self.maxSpeedCtrl           = self.addLabelAndText('Max Speed:',    floatFormat % self.maxspeed)
        self.maxSpeedInKnotsCtrl    = self.addLabelAndText('In Knots :',    floatFormat % self.maxspeedInKnots)
        self.turningSpeedCtrl       = self.addLabelAndText('Turning Speed:',floatFormat % self.turningSpeed)
        self.accelTimeCtrl          = self.addLabelAndText('Accel Time:',   floatFormat % self.accelTime)
        self.turnAccelTimeCtrl      = self.addLabelAndText('Turn Accel Time:',   floatFormat % self.turnAccelTime)

        self.Bind                   (wx.EVT_TEXT, self.setMaxSpeed,         self.maxSpeedCtrl )
        self.Bind                   (wx.EVT_TEXT, self.setMaxSpeedInKnots,  self.maxSpeedInKnotsCtrl )
        self.Bind                   (wx.EVT_TEXT, self.setTurningSpeed,     self.turningSpeedCtrl )
        self.Bind                   (wx.EVT_TEXT, self.setAccelTime,        self.accelTimeCtrl )
        self.Bind                   (wx.EVT_TEXT, self.setTurnAccelTime,    self.turnAccelTimeCtrl )
        #self.calcParameters         ()

        self.saveButton             = wx.Button(self, -1, "Save")
        self.resetButton            = wx.Button(self, -1, "Reset")
        self.sizer.Add              ( self.saveButton,  (0,2), flag=wx.EXPAND)
        self.sizer.Add              ( self.resetButton, (1,2), flag=wx.EXPAND)
        self.Bind                   (wx.EVT_BUTTON, self.save,            self.saveButton)
        self.Bind                   (wx.EVT_BUTTON, self.resetParameters, self.resetButton)

        self.SetSizerAndFit( self.sizer )
    #/-----------------------------------------------------------------------------//
    def setMass(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.mass:
            f = float(f)
            if abs(f-self.mass) < .01: return
            self.mass = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setInertia(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.inertia:
            f = float(f)
            if abs(f-self.inertia) < .01: return
            self.inertia = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setFriction(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.friction:
            f = float(f)
            if abs(f-self.friction) < .01: return
            self.friction = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setRFriction(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.rfriction:
            f = float(f)
            if abs(f-self.rfriction) < .01: return
            self.rfriction = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setEnginePower(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.enginePower:
            f = float(f)
            if abs(f-self.enginePower) < .01: return
            self.enginePower = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setRudderTorque(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.rudderTorque:
            f = float(f)
            if abs(f-self.rudderTorque) < .01: return
            self.rudderTorque = f
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setMaxSpeed(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.maxspeed:
            f = float(f)
            if abs(f-self.maxspeed) < .01: return
            self.maxspeed = f
            self.maxspeedInKnots = convertMsToKnots(f)
            self.calcParameters()
    #/-----------------------------------------------------------------------------//
    def setMaxSpeedInKnots(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.maxspeed:
            f = float(f)
            if abs(f-self.maxspeed) < .01: return
            self.maxspeed = convertKnotsToMs(f)
            self.maxspeedInKnots = f
            self.calcParameters()
            self.calcPerformance()
    #/-----------------------------------------------------------------------------//
    def setTurningSpeed(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.turningSpeed:
            f = float(f)
            if abs(f-self.turningSpeed) < .01: return
            self.turningSpeed = f
            self.calcParameters()
    #/-----------------------------------------------------------------------------//
    def setAccelTime(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.accelTime:
            f = float(f)
            if abs(f-self.accelTime) < .01: return
            self.accelTime = f
            self.calcParameters()
    #/-----------------------------------------------------------------------------//
    def setTurnAccelTime(self, event):
        f = event.GetString()
        if f and float(f) and float(f) != self.turnAccelTime:
            f = float(f)
            if abs(f-self.turnAccelTime) < .01: return
            self.turnAccelTime = f
            self.calcParameters()
    #/-----------------------------------------------------------------------------//
    def save(self,event):
        print 'Saving to', self.filename
        for ed in self.ented.getElementsByTagName('var'):
            name  = str(ed.getAttribute('name'))
            if name == 'mass':                ed.setAttribute('value', str(self.mass))
            if name == 'inertia':             ed.setAttribute('value', str(self.inertia))
            if name == 'friction':            ed.setAttribute('value', str(self.friction))
            if name == 'rfriction':           ed.setAttribute('value', str(self.rfriction))
            if name == 'enginePower':         ed.setAttribute('value', str(self.enginePower))
            if name == 'rudderTorque':        ed.setAttribute('value', str(self.rudderTorque))
        file = open(self.filename, 'w')
        file.write(self.document.toxml())
        file.close()
#/-----------------------------------------------------------------------------//
i = 0
def parseFile(filename):
    global i
    file                            = open(filename)
    document                        = minidom.parse( file )
    file.close()
    for ented in document.getElementsByTagName('entityDefinition'):
        try:
            panel = EntEditorPanel( frame, -1, document, ented, filename )
        except:
            printExcept( 'failed to parse file %s' % filename )
            continue

        sizer.Add( panel, (i, 0), flag=wx.EXPAND)
        i += 1
        sizer.Add( wx.Panel(frame,-1, size=(50,0)), (i,0))
        i += 1
#/-----------------------------------------------------------------------------//
app = wx.App()
frame = wx.Frame(None,-1)
frame.Show()
sizer = wx.GridBagSizer(0,0)
sizer.AddGrowableCol(0)
sizer.AddGrowableRow(0)
for filename in sys.argv[1:]:
    parseFile( filename )
frame.SetSizerAndFit( sizer )
frame.SetSize( (800,800))
app.MainLoop()
