namespace eval InfluenceMap {
    variable gridsize 128
    variable info
    variable x -1
    variable z -1
    variable imdir /game/im
    proc parseVal { command ent } {
        foreach word [lindex $command 0] {
            if {[string index $word 0] == "#" } {
	set varname [string range $word 1 end]
	set val [$ent.gf $varname]
	set command [string map "$word $val" $command]
            }
        }
        set val 0
        if { [catch { set val [expr $command] } err] } {
            puts $err
        }
        return $val
    }
    proc closeness { ent maxDist } { 
        set md [parseVal $maxDist $ent]
        set d [dist $ent]
        if { $d > $md } { 
            return 0
        } else {
            return [expr 1.0 - $d/$md]
        }
    }
    proc within { ent maxDist } {
        set md [parseVal $maxDist $ent]
        set d [dist $ent]
        if { $d > $md } { 
            return 0
        } else {
            return 1
        }
    }
    proc dist { ent } {
        variable x 
        variable z
        set p [$ent.gv pos]
        set px [lindex $p 0]
        set pz [lindex $p 2]
        return [eucl $x $z $px $pz]
    }
    proc eucl { x0 y0 x1 y1 } {
        return [expr sqrt(($x0 - $x1)*($x0-$x1)+($y0-$y1)*($y0-$y1))]
    }
    proc sum { func ents {args ""} } {
        set val 0
        sel /game/ents/$ents
        foreach ent [dir] {
            set val [expr $val + [eval $func $ent $args]]
        }
        return $val
    }
    proc sumim { im0 im1 coef } {
        vars i j imdir
        set val0 [$imdir/$im0.getval $i $j]
        set val1 [$imdir/$im1.getval $i $j]
        set val [expr $val0 + $val1]
        return $val
    }
    #okay so i want the sum of values in the other im in this one
    proc sumRadiusIM { r im } {
        vars i j x z imdir squaresize gridsize
        set sr [expr int($r / $squaresize + 1.0)]

        set mink [expr $i - $sr]
        if { $mink < 0 } { set mink 0 }
        set maxk [expr $i + $sr]
        if { $maxk > $gridsize } { set maxk $gridsize }
        set minl [expr $i - $sr]
        if { $minl < 0 } { set minl 0 }
        set maxl [expr $i + $sr]
        if { $maxl > $gridsize } { set maxl $gridsize }

        set val 0
        for { set k $mink } { $k < $maxk }  { incr k } {
            for { set l $minl } { $l < $maxl } { incr l } {
	set m [expr $squaresize * $k]
	set n [expr $squaresize * $l]
	if { [eucl $x $z $m $n] < $r } {
	    set val [expr $val + [$imdir/$im.getval $k $l]]
	}
            }
        }
        return $val
    }
    proc calc { im {newexp "none"} } { 
        vars imdir gridsize i j x z display info
        exportVars
        if { [$imdir/$im.calculate] == "false" } { 
            variable squaresize [expr $::Map::worldsize/($gridsize-1)]
            if { $newexp != "none" } { set info($im.exp) $newexp }
            if { [$imdir/$im.getsize] != $gridsize } {
	$imdir/$im.setsize $gridsize
            }
            for { set i 0 } { $i < $gridsize } { incr i } {
	for { set j 0 } { $j < $gridsize } { incr j } {
	    set x [expr $squaresize * $i]
	    set z [expr $squaresize * $j]
	    set val [expr $info($im.exp)]
	    $imdir/$im.setval $i $j $val
	}
            }
            $imdir/$im.normalize
            set display($im) "display"
            $imdir/$im.display
        }
        importVars
    }
    proc open { } {
        TL InfluenceMap
        variable imframe $f
        pushf [PW $imframe.opts "Globals"]
            glabel -text "gridsize:"
            gentry -textvariable ::InfluenceMap::gridsize
            gbutton -text "importvars" -command ::InfluenceMap::importVars
            gbutton -text "exportvars" -command ::InfluenceMap::exportVars
            packall
        popf
        importVars
    }
    proc openIM { im { e "" } } {
        vars display gridsize imdir imframe info
        if { $e != "" } 		{ set info($im.exp) $e }
        if { [$imdir.find $im] == "null" } 	{ new ninfluencemap $imdir/$im } else { set info($im.exp) [$imdir/$im.getexp] }
        if { [$imdir/$im.getsize] != $gridsize } { $imdir/$im.setsize $gridsize }
        if { ![info exists imframe] } { return }
        set info($im.a) [$imdir/$im.geta]
        set info($im.b) [$imdir/$im.getb]
        set info($im.c) [$imdir/$im.getc]
        importVars
        pushf $imframe
            pushf [PW $f.$im $im]
	pushf [frame $f.info]
	    pack $f
	    gcheck -text "Display" -variable ::InfluenceMap::display($im) -onvalue display -offvalue undisplay -command "$imdir/$im.\$::InfluenceMap::display($im)"
	    glabel -text "min:"
	    gentry -textvariable ::InfluenceMap::info($im.min) -width 8
	    glabel -text "max:"
	    gentry -textvariable ::InfluenceMap::info($im.max) -width 8
	    glabel -text "coef:"
	    gentry -textvariable ::InfluenceMap::info($im.coef) -width 8
	    glabel -text "A:"
	    gentry -textvariable ::InfluenceMap::info($im.a) -width 8
	    glabel -text "B:"
	    gentry -textvariable ::InfluenceMap::info($im.b) -width 8
	    glabel -text "C:"
	    gentry -textvariable ::InfluenceMap::info($im.c) -width 8
	    gbutton -text "calc" -command "InfluenceMap::calc $im"
	    packall -side left
	popf
	pushf [frame $f.exp]
	    pack $f
	    glabel -text "exp:"
	    gentry -textvariable ::InfluenceMap::exp($im) -width 30
	    packall -side left
	popf
            popf
        popf
    }
    proc importVars { } {
        vars info imdir
        pushsel
        sel /game/im
        foreach im [dir] {
            set info($im.exp) 	[$imdir/$im.getexp]
            set info($im.min) 	[format %.3f [$imdir/$im.getmin]]
            set info($im.max) 	[format %.3f [$imdir/$im.getmax]]
            set info($im.coef) 	[format %.3f [$imdir/$im.getcoef]]
            set info($im.a) 	[format %.3f [$imdir/$im.geta]]
            set info($im.b) 	[format %.3f [$imdir/$im.getb]]
            set info($im.c) 	[format %.3f [$imdir/$im.getc]]
        }
        popsel
    }
    proc exportVars { } {
        vars info imdir
        pushsel
        sel /game/im
        foreach im [dir] {
            foreach var "coef a b c" {
	if { [info exists info($im.$var)] } { 
	    if { $info($im.$var) != ""} {
	        $imdir/$im.set$var $info($im.$var)
	    } else {
	        puts "$info($im.$var) == "
	    }
	} else {
	    puts "cannot find info($im.$var)"
	}
            }
        }
        popsel
    }
}
