#**************************************************************************
#displays a result file in a nice graphical format
#**************************************************************************
namespace eval Results {
    variable width 800
    variable height 800
    variable canvas
    variable types "evals generation timetaken fitness genmodified injectedamount cachehits hammingDistance"
    variable colors
    variable lineWidth 1
    variable pad .1
    variable results
    variable dashAverages 1
    variable filename $::xmldir/results/results.xml
    variable xgraphFilename "results.xg"
#**************************************************************************
#opens this widget - loading everything
#**************************************************************************
    proc open { } {
        lb
        variable width
        variable height
        variable colors
        array set colors {
            background	white
            1,evals 	black
            1,generation 	red
            1,timetaken 	green
            1,fitness 	blue
            1,genmodified 	yellow
            1,injectedamount 	indianred	
            1,cachehits	purple
            1,hammingDistance	maroon
            0,evals 	gray
            0,generation 	darkred
            0,timetaken 	darkgreen
            0,fitness 	darkblue
            0,genmodified 	orangered
            0,injectedamount 	indianred	
            0,cachehits	darkviolet
            0,hammingDistance	maroon1
        }
        TL Results
        pushf [frame $f.canvas]
            grid config $f -row 0 -column 0 -rowspan 2 -columnspan 1 -sticky nsew
            variable canvas [canvas $f.canvas -width $width -height $height -background $colors(background) -relief raised]
            pack $canvas
        popf
        pushf [frame $f.tree]
            grid config $f -row 0 -column 1 -sticky nsew
            openTree
        popf
        pushf [frame $f.buttons]
            grid config $f -row 1 -column 1 -sticky nsew
            variable checkFrame [frame $f.checks]
            pack $checkFrame -side left
            pushf [frame $f.drawOptions]
	pack $f -side right -anchor n -fill both -expand yes
	pushf [frame $f.options]
	    pack $f -anchor n -fill both -expand yes
	    gc 0 0 glabel -text "Pad:"
	    gc 0 1 gentry -textvariable ::Results::pad
	    gc 1 0 glabel -text "Line Width: "
	    gc 1 1 gentry -textvariable ::Results::lineWidth
	    gc 2 0 gcheck -variable ::Results::dashAverages -text "Dash Averages" -command Results::draw
	popf
	gbutton -text "Calc max" 	-command "Results::calcMax"
	gbutton -text "Redraw" 	-command "$Results::canvas delete; Results::draw"
	gbutton -text "View File" 	-command {XMLWindow .resultFile Results $::Results::results "result gamedata"}
	gentry -textvariable ::Results::xgraphFilename
	gbutton -text "Gen XGraph"	-command "Results::drawXGraph"
	packall
            popf
            variable maxFrame [frame $f.maxs]
            pack $maxFrame -side right
        popf
        build
        buildTree
        calcMax
        draw root
        lb
    }
#**************************************************************************
#construct the points from the files
#**************************************************************************
    proc build { } {
        variable filename
        variable results [fileToString $filename]
        parseData $results
    }
#**************************************************************************
#calculate maxs - a recursive tree search thingy
#**************************************************************************
    proc calcMax { {node root} } {
        variable draw
        variable max
        variable types
        variable tree
        variable maxx
        variable info
        if { $node == "root" } {
            #clear the current array
            set maxx 1
            foreach type $types {
	set max($type) .000000001
            } 
        }
        #if its a node i should draw this node include it in the max calculation
        if { [info exists draw(run,$node)] } {
            if { $draw(run,$node) } {
	foreach type $types {
	    for { set j 0 } { [info exists info($type,$node,$j)] } { incr j } {
	        if { $j > $maxx } { 
	            set maxx $j 
	        }
	        if { $info($type,$node,$j) > $max($type) } {
	            set max($type) $info($type,$node,$j)
	        }
	    }
	}
            }
        }
        foreach child [$tree nodes $node] {
            calcMax $child
        }
    }
#**************************************************************************
#recursive draw func - draws this node and all its children
#only if it is on the drawlist however
#note: recursion works even if parent is not drawn
#**************************************************************************
    proc draw { {node root} } {
        variable info
        variable tree
        variable max
        variable types
        variable colors
        variable maxx
        variable width
        variable height
        variable runs
        variable canvas
        variable draw
        variable widgets
        variable pad
        if { [info exists widgets($node)] } {
            foreach widget $widgets($node) {
	$canvas delete $widget
            }
        }
        set children [$tree nodes $node]
        foreach child $children {
            draw $child
        }
        set widgets($node) ""
        if { [info exists draw(run,$node)] } {
            if { $draw(run,$node) } {
	foreach type $types {
	    if { $draw(type,$type) } {
	        for { set j 1 } { [info exists info($type,$node,$j)] } { incr j } {
	            set p0 $info($type,$node,[expr $j-1])
	            set p1 $info($type,$node,[expr $j])
	            set x0 [expr ($j-1) 	* $width / ($maxx)]
	            set x1 [expr ($j) 	* $width / ($maxx)]
	            set y0 [expr $p0 	* $height / $max($type)]
	            set y1 [expr $p1 	* $height / $max($type)]
	            set y0 [expr $height - $y0]
	            set y1 [expr $height - $y1]
	            set x0 [expr $x0 * (1-$pad) + $width  * $pad / 2.0]
	            set y0 [expr $y0 * (1-$pad) + $height * $pad / 2.0]
	            set x1 [expr $x1 * (1-$pad) + $width  * $pad / 2.0]
	            set y1 [expr $y1 * (1-$pad) + $height * $pad / 2.0]
	            set colorGroup 1
	            if { [string match "*avg*" $node] } { 
		set colorGroup [string index $node 3]
		while { $colorGroup > 1 } { incr colorGroup -2}
		variable dashAverages
	               	variable lineWidth
		    
		if { $dashAverages } {
		    lappend widgets($node) [$canvas create line $x0 $y0 $x1 $y1 -fill $colors($colorGroup,$type) -width $::Results::lineWidth -dash {5 3} ]
		} else {
		    lappend widgets($node) [$canvas create line $x0 $y0 $x1 $y1 -fill $colors($colorGroup,$type) -width $::Results::lineWidth]
		}
	            } elseif { [string match "*avg*" [$tree parent $node] ]} {
		set colorGroup [string index [$tree parent $node] 3]
		while { $colorGroup > 1 } { incr colorGroup -2}
		lappend widgets($node) [$canvas create line $x0 $y0 $x1 $y1 -fill $colors($colorGroup,$type) -width $::Results::lineWidth]
	            } else {
		lappend widgets($node) [$canvas create line $x0 $y0 $x1 $y1 -fill $colors($colorGroup,$type) -width $::Results::lineWidth]
	            }
	        }
	    }
	}
            } 
        }
    }


    proc drawXGraph { {node root} } {
        variable info
        variable tree
        variable max
        variable types
        variable colors
        variable maxx
        variable width
        variable height
        variable runs
        variable canvas
        variable draw
        variable widgets
        variable pad
        variable xg
        if { $node == "root" } {
            set xg ""
        }

        set children [$tree nodes $node]
        foreach child $children {
            drawXGraph $child
        }
        if { [info exists draw(run,$node)] } {
            if { $draw(run,$node) } {
	foreach type $types {
	    if { $draw(type,$type) } {
	        for { set j 0 } { [info exists info($type,$node,$j)] } { incr j } {
	            set x [expr $j]
	            set y [expr $info($type,$node,$j)]
	            append xg "$x $y\n"
	        }
	        append xg "\n"
	    }
	}
            } 
        }
        if { $node == "root" } {
            LogWindow .xg "XG" $xg
            variable xgraphFilename
            stringToFile "$::xmldir/results/$xgraphFilename" $xg
        }
    }


#**************************************************************************
#okay parse out the data we want to display
#**************************************************************************
    variable idCounter 0
    proc parseId { txt } { 
        variable idCounter

        if { $txt == "" } {
            set answer run$idCounter
            incr idCounter
        } else {
            set answer $txt
        }
        return $answer
    }

    proc hammingDistance { a b } {
        set distance 0
        if { [string length $a] != [string length $b] } { puts "warning lengths do not match" } {
            for { set i 0 } { $i < [string length $a] } { incr i } {
	if { [string index $a $i] != [string index $b $i] } {
	    incr distance
	}
            }
        }
        return $distance
    }
    proc checkfit { fitness } {
        if { $fitness == -1 } {
            uplevel { lappend tags(bad) $run }
        }
    }
    proc calcHammingDistance { bitstring } {
#okay so we want to umm
#this is so easy
        variable info
        upvar run run
        upvar num num
        set info(bitstring,$run,$num) $bitstring
        if { $num > 0 } {
            set oldBitString $info(bitstring,$run,[expr $num-1])
            set info(hammingDistance,$run,$num) [hammingDistance $bitstring $oldBitString]
        } else {
            set info(hammingDistance,$run,$num) 0.0
        }
    }
#**************************************************************************
#parses the result xml file
#**************************************************************************
    proc parseData { plan } [XML::buildParser {
        init		{ 
		    variable runs ""
		    variable num 0 
		    variable info
		    variable tags
		}
        result		{ 
		    set num 0
		    set run [parseId $txt]
		    lappend runs $run 
		} { }
        resulttag	{ lappend tags($txt) $run }
        gamedata	{ }
        bluedamage	{ }
        reddamage 	{ }
        commandtime	{ }
        gadata		{ }
        garun		{ } { incr num }
        data		{ }
        year		{ }
        month		{ }
        day		{ }
        hour		{ }
        minute		{ }
        second		{ }
        evals		{ set info(evals,$run,$num) 		$txt }
        generation	{ set info(generation,$run,$num) 	$txt } 
        timetaken	{ set info(timetaken,$run,$num) 	$txt }
        fitness		{ set info(fitness,$run,$num)		$txt; checkfit $txt }
        rank		{ set info(rank,$run,$num)		$txt }
        genmodified	{ set info(genmodified,$run,$num)	$txt }
        injectedamount	{ set info(injectedamount,$run,$num)	$txt }
        cachehits	{ set info(cachehits,$run,$num)	$txt }
        bitstring	{ calcHammingDistance 		$txt }
    }]

#**************************************************************************
#loads the tree widget and its supporters
#**************************************************************************
    proc openTree { } { 
        upvar f f
        variable sw [ScrolledWindow $f.sw -relief sunken -borderwidth 2]
        variable tree $f.tree
        Tree $tree\
            -relief        	 	sunken		\
            -borderwidth    		2       			\
            -highlightthickness 	0   		\
            -redraw         		0       			\
            -dropenabled    		1       			\
            -dragenabled    		1       			\
            -height 		40		\
            -width 		30		\
            -opencmd   		"Results::moddir 1 $tree" 	\
            -closecmd  		"Results::moddir 0 $tree"	\
            -dropcmd		"Results::dropcmd"	\
            -deltay		18
        $sw setwidget $tree
        $tree bindText  <ButtonPress-1>        "Results::select tree 1 $tree"
        $tree bindText  <Double-ButtonPress-1> "Results::select tree 2 $tree"
        $tree configure -redraw 1
        pack $sw -fill both -expand yes
        $tree insert end root averages 	-text "Averages" 	-open 1 -font {Times 10 bold}
        $tree insert end root free 	-text "Free" 		-open 1 -font {Times 10 bold}
        $tree insert end root tags 	-text "Tags" 		-open 1 -font {Times 10 bold}
    }
#**************************************************************************
#constructs the tree from parsed info
#**************************************************************************
    proc buildTree { } {
        variable tree
        variable info
        variable runs
        variable types
        variable max
        variable tree
        variable draw
        variable id
        variable tags
        variable drawTags
        foreach run $runs {
            set draw(run,$run) yes
            $tree insert end free $run -text "                                             " -window [genNode $run]
        }
        foreach type $types {
            variable maxFrame
            set draw(type,$type) yes
            pushf $maxFrame
	pushf [frame $f.$type]
	    pack $f -expand yes -fill both
	    pack [gentry -textvariable ::Results::max($type)] -side right
	popf
            popf
            variable checkFrame
            variable draw
            pushf $checkFrame
	pushf [frame $f.$type]
	    pack $f -expand yes -fill both
	    pack [glabel -text $type] -side left
	    pack [checkbutton $f.cb -variable ::Results::draw(type,$type) -command "Results::draw" -onvalue yes -offvalue no] -side right
	popf
            popf
        }
        foreach tag [array names tags] {
            set drawTags($tag) yes
            $tree insert end tags $tag -text "   $tag" -window [checkbutton $tree.tag$tag -variable ::Results::drawTags($tag) -onvalue yes -offvalue no -command "Results::updateTags {$tag}"]
        }
    }

#**************************************************************************
#called when a node is opened / closed
#**************************************************************************
    proc moddir { index tree node } {
    }
#**************************************************************************
#called when a node is selected
#**************************************************************************
    proc select { where num tree node } {
    }
#**************************************************************************
#called when nodes are dropped from one to another
#if dropped on averages it creates a new average node
#otherwise just mvoes things around
#**************************************************************************
    variable cavg 0
    proc dropcmd { tree srcpath drop op type data } {
        variable cavg
        variable draw
        set from $data
        set to [lindex $drop 1]
        if {$to == "averages" } {
            set node "avg$cavg"
            incr cavg
            set draw(run,$node) yes
            $tree insert end averages $node -open 1 -window [genNode $node]
            set to $node
        }
        if { $from != $to } {
            variable tags
            if { [lsearch [array names tags] $from ] != -1 } {
	variable tags
	foreach node $tags($from) {
	    $tree move $to $node end
	}
            } else {
	$tree move $to $data end
            }
        } else {
        }
        updateAverages
    }
    #parse the tree structure and build our average results accordingly
#**************************************************************************
#recalculates the file averages
#**************************************************************************
    proc updateAverages { } {
        variable tree
        variable info
        variable types
        set avgNodes [$tree nodes averages]
        array unset info *avg*
        foreach avg $avgNodes {
            set children [$tree nodes $avg]
            foreach type $types {
	foreach child $children {
	    for { set j 0 } { [info exists info($type,$child,$j)] } { incr j } {
	        if {[info exists info($type,$avg,$j)]} { 
	            #we have some averaging information for that type
	            #incr the number of elements we have for it
	            incr count($type,$avg,$j)
	            set c 		$count($type,$avg,$j)
	            set info($type,$avg,$j) 	[expr $info($type,$avg,$j) * ( ($c-1.0) / $c) + $info($type,$child,$j) * ( 1.0 / $c )]
	        } else {
	            set info($type,$avg,$j) 	$info($type,$child,$j)
	            set count($type,$avg,$j) 	1
	        }
	    }
	}
            }
            draw $avg
        }
    }
#**************************************************************************
#creates a new node in the tree
#**************************************************************************
    variable nodec 0
    proc genNode { node } {
        variable tree 
        variable nodec
        set window 	[frame $tree.node_$nodec]
        pushf $window
            checkbutton $f.cb -variable Results::draw(run,$node) -onvalue yes -offvalue no -command "Results::draw $node"
            label $f.l -text $node
            pack $f.cb $f.l -side left
        popf
        incr nodec
        return $window
    }
    proc updateTags { tag } {
        variable tags
        variable drawTags
        variable draw
        foreach node $tags($tag) {
	if {$drawTags($tag)} {
	set draw(run,$node) yes
            } else {
	set draw(run,$node) no
            }
        }
        draw
    }
}
