#goal
#to coordinate all game play
#the battle master is responsible for choosing the map, the mission
#as well as master control over both players
#it should show the current map / mission
#allow for the changing of that
#should be able to bring up the free manipulate window - allowing total manipulation of all game objects
#sould be able to save these manipulations into a new mission file
#it should also be able to control the players
#should allow connections from the java interfaces
#should be able to boot / connect to either of the ga's as well
#parameters to those players should be changeable

#phase 1
#simple playability - GA:blue RED:static

namespace eval StrikeServer {
    variable info
    variable state 	"planning"
    variable gameState	""

    variable procIDs	""
    variable timer 	0.0000
    variable inited	no
#**************************************************************************
#log rerouting function - all logs made here are routed to both all and strikeserver
    proc log { txt } {
        logto "all strikeserver" $txt
    }
#initialization function
    proc init { } {
        variable info
        global defaults
        array set info {
            map		"default.xml"
            mission	"default.xml"
            blue.player.type 	"none"
            blue.player.url  	"unconnected"
            blue.player.ready	"no"
            blue.plan	""
            blue.plan.type	"none"
            blue.ga.connected	"false"
            blue.ga.hostnames	"chunli.cse.unr.edu enkiduo.cse.unr.edu"
            red.player.type 	"none"
            red.player.url  	"unconnected"
            red.player.ready	"no"
            red.plan	""
            red.plan.type	"none"
        }

        global socket
        Socket::init "strikeserver"
        Socket::openServer
        Pathfinder::init
        $socket.addcallback "blueplan" 	script StrikeServer::ReceiveBluePlan
        $socket.addconnectiondiedcallback 	script StrikeServer::ConnectionDied
        $socket.addcallback "getgamestate" 	script StrikeServer::GetGameState
        $socket.addcallback "red" 	script StrikeServer::ReceiveRed
        $socket.addcallback "redready" 	script StrikeServer::ReceiveRedReady
        $socket.addcallback "garun" 	script StrikeServer::ReceiveGARun
        $socket.addcallback "cases" 	script StrikeServer::ReceiveCases

        EventCallbackGroup::addcallback "commandexecuted"  StrikeServer::replan
        #pause the game when the strikeserver loads - so plats dont move, threats dont shoot, etc.
        after 1000 holdall
#load the default map
        if { [info exists defaults(map)] } {
            set info(map) $defaults(map)
        }
        if { [info exists defaults(mission)] } {
            set info(mission) $defaults(mission)
        }
        SetupGame
        if { [info exists defaults(plan.blue)] } {
            set f $defaults(plan.blue)
            set info(blue.plan.type) $f
            loadPlanFromFile blue $f
        }
        if { [info exists defaults(plan.red)] } {
            set f $defaults(plan.red)
            set info(red.plan.type) $f
            loadPlanFromFile red $f
        }
    }

#**************************************************************************
#load the window
#**************************************************************************
    proc open { } {
        TL "StrikeServer" 
        pushf [PW $f.player "Mission" -anchor w]
            pack $f -expand yes -fill both -padx 5
            pushf [frame $f.mapinfo]
            pack $f -side top
	gc 0 0 glabel -text "Map:"
	gc 0 1 gentry -textvariable ::StrikeServer::info(map) -width 15
	gc 0 2 gbutton -text "Change" -command StrikeServer::ChangeMap
	gc 1 0 glabel -text "Mission:"
	gc 1 1 gentry -textvariable ::StrikeServer::info(mission) -width 15
	gc 1 2 gbutton -text "Change" -command StrikeServer::ChangeMission
            popf
        popf
        pushf [PW $f.red "Red Plan"]
            pushf [frame $f.f]
	pack $f -anchor w
	glabel -text "Current:" -width 10
	gentry -textvariable ::StrikeServer::info(red.plan.type) -width 20
	packall -side left -anchor w
            popf
            pushf [gframe]
	packall
	gbutton -text "Load" 	-command "StrikeServer::loadplan red" 	-width 7
	gbutton -text "Save" 	-command "StrikeServer::saveplan red" 	-width 7
	gbutton -text "View" 	-command "StrikeServer::viewplan red" 	-width 7
	packall -side left
            popf
            gbutton -text "Run GA Planner" 	-command "StrikeServer::plan red"	-width 30
            gbutton -text "Edit"	-command "StrikeServer::editplan red"	-width 30
            packall -anchor w
        popf
        pushf [PW $f.blue "Blue Plan"]
            pushf [frame $f.f]
	pack $f -anchor w
	glabel -text "Current:" -width 10
	gentry -textvariable ::StrikeServer::info(blue.plan.type) -width 20
	packall -side left -anchor w
            popf
            pushf [gframe]
	packall
	gbutton -text "Load" 	-command "StrikeServer::loadplan blue" 	-width 7
	gbutton -text "Save" 	-command "StrikeServer::saveplan blue" 	-width 7
	gbutton -text "View" 	-command "StrikeServer::viewplan blue" 	-width 7
	packall -side left
            popf
            gbutton -text "Run GA Planner" 	-command "StrikeServer::plan blue" 	-width 30
            gbutton -text "Edit"	-command "StrikeServer::editplan blue"	-width 30
            packall -anchor w
        popf
        gbutton -text "Execute Plans" -command { StrikeServer::executeplans } -font {Times 14 bold} -width 25
        packall
        pushf [PW $f.gameOptions "Game Commands"]
            gbutton -text "Hold Running Game" 	-command "StrikeServer::holdgame"	-width 30
            gbutton -text "Resume Held Game"	-command "StrikeServer::resumegame"	-width 30
            gbutton -text "Reset Game"	-command "StrikeServer::resetGame"	-width 30
            packall
        popf
    }
#**************************************************************************
#changes the current mission
#**************************************************************************
    proc ChangeMission { } {
        global xmldir
        vars info
        set fullFilename [tk_getOpenFile -initialdir $xmldir/missions -filetypes {{"Xml Files" .xml}} -title "Load Map"]
        if { $fullFilename == "" } { return }
        set info(mission) 	[file tail $fullFilename]
        global defaults
        set defaults(mission) 	$info(mission)
        SetupGame
    }
#**************************************************************************
#changes the current map
#**************************************************************************
    proc ChangeMap { } {
        global xmldir
        vars info
        set fullFilename [tk_getOpenFile -initialdir $xmldir/maps -filetypes {{"Xml Files" .xml}} -title "Load Map"]
        if { $fullFilename == "" } { return }
        set info(map) 	[file tail $fullFilename]
        global defaults
        set defaults(map) 	$info(map)
        SetupGame
    }
#**************************************************************************
#Restarts the game
#reloads the map
#reloads the mission - all units
#plans are not reloaded
#**************************************************************************
    proc SetupGame { } {
        global xmldir
        vars info inited
        Plan::clear
        puts "restarting game - map:$info(map) - mission:$info(mission)"
        if { $info(map) != "" } {
        puts "load map"
            catch { Plan::load [file_to_string $xmldir/maps/$info(map)]	yes }
        }
        if { $info(mission) != "" } {
            puts "load mission"
            catch { Plan::load [file_to_string $xmldir/missions/$info(mission)]	yes }
            puts "done loading mission"
        }
#if { !$inited } {
            Map::gen
#           set inited yes
#       }
    }

#**************************************************************************
#saves the current plan to a file
#**************************************************************************
    proc saveplan { side } {  
        variable info
        global xmldir
        set filename 		[tk_getSaveFile -initialdir $xmldir/plans/$side/ -filetypes {{"Xml Files" .xml}} -title "Save $side Plan"]
        if { $filename == "" } { return }
        log "saving plan side:$side file: $filename"
        stringToFile $filename $info($side.plan)
        set defaults(plan.$side) $filename
        set info($side.plan.type) 	[file tail $filename]
    }
#**************************************************************************
#loads a plan from a file
#**************************************************************************
    proc loadplan { side } {
        variable info
        global xmldir
        set filename 		[tk_getOpenFile -initialdir $xmldir/plans/$side/ -filetypes {{"Xml Files" .xml}} -title "Load $side Plan"]
        if { $filename == "" } { return }
        global defaults
        set defaults(plan.$side) $filename
        loadPlanFromFile $side $filename
    }
    proc loadPlanFromFile { side filename } {
        vars info
        log "loading plan side:$side file: $filename"
        set plan      [fileToString $filename]
        setplan $side [file tail $filename] $plan
    }
#**************************************************************************
#view the plan xml
#**************************************************************************
    proc viewplan { side } {
        vars info
        XMLWindow [cat .$side plan] "$side plan" $info($side.plan) "side"
    }
#**************************************************************************
#Trigger the GA to plan for a side
#**************************************************************************
    proc plan { side } { GALoader::open $side }
#**************************************************************************
#Load the manual editor for a plan
#**************************************************************************
    proc editplan { side } { 
        switch $side {
            blue 	{ BlueInterface::open }
            red 	{ RedInterface::open }
        }
    }
#**************************************************************************
#Halt a running game
#**************************************************************************
    proc executeplans { } {
        variable state "playing"
        resumegame
    }
#**************************************************************************
#Halt a running game
#**************************************************************************
    proc holdgame { } {
        holdall
    }
#/-----------------------------------------------------------------------------//
#Resume a halted game
#/-----------------------------------------------------------------------------//
    proc resumegame { } {
        unholdall
    }
#/-----------------------------------------------------------------------------//
    proc resetGame { } {
        vars info
        SetupGame
        foreach side "blue red" {
            Plan::load $info($side.plan)
        }
        holdgame
    }
#/-----------------------------------------------------------------------------//
    proc ConnectionDied { connection } {
        log "Lost Connection: $connection"
    }
#QUASI PROTECTED FUNCTIONS - not buttons
#/-----------------------------------------------------------------------------//
#Set Plans
#/-----------------------------------------------------------------------------//
    proc setplan { side type plan } {
        vars info
        log "Setting $side Plan type: $type"
        set info($side.plan)		$plan
        set info($side.plan.type) 	$type
        Plan::load $plan
    }
#/-----------------------------------------------------------------------------//
#RPCS
#/-----------------------------------------------------------------------------//
    proc ReceiveBluePlan { plan } {
        puts "ReceiveBluePlan: $plan"
        setplan blue "GA" $plan
        editplan blue
    }
    proc ReceiveGARun { m } {
        logto garun "ReceiveGARun::$m"
    }
    proc ReceiveCases { c } {
        logto gacases "ReceiveCases: $c"
    }
}
