namespace eval tkproc { 
    variable fstack ""
    namespace export fstack
}

proc pushf { framename } {
    upvar f f	
    catch { lappend ::tkproc::fstack $f } err
    set f $framename
}
proc popf { } {
    upvar f f
    set frame	[lindex $::tkproc::fstack end]
    set f $frame
    set ::tkproc::fstack [lrange $::tkproc::fstack 0 end-1]
}

proc PW args {
    set path [lindex $args 0]
    set lbl [lindex $args 1]
    set args [lrange $args 2 end]
    set p [PanedWindow $path -side top]
    set pane [$p add -weight 2]
    set title [TitleFrame $p.tf -text $lbl]
    pack $pane $title -expand yes -fill both -anchor nw
    eval pack $p -anchor nw -expand yes -fill both -padx 5 $args
    return [$title getframe]
}

proc Log { path { txt "" } {width 50} {height 20} } {
    pushf [frame $path]
        pack $f -expand yes -fill both
        set lg [
            text $f.txt -wrap word\
            -width $width -height $height	\
            -xscrollcommand "$f.xscroll set" \
            -yscrollcommand "$f.yscroll set" \
            -setgrid 1 -highlightthickness 0 -pady 2 -padx 3
        ]
        $lg insert end $txt
        scrollbar $f.yscroll -command "$f.txt yview" -highlightthickness 0 -orient vertical
        pack $f.txt -side left -expand 1 -fill both -anchor n
        pack $f.yscroll -side right -fill y -anchor n
    popf
    return $lg
}
proc LogWindow { path lbl txt } {
    destroy $path
    set tl [toplevel $path]
    wm title $tl $lbl
    Log $tl.f $txt
}
proc Table { f rows cols func } {
    pushf [frame $f]
        pack $f -expand yes -fill both
        set rowIndex 0
        set colIndex 1
        #layout column headers
        foreach col $cols {
            set suffix $rowIndex
            append suffix _$colIndex
            grid config [label $f.l$suffix -text [lindex $col 0]] -row $rowIndex -column $colIndex -sticky nsew
            incr colIndex
        }
        set rowIndex 1
        set colIndex 0
        foreach row $rows {
            set suffix $rowIndex
            append suffix _$colIndex
            grid config [label $f.l$suffix -text [lindex $row 0]] -row $rowIndex -column $colIndex -sticky nsew
            incr rowIndex
        }
        set rowIndex 1
        set colIndex 1
        foreach row $rows {
            set colIndex 1
            foreach col $cols {
	set e "none"
	set suffix $rowIndex
        	append suffix _$colIndex
	set p $f.e$suffix
	if { [ catch { $func $p $row $col } err ]} { puts $err } else {
	    catch { grid config $p -row $rowIndex -column $colIndex -sticky nsew }
	}
	incr colIndex
            }
            incr rowIndex
        }
    popf
}
proc TL { title {geom "default"} } {
    TK::init
    upvar tl tl
    regsub " " "$title" "" path
    set path .[string tolower $path]
    puts "path:$path"
    Defaults::registerWindow $path
    destroy $path
    set tl [toplevel $path -background white -colormap new]
    wm title $tl $title
    wm resizable $tl 1 1
    if { $geom != "default" } {
        wm geometry $tl $geom
    } else {
        set winDefaults [Defaults::getWindowDefaults $path]
        if { $winDefaults != "none" } { wm geometry $tl $winDefaults }
    }
    upvar f f
    pushf [frame $tl.f]
    pack $f -expand yes -fill both
    return $path
}
proc getColumn { path } {
    regexp -- {-column ([0-9]+)} [grid info $path] all column
    return $column
}
proc getRow { path } {
    regexp -- {-row ([0-9]+)} [grid info $path] all row
    return $row
}

global 	widgetCounter
set 	widgetCounter 0
proc glabel args {
    upvar f f
    upvar widgets widgets
    global widgetCounter
    set lbl [eval label $f.label$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gtext args {
    upvar f f
    upvar widgets widgets
    global widgetCounter
    set lbl [eval text $f.text$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}

proc gentry args {
    upvar f f
    upvar widgets widgets
    global widgetCounter
    set lbl [eval entry $f.entry$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gbutton args {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [eval button $f.entry$widgetCounter -border 1 $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gcheck args {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [eval checkbutton $f.check$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}

proc gradio args {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [eval radiobutton $f.radio$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gscale args {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [eval scale $f.scale$widgetCounter -orient h $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gframe args {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [eval frame $f.frame$widgetCounter $args]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc glog { tag {width default} {height default} } {
    upvar widgets widgets
    upvar f f
    global widgetCounter
    set lbl [Log::gLog $f.log$widgetCounter $tag $width $height]
    incr widgetCounter 
    lappend widgets $lbl
    return $lbl
}
proc gc args {
    upvar f f
    set cmd [lrange $args 2 end]
    set widget [eval $cmd]
    grid config $widget -row [lindex $args 0] -column [lindex $args 1] -sticky nsew
}
proc packall args {
    upvar widgets widgets
    eval pack $widgets $args
    set widgets ""
}
proc XMLWindow { path title xml {opentags ""} } {
    destroy $path
    set tl [toplevel $path]
    wm title $tl $title
    wm resizable $tl 1 1
    XMLViewer::open $tl.xml $xml $opentags
}
proc emph { path } {
    $path configure -font {Helvetica 16 bold}
}
proc demph { path } {
    $path configure -font {Helvetica -12 bold}
}
