#idea
#build a table allocating assets to targets
#list each asset / target
#buttons are enabled only if the platform is going to that target
#todo! a switch would be nice that enables / disables showing unallocated targets

namespace eval AssetAllocation {
    variable showUnTargetted "no"
    proc log { txt } {
        logto "all assetallocation" $txt
    }

    proc open { } {
        EventCallbackGroup::addcallback "targetsChanged" AssetAllocation::PlatformAllocationChanged
        variable allocation
        variable showUnTargetted false
#array set allocation ""

        set row 2
        set col 0

        pushsel
        TL assetallocation
        pushf [PW $f.alloc "allocation"]
            identifyPlatforms
            identifyWeapons
            identifyTargets
            identifyPlatformTargets
            identifyAllocation
            defaultAllocation
            identifyTargettedTargets

            createLabels 		$f
            createTargetLabels 		$f
            createPlatformLabels 	$f
            createAllocationButtons 	$f
            createTargetHealthBars 	$f
        popf
        pack $f.alloc -side right
        pushf [PW $f.info "controls"]
            gcheck -text "Show Untargetted" -variable ::AssetAllocation::showUnTargetted -command { AssetAllocation::open }
            packall
            pushf [PW $f.td "total damage"] 
	gentry -textvariable ::AssetAllocation::totalDamage -width 5
	packall
            popf
        popf
        pack $f.info -side left
        popsel
    }
    
#*******************************************************************************
    proc identifyPlatforms { } {
        variable platforms
        sel /game/ents/platform
        set platforms [dir]
    }
#*******************************************************************************
    proc identifyWeapons { } {
        variable platforms
        variable platformWeapons
        variable types
        variable refs
        foreach platform $platforms {
            set weapons ""
            sel $platform/hardpoints
            foreach hardpoint [dir] {
	if { ![$hardpoint.open] } {
	    set ref [$hardpoint.getattached]
	    set name [$ref.getname]

	    lappend weapons $name
	    set types($name) 	[$ref.gs type]
	    set refs($name) 	$ref
	}
            }
            set platformWeapons($platform) $weapons
            sel ../..
        }
    }
#*******************************************************************************
    proc identifyTargets { } {
        variable targets ""
        variable damage
        sel /game/ents/target
        append targets [dir]
        sel /game/ents/threat
        append targets [dir]
    }
#*******************************************************************************
    proc identifyPlatformTargets { } {
        variable platforms
        variable platformTargets
        variable refs
        
        foreach platform $platforms {
            set mytargets ""
            sel /game/ents/platform/$platform
            if { [.find "wp"] == "null" } { continue } 
            sel wp
            foreach wp [dir] {
	set d [$wp.gv pos]
	if { [$wp/vars.find attack] != "null" } {  
	    set target [$wp.gref attack]
	    if { $target == "null" || $target == "none" } {
	    }  else {
	        set name [$target.getname]
	        lappend mytargets $name
	        set refs($name) $target
	    }
	}
	set platformTargets($platform) $mytargets
            }
        }
    }
#*******************************************************************************
    proc identifyAllocation { } {
        variable platforms
        variable platformWeapons
        variable usedWps ""
        variable types
        variable allocation
        log "identifyAllocation"
        foreach platform $platforms {
            foreach weapon $platformWeapons($platform) {
	set weaponType $types($weapon)
	sel /game/ents/platform/$platform/wp
	set found false
	foreach wp [dir] {
	    if { $found } { break }
	    set wpRef [$wp.getfullname]
	    if { [$wp/vars.find attack] == "null" } { continue }
	    set targetRef [$wp.gref attack]
	    set target [$targetRef.getname]
	    set asset [$wp.gs withasset]
	    if { $asset == $weaponType } {
	        if { [lsearch $usedWps $wpRef] == -1 } {
	            set allocation($weapon) $target
	            log "set allocation($weapon) $target"
            	            lappend usedWps $wpRef
	            set found true
	        } 
	    }
	}
	if { !$found } {
	    log "$weapon unallocated"
	}
            }
        }
    }
#*******************************************************************************
    proc defaultAllocation { } {
        variable platforms
        variable platformWeapons
        variable platformTargets
        variable allocation
        set allocatedWeapons [array names allocation]
        log "allocated weapons : $allocatedWeapons"
        foreach platform $platforms { 
            foreach weapon $platformWeapons($platform) {
	if { [lsearch $allocatedWeapons $weapon] == -1 } {
	    set targets $platformTargets($platform)
	    set target [lindex $targets 0]
	    if { $target == "" } { set target "unallocated" }
	    set allocation($weapon) 	$target
	}
            }
        }
        updateDamages
    }
#*******************************************************************************
    proc identifyTargettedTargets { } {
        variable platforms
        variable platformTargets
        variable allocation
        variable targettedTargets ""

        set targettedTargetsUnsorted ""
        foreach platform $platforms {
            foreach target $platformTargets($platform) {
	if {[lsearch $targettedTargetsUnsorted $target] == -1 } {
	    lappend targettedTargetsUnsorted $target
	}
            }
        }
        log "targettedTargetsUnsorted: $targettedTargetsUnsorted"
#make sure they are in the same order as the regular list
        variable targets
        foreach target $targets {
            if { [lsearch $targettedTargetsUnsorted $target] != -1 } {
	lappend targettedTargets $target
            }
        }
        log "targettedTargets: $targettedTargets"
    }
#*******************************************************************************
    proc createLabels { f } {
        grid config [label $f.row0 -text "Platforms" 	-font {Helvetica 14} ] -row 0 -column 0
        grid config [label $f.row1 -text "Assets"	-font {Helvetica 14} ] -row 0 -column 1
        grid config [label $f.row2 -text "Targets"	-font {Helvetica 14} ] -row 0 -column 2 -columnspan 50
    }
#*******************************************************************************
    proc createTargetLabels { f } {
        variable showUnTargetted
        variable targets
        variable targettedTargets
        set row 1
        set col 2
        if { $showUnTargetted } {
            foreach target $targets {
	grid config [
	    label $f.target_$target -text $target -font {Helvetica 10}
	] -row $row -column $col
	incr col
            }
        } else {
            foreach target $targettedTargets {
	grid config [
	    label $f.target_$target -text $target -font {Helvetica 10}
	] -row $row -column $col
	incr col
            }
        }
    }
#*******************************************************************************
    proc createPlatformLabels { f } {
        variable platforms
        variable platformWeapons
        variable types
        set row 2
        set col 0
        foreach platform $platforms {
            grid config [
	label $f.platform_$platform -text $platform -font {Helvetica 12}
            ] -row $row -column $col 
            #-rowspan [llength $platformWeapons($platform)]
            foreach weapon $platformWeapons($platform) {
	set weaponType $types($weapon)
	grid config [
	    label $f.asset_$weapon -text $weaponType -font {Helvetica 10}
	] -row $row -column 1
	incr row
            }
        }
    }
#*******************************************************************************
    proc createAllocationButtons { f } {
        variable platforms
        variable platformWeapons
        variable platformTargets
        foreach platform $platforms {
            foreach weapon $platformWeapons($platform) {
	foreach target $platformTargets($platform) {
	    set row 	[getRow $f.asset_$weapon]
	    set column 	[getColumn $f.target_$target]
	    log "row;$row col:$column"
	    if { [winfo exists $f.rb_$weapon$target] } { } { 
	        grid config [
	            radiobutton $f.rb_$weapon$target -variable AssetAllocation::allocation($weapon) -value $target -command AssetAllocation::updateDamages 
	        ] -row $row -column $column
	    }
	}
#-command AssetAllocation::export
            }
        }
    }

#*******************************************************************************
    proc createTargetHealthBars { f } {
        variable showUnTargetted
        variable targets
        variable targettedTargets
        log "createTargetHealthBars"
        set row 	[lindex [grid size $f] 1]
        if { $showUnTargetted } {
            foreach target $targets {
	set column 	[getColumn $f.target_$target]
	ProgressBar::create $f.pb_$target -width 30 -height 20 -variable AssetAllocation::damagei($target) -maximum 1000
	grid config $f.pb_$target -row $row -column $column
            }
        } else {
            foreach target $targettedTargets {
	set column 	[getColumn $f.target_$target]
	ProgressBar::create $f.pb_$target -width 30 -height 20 -variable AssetAllocation::damagei($target) -maximum 1000
	grid config $f.pb_$target -row $row -column $column
            }
        }
    }
#*******************************************************************************
    proc updateDamages { } {
        variable platforms
        variable platformWeapons
        variable allocation
        variable types
        variable refs
        variable damage
        variable damagei

        variable totalDamage


        variable targets
        foreach target $targets {
            set damage($target) 0.0
            set damagei($target) 0
        }

        foreach platform $platforms  {
            foreach weapon $platformWeapons($platform) {
	if { $allocation($weapon) == "unallocated" } { continue }
	set weaponRef  $refs($weapon)
	set weaponType $types($weapon)
	set target 	$allocation($weapon)

	set targetRef  $refs($target)

	if { $targetRef != "" } {
	    set armorclass [$targetRef.gs armorclass]
	    log "armorclass: $armorclass"
	    set weaponDamage [/info/armor/$armorclass.gf $weaponType]
	    log "damage: $weaponDamage"
	    set damage($target)	[format %.2f [expr 1- (( 1- $damage($target)) * (1-$weaponDamage))] ]
	    set damagei($target)	[expr int(1000.0 * $damage($target))]
	    log "set damagei($target)	[expr int(1000.0 * $damage($target))]"
	}
            }
        }
        set totalDamage 0
        foreach target $targets {
            set totalDamage [expr $totalDamage + $damage($target)]
        }
        export
    }

    proc PlatformAllocationChanged { param } {
        log "PlatformAllocationChanged $param"
    }
    
    proc export { } {
        variable allocation
        variable refs
        variable platforms
        variable platformWeapons
        variable types
        pushsel
        foreach platform $platforms  {
            sel /game/ents/platform/$platform/wp
            foreach wp [dir] {
	$wp.cs withasset "null"
            }
            foreach weapon $platformWeapons($platform) {
	if { $allocation($weapon) == "unallocated" } { continue }
	set weaponRef  $refs($weapon)
	set target 	$allocation($weapon)
	sel /game/ents/platform/$platform/wp
	foreach wp [dir] {
	    if { [$wp.hasvar attack] } {
	        set f [[$wp.gref attack].getname]
	        set t $allocation($weapon)
	        if { $f == $t } {
	            $wp.cs withasset $types($weapon)
	        }
	    }
	}
            }
        }
        popsel
    }
}
