#stati
#unconnected
#connected - no slaves
#booting slaves
#reading
#running
namespace eval GALoader { 
    variable info
#/-----------------------------------------------------------------------------//
    proc log { txt } { logto "all galoader" $txt }
#/-----------------------------------------------------------------------------//
    proc autoload { side } {
        vars buttons
        foreach button [array names buttons] {
            set b $buttons($button)
            $b configure -state disable
        }
        log "AutoLoader - Trying to connect to existing GA"
        set connected [connect $side]
        if { !$connected } {
            log "AutoLoader - GA not found - booting a new GA"
            bootGA $side
            connect $side
        } else {
            log "AutoLoader - GA Found"
        }
        log "AutoLoader - Checking for slaves"
        vars numslaves
        set numslaves -1
        getNumSlaves $side
    }
#/-----------------------------------------------------------------------------//
    proc autoload_b { side } {
        vars info numslaves
        log "AutoLoader - GA has $numslaves slaves"
        if { $numslaves < 5 } {
            if { !$info($side.slavesbooted) } {
	set info($side.slavesbooted) true
	bootSlaves $side
            }
            after 1000 GALoader::getNumSlaves $side
        } else {
            log "AutoLoader - Starting Planning"
            plan $side
        }
    }
#/-----------------------------------------------------------------------------//
    proc init { } {
        vars info
        array set info {
            blue.hostnames 	"chunli.cse.unr.edu"
            blue.ga.bootquiet	"true"
            blue.pids	""
            blue.slavesbooted	false
            red.hostnames  	"chunli.cse.unr.edu"
            red.ga.bootquiet	"true"
            red.pids	""
            red.slavesbooted	false
        }
    }
#/-----------------------------------------------------------------------------//
    proc open { side } {
        global socket
        $socket.addcallback "numslaves" 	script GALoader::receiveNumSlaves

        vars info
        vars buttons
        set info($side.status) "unconnected"
        TL "GALoader $side"
        pushf [PW $f.local "Run Locally"]
            gbutton -text "Run" -command "GALoader::runLocal $side"
            packall
        popf
        pushf [PW $f.remote "Run Remotely"]
            pushf [gframe]
	packall 
	glabel -text "Status:" -width 10
	gentry -textvariable ::GALoader::info($side.status) -width 20
	packall -side left -expand yes -fill both
            popf
            set buttons($side.connect) 	[gbutton -text "Connect to GA" 	-width 30 	-command "GALoader::connect	$side"]
            pushf [PW $f.hostnames "Hostnames:"]
	gentry -textvariable ::GALoader::info($side.hostnames) -width 28
            popf
            pushf [gframe]
	packall  -expand yes -fill both
	set buttons($side.boot) 	[gbutton -text "Boot GA" 	-width 20	-command "GALoader::bootGA	$side"]
	set buttons($side.quietly) 	[gcheck -text  "Quietly" -variable ::GALoader::info($side.ga.bootquiet) -onvalue true -offvalue false]
	packall -side left
            popf
            set buttons($side.bootslaves) 	[gbutton -text "Boot Slaves" 	-width 30	-command "GALoader::bootSlaves	$side"]
            set buttons($side.plan) 	[gbutton -text "Plan"	-width 30	-command "GALoader::plan	$side"]
            set buttons($side.killga) 	[gbutton -text "Kill GA" 	-width 30	-command "GALoader::killGA 	$side"]
            set buttons($side.killslaves)	[gbutton -text "Kill Slaves" 	-width 30	-command "GALoader::killSlaves 	$side"]
        popf
        glog ga 30 20
        glog galoader 30 20
        packall -expand yes -fill both
    }
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/		LOCAL GA CODE
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
    proc runLocal { side } { 
        log "GALoader::runLocal $side"
        initGALocal $side
        runGALocal $side
        EventCallbackGroup::addcallback gaFinished GALoader::gaFinishedLocal
    }
#/-----------------------------------------------------------------------------//
    proc initGALocal { side } {
        log "GALoader::initGALocal $side"
        sn nent /ga
        #fills all the ent vars from ga.xml
        GA::parseParams

        variable ga [new nga ga]
        new nga$side problem
        GAInfo::open
        CaseBase::build

        puts "INJECTION"
#[new ncaseextractor extractor].initvars
        [new ncaseinjector injector].initvars
#EventCallbackGroup::addcallback "nextgeneration" /ga/extractor.extract
        EventCallbackGroup::addcallback "nextgeneration" /ga/injector.inject
    }
#/-----------------------------------------------------------------------------//
    proc runGALocal { side } {
        log "runGALocal $side"
        vars ga info
        set situation [Plan::from_gamestate]
        Scenario::clear
        Scenario::load $situation
        $ga.setsituation $situation
        $ga.run
    }
#/-----------------------------------------------------------------------------//
    proc gaFinishedLocal { args } {
        vars ga info
        log "gaFinished $args"
        set info(plan) [$ga.getbestplan]
        StrikeServer::setplan blue "BlueGA - Local" $info(plan)
        StrikeServer::resetGame
    }
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#/		REMOTE GA CODE
#/-----------------------------------------------------------------------------//
#/-----------------------------------------------------------------------------//
#**************************************************************************
#connects to an existing gamaster
#**************************************************************************
    proc connect { side } {
        global socket
        vars info buttons
        emph $buttons($side.connect)
        log "Connecting to GA for side: $side hostnames: $info($side.hostnames)"
        foreach host $info($side.hostnames) {
            log "Trying host: $host"
            set retval [$socket.connect $host]
            if { $retval == "true" } {
	log "connected to $host"
	set info($side.status) 	connected
	set info($side.url) 	$host
	demph $buttons($side.connect)
	return true
            } else {
	log "connection failed"
            }
        }
        log "could not connect to GA Master"
        demph $buttons($side.connect)
        return false
    }
#**************************************************************************
#boots the gamaster (on hostname 0)
#**************************************************************************
    proc bootGA { side } {
        vars info buttons
        emph $buttons($side.boot)
        set host [lindex $info($side.hostnames) 0]
        set cmd "go gamaster"
        if { $info($side.ga.bootquiet) } { lappend cmd "remote" }
        opt injection { lappend cmd "injection" } 
        log "exec ssh $host $cmd &"
        lappend info($side.pids) [exec ssh $host $cmd &]
        after 5000
        demph $buttons($side.boot)
    }
#**************************************************************************
#kills the gamaster
#**************************************************************************
    proc killGA { side } {
        vars info 
        set info($side.status) 	"unconnected"
        foreach pid $info($side.pids) {
            log "kill $pid"
            exec kill $pid
        }
        set info($side.pids) ""
    }
#**************************************************************************
#boots up slaves
#**************************************************************************
    proc bootSlaves { side } {
        global socket
        vars info buttons
        emph $buttons($side.bootslaves)
        if { $info($side.status) != "unconnected" } {
            $socket.send "bootslaves" "now"
        } else {
            log "cannot boot slaves - not connected to gamaster"
        }
        after 1000
        demph $buttons($side.bootslaves)
    }
#**************************************************************************
#Finds out the number of active slaves
#**************************************************************************
    proc getNumSlaves { side } {
        global socket
        $socket.send "getnumslaves" "$side"
    }
    proc receiveNumSlaves { txt } {
        vars numslaves
        set side	[lindex $txt 0]
        set numslaves 	[lindex $txt 1]
        puts "received slaves : side:$side numslaves:$numslaves"
        autoload_b $side
    }
#**************************************************************************
#Kills any active slaves
#**************************************************************************
    proc killSlaves { side } {
        global socket
        vars info
        if { $info($side.status) != "unconnected" } {
        } else {
            puts "cannot kill slaves - not connected to gamaster"
        }
    }
#**************************************************************************
#Actually runs the ga
#**************************************************************************
    proc plan { side } {
        global socket
        set situation [Plan::from_gamestate]
        $socket.send "situation" $situation
        $socket.send "plan" "$side"
    }
}
GALoader::init
