# Mission chooser
# displays all the available missions to be choosen
# define files that describe other files to choose
# add tags in their for description / etc
# list those files info here
# when selected load the appropriate mission and continue
# open window
# parse out list of available missions
# add the to appropriate trees

namespace eval MissionChooser {
    variable count 0
    variable map                "none"
    variable mapbrief           "none"
    variable mapdesc            "none"
    variable mission            "none"
    variable missionbrief       "none"
    variable missiondesc        "none"
    variable inited	no
#***********************************************************************************************
#   WHEN A DIR IS opened - doesn't do anything
#***********************************************************************************************
    proc moddir { index tree node } {
        puts "moddir($index,$node)"
    }
#***********************************************************************************************
#   WHEN an element is selected - select it as the current map
#***********************************************************************************************
    proc select { where num tree node } {
        puts "select $where $num $tree $node"
        variable map
        variable mapfile
        variable mapdesc
        variable mission
        variable missionfile
        variable missiondesc

        set spl [split $node :]
        set type [lindex $spl 0]
        set nodename "[lindex $spl 0]:[lindex $spl 1]"

        switch $type {
            map { 
                set map         [$tree itemcget $nodename -text]
                set mapfile     [$tree itemcget $nodename:file -text]
                set mapdesc     [$tree itemcget $nodename:desc -text]
            }
            mission { 
                set mission         [$tree itemcget $nodename -text]
                set missionfile     [$tree itemcget $nodename:file -text]
                set missiondesc     [$tree itemcget $nodename:desc -text]
            }
            default {
                puts "unrecognized type:$type"
            }
        }
    }
#***********************************************************************************************
#   RELOAD the map being played
#***********************************************************************************************
    proc update { } {
        global xmldir
        variable mapfile
        variable missionfile
        if { $mapfile != "none" && $missionfile != "none" } {
            Plan::clear
            Plan::load [file_to_string $xmldir/maps/$mapfile]		yes
            Plan::load [file_to_string $xmldir/missions/$missionfile]	yes
            vars inited
            if { !$inited } {
	Map::gen
	set inited yes
            }
        }
    }
#***********************************************************************************************
#   Chooses this map to play
#***********************************************************************************************
    proc go { } {
        global xmldir
        variable tl
        variable mapfile
        variable missionfile
        if { $mapfile != "none" && $missionfile != "none" } {
            Scenario::load [file_to_string $xmldir/maps/$mapfile]
            Scenario::load [file_to_string $xmldir/missions/$missionfile]
        }
        destroy $tl
    }
#***********************************************************************************************
#***********************************************************************************************
#   PARSE OUT available missions, and add them to the tree
#***********************************************************************************************
    proc buildlist { tree } {
        variable count
        global xmldir
        set mapdir $xmldir/maps
        if { [ file exists $mapdir ] } {
            set currentdir [exec pwd]
            cd $mapdir
            set maps [lsort [exec dir]]
            set count 0
            foreach map $maps {
	if { $map == "CVS" } { continue }
                set file $map
                XML::parse [file_to_string $mapdir/$map]
                global ::XML::type ::XML::tag ::XML::txt
                while { [XML::next] } {
#puts "BUILDLIST: $type $tag $txt"
                    switch $type {
                        open { 
                            switch $tag {
                                map         { set brief $txt }
                                description { set desc $txt }
                            }
                        }
                    }
                }
                set rnode   map:$count
                $tree insert end maps   $rnode      -text $brief -open 1
                $tree insert end $rnode $rnode:file -text $file
                $tree insert end $rnode $rnode:desc -text $desc
                incr count
            }
            cd $currentdir
        } else { 
            puts "error: $mapdir does not exist"
        }
        global xmldir
        set missiondir $xmldir/missions
        if { [ file exists $missiondir ] } {
            set currentdir [exec pwd]
            puts "currentdir: $currentdir"
            cd $missiondir
            set missions [exec dir]
            set count 0
            foreach mission $missions {
	if { $mission == "CVS" } { continue }
                set file $mission
                XML::parse [file_to_string $missiondir/$mission]
                global ::XML::type ::XML::tag ::XML::txt
                while { [XML::next] } {
                    switch $type {
                        open { 
                            switch $tag {
                                mission     { set brief $txt }
                                description { set desc $txt }
                            }
                        }
                    }
                }
                set node mission:$count
                $tree insert end missions $node -text $brief -open 1
                $tree insert end $node    $node:file -text $file 
                $tree insert end $node    $node:desc -text $desc
                incr count
            }
            cd $currentdir
        } else { 
            puts "error: $missiondir does not exist"
        }
 
    }
#***********************************************************************************************
#   CREATES THE WINDOW
#***********************************************************************************************
    proc open { } {
        variable tl
        TL missionchooser
        wm resizable $tl 1 1
        pushf [PW $f.info "Mission Info"]
            pack $f -anchor n
            grid config [label $f.ml  -text "Map:"                                ] -row 0 -column 0 -sticky w
            grid config [label $f.m   -textvariable ::MissionChooser::map         ] -row 0 -column 1 -sticky w
            grid config [label $f.mf  -textvariable ::MissionChooser::mapfile     ] -row 0 -column 2 -sticky w
            grid config [label $f.mdl -text "Description:"                        ] -row 1 -column 0 -sticky w -columnspan 2
            grid config [label $f.md  -textvariable ::MissionChooser::mapdesc     ] -row 1 -column 1 -sticky w
            grid config [label $f.sl  -text "Mission:"                            ] -row 2 -column 0 -sticky w
            grid config [label $f.s   -textvariable ::MissionChooser::mission     ] -row 2 -column 1 -sticky w
            grid config [label $f.sf  -textvariable ::MissionChooser::missionfile ] -row 2 -column 2 -sticky w
            grid config [label $f.sdl -text "Description:"                        ] -row 3 -column 0 -sticky w
            grid config [label $f.sd  -textvariable ::MissionChooser::missiondesc ] -row 3 -column 1 -sticky w -columnspan 2
        popf
        pushf [frame $f.chooser]
            pack $f
            set pw          [PanedWindow $f.pane -side top]
            set pne         [$pw add -weight 1]
            set title       [TitleFrame $pne.ttl -text "Load Mission"]
            set sw          [ScrolledWindow [$title getframe].sw -relief sunken -borderwidth 1]
            set tree        [Tree $sw.tree              \
		-relief         flat    \
		-borderwidth    0       \
		-width          15      \
		-highlightthickness 0   \
		-redraw         0       \
		-dropenabled    1       \
		-dragenabled    1       \
		-height         20      \
		-width          50     \
		-opencmd   "MissionChooser::moddir 1 $sw.tree" \
		-closecmd  "MissionChooser::moddir 0 $sw.tree"
	            ]
            $sw setwidget $tree
            pack $sw    -side top  -expand yes -fill both
            pack $title -fill both -expand yes
            pack $pw    -fill both -expand yes
#bind selection commands
            $tree bindText  <ButtonPress-1>        "MissionChooser::select tree 1 $tree "
            $tree bindText  <Double-ButtonPress-1> "MissionChooser::select tree 2 $tree "
#build default tree
            $tree insert end root home      -text "DATA:"    -open 1
            $tree insert end home maps      -text "MAPS"     -open 1
            $tree insert end home missions  -text "MISSIONS" -open 1
            MissionChooser::buildlist $tree 
            $tree configure -redraw 1
        popf
        pushf [frame $f.buttons]
            pack $f
            button $f.update -text "Update Map" -command MissionChooser::update
            grid config $f.update -row 2 -column 0 -sticky s
            button $f.go -text "Go" -command MissionChooser::go
            grid config $f.go -row 2 -column 1 -sticky s
        popf
    }
}
