#ifndef GENE_H
#define GENE_H
#include "kernel/nkernelserver.h"

/*
    chromosome = tree
    each node can be of several types
    1) void -> just a header
    2) float -> holds a float, range, numbits
    3) int -> hots an int, range, numbits
    4) enum -> holds an int,  pointer to array, range, numbits

    now we need some measure for case injection..
    how injected are we??
    we can just use 0.1s..
    or mayeb some small metrics..
    how about just straight 1s..okay?
    okay sounds good

*/

    
class Gene 
{
    enum GENE_TYPES {
        EMPTY,
        FLOAT,
        INT,
        ENUM,
        ENUMPROXY,
    };
public:
    Gene():
        type( EMPTY ),
        name( 0 ),
        parent(0),
        floatPrecision( .1f ),
        enumArray(0),
        injectedAmount( 0.0 ),
        injectedValue( 1.0 )
    {}
    ~Gene(){
        for( int i = 0; i < children; i ++ )
            delete children[i];
    }

    nArray<nString> Sum ( int indent = 0 ){
        int i;
        nArray<nString> strings;
        nString myString;
        for( i = 0; i < indent; i ++ )
            myString += "  ";
        myString += this->toString();
        strings.Append( myString );
        for( i = 0; i < children.Size(); i ++ ){
            nArray<nString> childStrings = children[i]->Sum( indent+1 );
            for( int j = 0; j < childStrings.Size(); j ++ )
	strings.Append( childStrings[j] );
        }
        return strings;
    }
    void ToXML( XMLString& xml ) {
        xml.open("GENE", Name() );
        nString bitStringString = this->BitStringString();
        xml.wrap("BITS", bitStringString.Get());
        nString s;
        s.PrintF("%f", injectedValue );
        xml.wrap("INJECTEDVALUE", s.Get() );
        s.PrintF("%f", injectedAmount );
        xml.wrap("INJECTEDAMOUNT", s.Get() );
        switch( type ){
            case EMPTY: 
	break;
            case FLOAT: 	
	xml.wrap("FLOAT", GetF() );
	xml.wrap("MINF", minF );
	xml.wrap("MAXF", maxF );
	break;
            case INT:   	
	xml.wrap("INT", GetI() );
	xml.wrap("MINI", minI );
	xml.wrap("MAXI", maxI );
	break;
            case ENUM:
	xml.wrap("ENUM", GetE() );
	break;
            case ENUMPROXY:
	xml.wrap("ENUMPROXY", enumProxy.Get() );
	break;
        }
        for( int i=  0; i < children; i ++ ) {
            children[i]->ToXML( xml );
        }

        xml.close("GENE");
    }
    nString toString(){
        char string[1024];
        switch( type ){
            case EMPTY:  	sprintf(string, "<EMPTY> %8s"                    , name ? name : "" );                          break;
            case FLOAT: 	sprintf(string, "<FLOAT> %8s %f (%f->%f)\n"      , name ? name : "" , GetF(), minF, maxF );     break;
            case INT:   	sprintf(string, "<INT>   %8s %i (%i->%i)\n"      , name ? name : "" , GetI(), minI, maxI );     break;
            case ENUM:  	sprintf(string, "<ENUM>  %8s %s"                 , name ? name : "" , GetE() );                 break;
            case ENUMPROXY:  	sprintf(string, "<ENUMPROXY>  %8s %s"                 , name ? name : "" , enumProxy.Get() );   break;
            default: n_error("Gene::toString() unrecognized type %i\n"  , type);
        }
        return nString(string);
    }
    nArray<bool> BitString(){
        this->updateBits();	//just recalc wheenver someone wants it
        nArray<bool> bits;
        for( int i = 0; i < _bits; i ++ ){
            //n_assert( _bits[i] == 0 || _bits[i] == 1 );
            bits.Append( _bits[i] );
        }
        for( int i = 0; i < children.Size(); i ++ ){
            nArray<bool> childBits = children[i]->BitString();
            for( int j = 0; j < childBits.Size(); j ++ )
	bits.Append( childBits[j] );
        }
        return bits;
    }

    nString BitStringString() {
        nString bitStringString;
        nArray<bool> bits = this->BitString();
        char c[4];
        for( int i = 0; i < bits; i ++ ){
            //n_assert( bits[i] == 0 || bits[i] == 1 );
            sprintf( c, "%i", bits[i] );
            bitStringString += c;
        }
        return bitStringString;
    }
    int NumChildren(){
        return children.Size();
    }
    Gene* GetChild( int i ){
        return children[i];
    }
    const char* GetName(){
        return this->name;
    }
    int GetI(){
        n_assertf( valI == minI || valI == maxI || valI >= minI && valI <= maxI, "warning val:%i escaped range (%f->%f)", valI, minI, maxI );
        return valI;
    }
    float GetF(){
        n_assertf( valF == minF || valF == maxF || valF >= minF - TINY && valF <= maxF + TINY, "warning val:%i escaped range (%f->%f)", valF, minF, maxF );
        return valF;
    }
    const char* GetE(){
        if(!( valI >= minI && valI <= maxI ))
            n_error("valI : %i minI : %i maxI : %i", valI, minI, maxI );
        return enumArray[valI];
    }
    bool SetI( int i ){
        CheckType( INT );
        if( this->valI != i ){
            this->valI = i;
            return true;
        }
        return false;
    }
    bool SetE( int i ) {
        CheckType( ENUM );
        if( this->valI != i ){
            this->valI = i;
            return true;
        } return false;
    }
    bool SetMinI( int i){
        if( type != INT && type != ENUM )
            CheckType( INT );
        if( this->minI != i ){
            this->minI = i;
            calcBitLength();
            return true;
        }
        return false;
    }
    bool SetMaxI( int i ){
        if( type != INT && type != ENUM )
            CheckType( INT );
        if( this->maxI != i ){
            this->maxI = i;
            calcBitLength();
            return true;
        }
        return false;
    }
 
    bool SetF( float f ){
        CheckType( FLOAT );
        if( this->valF != f ){
            this->valF = f;
            return true;
        }
        return false;
    }
    bool SetMinF( float f ){
        CheckType( FLOAT );
        if( this->minF != f ){
            this->minF = f;
            return true;
            calcBitLength();
        }
        return false;
    }
    bool SetMaxF( float f ){
        CheckType( FLOAT );
        if( this->maxF != f ){
            this->maxF = f;
            return true;
            calcBitLength();
        }
        return false;
    }
    bool SetEnumProxy( const char * s ) {
        CheckType(ENUMPROXY);
        if( enumProxy != s ) {
            this->enumProxy = s;
            return true;
        }
        return false;
    }
    Gene* AddV( const char* geneName ){
        Gene* g = new Gene();
        g->name = geneName;
        g->type = EMPTY;
        g->parent = this;
        children.Append(g);
        //this->Randomize();
        return g;
    }
    Gene* AddI( const char* geneName, int min, int max ){
        Gene* g = new Gene();
        g->name = geneName;
        g->type = INT;
        g->minI = min;
        g->maxI = max;
        g->parent = this;
        //this->Randomize();
        children.Append( g );
        return g;
    }
    Gene* AddF( const char* geneName, float min, float max ){
        Gene* g = new Gene();
        g->name = geneName;
        g->type = FLOAT;
        g->minF = min;
        g->maxF = max;
        g->parent = this;
        //this->Randomize();
        children.Append( g );
        return g;
    }
    Gene* AddE( const char* geneName, const char** enumArray, int arraySize ){
        Gene* g = new Gene();
        g->enumArray = enumArray;
        g->name = geneName;
        g->type = ENUM;
        g->minI = 0;
        g->maxI = arraySize-1;
        g->parent = this;
        //this->Randomize();
        children.Append( g );
        return g;
    }
    Gene* Clone(){
        Gene* g = new Gene();
        g->type = this->type;
        g->name = this->name;
        g->enumArray = this->enumArray;
        g->injectedAmount = this->injectedAmount;
        g->injectedValue = this->injectedValue;
        switch ( this->type ){
            case EMPTY:
	break;
            case FLOAT:
	g->valF = this->valF;
	g->minF = this->minF;
	g->maxF = this->maxF;
	break;
            case INT:
            case ENUM:
	g->valI = this->valI;
	g->minI = this->minI;
	g->maxI = this->maxI;
	break;
            case ENUMPROXY:
	g->enumProxy = this->enumProxy;
        }
        for( int i = 0; i < children.Size(); i ++ )
            g->children.Append( children[i]->Clone() );
        return g;
    }
    void Randomize(){
        switch( this->type ){
            case EMPTY:
	break;
            case FLOAT:
	this->valF = randF( this->minF, this->maxF );
	this->SetDirty();
	break;
            case INT:
            case ENUM:
	this->valI = randI( this->minI, this->maxI );
	this->SetDirty();
	break;
            case ENUMPROXY:
	n_assert( 0 && "trying to randomize a proxy");
	break;
        }
        for( int i = 0; i < children.Size(); i ++ )
            children[i]->Randomize();
    }
    void CrossoverWith( Gene* rhs, float crossoverProb ){
        float r, f1, f2;
        int i1, i2;
        if( roll( crossoverProb ) ){
            switch( type ){
	case EMPTY:
	    break;
	case FLOAT:
	    r = randF(0,1);
	    f1 = this->valF;
	    f2 = rhs->valF;
	    if( this->SetF ((f1 * r) + (f2 * (1.0f-r)) ) )
	        this->SetDirty();
	    if(  rhs->SetF ((f2 * r) + (f1 * (1.0f-r))) )
	        this->SetDirty();
	    break;
	case ENUM:
	    r = randF(0,1);
	    i1 = this->valI;
	    i2 = rhs->valI;
	    if( this->SetE((int) ((i1 * r) + (i2 * (1.0f-r))))) {
	        this->SetDirty();
	    }
	    if(  rhs->SetE((int) ((i2 * r) + (i1 * (1.0f-r))))) {
	        this->SetDirty();
	    }
	    break;
	case INT:
	    r = randF(0,1);
	    i1 = this->valI;
	    i2 = rhs->valI;
	    if( this->SetI((int) ((i1 * r) + (i2 * (1.0f-r))))) {
	        this->SetDirty();
	    }
	    if(  rhs->SetI((int) ((i2 * r) + (i1 * (1.0f-r))))) {
	        rhs->SetDirty();
	    }
	    break;
	case ENUMPROXY:
	    n_assert( 0 && "trying to crossover a proxy" );
	    break;
            }
        }
        for( int i = 0; i < children.Size() && i < rhs->children.Size(); i ++ )
            children[i]->CrossoverWith( rhs->children[i], crossoverProb );
    }
    void Mutate( float mutationProb, bool& evaluated ){
        switch( type ){
            case EMPTY:
	break;
            case FLOAT:
	if( roll( mutationProb ) ){
	    this->SetF( randF( minF, maxF ) );
	    float f = randF( minF, maxF );
	    if( f != this->valF ) {
	        this->valF = f;
	        this->SetDirty();
	    }
	}
	break;
            case INT:
            case ENUM:
	if( roll( mutationProb ) ){
	    this->valI = randI( minI, maxI );
	    this->SetDirty();
	}
	break;
            case ENUMPROXY:
	n_assert( 0 && "trying to mutate a proxy");
	break;
        }
        for( int i = 0; i < children.Size(); i ++)
            children[i]->Mutate( mutationProb, evaluated );
    }
    const char* Name(){
        return name? name : "";
    }
    Gene* GetParent(){ return this->parent; }
    void CastTo( Gene* rhs ){
        log("%s casting to %s", name, rhs->name );
        switch( this->type ) {
            case ENUMPROXY:
	if( rhs->type != ENUM ){
	    log("%s trying to cast a ENUMPROXY into a non ENUM type = %i", GetName(), rhs->type );
	    break;
	}
	this->enumArray = rhs->enumArray;
	n_assert( this->enumArray );
	this->maxI = rhs->maxI;
	this->minI = rhs->minI;
	int i;
	for( i = minI; i <= maxI; i ++ ){
	    if( enumProxy == enumArray[i] ) {
	        this->valI = i;
	        break;
	    }
	}
	if( i > maxI ) {
	    log("%s could not find my enum value: %s in rhs enumarray - defaulting to value 0", name, enumProxy.Get() );
	    this->valI = minI;
	}
	this->type = ENUM;
            break;
            default:
	if( this->type != rhs->type )
	    log("warning, types do not match ( %i != %i )", this->type, rhs->type);
	break;
        }
        int i;
        for( i = 0; i < children; i ++ ) {
            if( i < rhs->children ) { //we both have this many children
	children[i]->CastTo( rhs->children[i] );
            } else {
	log("my child: %s does not exist in the rhs, deleting it", children[i]->GetName() );
	delete children[i];
            }
        }
    }
    void SetInjectedAmount( float f ) {
        this->injectedAmount = f;
    }
    float GetInjectedAmount( ) {
        return this->injectedAmount;
    }
    void SetInjectedValue( float f ){
        this->injectedValue = f;
    }
    float GetInjectedValue( ) {
        return this->injectedValue;
    }
    float GetTotalValue(){
        float value = GetInjectedValue();
        for( int i = 0; i < children; i ++ )
            value += children[i]->GetTotalValue();
        return value;
    }
    float GetTotalInjectedAmount(){
        //at each level find the sum of values / amounts from our children
        if( children.Size() == 0 ) 
            return this->GetInjectedAmount();
        float points = 0.0f;
        float amount, value;
        float totalValue = 0.0f;
        for( int i= 0; i < children; i++ ) {
            amount = children[i]->GetTotalInjectedAmount();
            value  = children[i]->GetInjectedValue();
            points += amount * value;
            totalValue += value;
        }
        this->SetInjectedAmount( points / totalValue );
        return GetInjectedAmount();
    }
    void SetInjected() {
        this->SetInjectedAmount( 1.0 );
        for( int i = 0; i < children; i ++ )
            children[i]->SetInjected();
    }
    static nKernelServer* kernelServer;
    void SetDirty() { //i have changed
        *evaluated = false;
        if( parent )
            parent->SetDirty();
    }
protected:
    void CheckType( GENE_TYPES type ) {
        if( type != this->type ) {
            if( this->type != EMPTY )
    	log("warning..changing gene from type:%i to type:%i", this->type, type );
            if( type == INT )
	n_assert( 0 );
            this->type = type;
        }
    }
    void log( const char* format, ... ) {
        if( kernelServer == 0 )
            return;
        va_list arglist;
        const char* logName = "all gene";
        va_start(arglist, format);
        char s [1024];
        vsprintf( s, format, arglist );
        char s2 [1024];
        sprintf( s2, "logto {%s} {%s}", logName, s );
        RUNSCRIPT( s2 );
        va_end(arglist);
    }
    void calcBitLength(){
        int irange;
        double ilog;
        int numBits;
        float frange;
        float precision;
        switch( type ){
            case EMPTY:
            case ENUMPROXY:
	//_bits.SetFixedSize( 0 );
	break;
            case ENUM:
            case INT:
	irange = maxI - minI +1;
	ilog = log10( irange) / log10(2.0);
	numBits = (int) ceil(ilog);
	log("range: %i lgrange:%f bits:%i", irange, ilog, numBits );
	_bits.SetFixedSize( numBits );
	break;
            case FLOAT:
	frange = maxF - minF;
	precision = frange;
	numBits = 1;
	while( precision > floatPrecision ) {
	    numBits++;
	    precision /= 2.0;
	}
	log("range: %f precision:%f bits:%i", frange, precision, numBits );
	_bits.SetFixedSize( numBits );
	break;
        }
    }
    void updateBits(){
        calcBitLength();
        float f;
        int vi;
        switch(type){
            case EMPTY:
            case ENUMPROXY:
	break;
            case ENUM:
            case INT:
	vi = valI;
	for( int i = _bits.Size()-1; i >= 0; i-- ){
	    _bits[i] = vi%2;
	    vi /= 2;
	}
	n_assert( vi == 0 && "failed to properly convert integer into bitstring");
	break;
            case FLOAT:
	f = ( valF - minF ) / ( maxF - minF );
	float pow = 0.5f;
	for( int i = 0; i < _bits.Size(); i ++ ){
	    if( f > pow ){
	        f -= pow;
	        _bits[i] = 1;
	    } else {
	        _bits[i] = 0;
	    }
	    pow /= 2.0f;
	}
	float precision = f * (maxF - minF);
	log("precision:%f", precision );
	n_assert( precision < floatPrecision );
	break;
        }
    }
    GENE_TYPES type;
    Gene* parent;
    nArray<Gene*> children;
    nArray<bool> _bits;
    const char* name;
    const char** enumArray;
    union {
        int valI;
        float valF;
    };
    union {
        int minI;
        float minF;
    };
    union {
        int maxI;
        float maxF;
    };
    ///minimum precision for float values - changes number of bits used to store data
    float floatPrecision;	//so how do we do the bitstring??
		//well we just have to keep it concurrent with oru values..
		//okay..so doing minI/maxI numbits is easy..just the log2 whatever of it..
		//for floats..we can either define the number of bits..or the precision
		//lets do the precision..that is much better
    nString enumProxy;

    float injectedValue;	//how important is me being injected
    float injectedAmount;	//am I injected?
    bool* evaluated;
};
//------------------------------------------------------------------------------s.Get() );
#endif
